/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 */

/*
aie_qaa.js - AIE module for Question and answers
*/

var AIE = AIE || {};

AIE.Qaa = {
	evalMap: {},
	
	feedbackOptions: {},
	
	dadMap: {},
	
	init: function() {
		this.generateID();
		this.createEvaluationLookup();
		this.createFeedbackLookup();
		AIE.Qaa.TrueFalse.init();
		AIE.Qaa.MultiTrueFalse.init();
		AIE.Qaa.MultiChoice.init();
		AIE.Qaa.MultiResponse.init();
		AIE.Qaa.TextMatch.init();
		AIE.Qaa.MultiTextMatch.init();
		AIE.Qaa.Association.init();
		AIE.Qaa.Association_tap.init();
		AIE.Qaa.Sequence.init();
		AIE.Qaa.UserResultStorage.init();	
		AIE.Qaa.QASet.init();
		AIE.Qaa.SortWords.init();
	},
	
	generateID: function() {
		//Generate ID for each qaa
		//ID pattern qaa{n}
		$('.qaa-rw').each(function(index) {
			var qaaid = 'qaa'+index;
			//Dont lose the original id as it could be used for CSS mapping
			var origid = $(this).attr('id');
			if (origid){ 
				$(this).attr('id', origid);
				qaaid = origid + "___" + qaaid;
			} else {
				qaaid = qaaid + "___" + qaaid;
				$(this).attr('id', qaaid);
			}
			
			//Generate ID for each option source inside the qaa
			//ID pattern for ol option source - qaa{n}_os{n}
			$(this).find('.option-source').each(function(osi){
				var osid = qaaid + "_" + "os" + osi;
				$(this).attr('id', osid);
				$(this).find('li').each(function(lii){
					//ID pattern for LI qaa{n}_os{n}_li{n}
					var liid = osid + "_" + "li" + lii;
					$(this).attr('id', liid);
					$(this).find('span[contenteditable]').each(function(sii){
						//ID pattern for LI qaa{n}_os{n}_li{n}
						var siid = liid + "_sp" + sii;
						$(this).attr('id', siid);
					});
					
					$(this).find('.inputcontent').each(function(sii){
						//ID pattern for LI qaa{n}_os{n}_li{n}
						var siid = liid + "_sp" + sii;
						$(this).attr('id', siid);
					});
				});
			});
			//Generate ID for options target
			$(this).find('.option-target').each(function(oti){
				var osid = qaaid + "_" + "ot" + oti;
				$(this).attr('id', osid);
				$(this).find('li').each(function(olii){
					//ID pattern for LI qaa{n}_os{n}_li{n}
					var liid = osid + "_" + "li" + olii;
					$(this).attr('id', liid);
				});
			});
			
			//Generate ID for each Question control answer map inside the qaa
			//ID pattern for ol option source - qaa{n}_am{n}
			$(this).find('.answer-map').each(function(osi){
				var osid = qaaid + "_" + "am" + osi;
				$(this).attr('id', osid);
				$(this).find('li').each(function(lii){
					//ID pattern for LI qaa{n}_am{n}_li{n}
					var liid = osid + "_" + "li" + lii;
					$(this).attr('id', liid);
				});
			});
		});
	},
	
	/*
		Create a lookup structure to map option source to its target value
	*/
	createEvaluationLookup: function() {
		//Create a lookup structure for li
		$('.qaa-rw').find('.option-source > li').each( function() {
			var optid = $(this).attr('id');
			var targetid = "#" + optid.replace("_os", "_am");
			AIE.Qaa.evalMap[optid] = "";
			if($(targetid).text() || $(targetid).val()) {
				AIE.Qaa.evalMap[optid] = $(targetid).text() || $(targetid).val();
			}
		});
		
		//Now iterate throught all option source
		//AIE.Qaa.evalMap[qaa0_os0_li2_sp0] = actual answer goes here
		$('.qaa-rw').find(".option-source").each(function(){
			//Iterate each span inside the option source
			$(this).find('span[contenteditable]').each( function(counter) {
				var optid = $(this).attr('id');
				var li_id = $(this).parent('li').attr('id');
				
				var opt_index_index = optid.indexOf("___");
				var prefix = optid.substring(0, opt_index_index+3);
				var suffix = optid.substring(opt_index_index+3);
				
				//Span id structure is: qaa0_os0_li1_sp0
				//We are trying to get the index from the span 
				var opt_index_l = optid.split("_");
				
				//Get the last item from the list
				var opt_index_s = opt_index_l[opt_index_l.length-1];
				//Remove sp to get just the number 
				opt_index_s = opt_index_s.replace("sp", "");
				//Convert the string into a number
				var opt_index_index = counter + 1;
				//create the id for the equivalent answermap for this option source
				//Split the suffix after ___ to get the first two parts of the id
				var t = suffix.split("_");
				var temp = prefix + t[0] + "_" + t[1] ;
				var am_id = temp.replace("_os", "_am");
				
				//Now create a filter to get the target answer based on nth-child selector
				var targetfilter =  "#" + am_id + " > li:nth-child(" + opt_index_index + ")";
				var answer = $(targetfilter).text() ||  $(targetfilter).val();
				
				AIE.Qaa.evalMap[optid] = "";
				if(answer) {
					AIE.Qaa.evalMap[optid] = answer;
				}
			});
			
			
			
			$(this).find('.inputcontent').each( function(counter) {
				var optid = $(this).attr('id');
				var li_id = $(this).parent('li').attr('id');
				
				var opt_index_index = optid.indexOf("___");
				var prefix = optid.substring(0, opt_index_index+3);
				var suffix = optid.substring(opt_index_index+3);
				
				//Span id structure is: qaa0_os0_li1_sp0
				//We are trying to get the index from the span 
				var opt_index_l = optid.split("_");
				
				//Get the last item from the list
				var opt_index_s = opt_index_l[opt_index_l.length-1];
				//Remove sp to get just the number 
				opt_index_s = opt_index_s.replace("sp", "");
				//Convert the string into a number
				var opt_index_index = counter + 1;
				//create the id for the equivalent answermap for this option source
				//Split the suffix after ___ to get the first two parts of the id
				var t = suffix.split("_");
				var temp = prefix + t[0] + "_" + t[1] ;
				var am_id = temp.replace("_os", "_am");
				
				//Now create a filter to get the target answer based on nth-child selector
				var targetfilter =  "#" + am_id + " > li:nth-child(" + opt_index_index + ")";
				var answer = $(targetfilter).text() ||  $(targetfilter).val();
				
				AIE.Qaa.evalMap[optid] = "";
				if(answer) {
					AIE.Qaa.evalMap[optid] = answer;
				}
			});
		});
	},
	
	/* */
	createFeedbackLookup: function() {
		//Generate ID for each qaa
		//ID pattern qaa{n}
		$('.qaa-rw').each(function(index) {
			var qaaid = $(this).attr('id');
			//Load the feedback messages class and message for this qaa div
			AIE.Qaa.feedbackOptions[qaaid] = {};
			var qacl = $(this).attr('class');
			qacl = qacl.replace("qaa-rw", "");
			qacl = jQuery.trim(qacl);
		
			AIE.Qaa.feedbackOptions[qaaid]["qaatype"] = qacl;
			AIE.Qaa.feedbackOptions[qaaid]["evaluation"] = $(this).find(".evaluation").text();
			AIE.Qaa.feedbackOptions[qaaid]["feedback"] = $(this).find(".feedback").text();
			AIE.Qaa.feedbackOptions[qaaid]["results"] = $(this).find(".results").text();
			$(this).find('.feedback-messages').children().each(function(){
				var cl = $(this).attr('class');
				AIE.Qaa.Association_tapl = cl.replace("fbm", "");
				cl = jQuery.trim(cl);
				
				switch (cl) {
					case "correct":
						AIE.Qaa.feedbackOptions[qaaid]["correct"] = $(this).text();
						break;
					case "wrong":
						AIE.Qaa.feedbackOptions[qaaid]["wrong"] = $(this).text();
						break;
					case "reinforcement":
						AIE.Qaa.feedbackOptions[qaaid]["reinforcement"] = $(this).html();
						break;
				}
			});
		});
	},
	
	showFeedback: function($parentobj, state) {
		var rl = 'reinforcement';
		//Create the dynamic eventname
		var eventname = $parentobj.attr("id") + "-";
		
		if (state == true) {
			//user has got it right
			//Show the correct message
			var cl = AIE.Qaa.getCorrectFeedbackClass($parentobj.attr("id"));
			$parentobj.find(" > .feedback-messages > ." + cl).show();
			$parentobj.find(" > .feedback-messages > .wrong").hide();
			//Hide the reinforcement if it is displayed
			$parentobj.find(" > .feedback-messages > ." + rl).hide();
			$parentobj.find(" > .feedback-messages").show();
			eventname = eventname + "correct";
		} else {
			//user has got it wrong
			//hide the correct message
			var cl = AIE.Qaa.getCorrectFeedbackClass($parentobj.attr("id"));
			
			$parentobj.find(" > .feedback-messages > ." + cl).hide();
			$parentobj.find(" > .feedback-messages > .wrong").show();
			//show the reinforcement if it is displayed
			$parentobj.find(" > .feedback-messages > ." + rl).show();
			$parentobj.find("> .feedback-messages").show();
			eventname = eventname + "wrong";
		}
		//Publish the dynamic event
		$.publish(eventname, []);
	
	},
	
	evaluate: function(qaaobj, selobj) {
		var qaaid = qaaobj.attr('id');
		var fb = AIE.Qaa.feedbackOptions[qaaid]["feedback"];
		if(fb){
			//Handle the different types of feedback here
			switch (fb) {
				
				default:
					//Default is no feedback on a user event
					break;
			}
		}
	},
	
	getAnswer: function(elid) {
		var ans = AIE.Qaa.evalMap[elid];
		ans = jQuery.trim(ans);
		return ans;
	},
	
	getWrongFeedbackClass: function(qaaid) {
		return "wrong";
	},
	
	getCorrectFeedbackClass: function(qaaid) {
		return "correct";
	},
	
	resetOptions: function($parentobj) {
		$parentobj.find('.option-source >li').each(function(){
			$(this).removeClass('wrong');
			$(this).removeClass('correct');
			$(this).removeClass('selected-aie');
		});
		//Hide the feedback message
		$parentobj.find(".feedback-messages").hide();
	},
	
	retryOptions: function($parentobj) {

		var  id = $parentobj.attr("id")
		$("#"+id+" ol").find('.wrong').each(function() {
			$(this).removeClass('wrong');
		});
		//Hide the feedback message
		$parentobj.find(".feedback-messages").hide();
	},
	
	checkAnswers: function($parentobj){
		//we declare the variable here to check if any one of the answers
		//is wrong. If that happens, we need to show wrong message
		var eval_status = false;
		var once = true;
		$parentobj.find('.option-source >li').each(function() {
			if (once) {
				//set the eval_status to true for first time
				eval_status = true;
				once = false;
			}
			
			var answer = AIE.Qaa.getAnswer($(this).attr("id"));
			 
			if ($(this).hasClass('selected-aie') == false) {
				//We need to make sure that we show a reenforcement
				//if user does not attempt something, But the answer is true
				if ((answer == "1") && ($(this).hasClass('correct') == false)) eval_status = false;
			} else {
				switch (answer) {
					case "0":
						var cl = AIE.Qaa.getWrongFeedbackClass($parentobj.attr("id"));
						$(this).addClass(cl).removeClass('selected-aie');
						eval_status = false;
						break;
					case "1":
						var cl = AIE.Qaa.getCorrectFeedbackClass($parentobj.attr("id"));
						$(this).addClass(cl).removeClass('selected-aie');
						break;
					default:
						var cl = AIE.Qaa.getWrongFeedbackClass($parentobj.attr("id"));
						eval_status = false;
						$(this).addClass(cl).removeClass('selected-aie');
				}
			}
		});
		AIE.Qaa.showFeedback($parentobj, eval_status);
	},
	
	checkTextMatchAnswers: function($parentobj){
		var eval_status = false;
		var once = true;
		var parentid = $parentobj.attr('id');
	 
		if($("#" + parentid + " input").hasClass('inputcontent')){
			
			var filter = "#" + parentid + ' .inputcontent';
			
		 }
		 else{
				var filter = "#" + parentid + ' span[contenteditable]' ;
		 }
		
		 
		$(filter).each(function() {
			if (once) {
				//set the eval_status to true for first time
				eval_status = true;
				once = false;
			}
			var user_answer = $(this).text() || $(this).val();
			 
			 
			user_answer = jQuery.trim(user_answer);
			var sys_answer = AIE.Qaa.getAnswer($(this).attr("id"));
			$(this).removeClass('selected-aie');
			$(this).removeClass('wrong');
			$(this).removeClass('correct');
			
			var correct_answer = false;
			//Handle the multi answers here
			var multi = [];
			//multiple options are specified as quotes wrapped comma separated
			if (sys_answer[0] == "\"") {
				var tmulti = sys_answer.split('"');
				for (var k=0; k<tmulti.length; k++){
					var option = tmulti[k];
					option = jQuery.trim(option);
					if (option && (option!=",") && (option!="\"") ) multi.push(tmulti[k]);
				}
			} else {			//multiple options are specified as comma separated
				var multi = sys_answer.split(',');
			}
			if (multi.length > 1) {
				for (var k=0; k<multi.length; k++){
					var option = multi[k];
					option = jQuery.trim(option);
					if (user_answer == option){
						correct_answer = true;
						break
					}
				}
			} else {
				if (multi.length == 1) sys_answer =  multi[0];
				//Single answer option
				if (user_answer == sys_answer){
					correct_answer = true;
				}
			}

			if (correct_answer) {
				var cl = AIE.Qaa.getCorrectFeedbackClass($parentobj.attr("id"));
				$(this).addClass(cl);
			} else {
				if (($(this).get(0).innerHTML == "&nbsp;") ||  (user_answer == "")){
					//Dont set a error class if user has not attemped the option
					eval_status = false;
				} else {
					var cl = AIE.Qaa.getWrongFeedbackClass($parentobj.attr("id"));
					eval_status = false;
					$(this).addClass(cl);
				}
			}
		});
		AIE.Qaa.showFeedback($parentobj, eval_status);
	},
	
	resetTextMatch: function($parentobj){
		var parentid = $parentobj.attr('id');
	 
		if($("#" + parentid + " input").hasClass('inputcontent')){
			
			var filter = "#" + parentid + ' .inputcontent';
			
		 }
		 else{
				var filter = "#" + parentid + ' span[contenteditable]' ;
		 }
		
		//var filter = "#" + parentid + ' span[contenteditable]' || "#" + parentid + ' .inputcontent';
		$(filter).each(function() {
			$(this).removeClass('wrong');
			$(this).removeClass('correct');
			$(this).removeClass('selected-aie');
			if($(this).hasClass('inputcontent')) {
				$(this).val(" ");	
			} else {
				$(this).text(" ");	
			}	
			var index = $(this).attr('class').replace(/\D/g,'') -1 ;
			$($parentobj.children('.ans-suggestions').find('li:eq('+index +')').show()).css({left:'0px',top:'0px'});
		});
		//Hide the feedback message
		$parentobj.find(".feedback-messages").hide();
	},
	
	retryTextMatch: function($parentobj){
		var parentid = $parentobj.attr('id');
		 
		//var filter = "#" + parentid + ' span[contenteditable]' || "#" + parentid + ' .inputcontent';
		if($("#" + parentid + " input").hasClass('inputcontent')){
			
			var filter = "#" + parentid + ' .inputcontent';
			
		 }
		 else{
				var filter = "#" + parentid + ' span[contenteditable]' ;
		 }
		
	 
		$(filter).each(function() {
			if ($(this).hasClass('wrong')) {
				var index = $(this).attr('class').replace(/\D/g,'') -1 ;
				$($parentobj.children('.ans-suggestions').find('li:eq('+index +')').show()).css({left:'0px',top:'0px'});
				$(this).removeClass('wrong');
				$(this).removeClass('correct');
				$(this).removeClass('selected-aie');
				if($(this).hasClass('inputcontent')) {
	
						$(this).val(" ");	
	
					} else {
						$(this).text(" ");	
					}	
				
				
							
			}
		});
		//Hide the feedback message
		$parentobj.find(".feedback-messages").hide();
	
	},
	
	
	checkDADMatchLive: function($parentobj) {
		var result = true;
		$parentobj.find('.option-target').children().each(function(){
			if($(this).hasClass('selected-aie')) {
				var target_id = $(this).attr('id');
				//check if the map has src for the selected target
				if (AIE.Qaa.dadMap[target_id]) {
					//get the src id
					var src_id = AIE.Qaa.dadMap[target_id];
					//now use the src id to get the class
					var sClass = $("#" + src_id).attr('class').split(' ')[0];
					//extract the src class index
					var sClassIndex = sClass.replace(/\D/g,'');
					//now use the target id to get the class
					var tClass = $(this).attr('class').split(' ')[0];
					var tClassIndex = tClass.replace(/\D/g,'');
					//Compare the src and target index
					if (sClassIndex == tClassIndex) {
						//do nothing
					} else {
						var srcdata = $("#" + src_id).text();
						var targetid = "#" + target_id.replace("_ot", "_am");
						var targetanswer = $(targetid).text();												if (srcdata != targetanswer) {
							result = false;
						}
					}
				} else {
					result = false;
				}
			}else {
				//looks like user has some items remaining to drop
				result = false;
			}
		});
		return result;		
	},
	
	hasUserFinishedDrop: function($parentobj) {
		var result = true;
		$parentobj.find('.option-target').children().each(function(){
			if($(this).hasClass('selected-aie')) {
				
			}else {
				//looks like user has some items remaining to drop
				result = false;
			}
		});
		return result;	
	},
	
	checkDADMatch: function($parentobj) {
		//$(".non-draggable").removeClass("non-draggable").addClass("ui-draggable");  
		$(".non-draggable").removeClass("selected-aie") 
		//Variable used for displaying the feedback
		var eval_status = false;
		var once = true;

		$parentobj.find('.option-target').children().each(function(){
			if (once) {
				//set the eval_status to true for first time
				eval_status = true;
				once = false;
			}
			if($(this).hasClass('selected-aie')) {
				var target_id = $(this).attr('id');
				//check if the map has src for the selected target
				if (AIE.Qaa.dadMap[target_id]) {
					//get the src id
					var src_id = AIE.Qaa.dadMap[target_id];
					//now use the src id to get the class
					var sClass = $("#" + src_id).attr('class').split(' ')[0];
					//extract the src class index
					var sClassIndex = sClass.replace(/\D/g,'');
					//now use the target id to get the class
					var tClass = $(this).attr('class').split(' ')[0];
					var tClassIndex = tClass.replace(/\D/g,'');
					//Compare the src and target index
					if (sClassIndex == tClassIndex) {
						//Show correct response
						var cl = AIE.Qaa.getCorrectFeedbackClass($parentobj.attr("id"));
						$(this).addClass(cl);
					} else {
						//Show error response
						var cl = AIE.Qaa.getWrongFeedbackClass($parentobj.attr("id"));
						eval_status = false;
						$(this).addClass(cl).removeClass('selected-aie');
					}
					$(this).removeClass('selected-aie');
				} else {
					//Show error response
					var cl = AIE.Qaa.getWrongFeedbackClass($parentobj.attr("id"));
					eval_status = false;
					$(this).addClass(cl).removeClass('selected-aie');
				}
			}else {
				//Make sure that you handle the case where something was 
				//checked properly in the previous check event and is not selected
				if ($(this).hasClass('correct') == false) {
					eval_status = false;
				}
			}
		});
		
		AIE.Qaa.showFeedback($parentobj, eval_status);
						
	},
	
	retryDAD: function($parentobj) {
		
		$(".non-draggable").removeClass("non-draggable").addClass("ui-draggable");  
		$( ".non-droppable").removeClass("non-droppable").addClass("ui-droppable"); 
		//$("option-source .ui-draggable").removeClass("selected-aie");
		$parentobj.find('.option-target').children().each(function(){
			//Check if user has done a drag operation and it is wrong
			if($(this).hasClass('wrong')) {
				var target_id = $(this).attr('id');
				if (AIE.Qaa.dadMap[target_id]) {
					//get the src id
					var src_id = AIE.Qaa.dadMap[target_id];
					//now use the src id to get the class
					var sClass = $("#" + src_id).attr('class').split(' ')[0];
					//extract the src class index
					var sClassIndex = sClass.replace(/\D/g,'');
					var srcclass ='.s'+ sClassIndex;
					var src_selector = '.option-source >' + srcclass;
					$parentobj.find(src_selector).css({top:'0px',left:'0px'});
					
					$(this).removeClass('selected-aie');
					$(this).removeClass('wrong');
					$(this).removeClass('correct');
				}
				if (AIE.Qaa.dadMap[target_id]) {
					delete AIE.Qaa.dadMap[target_id];
				}
			}
		});
		//Hide the feedback message
		$parentobj.find(".feedback-messages").hide();
	},
	
	resetDAD: function($parentobj) {
		
	    $($parentobj).attr('id');
		var elem=  $($parentobj).attr('id');
		
		var association_id= elem
		AIE.Qaa.LocalStorage.remove('pathname_user');
		var pathname_user= window.location.pathname;
		AIE.Qaa.LocalStorage.save(pathname_user, pathname_user);
	  	AIE.Qaa.LocalStorage.remove(pathname_user+'result_combination');
	 	AIE.Qaa.LocalStorage.remove(pathname_user+'Source_target_combination')
	  	AIE.Qaa.LocalStorage.remove('pathname1');
		var pathname1 = window.location.pathname;
	 	AIE.Qaa.LocalStorage.save(pathname1, pathname1);
        $('#'+association_id+' .check').attr("disabled", false);
  		$('#'+association_id+' .try-again').attr("disabled", false);
			var wrong_cookie = (AIE.Qaa.LocalStorage.remove(pathname1+association_id+'wrong'));
		  	var correct_cookie = (AIE.Qaa.LocalStorage.remove(pathname1+association_id+'correct_ans'));
		  	var reinforcement_cookie = (AIE.Qaa.LocalStorage.remove(pathname1+association_id+'reinforcement'));
		  	AIE.Qaa.LocalStorage.remove(pathname_user+association_id+'Source_target_combination')
			AIE.Qaa.LocalStorage.remove('pathname_user');
			var pathname_user= window.location.pathname;
			AIE.Qaa.LocalStorage.save(pathname_user, pathname_user);
			$parentobj.find('.option-target').children().each(function(){
			var target_id = $(this).attr('id');
			$(this).removeClass('wrong');
			$(this).removeClass('correct')
			
			if(AIE.Qaa.dadMap[target_id]) {
				//get the src id
				var src_id = AIE.Qaa.dadMap[target_id];
				//now use the src id to get the class
				var sClass = $("#" + src_id).attr('class').split(' ')[0];
				//extract the src class index
				var sClassIndex = sClass.replace(/\D/g,'');
					
				var srcclass ='.s'+ sClassIndex;
				var src_selector = '.option-source >' + srcclass;
				$parentobj.find(src_selector).css({top:'0px',left:'0px'});
		 
				$(this).removeClass('selected-aie');
				$(this).removeClass('wrong');
				$(this).removeClass('correct');	 
			}
			
			if (AIE.Qaa.dadMap[target_id]) {
				delete AIE.Qaa.dadMap[target_id];
			}
		});
		//Hide the feedback message
		$parentobj.find(".feedback-messages").hide();
		
		//Setup event handler for evaluation
		$('.'+association_id+ ' > .option-source').each(function(){
			//Make sure you shuffle only if the author wants it
			if ($(this).hasClass('shuffle')) {
				$(this).shuffle();
			}
		});
		
		var content_area='$$$$'
			$('.option-target li').each( function(index) {
				var tcontent_area= (index + ': ' + $(this).attr('class').split(' ')[2]);
				content_area= content_area + tcontent_area+"$$$$";
			});
			var result_combination= content_area;
			AIE.Qaa.LocalStorage.save(pathname_user+'result_combination', result_combination);
	
		
		$(".option-target li").removeClass("  undefined")
		//	$( ".non-droppable").removeClass("non-droppable")
		$(".option-target li").removeClass("selected-aie wrong correct undefined")
		
		$( ".non-droppable").removeClass("non-droppable").addClass("ui-droppable"); 
		$(".ui-draggable").removeClass("selected-aie");
		
		var user_ans=(AIE.Qaa.LocalStorage.load(pathname_user+'result_combination'));
		if (user_ans != 'null') {	 
			 
			var user_ans_split = user_ans.split('$$$$');
			var user_ans_split_data = (user_ans_split.length-1);
			var user_ans_split_index = 0;
		
			for (var i=0; i < user_ans_split_data; i++) {
				user_ans_split_index = user_ans_split[i].split(': ');
				  $('.option-target li').each( function(index) {	
					for (var i=0; i < user_ans_split_index.length; i++) {
			 			if(index == user_ans_split_index[0]) {
			 				 
			 				$(this).addClass(user_ans_split_index[1]);
			 			}
					};
				 });			 
			};
		}		
	 
	 $('#'+elem+" .option-source li").each( function(index) {
	 		 $(this).css( {"left" :0 , "top" :0});
	  });
	
		 
		//  location.reload();
	},
	
	evaluateOptions: function($sel_obj){
		var isEval = false;
		$qaaobj = $sel_obj.parents('.qaa-rw');
		AIE.Qaa.evaluate($qaaobj, $sel_obj);
		//Reset selection of other classes
		$optionsrc = $sel_obj.parent();
		$optionsrc.children().each(function(){
			if (($(this).hasClass('correct')) ||  ($(this).hasClass('wrong'))){
				isEval = true;
			}
		});
		
		if (isEval == false) {
			$optionsrc.children().removeClass('selected-aie');
			//set selection on the current item
			$sel_obj.addClass('selected-aie');
		}
	}
};

AIE.Qaa.TrueFalse = {
	init:function(){
		//Setup event handler for evaluation
		$('.truefalse-rw >.option-source >li').click(function(){
			AIE.Qaa.evaluateOptions($(this));
		});	
		
		//Setup event handler for CHECK button
		$('.truefalse-rw >.buttons-rw > .check').click(function(){




			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkAnswers($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.truefalse-rw > .buttons-rw > .try-again').click(function(){


















			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryOptions($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.truefalse-rw > .buttons-rw > .reset').click(function(){










			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetOptions($parentobj);
		});	
	}	
};

AIE.Qaa.MultiTrueFalse = {
	init:function(){
		//Setup event handler for evaluation
		$('.multi-truefalse-rw').find('.option-source >li').click(function(){
			AIE.Qaa.evaluateOptions($(this));
		});	
		
		//Setup event handler for CHECK button
		$('.multi-truefalse-rw >.buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkAnswers($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.multi-truefalse-rw > .buttons-rw > .try-again').click(function(){



			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryOptions($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.multi-truefalse-rw > .buttons-rw > .reset').click(function(){
















			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetOptions($parentobj);
		});	
	}	
};
	
AIE.Qaa.MultiChoice = {
	init:function(){
		//Setup event handler for evaluation
		$('.multichoice-rw').find('.option-source >li').click(function(){
			AIE.Qaa.evaluateOptions($(this));
		});	
		
		//Setup event handler for CHECK button
		$('.multichoice-rw >.buttons-rw > .check').click(function(){


			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkAnswers($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.multichoice-rw > .buttons-rw > .try-again').click(function(){


			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryOptions($parentobj);

		});	
		
		//RESET  EVENT HANDLER
		$('.multichoice-rw > .buttons-rw > .reset').click(function(){























			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetOptions($parentobj);
		});	
	}	
};

AIE.Qaa.MultiResponse = {
	init:function(){
		//Setup event handler for evaluation
		$('.multiresponse-rw').find('.option-source >li').click(function(event){
			//Removed the requirement for CTRL key as per client reqt - 20111121 - DC
			// if (event.ctrlKey) {
				$qaaobj = $(this).parents('.qaa-rw');
				AIE.Qaa.evaluate($qaaobj, $(this));
				//Reset selection of other classes
				$optionsrc = $(this).parent();
				//set selection on the current item
				$(this).toggleClass('selected-aie');
			// }
		});	
		
		//Setup event handler for CHECK button
		$('.multiresponse-rw >.buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkAnswers($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.multiresponse-rw > .buttons-rw > .try-again').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryOptions($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.multiresponse-rw > .buttons-rw > .reset').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetOptions($parentobj);
		});	
	}	
};

AIE.Qaa.TextMatch = {
	dragItems:function(){
		
		$('.textmatch-rw >.ans-suggestions >li').draggable({
			revert:"invalid"
//			$(this).addClass('selected-aie');	
		});
		
		$('.textmatch-rw >.option-source >li').find('span:eq('+1+')').droppable({
			drop:function(e,ui){
				$(ui.draggable).offset($(this).offset())
				$(this).text($(ui.draggable).text());
				$($(ui.draggable)).hide();
			}
//			$(this).addClass('selected-aie');	
		});
		
	}, 	
	init:function(){
		//Setup event handler for evaluation
		$('.textmatch-rw > .option-source >li >span').blur(function(){
			$(this).addClass('selected-aie');	
		});
		
		//Setup event handler for CHECK button
		$('.textmatch-rw >.buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkTextMatchAnswers($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.textmatch-rw > .buttons-rw > .try-again').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryTextMatch($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.textmatch-rw > .buttons-rw > .reset').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetTextMatch($parentobj);
		});	
	}	
};



AIE.Qaa.LocalStorage = {
	getAzardiLS: function() {
		var url = "http://infogridpacific.com";
		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var ssm = Components.classes["@mozilla.org/scriptsecuritymanager;1"].getService(Components.interfaces.nsIScriptSecurityManager);
		var dsm = Components.classes["@mozilla.org/dom/storagemanager;1"].getService(Components.interfaces.nsIDOMStorageManager);
 
   		var uri = ios.newURI(url, "", null);
		var principal = ssm.getCodebasePrincipal(uri);
		var storage = dsm.getLocalStorageForPrincipal(principal, "");
		
		return storage;
	},
	
	save: function(key, content) {
		return;
		if(parent.AME.getAppName()) {
			var storage = this.getAzardiLS();
			var cdpId = parent.Azd.getCDPID();
			var acoId = parent.CdpNav.getACOID();
			key = cdpId+'-'+acoId+'-'+key;
			storage.setItem(key, content);
		} else {
			window.localStorage.setItem(key, content);
		}
	},

	load: function(key) {
		return;
		if(parent.AME.getAppName()) {
			var storage = this.getAzardiLS();
			var cdpId = parent.Azd.getCDPID();
			var acoId = parent.CdpNav.getACOID();
			key = cdpId+'-'+acoId+'-'+key;
			return  storage.getItem(key);
		} else {
			return  localStorage.getItem(key);
		}
	},
	
	remove: function(key) {
		return;
		if(parent.AME.getAppName()) {
			var storage = this.getAzardiLS();
			var cdpId = parent.Azd.getCDPID();
			var acoId = parent.CdpNav.getACOID();
			key = cdpId+'-'+acoId+'-'+key;
			storage.removeItem(key);
		} else {
			window.localStorage.removeItem(key);
		}
	}
};


AIE.Qaa.UserResultStorage = {
	init: function() {
		
		AIE.Qaa.LocalStorage.remove('pathname1');
		var pathname1 = window.location.pathname;
	 	AIE.Qaa.LocalStorage.save(pathname1, pathname1);
		var getstr = AIE.Qaa.LocalStorage.load(pathname1);
	// Remove the local storage varible saved by the user.	 
	  	$(".reset").click(function(){
	  		AIE.Qaa.LocalStorage.remove(pathname1+'wrong');	
	  		AIE.Qaa.LocalStorage.remove(pathname1+'correct_ans');	
	  		AIE.Qaa.LocalStorage.remove(pathname1+'reinforcement');	
	  	});
	  	
	  	AIE.Qaa.UserResultStorage.saveUserResult();
  		AIE.Qaa.UserResultStorage.loadUserResult();
	},
	
	// Function to store the result of the user in local storage
	 saveUserResult: function() {
	  
		AIE.Qaa.LocalStorage.remove('pathname1');
		var pathname1 = window.location.pathname;
	 	AIE.Qaa.LocalStorage.save(pathname1, pathname1);
		var getstr = AIE.Qaa.LocalStorage.load(pathname1);
	  
		var flag_check = 0;
	 		$(".check").click(function(){	
	 			$(".non-droppable").removeClass("undefined")  	
			flag_check = 1;
		});
		
		$(document).mousedown(function() {
			
			if (flag_check == 1) {
				AIE.Qaa.LocalStorage.remove('pathname_user');
				var pathname_user= window.location.pathname;
				AIE.Qaa.LocalStorage.save(pathname_user, pathname_user);
				var getstr = AIE.Qaa.LocalStorage.load(pathname_user);

				//to store color value of the user last result
				var content_area='$$$$'
				$('.option-target li').each( function(index) {
					var tcontent_area= (index + ': ' + $(this).attr('class').split(' ')[2]);
					content_area= content_area + tcontent_area+"$$$$";
				});
				var result_combination= content_area;
				AIE.Qaa.LocalStorage.save(pathname_user+'result_combination', result_combination);

				$('.association-rw').each( function(index) {






					var association_id= $(this).attr("id")
					AIE.Qaa.LocalStorage.remove(pathname1+association_id+'wrong');
					AIE.Qaa.LocalStorage.remove(pathname1+association_id+'correct_ans');
					AIE.Qaa.LocalStorage.remove(pathname1+association_id+'reinforcement');
					var wrong = $("#"+association_id+' .wrong').is(':visible');
					var correct_ans = $("#"+association_id+" .correct").is(':visible');
					var reinforcement = $("#"+association_id+" .reinforcement").is(':visible');
					AIE.Qaa.LocalStorage.save(pathname1+association_id+'wrong', wrong);
					AIE.Qaa.LocalStorage.save(pathname1+association_id+'correct_ans', correct_ans);
					AIE.Qaa.LocalStorage.save(pathname1+association_id+'reinforcement', reinforcement);


				});
				
				
				

				$('.qaa-rw').each( function(index) {
					





					var truefalse_id= $(this).attr("id")
					 
					AIE.Qaa.LocalStorage.remove(pathname1+truefalse_id+'wrong');	
					AIE.Qaa.LocalStorage.remove(pathname1+truefalse_id+'correct_ans');	
					AIE.Qaa.LocalStorage.remove(pathname1+truefalse_id+'reinforcement');	
					var wrong = $("#"+truefalse_id+' .wrong').is(':visible');
					var correct_ans = $("#"+truefalse_id+" .correct").is(':visible');
					var reinforcement = $("#"+truefalse_id+" .reinforcement").is(':visible');
					AIE.Qaa.LocalStorage.save(pathname1+truefalse_id+'wrong', wrong); 
					AIE.Qaa.LocalStorage.save(pathname1+truefalse_id+'correct_ans', correct_ans);
					AIE.Qaa.LocalStorage.save(pathname1+truefalse_id+'reinforcement', reinforcement);





				});







































			} 
		});
		 
		// to save the user result on any unload function of the window 
		$(window).unload( function() {
			
			if (flag_check == 1) {

				AIE.Qaa.LocalStorage.remove('pathname_user');
				var pathname_user= window.location.pathname;
				AIE.Qaa.LocalStorage.save(pathname_user, pathname_user);
				var getstr = AIE.Qaa.LocalStorage.load(pathname_user);

				//to store color value of the user last result
				var content_area='$$$$'
				$('.option-target li').each( function(index) {
					var tcontent_area= (index + ': ' + $(this).attr('class').split(' ')[2]);
					content_area= content_area + tcontent_area+"$$$$";
				});
				var result_combination= content_area;
				AIE.Qaa.LocalStorage.save(pathname_user+'result_combination', result_combination);


				$('.association-rw').each( function(index) {
					

					var association_id= $(this).attr("id")
					AIE.Qaa.LocalStorage.remove(pathname1+association_id+'wrong');	
					AIE.Qaa.LocalStorage.remove(pathname1+association_id+'correct_ans');	
					AIE.Qaa.LocalStorage.remove(pathname1+association_id+'reinforcement');	
					var wrong = $("#"+association_id+' .wrong').is(':visible');
					var correct_ans = $("#"+association_id+" .correct").is(':visible');
					var reinforcement = $("#"+association_id+" .reinforcement").is(':visible');
					AIE.Qaa.LocalStorage.save(pathname1+association_id+'wrong', wrong); 
					AIE.Qaa.LocalStorage.save(pathname1+association_id+'correct_ans', correct_ans);
					AIE.Qaa.LocalStorage.save(pathname1+association_id+'reinforcement', reinforcement);


				});
				
				
				$('.qaa-rw').each( function(index) {
					
					var truefalse_id= $(this).attr("id")
					 
					AIE.Qaa.LocalStorage.remove(pathname1+truefalse_id+'wrong');	
					AIE.Qaa.LocalStorage.remove(pathname1+truefalse_id+'correct_ans');	
					AIE.Qaa.LocalStorage.remove(pathname1+truefalse_id+'reinforcement');	
					var wrong = $("#"+truefalse_id+' .wrong').is(':visible');
					var correct_ans = $("#"+truefalse_id+" .correct").is(':visible');
					var reinforcement = $("#"+truefalse_id+" .reinforcement").is(':visible');
					AIE.Qaa.LocalStorage.save(pathname1+truefalse_id+'wrong', wrong); 






					AIE.Qaa.LocalStorage.save(pathname1+truefalse_id+'correct_ans', correct_ans);
					AIE.Qaa.LocalStorage.save(pathname1+truefalse_id+'reinforcement', reinforcement);






				});



































			}
		});
	 },
	 
     // Function to load the result of the user from local storage 
	 loadUserResult: function() {

		AIE.Qaa.LocalStorage.remove('pathname1');
		var pathname1 = window.location.pathname;
	 	AIE.Qaa.LocalStorage.save(pathname1, pathname1);
		var getstr = AIE.Qaa.LocalStorage.load(pathname1);




	$('.association-rw').each( function(index) {
		
		var association_id= $(this).attr("id")
		var wrong_cookie = (AIE.Qaa.LocalStorage.load(pathname1+association_id+'wrong'));
	  	var correct_cookie = (AIE.Qaa.LocalStorage.load(pathname1+association_id+'correct_ans'));
	  	var reinforcement_cookie = (AIE.Qaa.LocalStorage.load(pathname1+association_id+'reinforcement'));
	  	if (wrong_cookie == "true") {
	  		$("#"+association_id+" .wrong").show();
	  	}
	  	
	  	if (correct_cookie == "true" && reinforcement_cookie == "false") {
	  		$("#"+association_id+" .correct").show();
	  	}
	  	
	  	if (reinforcement_cookie == "true") {
	  		$("#"+association_id+" .reinforcement").show();
	  	}
	  	
	});
	
	
	
	
	$('.qaa-rw').each( function(index) {
		
		var truefalse_id= $(this).attr("id")
		var wrong_cookie = (AIE.Qaa.LocalStorage.load(pathname1+truefalse_id+'wrong'));
	  	var correct_cookie = (AIE.Qaa.LocalStorage.load(pathname1+truefalse_id+'correct_ans'));
	  	var reinforcement_cookie = (AIE.Qaa.LocalStorage.load(pathname1+truefalse_id+'reinforcement'));
	  	if (wrong_cookie == "true") {
	  		$("#"+truefalse_id+" .wrong").show();
	  	}
	  	









	  	if (correct_cookie == "true" && reinforcement_cookie == "false") {
	  		$("#"+truefalse_id+" .correct").show();
	  	}
	  	

	  	if (reinforcement_cookie == "true") {
	  		$("#"+truefalse_id+" .reinforcement").show();
	  	}
	  	

	});
	  	


















	  //	var user_ans = '';
  		AIE.Qaa.LocalStorage.remove('pathname_user');
		var pathname_user= window.location.pathname;
		AIE.Qaa.LocalStorage.save(pathname_user, pathname_user);
		//var getstr = AIE.Qaa.LocalStorage.load(pathname_user);
		var user_ans=(AIE.Qaa.LocalStorage.load(pathname_user+'result_combination'));
		  
		
	if (user_ans != "null") {		
			
			if (user_ans) { 
			 
			var user_ans_split = user_ans.split('$$$$');
			var user_ans_split_data = (user_ans_split.length-1);
			var user_ans_split_index = 0;
		
			for (var i=0; i < user_ans_split_data; i++) {
				user_ans_split_index = user_ans_split[i].split(': ');
				  $('.option-target li').each( function(index) {	
					for (var i=0; i < user_ans_split_index.length; i++) {
			 			if(index == user_ans_split_index[0]) {
			 				$(this).addClass(user_ans_split_index[1]);
			 			}
					};
				 });			 
			};
		}
	}			
	  	















































		$('.association-rw .option-source .ui-draggable').each( function(index) {
			var elem= $(this) 
   		    var association_id= $(elem).parents('.qaa-rw').attr("id")
		  	//AIE.Qaa.LocalStorage.save(pathname_user+'Source_target_combination', Source_target_combination);
		  	var user_ans_value=(AIE.Qaa.LocalStorage.load(pathname_user+association_id+'Source_target_combination'));
		   	$('#'+association_id+' .check').attr("disabled", false);
			$('#'+association_id+' .try-again').attr("disabled", false);
		   
	  	if (user_ans_value != null) {		 
	  		  	$('#'+association_id+' .check').attr("disabled", true);
			  	$('#'+association_id+' .try-again').attr("disabled", true);
			  
				var user_ans_value_split = user_ans_value.split('$$$$');
				var user_ans_value_split_data = (user_ans_value_split.length-1);
				var user_ans_value_split_index = 0;
			
				for (var i=0; i < user_ans_value_split_data; i++) {
					user_ans_value_split_index = user_ans_value_split[i].split(': ');
					  $('#'+association_id+' .option-source li').each( function(index) {	
						for (var i=0; i < user_ans_value_split_index.length; i++) {
								var source = "s"+user_ans_value_split_index[1]		
								var target = "t"+user_ans_value_split_index[0]
							    $('#'+association_id+" .option-source"+" ."+source).offset($('#'+association_id+" .option-target"+" ."+target).offset());
						};
					});			 
				};
			}
		});
	}
};



AIE.Qaa.MultiTextMatch = {
	init:function(){
	
		//Setup event handler for evaluation
		$('.multi-textmatch-rw > .option-source >li >span').blur(function(){
			$(this).addClass('selected-aie');	
		});
		
		//Setup event handler for CHECK button
		$('.multi-textmatch-rw >.buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkTextMatchAnswers($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.multi-textmatch-rw > .buttons-rw > .try-again').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryTextMatch($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.multi-textmatch-rw > .buttons-rw > .reset').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetTextMatch($parentobj);
		});	
	}	
};



AIE.Qaa.Association_tap = {
	init: function() {
		
		$('.association-rw .option-source .ui-draggable').each( function(index) {
			var association_id= $(this).attr("id")
			AIE.Qaa.Association_tap.save_tap_results(this);
		});
	},
	
	save_tap_results: function(elem) {

		var Source_target_string=''
		AIE.Qaa.LocalStorage.remove('pathname_user');
		var pathname_user= window.location.pathname;
		AIE.Qaa.LocalStorage.save(pathname_user, pathname_user);
		var association_id= $(elem).parents('.qaa-rw').attr("id")
	
		$(elem).click( function() {
			$('.association-rw .option-source .ui-draggable').removeClass("selected-aie")
			var select_this= $(this).attr("class").split(' ')[1];
	
			$(elem).addClass("selected-aie");
				if(	$('.'+select_this).hasClass("selected-aie")) {
					$('.'+select_this).removeClass("selected-aie")
					$(elem).addClass("selected-aie");
				}
		
			$(".option-target li").removeClass("undefined")
			var source_target= true;

				var source = elem;
				var src_id =  $(elem).attr('id');
		
				$(elem).parents('.qaa-rw').find('.option-target> *').unbind();
				$(elem).parents('.qaa-rw').find('.option-target> *').bind( "click", function(e) {

				if( (source_target)) {
	
					$(source).addClass("selected-aie");
					$(this).addClass("selected-aie");
					var target_id = $(this).attr('id');
					
					var tag=$(e.target).get(0).tagName
		 				
					
					//	alert($(e.target).hasClass('inline-target') +"   "+ $(source).hasClass('ui-draggable')+"   "+ $(source).hasClass('selected-aie')+"   "+ $(e.target).parents("li").hasClass('ui-droppable')+"   "+ tag)
					
		
						if($(source).hasClass('ui-draggable') && $(source).hasClass('selected-aie') && !$(e.target).hasClass('inline-target') && $(e.target).hasClass('ui-droppable') && tag=='LI') {
							$(source).offset($(e.target).offset());    
							$(source).removeClass("ui-draggable").addClass("non-draggable");
		
						}
						//alert($(e.target).hasClass('inline-target') +"   "+ $(source).hasClass('ui-draggable')+"   "+ $(source).hasClass('selected-aie')+"   "+ $(e.target).parent().hasClass('ui-droppable'))
						
						if($(source).hasClass('ui-draggable') && $(source).hasClass('selected-aie')  && $(e.target).parent().hasClass('ui-droppable') && tag=="SPAN") {
							
						 
							$(source).offset($(e.target).parents("li").offset());
							$(source).removeClass("ui-draggable").addClass("non-draggable");
		
						}
	
	
						if($(source).hasClass('ui-draggable') && $(source).hasClass('selected-aie')  && $(e.target).parents("li").hasClass('ui-droppable') && tag== "IMG") {
						 
							 
							$(source).offset($(e.target).parents("li").offset());
							$(source).removeClass("ui-draggable").addClass("non-draggable");
		
						}
	
	
						if($(this).hasClass('selected-aie')) {
							$(this).removeClass("ui-droppable").addClass("non-droppable");
						}
						
						AIE.Qaa.dadMap[target_id] = src_id;
					
						var $parentobj = $(this).parents('.qaa-rw');
							var finished = AIE.Qaa.hasUserFinishedDrop($parentobj);
							if (finished) {
								var parentid = $parentobj.attr('id');
								var result = AIE.Qaa.checkDADMatchLive($parentobj);
								if (result) {
									$.publish("qaaAssociationCorrect", [parentid, result]);
								} else {
									$.publish("qaaAssociationWrong", [parentid, result]);
								}
							}
						source_target = false;
	
						var sClass = $(source).attr('class').split(' ')[0];
						//extract the src class index
						var sClassIndex = sClass.replace(/\D/g,'');
		
						//now use the target id to get the class
						var tClass = $(this).attr('class').split(' ')[0];
						var tClassIndex = tClass.replace(/\D/g,'');
		
						//
						var Source_target= (tClassIndex+': '+sClassIndex);
						Source_target_string= Source_target_string + Source_target+"$$$$";
						//  alert(Source_target_string)
						var a= AIE.Qaa.LocalStorage.load(pathname_user+association_id+'Source_target_combination');
						var f=''
							if(a!=null) {
								f= AIE.Qaa.LocalStorage.load(pathname_user+association_id+'Source_target_combination');
							}
						var Source_target_combination= Source_target_string;
		
						AIE.Qaa.LocalStorage.save(pathname_user+association_id+'Source_target_combination', Source_target_combination);
						var g = AIE.Qaa.LocalStorage.load(pathname_user+association_id+'Source_target_combination' );
		
						f=f+g
		
						AIE.Qaa.LocalStorage.save(pathname_user+association_id+'Source_target_combination', f);
						f= AIE.Qaa.LocalStorage.load(pathname_user+association_id+'Source_target_combination');
					}
				})
			});
		}
	};
		


AIE.Qaa.Association = {
	init:function(){
		//Setup event handler for evaluation
		$('.association-rw > .option-source').each(function(){
			//Make sure you shuffle only if the author wants it
			if ($(this).hasClass('shuffle')) {
				$(this).shuffle();
			}
		});
		
		//Setup draggable
		$('.association-rw >.shuffle >li').draggable({
			revert:'invalid'				
		});	
		
		//Setup dropable	
		$('.association-rw >.option-target >li').droppable({
			drop:function(event,ui){
				var target_id = $(this).attr('id');
				//Is there something already there on the target
				if ($(this).hasClass('selected-aie')) {
					//We dont want to stack another object on top of that
					$(ui.draggable).css({top:'0px',left:'0px'});
					return false;
				}
				
				var src_id =  $(ui.draggable).attr('id');
				//Snap the element to the top of target
				$(ui.draggable).offset($(this).offset());
				$(this).addClass('selected-aie');
				//Map the src id to element on which it is mapped
				AIE.Qaa.dadMap[target_id] = src_id;
				var $parentobj = $(this).parents('.qaa-rw');
				var finished = AIE.Qaa.hasUserFinishedDrop($parentobj);
				if (finished) {
					var parentid = $parentobj.attr('id');
					var result = AIE.Qaa.checkDADMatchLive($parentobj);
					if (result) {
						$.publish("qaaAssociationCorrect", [parentid, result]);
					} else {
						$.publish("qaaAssociationWrong", [parentid, result]);
					}
				}
			}
		});	
			
		
		//Setup event handler for CHECK button
		$('.association-rw >.buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkDADMatch($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.association-rw > .buttons-rw > .try-again').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryDAD($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.association-rw > .buttons-rw > .reset').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetDAD($parentobj);
		});	
	}	
};

AIE.Qaa.Sequence = {
	init:function(){
		//Setup event handler for evaluation
		$('.sequence-rw > .option-source').each(function(){
			//Make sure you shuffle only if the author wants it
			if ($(this).hasClass('shuffle')) {
				$(this).shuffle();
			}
		});
		
		//Setup draggable
		$('.sequence-rw >.shuffle >li').draggable({
			revert:'invalid'				
		});	
		
		//Setup dropable	
		$('.sequence-rw >.option-target >li').droppable({
			drop:function(event,ui){
				var target_id = $(this).attr('id');
				//Is there something already there on the target
				if ($(this).hasClass('selected-aie')) {
					//We dont want to stack another object on top of that
					$(ui.draggable).css({top:'0px',left:'0px'});
					return false;
				}
				
				var src_id =  $(ui.draggable).attr('id');
				//Snap the element to the top of target
				$(ui.draggable).offset($(this).offset());
				$(this).addClass('selected-aie');
				//Map the src id to element on which it is mapped
				AIE.Qaa.dadMap[target_id] = src_id;
			}
		});	
			
		
		//Setup event handler for CHECK button
		$('.sequence-rw >.buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.checkDADMatch($parentobj);
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.sequence-rw > .buttons-rw > .try-again').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.retryDAD($parentobj);
		});	
		
		//RESET  EVENT HANDLER
		$('.sequence-rw > .buttons-rw > .reset').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			AIE.Qaa.resetDAD($parentobj);
		});	
	},
	
};

AIE.Qaa.QASet = {
	init:function(){	
		function scoreSet(setobj, scoredata) {
			$(".qaa-rw", $parentobj).each( function() {
				total_correct = 0;
				total_wrong = 0;
				for (var i =0; i < scoredata.length; i++) {
					var t_score = scoredata[i];
					if (t_score["status"] == "correct") {
						total_correct = total_correct + t_score["score"];
					} else {
						total_wrong = total_wrong + t_score["score"];
					}
				}
				
				$(".qaa-set-results-rw > .correct-score", setobj).text(total_correct);
				$(".qaa-set-results-rw > .wrong-score", setobj).text(total_wrong);
				
			});
		}
		
		//Setup event handler for CHECK button
		$('.qaa-set-rw > .qaa-set-buttons-rw > .qaa-set-submit').click(function(){
			$parentobj = $(this).parents('.qaa-set-rw');
			
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.qaa-set-rw > .qaa-set-buttons-rw > .qaa-set-check').click(function(){
			$parentobj = $(this).parents('.qaa-set-rw');
			var score = [];
			$(".qaa-rw", $parentobj).each( function() {
				var classname = $(this).attr("class");
				if ($(this).hasClass("multichoice-rw")) { //Multichoice item in set
					AIE.Qaa.checkAnswers($(this));
				} else if ($(this).hasClass("multiresponse-rw")) { //Multiresponse item in set
					AIE.Qaa.checkAnswers($(this));
				} else if ($(this).hasClass("truefalse-rw")) { //True false item in set
					AIE.Qaa.checkAnswers($(this));
				} else if ($(this).hasClass("multi-truefalse-rw")) { //Multi True false item in set
					AIE.Qaa.checkAnswers($(this));
				} else if ($(this).hasClass("textmatch-rw")) { //Text Match item in set
					AIE.Qaa.checkTextMatchAnswers($(this));
				} else if ($(this).hasClass("multi-textmatch-rw")) { //Multi Text match item in set
					AIE.Qaa.checkTextMatchAnswers($(this));
				} else if ($(this).hasClass("sequence-rw")) { //Sequence  in set
					AIE.Qaa.checkDADMatch($(this));
				} else if ($(this).hasClass("association-rw")) { //Association  in set
					AIE.Qaa.checkDADMatch($(this));
				}
				
				//now create a structure for scoring
				var wrong_feedback = $(".feedback-messages > .wrong", $(this));
				var correct_score = $(this).find(".question-control > .correct-score").text();
				var wrong_score = $(this).find(".question-control > .wrong-score").text();
				var qaa_res =  {"id": "", "status": "", "score": 0};
				if ($(wrong_feedback).is(":visible")) {
					qaa_res["id"] = $(this).attr("id");
					qaa_res["status"] = "wrong";
					qaa_res["score"] = parseInt(wrong_score);
				} else {
					qaa_res["id"] = $(this).attr("id");
					qaa_res["status"] = "correct";
					qaa_res["score"] = parseInt(correct_score);
				}
				//Push the info regarding this qaa item into array for later eval
				score.push(qaa_res);
			});
			//Now pass the eval array to function for final processing
			scoreSet($parentobj, score);
		});	
		
		//RESET  EVENT HANDLER
		$('.qaa-set-rw > .qaa-set-buttons-rw > .qaa-set-reset').click(function(){
			$parentobj = $(this).parents('.qaa-set-rw');
			$(".qaa-rw", $parentobj).each( function() {
				var classname = $(this).attr("class");
				if ($(this).hasClass("multichoice-rw")) { //Multichoice item in set
					AIE.Qaa.resetOptions($(this));
				} else if ($(this).hasClass("multiresponse-rw")) { //Multiresponse item in set
					AIE.Qaa.resetOptions($(this));
				} else if ($(this).hasClass("truefalse-rw")) { //True false item in set
					AIE.Qaa.resetOptions($(this));
				} else if ($(this).hasClass("multi-truefalse-rw")) { //Multi True false item in set
					AIE.Qaa.resetOptions($(this));
				} else if ($(this).hasClass("textmatch-rw")) { //Text Match item in set
					AIE.Qaa.resetTextMatch($(this));
				} else if ($(this).hasClass("multi-textmatch-rw")) { //Multi Text match item in set
					AIE.Qaa.resetTextMatch($(this));
				} else if ($(this).hasClass("sequence-rw")) { //Sequence  in set
					AIE.Qaa.resetDAD($(this));
				} else if ($(this).hasClass("association-rw")) { //Association  in set
					AIE.Qaa.resetDAD($(this));
				}
			});
			//Set the score to empty
			$(".qaa-set-results-rw > .correct-score", $parentobj).text("0");
			$(".qaa-set-results-rw > .wrong-score", $parentobj).text("0");
		});	
		
	},
	
	
};

AIE.Qaa.SortWords = {
	init:function(){
		//Setup event handler for evaluation
		
		$('.sortword-multi-rw > .qaa-item > .option-source').each(function(){
			//Make sure you shuffle only if the author wants it
			if ($(this).hasClass('shuffle')) {
				var spans = $(this).find("li");
				spans.shuffle();
				//Append a blank to make sure that li does not collapse
				//after all spans inside it are hidden
				spans.append("&#160;");
			}
		});
		
		//Setup draggable
		$('.sortword-multi-rw > .qaa-item >.shuffle >li > span').draggable({
			revert:'invalid'				
		});	
		
		//Setup dropable	
		$('.sortword-multi-rw > .qaa-item >.option-target >li').droppable({
			drop:function(event,ui){
				var target_id = $(this).attr('id');
				var src_id =  $(ui.draggable).attr('id');
				var topy = $(this).parent().css("top");
				var offset = $(this).parent().position();
				$(this).addClass('selected-aie');
				$(ui.draggable).addClass('aieapp-dropped');
				var text = $(ui.draggable).text();
				$(this).append("<span>" + text + "</span>");
				$(ui.draggable).css({top:'0px',left:'0px'});
				$(ui.draggable).hide();
			}
		});	
			
		
		//Setup event handler for CHECK button
		$('.sortword-multi-rw > .buttons-rw > .check').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			var eval_status = true;
			$(".qaa-item > .option-target > li ", $parentobj).each(function(index){
				var res = {};
				var user_answer = "";
				$("span", $(this)).each(function(){
					user_answer = user_answer + $(this).text() + " ";
				});
				var questioncontrol = $parentobj.find(".question-control > .answer-map");
				var curr_answer_map_ol = questioncontrol[index];
				var answer_text = $(curr_answer_map_ol).find("li").text();
				answer_text = jQuery.trim(answer_text);
				user_answer = jQuery.trim(user_answer);
				console.log(answer_text, user_answer);
				if (answer_text != user_answer) {
					eval_status = false;
					$(this).addClass("wrong");
				} else {
					$(this).addClass("correct");
				}
				
				AIE.Qaa.showFeedback($parentobj, eval_status);
				return;
			});
		});	
		
		//TRY AGAIN EVENT HANDLER
		$('.sortword-multi-rw  > .buttons-rw > .reset').click(function(){
			$parentobj = $(this).parents('.qaa-rw');
			$(".qaa-item > .option-target > li").html("&#160;");
			$(".qaa-item > .option-target > li").removeClass("correct");
			$(".qaa-item > .option-target > li").removeClass("wrong");
			$(".qaa-item > .option-target > li").removeClass("selected-aie");
			
			$parentobj.find(".feedback-messages").hide();
			$(".qaa-item > .option-source > li ", $parentobj).each(function(index){
				$("span", $(this)).each(function(){
					$(this).removeClass("aieapp-dropped");
					$(this).css({top:'0px',left:'0px'});
					$(this).show();
					
				});
			});
		});	
	},
};

/*
$(document).ready(function() {
		
	try{		
		AIE.Qaa.TextMatch.dragItems();
		AIE.Qaa.init();
	}catch(e){
		console.log(e);
	}
});

*/