/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for shelf operations.
 */
var Shelf = function(){
    var config = {
    	open: false,
    	prompts: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),
		XUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
		allBooksShelfId: '',
		currentShelfId: '1',
		currentShelfName: 'All Books',
		currentShelfType: '',
		currentShelfDescription: 'Contains all purchased or uploaded books.'
	}
	
	var getShelfInfo = function(inconditions) {
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return {"status": false};
		}
			
		var table_name = "shelf_info";
		var values = ['shelf_id', 'shelf_metadata'];
		var conditions = inconditions; 
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var shelf_md_obj = {"status": false}
		while (statement.executeStep()){
			var shelf_id = statement.row.shelf_id;
			var shelf_md = statement.row.shelf_metadata;
			shelf_md_obj = JSON.parse(shelf_md);
			shelf_md_obj["status"] = true;
			break
		}
		 statement.finalize
		 return shelf_md_obj;
	}
	
	var isPubShelfDocsAvailable = function(inconditions) {
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return false;
		}
		var available = false;
		var table_name = "pub_docs";
		var values = ['id'];
		var conditions = inconditions; 
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while (statement.executeStep()){
			var id = statement.row.id;
			available = true;
			break
		}
		 statement.finalize();
		 return available;
	}
	
	acfconnector = function() {
		var getAccountBooks = function(shelfid){
			var fileObj = getFileObject(AME.getDbFilePath());
			if (!fileObj.exists()){
				return false;
			}
			
			var table_name = "shelf_info";
			var values = ['shelf_id', 'shelf_metadata'];
			var conditions = new Array(['shelf_id', shelfid, 'TEXT']);
			var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
			var shelf_md_obj = {}
			while (statement.executeStep()){
				var shelf_id = statement.row.shelf_id;
				var shelf_md = statement.row.shelf_metadata;
				shelf_md_obj = JSON.parse(shelf_md);
				break
			}
			 statement.finalize();
			 
			try {
				if (shelf_md_obj.hasOwnProperty("domain_name")) {
					var caturl = shelf_md_obj["domain_name"] + '/acfs/newapi/useraccountcatalog/';
					var devicetoken = shelf_md_obj["devicetoken"];
					var pubacctuid = shelf_md_obj["pubacctuid"];
					var useracctuid = shelf_md_obj["useracctuid"];
					var session = shelf_md_obj["session"];
					var langcode = "en";
					if (AME.config.locale) {
						langcode = AME.config.locale;
					}
					var catParam = 'store_token=' + pubacctuid + '&device_token=' + devicetoken;
					catParam =  catParam + "&lang_code=" + langcode + "&cb=" + Date.now();
					
					var catData = catParam;
					var params = {
						inn:{
							url: caturl,
							type: 'acf_bs_items',
							params: catData
						},
						out:{
							out:null,
							bsitems: null,
						}
					};
					
					window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
					
					if (!params.out.out) {
						var msg = getPropertiesString("azd_ale_shelf_catRefreshFail");
						alert(msg);
						return { 
							"status": false,
							"bsitems": {},
							"reason": msg
						}
					} else {
						return { 
							"status": true,
							"bsitems": params.out.bsitems,
							"reason": ""
						}
					}
				}
			} catch(e) {
				var msg = getPropertiesString("azd_ale_shelf_networkError");
				return { 
					"status": false,
					"bsitems": {},
					"reason": msg + " " + e
				}
			}
		}
	
		var registerAccount = function(type, bsurl, bsacc, name, pas) {
			//Connect to bookstore and get the domain info
			var infourl = bsurl + "/js/acfinfo.json" + "?cb=" + Date.now();
			var params = {
				inn:{
					url: infourl,
					type: 'acf_bs_i',
				},
				out:{
					out:null,
					bsinfo: null,
				}
			};
			
			window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
			
			if (!params.out.out) {
				var msg = getPropertiesString("azd_ale_shelf_invalidBookstore");
				return { 
					"status": false,
					"name": "",
					"description": "",
					"status_mesg": msg,
					"bsinfo": {}
				}
			}
			else{
				try {
					var domain_name = params.out.bsinfo["domain_name"];
					var bs_account_uid = params.out.bsinfo["store_token"];
					var store_name = params.out.bsinfo["acr_bucket"];
				} catch(e) {
						var msg = getPropertiesString("azd_ale_shelf_invalidBookstore");
						return { 
						"status": false,
						"name": "",
						"description": "",
						"status_mesg": msg,
						"bsinfo": {}
					}
				}
			}
			
			var session = guidGenerator();
			var langcode = "en";
			if (AME.config.locale) {
				langcode = AME.config.locale;
			}
			
			var formp = "email=" + name + "&password=" + pas + "&user_id="+bsacc + "&store_token="+bs_account_uid + "&lang_code="+ langcode + "&device_name="+ AME.config.device_name + "&platformos="+ AME.config.platform_os + "&os_version="+ AME.config.os_version + "&azardi_version="+ AME.config.azardi_version + "&manufacturer="+ AME.config.manufacturer + "&model="+ AME.config.model;
			formp += '&devuid=0&device_type=DT&current_timestamp=' + Date.now();
			var params = formp;
			var eurl = domain_name + "/acfs/newapi/registerapp/";
			
			var params = {
				inn:{
					url: eurl,
					type: 'acf_bs_ra',
					params: params
				},
				out:{
					out:null,
					bsinfo: null,
				}
			};
			
			window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
			if (!params.out.out) {
				return { 
					"status": false,
					"name": "",
					"description": "",
					"status_mesg": params.out.bsinfo["failure_reason"],
					"bsinfo": {}
				}
			} else {
				var bsinfo = params.out.bsinfo;
				store_name =  bsinfo["account"];
				bsinfo["domain_name"] = domain_name;
				bsinfo["pubacctuid"] = bs_account_uid;
				bsinfo["useracctuid"] = bsacc;
			}
			
			return { 
				"status": true,
				"name": store_name,
				"description": "",
				"status_mesg": "",
				"bsinfo": bsinfo
			}
		}
	
		var changeDevice = function(type, bsurl, bsacc, name, pas) {
			//Connect to bookstore and get the domain info
			var store_name = "bsurl";
			var infourl = bsurl + "/js/acfinfo.json";
			var params = {
				inn:{
					url: infourl,
					type: 'acf_bs_i',
				},
				out:{
					out:null,
					bsinfo: null,
				}
			};
			window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
			
			if (!params.out.out) {
				var msg = getPropertiesString("azd_ale_shelf_invalidBookstore");
				return { 
					"status": false,
					"name": "",
					"description": "",
					"status_mesg": msg,
					"bsinfo": {}
				}
			}
			else{
				try {
					var domain_name = params.out.bsinfo["domain_name"];
					var bs_account_uid = params.out.bsinfo["store_token"];
					store_name = params.out.bsinfo["account"];
				} catch(e) {
						var msg = getPropertiesString("azd_ale_shelf_invalidBookstore");
						return { 
						"status": false,
						"name": "",
						"description": "",
						"status_mesg": msg,
						"bsinfo": {}
					}
				}
			}
			
			var session = guidGenerator();
			var langcode = "en";
			if (AME.config.locale) {
				langcode = AME.config.locale;
			}
			
			var formp = "&username=" + name + "&store_token="+bs_account_uid + "&password=" + pas + "&lang_code=" + langcode + "&device_name="+ AME.config.device_name + "&platformos="+ AME.config.platform_os + "&os_version="+ AME.config.os_version + "&azardi_version="+ AME.config.azardi_version + "&manufacturer="+ AME.config.manufacturer + "&model="+ AME.config.model;
			
			formp += '&device_type=DT&current_timestamp=' + Date.now();
			var params = formp;
			var eurl = domain_name + "/acfs/newapi/changedevice/";
			
			var params = {
				inn:{
					url: eurl,
					type: 'acf_bs_cd',
					params: params
				},
				out:{
					out:null,
					bsinfo: null,
				}
			};
			
			window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
			
			if (!params.out.out) {
				return { 
					"status": false,
					"name": "",
					"description": "",
					"status_mesg": params.out.bsinfo["failure_reason"],
					"bsinfo": {}
				}
			} else {
				var bsinfo = params.out.bsinfo;
			}
			store_name = bsinfo["account"];
			return { 
				"status": true,
				"name": store_name,
				"description": "",
				"status_mesg": "",
				"bsinfo": bsinfo
			}
		}
		
		return {
			registerAccount: registerAccount,
			changeDevice: changeDevice,
			getAccountBooks: getAccountBooks
		}
	}();
	
	var init = function(){
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		
		//DC - 201301 - Create new publisher table [if not exists]
		try {
			createPubTables(fileObj);
		} catch(e){
			
		}
		self.clear();
		var shelves = self.get();
		shelves = self.sort(shelves);
		self.generate(shelves);
		if(config.currentShelfId != '') {
			$('#'+config.currentShelfId).addClass('shelf-clicked');
		} else {
			$('#1').addClass('shelf-clicked');
		}
    }
	
	var generate = function(shelves){
		try {
			var self = this;
			
			for(var i = 0; i < shelves.length; i++){
				var isAllBooksShelf = false;
				var shelfId = shelves[i][0],
					shelfType = shelves[i][1],
					shelfName = escape(shelves[i][2]),
					shelfDescription = escape(shelves[i][3]);
					shelfDescriptionorig = shelves[i][3];
				
				if (shelves[i][2] == "All Books"){
					var allb_label = getPropertiesString("azd_ale_shelf_allBookLabel");
					shelfName = allb_label;
					config.allBooksShelfId = shelfId;
					isAllBooksShelf = true;
				}
				
				var extdisp = self.getPubShelfDisplayInfo(shelfId);
				
				var shelfItem = document.createElementNS(self.config.XUL_NS, "description");
				shelfItem.setAttribute("id", shelfId);
				shelfItem.setAttribute("value", unescape(shelfName));
				shelfItem.setAttribute("context", "shelf_context_menu");
				var extdisplaytext = "";
				
				shelfDescriptionorig = jQuery.trim(shelfDescriptionorig);
				if (shelfDescriptionorig == "Contains all purchased or uploaded books.") {
					var desc_label = getPropertiesString("azd_ale_shelf_allBookDesc");
					shelfDescription = desc_label;
				}
					
				if (extdisp["status"]) {
					if (shelfDescription == "Contains all purchased or uploaded books") {
						shelfDescription = "";
					}
					
					var src_label = getPropertiesString("azd_ale_shelf_Source");
					extdisplaytext = "\n " + src_label + shelfName;
					if (extdisp.hasOwnProperty("email")) {
						var uname_label = getPropertiesString("azd_ale_shelf_unameLabel");
						extdisplaytext = extdisplaytext + "\n " + uname_label + extdisp["email"];
					}
					if (extdisp.hasOwnProperty("shelf_uid") && extdisp["shelf_uid"]) {
						var uref_label = getPropertiesString("azd_ale_shelf_userRefLabel");
						extdisplaytext += "\n " + uref_label + extdisp["shelf_uid"];
					}
				}
				
				shelfItem.setAttribute("tooltiptext", unescape(shelfName)+' :\n'+unescape(shelfDescription) +'\n' + unescape(extdisplaytext));
				//DC - 201301 
				shelfItem.setAttribute("onclick", "Shelf.showBooks(id, '"+shelfName+"', '"+shelfDescription+"', '" + shelfType + "')");
				
				if(!isAllBooksShelf){
					shelfItem.setAttribute("ondragenter", "Shelf.bookDragEnter(event)");
					shelfItem.setAttribute("ondragover", "Shelf.bookDragOver(event)");
					shelfItem.setAttribute("ondrop", "AME.addBookToShelf(event, '"+shelfName+"')");
				}
				
				document.getElementById('leftcol-aza').appendChild(shelfItem);
			}
			
		}catch(e){
			alert(e);
		}
	}
	
	var bookDragEnter = function(event){
		event.stopPropagation();
		event.preventDefault();
	}
	
	var bookDragOver = function(event){
		event.stopPropagation();
		event.preventDefault();
	}
	
	var clear = function(){
		$("#leftcol-aza > description").remove();
	}
	
	/**
	 * Function to get the list of available shelves.
	 * @returns {array} returns an array of shelves.
	 */
	var get = function(){
		var self = this;
		
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return [];
		}
		
		//Get the information of all the shelves from the database and return.
		var values = ['id', 'shelf_type', 'shelf_name', 'shelf_description'];
		var table_name = 'shelf_master';
		var statement = getValuesFromDatabase(fileObj, values, table_name);
		if(!statement){
			return [];
		}
		
		var shelves = new Array();
		while (statement.executeStep()) {
			var data = new Array();
			data.push(statement.row.id);
			data.push(statement.row.shelf_type);
			data.push(statement.row.shelf_name);
			data.push(statement.row.shelf_description);
			shelves.push(data);
		}
		statement.finalize();
		
		return shelves;
	}
	
	var sort = function(shelves) {
		var self = this;
		
		var unsortableShelves = new Array();
		for(var i = 0; i < shelves.length; i++) {
			if(shelves[i][1] == 'default') {
				var a = shelves.splice(i, 1);
				unsortableShelves.push(a[0]);
			}
		}
		 
		var columnSort = function(a, b) {
			if (a[2].toLowerCase() < b[2].toLowerCase()) return -1;
			if (a[2].toLowerCase() > b[2].toLowerCase()) return 1;
			return 0;
		}
		
		shelves.sort(columnSort);
		
		shelves = unsortableShelves.concat(shelves);
		
		return shelves;
	}
	
	/**
	 * Function to create a shelf.
	 * @param type Type of the shelf.
	 * @param name Name of the shelf.
	 * @param description A short description of the shelf.
	 * @returns {boolean} true if shelf created, false otherwise.
	 */
	var create = function(type, name, description) {
		var self = this;
		
		//Get the list of all shelves from the database.
		var shelves = self.get();
		
		//Check if the shelf is already created.
		for (i = 0; i < shelves.length; i++) {
			if (shelves[i][2] == name) {
				return false;
			}
		}
		
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return false;
		}
		
		//Add new shelf to the database.
		var table_name = 'shelf_master';
		var values = new Array(['shelf_type', type, 'TEXT'],
							   ['shelf_name', name, 'TEXT'],
							   ['shelf_description', description, 'TEXT']);
		insertRowIntoDatabase(fileObj, values, table_name);
		
		return true;
	}

	/**
	 * Function to create a shelf.
	 * @param type Type of the shelf.
	 * @param name Name of the shelf.
	 * @param description A short description of the shelf.
	 * @returns {boolean} true if shelf created, false otherwise.
	 */
	var createPub = function(type, bsurl, bsacc, name, pas) {
		try {
			var self = this;
			
			bsurl = jQuery.trim(bsurl);
			
			if (bsurl.endsWith("/")) {
				bsurl = bsurl.substring(0, bsurl.length-1);
			}
			
			if (!bsurl.startsWith("http://")) {
				bsurl = "http://" + bsurl;
			}
			
			//Get the list of all shelves from the database.
			var shelves = self.get();
			//Check if the database file exists.
			var fileObj = getFileObject(AME.getDbFilePath());
			if (!fileObj.exists()){
				return false;
			}
			
			if (Bootup.hasOwnProperty("single_bookstore") &&  
				Bootup.hasOwnProperty("single_bookstore_url") && 
				Bootup.single_bookstore && Bootup.single_bookstore_url ) {
				//for single account readers, check for only email
				var conditions = new Array(['email', name, 'TEXT']);
			} else {
				//for multiple bookstore support, check for email id and shelfuid
				var conditions = new Array(['shelf_uid', bsacc, 'TEXT']);
				conditions.push(['email', name, 'TEXT'])
			}
			
			var bsinfo = getShelfInfo(conditions);
			if (bsinfo["status"] == true) {
				var msg = getPropertiesString("azd_ale_shelf_bsAlreadyExists");
				alert(msg);
				return false;
			}
			
			//Todo - Talk to the server here using the url 
			//Get the details
			var result = acfconnector.registerAccount(type, bsurl, bsacc, name, pas);
			
			if (result["status"] == false) {
				var msg = getPropertiesString("azd_ale_shelf_createPubShelfFail");
				alert(msg + " " + result["status_mesg"]);
				return false;
			}
			
			var bsinfo =  {};
			try {
				var bsinfo = result["bsinfo"];
			} catch(e) {
				bsinfo = {};
			}
			
			if (bsinfo.hasOwnProperty("registration_flag") && bsinfo["registration_flag"] == 0) {
				var msg = getPropertiesString("azd_ale_shelf_createPubShelfFail");
				alert(msg + " " + result["status_mesg"]); 
				return false;
			}
			
			if (bsinfo.hasOwnProperty("registration_flag") && bsinfo["registration_flag"] == 2) {
				var userreponse = confirm(bsinfo["failure_reason"]);
				if (!userreponse) {
					return false;
				}
				
				var cd_result = acfconnector.changeDevice(type, bsurl, bsacc, name, pas);
				
				if (cd_result["status"] == false) {
					var msg = getPropertiesString("azd_ale_shelf_changeDeviceFail");
					alert(msg + " " + cd_result["status_mesg"]);
					return false;
				}
				
				if (cd_result.hasOwnProperty("bsinfo")) {
					var cd_resp = cd_result["bsinfo"];
					bsinfo["devicetoken"] = cd_resp["devicetoken"];
					result["bsinfo"] = bsinfo;
					result["name"] = cd_result["name"];
				}
			}
			
			var email = name;
			var description = "";
			name = result["name"];
			
			try {
				description = result["description"];
			} catch(e) {
			
			}
			
			
			
			//Check if the shelf is already created.
			/*for (i = 0; i < shelves.length; i++) {
				if (shelves[i][2] == name) {
					return false;
				}
			}*/
			
			//Add new shelf to the database.
			var table_name = 'shelf_master';
			var values = new Array(['shelf_type', type, 'TEXT'],
								   ['shelf_name', name, 'TEXT'],
								   ['shelf_description', description, 'TEXT']);
			var shelfid = insertRowIntoDatabase(fileObj, values, table_name);
			
			//Add new shelf to the database.
			var table_name = 'shelf_info';
			try {
				var shelf_md = result["bsinfo"];
				var shelf_md_s = JSON.stringify(shelf_md);
			} catch (e) {
				var shelf_md_s = "{}";
			}
			var values = new Array(['shelf_id', shelfid, 'INTEGER'],
								   ['shelf_uid', bsacc, 'TEXT'],
								   ['shelf_url', bsurl, 'TEXT'],
								   ['email', email, 'TEXT'],
								   ['shelf_metadata', shelf_md_s, 'TEXT']
								   );
			var shelfid = insertRowIntoDatabase(fileObj, values, table_name);
			
			return true;
		}catch(e){
			return false;
		}
	}

	
	var addBook = function(bookMetadata, shelfName){
		var self = this;
		
		var shelves = self.get();
		var shelfId = '',
		bookId = '';
		for(i = 0; i< shelves.length; i++){
			if(shelves[i][2] == shelfName){
				shelfId = shelves[i][0];
				break;
			}
		}
		if(typeof bookMetadata.id === 'undefined') {
			bookId = BookMetadata.getBookId(bookMetadata.title);
		} else {
			bookId = bookMetadata.id;
		}
		// var allBooks = BookMetadata.getAll();
		// for(i = 0; i< allBooks.length; i++){
			// if(allBooks[i]['title'] == bookMetadata.title){
				// bookId = allBooks[i]['id'];
			// }
		// }
		
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'shelf_document_join';
		var values = new Array(['lib_id', bookId, 'INTEGER'],
		   					  ['shelf_id', shelfId, 'INTEGER']);	
		insertRowIntoDatabase(fileObj, values, table_name);
	}
	
	/**
	 * Delete the book from shelf
	 * @params bookId Id of the book
	 * @params shelfId Id of the shelf
	 */
	var deleteBook = function(bookId, shelfId) {
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'shelf_document_join';
		var condition = new Array(['lib_id', bookId, 'INTEGER'],
								  ['shelf_id', shelfId, 'INTEGER']);
		deleteRowFromDatabase(fileObj, table_name, condition);
	};
	
	var getClicked = function(){
		var id = $('.shelf-clicked').attr('id');
		var label = $('.shelf-clicked').attr('value');
		
		return {'id': id, 'label': label}
	}
	
	var deleteShelf = function(clickedShelf){
		if(clickedShelf.id == config.allBooksShelfId){
			var msg = getPropertiesString("azd_ale_shelf_deleteMainShelf");
			alert(msg);
			return;
		}
		
		var msg = getPropertiesString("azd_ale_shelf_confirmShelfDelete");
		if(config.prompts.confirm(null, AME.getTitle(), msg)){
			var fileObj = getFileObject(AME.getDbFilePath());
		    if (!fileObj.exists()){
		        return false;
		    }
		    var table_name = 'shelf_master';
			var condition = new Array(['id', clickedShelf.id, 'INTEGER']);
			
			// remove shelf from shelf_master table.		
		    deleteRowFromDatabase(fileObj, table_name, condition);
		
			// remove shelf from shelf_document_join
			table_name = 'shelf_document_join';
			condition = new Array(['shelf_id', clickedShelf.id, 'INTEGER']);
			deleteRowFromDatabase(fileObj, table_name, condition);
			
			// remove shelf from shelf_document_join
			table_name = 'pub_docs';
			condition = new Array(['shelf_id', clickedShelf.id, 'TEXT']);
			deleteRowFromDatabase(fileObj, table_name, condition);
			
			var table_name = "shelf_info";
			condition = new Array(['shelf_id', clickedShelf.id, 'TEXT']);
			deleteRowFromDatabase(fileObj, table_name, condition);
			try {
				var evt = document.createEvent("MouseEvents");
				evt.initMouseEvent("click", true, true, window,
					0, 0, 0, 0, 0, false, false, false, false, 0, null);
				var cb = document.getElementById("1"); 
				var canceled = !cb.dispatchEvent(evt);
		    } catch(e) {
			
			}
			
			
		}
	}
	
	/**
	 * Highlights the shelf in shelf panel
	 * @params shelfId Id of the shelf
	 */
	var highlight = function(shelfId) {
		$("#leftcol-aza > description").removeClass('shelf-clicked');
		$('#'+shelfId).addClass('shelf-clicked');
	};
	
	/**
	 * Show all books in the shelf
	 * @params shelfId Id of the shelf
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Short description of the shelf 
	 */
	var showBooks = function(shelfId, shelfName, shelfDescription, shelfType) {
		var self = this;
		if(shelfType == "acfs") {
			shelfName = unescape(shelfName);
			shelfDescription = unescape(shelfDescription);
			//Highlight shelf in shelf panel
			self.highlight(shelfId);
			
			config.currentShelfId = shelfId;
			config.currentShelfName = shelfName;
			config.currentShelfDescription = shelfDescription;
			config.currentShelfType = shelfType;
			//Show the shelf books.
			Library.showPubShelfBooks(shelfId, shelfName, shelfDescription)
			
		} else {
		
			shelfName = unescape(shelfName);
			shelfDescription = unescape(shelfDescription);
			//Highlight shelf in shelf panel
			self.highlight(shelfId);
			
			config.currentShelfId = shelfId;
			config.currentShelfName = shelfName;
			config.currentShelfDescription = shelfDescription;
			config.currentShelfType = shelfType;
			
			//Show the shelf books.
			Library.showShelfBooks(shelfId, shelfName, shelfDescription)
		}
	};
	
	var updatePubShelf = function(shelfid) {
		try {
			var self = this;
			var result = acfconnector.getAccountBooks(shelfid);
			if (result.status) {
				try{
					//Iterate and check if the document already exists
					//add the existing documents into a object
					var shelf_md_obj = {};
					var fileObj = getFileObject(AME.getDbFilePath());	
					var table_name = "pub_docs";
					var values = ['workuid'];
					var conditions = new Array(['shelf_id', shelfid, 'INTEGER']);
					var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
					var shelf_b_obj = {}
					while (statement.executeStep()){
						var workuid = statement.row.workuid;
						shelf_md_obj[workuid] = true;
					}
					statement.finalize();
				}catch(e){
					
				}
				//now iterate the list and insert the documents that 
				//not exist in the account
				var currentTime = new Date();
				if (!result.bsitems.hasOwnProperty("cdps"))return false;
				var cdps = result.bsitems["cdps"];
				for (var i=0; i<cdps.length; i++) {
					var s_binfo = cdps[i];
					var s_binfo1 = cdps[i];
					var s_workuid =  s_binfo["agreement"];
					if (shelf_md_obj.hasOwnProperty(s_workuid)){
						continue;
					}
					var workuid = s_binfo["agreement"];
					var title = s_binfo["title"];
					var author = s_binfo["author"];
					
					var cover = s_binfo["cover"];
					var coverfn = cover.split("/").pop();
					var destObj = getFileObject(['ProfD', 'quizfx', 'covers']);
					destObj.append(coverfn);
					cover = destObj.path;
					var coverObject = getFileObjectFromPath(cover);
					var coverUrl = getURL(coverObject);
					cover = coverUrl;
					
					var covertype = "image";
					var product_description = s_binfo["product_description"];
					var identifier = s_binfo["identifier"];
					var published_date = s_binfo["published_date"];
					var format_type = s_binfo["epub_type"];
					var version = s_binfo["epub_version"];
					
					//Handle default status for "old" epubs
					if(format_type == "" || (typeof format_type == "undefined")) {
						format_type = "ePub"; 
					}
					
					if(version == "" || (typeof version == "undefined")) {
						version = ""; 
					}
					
					var state = "READYFORDOWNLOAD";
					var fileObj = getFileObject(AME.getDbFilePath());		
					var table_name = 'pub_docs';
					var values = new Array(['created_date', currentTime.getTime(), 'TEXT'],
										  ['shelf_id', shelfid, 'INTEGER'],
										  ['workuid', workuid, 'TEXT'],
										  ['bookstate', state, 'TEXT'],
										  ['title', title, 'TEXT'],
										  ['author',author, 'TEXT'],
										  ['publisher_identifier', identifier, 'TEXT'],
										  ['publish_date', published_date, 'TEXT'],
										  ['description', product_description, 'TEXT'],
										  ['location', "online", 'TEXT'],
										  ['type', format_type, 'TEXT'],
										  ['cover_image', cover, 'TEXT'],
										  ['cover_type', covertype, 'TEXT'],
										  ['docinfo', JSON.stringify(s_binfo), 'TEXT'],
										  ['version', version, 'TEXT']);
					var bookid = insertRowIntoDatabase(fileObj, values, table_name);
				}

				return true;
			}
			
		} catch(e) {

		
		}
	};
	
	var getPubAccountDetails = function(shelfid) {
		try {
			var self = this;
			var conditions = new Array(['shelf_id', shelfid, 'INTEGER']);
			if (isPubShelfDocsAvailable(conditions)) return true;
		
			self.updatePubShelf(shelfid);
			
			return false;
		} catch(e) {
			azDump("getPubAccountDetails EXCEPTION", e);
			return false;
		}
	}
	
	var getPubAcctMd = function(shelfid){
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return false;
		}
		
		var table_name = "shelf_info";
		var values = ['shelf_id', 'shelf_metadata'];
		var conditions = new Array(['shelf_id', shelfid, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var shelf_md_obj = {"status": false}
		while (statement.executeStep()){
			var shelf_id = statement.row.shelf_id;
			var shelf_md = statement.row.shelf_metadata;
			shelf_md_obj = JSON.parse(shelf_md);
			shelf_md_obj["status"] = true;
			break
		}
		 statement.finalize();
		 return shelf_md_obj;
	}
	
	var getPubShelfDisplayInfo = function(shelfid) {
		try{
			
			var fileObj = getFileObject(AME.getDbFilePath());
			if (!fileObj.exists()){
				return {status: false};
			}
			
			var table_name = "shelf_info";
			var values = ['shelf_uid', 'shelf_url', 'email'];
			var conditions = new Array(['shelf_id', shelfid, 'TEXT']);
			var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
			var shelf_md_obj = {"status": false};
			
			while (statement.executeStep()){
				var shelf_uid = statement.row.shelf_uid;
				var shelf_url = statement.row.shelf_url;
				var email = statement.row.email;
				shelf_md_obj["status"] = true;
				shelf_md_obj["shelf_uid"] = shelf_uid;
				shelf_md_obj["shelf_url"] = shelf_url;
				shelf_md_obj["email"] = email;
				break;
			}
			 statement.finalize();
			 
			 return shelf_md_obj;
		 }catch(e){
			
			shelf_md_obj = {"status": false};
			 return shelf_md_obj;
		 }
	}
	
    return {
		config: config,
		init: init,
		create: create,
		createPub: createPub,
		get: get,
		sort: sort,
		addBook: addBook,
		init: init,
		clear: clear,
		generate: generate,
		getClicked: getClicked,
		deleteShelf: deleteShelf,
		showBooks: showBooks,
		highlight: highlight,
		bookDragEnter: bookDragEnter,
		bookDragOver: bookDragOver,
		deleteBook: deleteBook,
		getPubAccountDetails: getPubAccountDetails,
		getPubAcctMd: getPubAcctMd,
		updatePubShelf: updatePubShelf,
		getPubShelfDisplayInfo: getPubShelfDisplayInfo,    
	};
}();

var encryptor  =  encryptor || {};
encryptor.init = {
	convert:function (term, key)
			{
			 var chars = getchars(); 
			 var key=key.split('').filter(function(itm,i,a){return i==a.indexOf(itm);}).sort().join('');
			 var item = key.split('');
			 var ch = chars.split('');
			 for (i in item){var ind = ch.indexOf(item[i]); delete ch[ind];}
			 var reversed = reverse(ch.join(''));
			 var center = parseInt(reversed.length/2);
			 var nwreversed = reversed.split('');
				 key = key.replace(/\d/g,"");
				 nwreversed.splice(center,0, key);
				 nwreversed = nwreversed.join('');
			 return encryptor.init.enccode(term, nwreversed);
			},
	enccode:function(term, refchars){
		var chars =getchars();
			term = reverse(term)
			term = term.split('');
		var b ="";
		for(i in term)
		{
			  var sind = chars.split('').indexOf(term[i]) 
	 	      var Aind = chars.split('').indexOf(term[i].toLowerCase());
		      b+=(sind == -1)?(Aind == -1)?term[i]:refchars.split('')[Aind].toUpperCase():refchars.split('')[sind];
	    }
		return btoa(b);
		//return Ti.Utils.base64encode(b); 
	}
}
function reverse(s){
    return s.split("").reverse().join("");
}
var getchars = function (){
	this.chars = new Array();
	for(i=97; i<=122; i++)this.chars.push(String.fromCharCode(i))
	return this.chars.join('');
} 

String.prototype.endsWith = function(suffix) {
    return this.indexOf(suffix, this.length - suffix.length) !== -1;
};

if (typeof String.prototype.startsWith != 'function') {
  // see below for better implementation!
  String.prototype.startsWith = function (str){
    return this.indexOf(str) == 0;
  };
}