/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for manipulating User Preferences.
 */
var UserPref = function(){
	/**
	 * Returns the user preferences stored in database.
	 * @returns {object} A key value pair for user preferences.
	 */
	var getFromDb = function(){
        var self = this;
        
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getDbFilePath());
        if (!fileObj.exists()) {
            return {};
        }
        
        //Get the user preferences from database.
        var values = ['key', 'value'];
        var table_name = 'global_user_pref';
        var statement = getValuesFromDatabase(fileObj, values, table_name);
        var key = new Array();
        var value = new Array();
        if (!statement) {
        	//If any error.
            return {};
        }
        while (statement.executeStep()) {
            key.push(statement.row.key);
            value.push(statement.row.value);
        }
        statement.finalize();
        
		//Convert the returned string to corrent format and assign them to newPref.
		var newPref = {};
        
        for (i = 0; i < key.length; i++) {
            switch (key[i]) {
                case 'columns':
                	//Before the release of AZARDI 11.0, variable columns in database used to contain value true|false.
                	//true being column mode and false being flow mode.
                	//After the release of AZARDI 11.0, varible columns in databse contains value 0|1|2|3|4.
                	//0- Flow mode, 1- 1 Column, 2- 2 Columns, 3- 3 Columns, 4- 4 Columns
                	//Following code was written to make the transition easy.
                	//The first if and else if condition takes care of transition,
                	//while the else condition will be used by AZARDI 11.0 and later.
                	//Do not remove the first two conditions as they will be used if someone tries to go from version before 11.0 to version after 11.0.
                	if(value[i] == 'true') {
                		newPref.columns = true;
                		newPref.noOfCols = 1;
                	} else if(value[i] == 'false') {
                		newPref.columns = false;
                		newPref.noOfCols = 0;
                	} else {
                		newPref.noOfCols = parseInt(value[i]);
                		if(newPref.noOfCols == 0) {
                			newPref.columns = false;
                		} else {
                			newPref.columns = true;
                		}
                	}
                    break;
                case 'libraryView':
                    newPref.libraryView = value[i];
                    break;
                case 'font':
                    newPref.font = value[i];
                    break;
                case 'hyphenationStatus':
                    newPref.hyphenationStatus = eval(value[i]);
                    break;
                case 'displayColumnRule':
                    newPref.displayColumnRule = eval(value[i]);
                    break;
                case 'justifyStatus':
                    newPref.justifyStatus = eval(value[i]);
                    break;
                case 'backgroundColor':
                    newPref.backgroundColor = value[i];
                    break;
                case 'openWidth':
                    newPref.openWidth = eval(value[i]);
                case 'openHeight':
                    newPref.openHeight = eval(value[i]);
                default:
            }
        }
        return newPref;
    };
    
    /**
     * Function to update the default preferences
     * @param newPref An object containing new User Preferences.
     */
    var set = function(newPref){
        var self = this;
        
        //If newPref is empty object then do not set it.
        if(!$.isEmptyObject(newPref)) {
            $.extend(self.config, newPref);
        }
    };
    
    var config = {
        columns: true,
        noOfCols: 2,
        displayColumnRule: true,
        libraryView: 'table',
        font: 'DejaVu Serif',
        fontSize: 1,
		lineSpacing: 1.2,
        margin: 30,
        hyphenationStatus: true,
        justifyStatus: true,
        backgroundColor: '#FFFFFF',
        openWidth: 700,
        openHeight: 700,
        locale: "en-US"
    };
    
    /**
     * Initilize User Preferences.
     */
    var init = function(){
        var self = this;
        
        //Get the stored preferences from the database.
        var newPref = self.getFromDb();
        //Set the new preferences as the default.
        self.set(newPref);
    };
	
	/**
	 * Sets the values in form from database
	 */
	var setForm = function(){
		$.each(config, function(key, value){
			switch(key) {
				case 'libraryView':
					$('#user_pref_lib_view').val(value);
					break;
				case 'openWidth':
					$('#user_pref_open_width').val(value);
					break;
				case 'openHeight':
					$('#user_pref_open_height').val(value);
					break;
				case 'font':
					$('#user_pref_font_list').val(value);
					break;
				// case 'fontSize':
					// $('#user_pref_font_size').val(value);
					// break;
//				case 'paragraphSpacing':
//					$('#user_pref_para_spacing_scale').attr('value', value);
//					break;
				// case 'lineSpacing':
					// $('#user_pref_line_space').val(value);
					// break;
				// case 'margin':
					// $('#user_pref_margin').val(value);
					// break;
				case 'backgroundColor':
					$('#user_pref_background_color').val(value);
					break;
				case 'hyphenationStatus':
					$('#user_pref_hyphenation_status').attr('checked', value);
					break;
				case 'displayColumnRule':
					$('#user_pref_column_rule_status').attr('checked', value);
					break;
				case 'justifyStatus':
					$('#user_pref_justify_status').attr('checked', value);
					break;
				case 'noOfCols':
					$('#user_preferences_column').val(value);
					break;
			}
		});
	};
	
	/**
	 * Get the new user preferences, set it, and save it to databse
	 */
	var save = function(){
		var self = this;
		
		var newPref = self.get();
		var curr_locale = getCurrentLocale();
		if (newPref.locale && (curr_locale != newPref.locale)) {
			var msg = getPropertiesString("azd_changelocale");
    		if(!msg){
				msg = "Language changed. Restart AZARDI.";
			}
			alert(msg);
			AME.setLocale(newPref.locale);
		}
		
		self.set(newPref);
		self.saveToDatabase();
	}
	
	var getCurrentLocale = function() {
		try {
			var chromeRegService = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService();
			var xulChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIXULChromeRegistry);
			var selectedLocale = xulChromeReg.getSelectedLocale("azardi2");
			return selectedLocale;
		} catch(e) {
			return "en-US";
		}
	}
	
	/**
	 * Save the new user preferences to the database overriding older one
	 */
	var saveToDatabase = function(){
		var fileObj = getFileObject(AME.getDbFilePath());
	    var table_name = 'global_user_pref';
		
		var preferenceArray = new Array(['libraryView', config.libraryView],
									   ['openWidth', config.openWidth],
									   ['openHeight', config.openHeight],
									   ['font', config.font],
									   // ['fontSize', config.fontSize],
//																   ['paragraphSpacing', config.paragraphSpacing],
									   // ['lineSpacing', config.lineSpacing],
									   // ['margin', config.margin],
									   ['backgroundColor', config.backgroundColor],
									   ['hyphenationStatus', config.hyphenationStatus],
									   ['displayColumnRule', config.displayColumnRule],
									   ['justifyStatus', config.justifyStatus],
									   ['columns', config.noOfCols]);
		for(var i = 0; i < preferenceArray.length; i++){
			var values = ['key'];
			var conditions = new Array(['key', preferenceArray[i][0], 'TEXT']);
			var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
			while (statement.executeStep()){
				var key = statement.row.key;
			}
			values = new Array(['key', preferenceArray[i][0], 'TEXT'], ['value', preferenceArray[i][1], 'TEXT']);
			
			//	Store value for first time.
			if (key == undefined){
				insertRowIntoDatabase(fileObj, values, table_name)
			}
			
			//	If value exists then update it.
			else{
				conditions = new Array(['key', preferenceArray[i][0], 'TEXT']);
				updateRowIntoDatabase(fileObj, values, table_name, conditions);
			}
			
			//	set key to undefined so if another key does not exists then it creates a new row.
			key=undefined;
		}
	};
	
	/**
	 * Get the new uer preferences from the form
	 */
	var get = function(){
		var newPref = {};
		newPref.noOfCols = $('#user_preferences_column').val();
		newPref.columns = (newPref.noOfCols == '0') ? false : true;
		newPref.libraryView = $('#user_pref_lib_view').val();
        newPref.font = $('#user_pref_font_list').val();
        // newPref.fontSize =	$('#user_pref_font_size').val();
//        newPref.paragraphSpacing = document.getElementById('user_pref_para_spacing_scale').value;
		// newPref.lineSpacing = $('#user_pref_line_space').val();
        // newPref.margin = $('#user_pref_margin').val();
        newPref.hyphenationStatus = $('#user_pref_hyphenation_status').is(':checked');
        newPref.displayColumnRule = $('#user_pref_column_rule_status').is(':checked');
        newPref.justifyStatus = $('#user_pref_justify_status').is(':checked');
        newPref.backgroundColor = $('#user_pref_background_color').val();
        newPref.openWidth = $('#user_pref_open_width').val();
        newPref.openHeight = $('#user_pref_open_height').val();
		var localeListbox = document.getElementById("locale-selector-menu");
		var myMenuitem = localeListbox.selectedItem;
		var myValue = myMenuitem.getAttribute('value');
		newPref.locale = myValue;
		return newPref;
	}
    return {
        config: config,
        init: init,
		getFromDb: getFromDb,
		set: set,
		setForm: setForm,
		save: save,
		get: get,
		saveToDatabase: saveToDatabase
    }
}();
