/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necesssary for adding multiple epubs using drag and drop.
 */
var AddMulEpub = function() {
	var config = {
		allBooksInfo: [],
		bookIndex: 0,
		coverIndex: 0,
		XHTML_NS: "http://www.w3.org/1999/xhtml"
	};
	
	/* This function checks the drag data to see if a simple text file or general purpose file types are available.
	   If the file types are present in the drag data, the function returns that dropping the data is allowed. */
	
	var dragOver = function(aEvent) {
		var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
	    var dragSession = dragService.getCurrentSession();
	
	    var supported = dragSession.isDataFlavorSupported("text/x-moz-url");
	    if (!supported)
			supported = dragSession.isDataFlavorSupported("application/x-moz-file");
	
	    if (supported)
			dragSession.canDrop = true;
	};
	
	var dragDrop = function(aEvent) {
		var uris = getURIS();
		var coverURLS = [];
		
		for(var i=0; i<uris.length; i++) {
			var file = getFileObjFromURL(uris[i]);
			if(getExtension(file) === 'epub'){
				try{
					var info = Epub.getInfo(file);
				}
				catch(e){
					//	If any error while loading epubs then display error message and continue with next ePub.
					var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
					var msg = getPropertiesString("azd_errLoadingEpub");
					prompts.alert(null, AME.getTitle(), '"'+file.leafName+'" '+msg);
					continue;
				}
				var epubInfo = extractInfo(info.metadata, 'ePub');
				var temp = {
					'bookLoc': epubInfo.location,
					'coverUrl': info.coverImagePath
				};
				var index = parseInt(config.bookIndex)+1;
				var temp2 = ['container_'+index, epubInfo];
				coverURLS.push(temp);
				config.allBooksInfo.push(temp2);
				generateAndDisplay(epubInfo);
			} else if(getExtension(file) === 'e0'){
				try{
					var info = Epub0.getE0Info(file);
				}
				catch(e){
					//	If any error while loading epubs then display error message and continue with next ePub.
					var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
					prompts.alert(null, AME.getTitle(), '"'+file.leafName+'" contain errors and will be skipped.');
					continue;
				}
				var epubInfo = extractInfo(info.metadata, "E0");
				var temp = {
					'bookLoc': epubInfo.location,
					'coverUrl': info.coverImagePath
				};
				var index = parseInt(config.bookIndex)+1;
				var temp2 = ['container_'+index, epubInfo];
				coverURLS.push(temp);
				config.allBooksInfo.push(temp2);
				generateAndDisplay(epubInfo);
			}
		}
		generateAndDisplayCover(coverURLS);
		AME.changeMulAddBookOkButtonStatus(config.allBooksInfo.length);
	};
	
	var generateAndDisplayCover = function(coverURLS) {
		if(coverURLS.length <= 0) {
			return;
		}
		
		config.coverIndex = config.coverIndex + 1;
		var temp = coverURLS.shift();
		var coverPath = temp.coverUrl;
		var bookLoc = temp.bookLoc;
		
		if(coverPath.split('.').pop() === 'svg') {
			var coverUri = generateDataURI(coverPath);
			$('#container_'+config.coverIndex+' .mul-book-cover').css('background-image', 'url('+coverUri+')');
			addCoverToBookInfo(bookLoc, coverUri);
			generateAndDisplayCover(coverURLS);
		} else {
			var coverObject = getFileObjectFromPath(coverPath);
			var coverUrl = getURL(coverObject);
			var canvas = document.getElementById('thumb_canvas');
			var ctx = canvas.getContext('2d');
			var img = new Image();
			img.onload = function() {
				var currW = img.width;
				var currH = img.height;
				var dim = AME.scaleSize(300, 300, currW, currH);
				canvas.width = dim[0];
				canvas.height = dim[1];
				ctx.mozImageSmoothingEnabled = true;
				ctx.drawImage(img, 0, 0, dim[0], dim[1]);
				var coverUri = canvas.toDataURL('image/png');
				$('#container_'+config.coverIndex+' .mul-book-cover').css('background-image', 'url('+coverUri+')');
				addCoverToBookInfo(bookLoc, coverUri);
				generateAndDisplayCover(coverURLS);
			};
			img.src = coverUrl;
		}
	};
	
	var addCoverToBookInfo = function(bookLoc, coverUri) {
		for(var i=0; i<config.allBooksInfo.length; i++) {
			if(bookLoc === config.allBooksInfo[i][1].location) {
				config.allBooksInfo[i][1].cover_image = coverUri;
				break;
			}
		}
	};
	
	var generateAndDisplay = function(epubInfo) {
		config.bookIndex = config.bookIndex + 1;
		var container = document.createElementNS(config.XHTML_NS, "html:div");
		container.setAttribute('class', 'multiple-book-container');
		container.setAttribute('id', 'container_'+config.bookIndex);
		// Create and append delete block.
		var bookDeleteContainer = document.createElementNS(config.XHTML_NS, "html:div");
		bookDeleteContainer.setAttribute('class', 'mul-book-delete');
		var bookDelete = document.createTextNode('Remove');
		bookDeleteContainer.appendChild(bookDelete);
		bookDeleteContainer.setAttribute('onclick', 'javascript:AddMulEpub.removeBook(this);');
		container.appendChild(bookDeleteContainer);
		// Create and append Cover block.
		var bookCoverContainer = document.createElementNS(config.XHTML_NS, "html:div");
		bookCoverContainer.setAttribute('class', 'mul-book-cover');
		container.appendChild(bookCoverContainer);
		// Create and append title block.
		var bookTitleContainer = document.createElementNS(config.XHTML_NS, "html:div");
		bookTitleContainer.setAttribute('class', 'mul-book-title');
		var bookTitle = document.createTextNode(epubInfo.title);
		bookTitleContainer.appendChild(bookTitle);
		container.appendChild(bookTitleContainer);
		// Create and append author block.
		var bookAuthorContainer = document.createElementNS(config.XHTML_NS, "html:div");
		bookAuthorContainer.setAttribute('class', 'mul-book-author');
		var bookAuthor = document.createTextNode(epubInfo.author);
		bookAuthorContainer.appendChild(bookAuthor);
		container.appendChild(bookAuthorContainer);
		// Create and append description block.
		var bookdescriptionContainer = document.createElementNS(config.XHTML_NS, "html:div");
		bookdescriptionContainer.setAttribute('class', 'mul-book-description');
		var bookdescription = document.createTextNode(epubInfo.description);
		bookdescriptionContainer.appendChild(bookdescription);
		container.appendChild(bookdescriptionContainer);
		document.getElementById('drop_container').appendChild(container);
	};
	
	var getExtension = function(file) {
		var fileName = file.leafName;
		return fileName.split('.').pop();
	};
	
	/* The getURIS function retrieves the file URI data from the drag session and returns an array of file URIs. */
	
	var getURIS = function() {
		var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
		var dragSession = dragService.getCurrentSession();
		var _ios = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces.nsIIOService);
		var uris = [];
	
	    // If sourceNode is not null, then the drop was from inside the application
	    if (dragSession.sourceNode)
	    	return;
	
	    // Setup a transfer item to retrieve the file data
	    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	    trans.addDataFlavor("text/x-moz-url");
	    trans.addDataFlavor("application/x-moz-file");
	
	    for (var i=0; i<dragSession.numDropItems; i++) {
			var uri = null;

			dragSession.getData(trans, i);
			var flavor = {}, data = {}, length = {};
			trans.getAnyTransferData(flavor, data, length);
			if(data) {
				try {
					var str = data.value.QueryInterface(Components.interfaces.nsISupportsString);
				} catch(e) {
				}

				if(str) {
					uri = _ios.newURI(str.data.split("\n")[0], null, null);
				} else {
					var file = data.value.QueryInterface(Components.interfaces.nsIFile);
					if (file)
						uri = _ios.newFileURI(file);
				}
			}
			
			if (uri)
				uris.push(uri);
	    }
	    
	    return uris;
	};
	
	var extractInfo = function(bookInfo, formattype) {
		var extractedInfo = {
			location: '',
			uid: '',
			title: '',
			author: '',
			publisher_identifier: '',
			publish_date: '',
			description: '',
			version: '',
			cover_image: '',
			cover_type: '',
			type: '',
			created_date: '',
			rendition_layout: 'reflowable', //DC - FLO UPDATE - 201211
			rendition_orientation: 'auto',
			rendition_spread: 'auto'
		};
	    var currentTime = new Date();
	    
		$.each(bookInfo, function(index, value){
			switch(value[0]){
				case 'dc:location':
					extractedInfo.location = value[1];
					break;
				case 'dc:identifier':
					extractedInfo.uid = value[1];
					break;
				case 'dc:title':
					extractedInfo.title = value[1];
					break;
				case 'dc:creator':
					extractedInfo.author = value[1];
					break;
				case 'dc:publisher':
					extractedInfo.publisher_identifier = value[1];
					break;
				case 'dc:date':
					extractedInfo.publish_date = value[1];
					break;
				case 'dc:description':
					extractedInfo.description = value[1];
					break;
				case 'dc:version':
					extractedInfo.version = value[1];
					break;
				case 'dc:cover_type':
					extractedInfo.cover_type = value[1];
					break;
				case 'rendition:layout': //DC - FLO UPDATE - 201211
					extractedInfo.rendition_layout = value[1];
					break;
				case 'rendition:orientation':
					extractedInfo.rendition_orientation = value[1];
					break;
				case 'rendition:spread':
					extractedInfo.rendition_spread = value[1];
					break;
			}
		});
		
		extractedInfo.type = formattype;
		extractedInfo.created_date = currentTime.getTime();
		return extractedInfo;
	};
	
	/**
	 * Initilize adding multiple epubs using drag and drop.
	 */
	var init = function() {
		var elem = document.getElementById('drop_container');
		//Add Event Listners to the drop container.
		elem.addEventListener("dragover", dragOver, true);
		elem.addEventListener("drop", dragDrop, true);
	};
	
	var clear = function() {
		//	Clear the drop container.
		$('#drop_container .multiple-book-container').remove();
		
		//	Clear the variables.
		config.allBooksInfo.splice(0, config.allBooksInfo.length);
		config.bookIndex = 0;
		config.coverIndex = 0;
	};
	
	var getAllBooksInfo = function() {
		return config.allBooksInfo;
	};
	
	var removeBook = function(ele) {
		var container = $(ele).parent();
		var containerId = container.attr('id');
		//	Remove the book information from the array.
		for(var i=0; i<config.allBooksInfo.length; i++) {
			if(config.allBooksInfo[i][0] === containerId){
				config.allBooksInfo.splice(i, 1);
				break;
			}
		}
		//	Remove the element from drop cotainer.
		container.remove();
		AME.changeMulAddBookOkButtonStatus(config.allBooksInfo.length);
	};
	  
	return{
		init: init,
		clear: clear,
		getAllBooksInfo: getAllBooksInfo,
		removeBook: removeBook
	};
}();