	/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * AZARDI MAIN ENGINE
 * This is the main js file.
 */
var AME = function(){

    /**
     * Private properties and methods.
     */
   
    var config = {
    	appName: 'AZARDI',
        context: null,
		locale: "en",
		defaultDirPath: ['Home', 'Infogrid Pacific'],
		dbFilePath: ['Home', 'Infogrid Pacific'],
		cdpDbFilePath: ['Home', 'Infogrid Pacific'],
		searchDbFilePath: ['Home', 'Infogrid Pacific'],
		tempDownloadPath: ['ProfD'],
		title: '',
		splashScreen: '',
		use_catalog: false,
		catalog_id: '',
		register_reader: false,
		register_url: '',
		db_version: '',
		readerToken: null,
		wasShelfOpen: false,
		XUL_NS: 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul',
		prompts: Components.classes['@mozilla.org/embedcomp/prompt-service;1'].getService(Components.interfaces.nsIPromptService),
		isCtrlPressed: false,
		isShiftPressed: false,
		istextbox: false,
		aud_playback_rate: 1,
		locale_index: -1,
		is_mac: false,
		azardi_version: "-",
		os_version: "-",
		device_name: "DT",
		platform_os: "-",
		manufacturer: "",
		model: ""
    };
	
	var isfullscreen = false;
	
    /**
     * Checks for the updates and downloads update file if available.
     */
    var checkUpdates = function() {
		var updateListener = {
			onProgress: function(request, position, totalSize) {
	                // alert('update check progress');
	        },
            onCheckComplete: function(request, updates, updateCount) {
	            //After retrieving updates.xml successfully, application will execute this code.
	
				if(updateCount > 0) {
	                //Don't ask user for applying updates. Do it at all.
	                var updater = Components.classes["@mozilla.org/updates/update-service;1"].createInstance(Components.interfaces.nsIApplicationUpdateService);
	
	                var update = updater.selectUpdate(updates, updateCount);
	                var download = updater.downloadUpdate(update, false);
	            }
            },
            onError: function(request, update) {
                // alert('update check error');
            },
            QueryInterface: function(aIID) {
                if (! aIID.equals(Components.interfaces.nsIUpdateCheckListener) && ! aIID.equals(Components.interfaces.nsISupports))
                        throw Components.results.NS_ERROR_NO_INTERFACE;
                return this;
            }
		};
	
        var checker = Components.classes["@mozilla.org/updates/update-checker;1"].createInstance(Components.interfaces.nsIUpdateChecker);

        checker.checkForUpdates(updateListener, true);
	}
	
	
	var loadLanguages = function () {
		try {
			// Query available and selected locales
			
			var chromeRegService = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService();
			var xulChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIXULChromeRegistry);
			var toolkitChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIToolkitChromeRegistry);
				
			var selectedLocale = xulChromeReg.getSelectedLocale("azardi2");
			var availableLocales = toolkitChromeReg.getLocalesForPackage("azardi2");
				
			// Render locale menulist by iterating through the query result from getLocalesForPackage()
			const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
				
			var localeListmenu = document.getElementById("locale-selector-menu");
			var localeListbox = document.getElementById("locale-selector");
				
			var selectedItem = null;
			var ctr = 0;
			while(availableLocales.hasMore()) {
					
				var locale = availableLocales.getNext();
					
				var listitem = document.createElementNS(XUL_NS, "menuitem");
				var locale_disp = getPropertiesString(locale);
				if(!locale_disp){
					locale_disp = locale;
				}
				listitem.setAttribute("value", locale);
				listitem.setAttribute("label", locale_disp);
				if (locale == selectedLocale) {
					// Is this the current locale?
					selectedItem = ctr;
				}
				localeListbox.appendChild(listitem);
				ctr = ctr + 1
			}
			
			// Highlight current locale
			localeListmenu.selectedIndex = ctr;
			config.locale_index = selectedItem;
			//localeListbox.selectedItem = selectedItem;
				
		} catch (err) {
			alert(err);
		}
	}
	
	var setLocale = function(newLocale) {
		// Write preferred locale to local user config
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefBranch);
		prefs.setCharPref("general.useragent.locale", newLocale);
	}
	
	/**
	 * This is the first function to be called when the application is started.
	 */
    var init = function(){
		var self = this;
		//Check if the updates are available.
		//Uncomment the following line before going live.
		//It should be commented while developing the code.
		self.checkUpdates();
		loadLanguages();
		try {
			var chromeRegService = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService();
			var xulChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIXULChromeRegistry);
			var toolkitChromeReg = chromeRegService.QueryInterface(Components.interfaces.nsIToolkitChromeRegistry);
				
			var selectedLocale = xulChromeReg.getSelectedLocale("global");
			var availableLocales = toolkitChromeReg.getLocalesForPackage("branding");
			while(availableLocales.hasMore()) {
				var locale = availableLocales.getNext();
				var locale_split = locale.split("-");
				config.locale = locale_split[0];
			}		
		} catch(e) {
		
		}
		
		//set a mac flag. This is used for video setup and webm vs mp4
		var osString = Components.classes["@mozilla.org/xre/app-info;1"]
               .getService(Components.interfaces.nsIXULRuntime).OS;
        if(osString.indexOf("Darwin") > -1){
			config.is_mac = true;
		}
		
		var info = Components.classes["@mozilla.org/xre/app-info;1"]
           .getService(Components.interfaces.nsIXULAppInfo);
		   
		config.azardi_version = info.version;
		config.os_version = window.navigator.oscpu;
		config.device_name = "DT";
		config.platform_os = osString;
		
		/*	
		 	If the application is not updating then to debug, comment above line and uncomment following six lines.
			They provide a decent ui for update procedure and also show errors if any.
		*/
		
		/*
		var um = Components.classes["@mozilla.org/updates/update-manager;1"].getService(Components.interfaces.nsIUpdateManager);
		var prompter = Components.classes["@mozilla.org/updates/update-prompt;1"].createInstance(Components.interfaces.nsIUpdatePrompt);
		if (um.activeUpdate && um.activeUpdate.state == "pending")
			prompter.showUpdateDownloaded(um.activeUpdate);
		else
			prompter.checkForUpdates();
		*/
		
		
		//Initilize the bootup preferences.
		self.setBootupPreferences();
		
		document.title = config.title;
		
		//Display the splash screen.
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
			$('#doc-viewer-azd').contents().find('#splash_image').attr('src', config.splashScreen);
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		$('#doc-viewer-azd').attr('src', 'html/welcome_azardi2.html');
		
		//Create directories and copy database tables if don't exist.
		config.defaultDirPath.push(config.title);
		
		config.dbFilePath.push(config.title);
		config.dbFilePath.push('azardi.azd');
		
		config.cdpDbFilePath.push(config.title);
		config.cdpDbFilePath.push('cdp_master.azd');
		
		config.searchDbFilePath.push(config.title);
		config.searchDbFilePath.push('azd_search.azd');
		
		createDirectory(config.defaultDirPath);
		self.copyDefaultDbTables();
		
		//Create default shelf, if not already created.
		Shelf.create('default', 'All Books', 'Contains all purchased or uploaded books.');
		
		//Initilize fonts.
		Fonts.init();
		
		//Initilize User Preferences with the values of database.
		UserPref.init();
		
		//Intilize Book Preferences with the values of user preferences.
		BookPref.init();
		
		//DC - FLO UPDATE - 201211
		Toolbar.setFLOToolbarDisplay(false);
		
		//DC - 201301 - Create new publisher table [if not exists]
		try {
			var fileObj = getFileObject(config.dbFilePath);
			createPubTables(fileObj);
		} catch(e){
			
		}
		//Check if you want to show catalog or just library.
		if (config.use_catalog) {
			//Show the splash screen for some time and then load the catalog.
			setTimeout(function() {
				if(config.register_reader) {
					if(!self.checkToken()) {
						$('#doc-viewer-azd').attr('src', 'html/azardi_register_reader.html');
					} else {
						Catalog.load(config.catalog_id);
					}
				}
			}, 2 * 1000);
		} else {
			//Check the database version.
			var dif = self.checkDbVersion();
			if(dif) {
				//Initilize the database updation.
				Update_DB.init();
			}
			//Initilize the toolbar.
			Toolbar.init();
			//Initilize Adding multiple epubs.
			AddMulEpub.init();
			//Initilize the keyboard events.
			self.setupKeyEvent();
			
		
		}
		
		window.resizeTo(UserPref.config.openWidth, UserPref.config.openHeight);
		
		//Finally show the main frame.
		$('#doc-viewer-azd').show();
		
		processCommandLineArgs();
		
		var aud_playback_rate = document.getElementById('aud_playback_rate');

		if (aud_playback_rate) {
			aud_playback_rate.addEventListener("change",function() {
				var slider_value = this.value;
				var ws = ((slider_value-5)/5) + 1;
				if (Epub.is_fixed_layout) {
					document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.changePlaybackRate(ws);
					return;
				}
				
				config.aud_playback_rate = ws;
				document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.changePlaybackRate(ws);
				
			});
		} 
    };
    
    /**
     * This function copies the default empty database tables to users home directory.
     */
    var copyDefaultDbTables = function () {
    	var self = this;
    	
		var searchDbObj = getFileObject(config.searchDbFilePath);
		var cdpDbObj = getFileObject(config.cdpDbFilePath);
		var azaDbObj = getFileObject(config.dbFilePath);
		var destObj = getFileObject(config.defaultDirPath);
		
		if (!searchDbObj.exists()) {
			var sourceObj = getFileObject(['CurProcD', 'chrome', 'content', 'azd_db', 'azd_search.azd']);
			sourceObj.copyTo(destObj, null);
		}
		
		if (!cdpDbObj.exists()) {
			var sourceObj = getFileObject(['CurProcD', 'chrome', 'content', 'azd_db', 'cdp_master.azd']);
			sourceObj.copyTo(destObj, null);
		}
		
		if (!azaDbObj.exists()) {
			var sourceObj = getFileObject(['CurProcD', 'chrome', 'content', 'azd_db', 'azardi.azd']);
			sourceObj.copyTo(destObj, null);
		}
    }
    
    /**
     * Checks if the database version differs.
     * @returns {boolean} true if version differs, false otherwise.
     */
    var checkDbVersion = function () {
		var self = this;
		
		//Get the database version from the database.
		var destinationFile = getFileObject(config.dbFilePath);
    	var table_name = 'System';
		var values = ['value'];
		var conditions = new Array(['key', 'db_version', 'TEXT']);
								   
		var statement = getValuesFromDatabase(destinationFile, values, table_name, conditions);
		var value_d;
		while(statement.executeStep()) {
			value_d = parseFloat(statement.row.value);
		}
		if(typeof value_d == 'undefined'){
			return true;
		}
		//Check the version no. from database with the version no. from azardi-bootup.js.
		if(value_d > parseFloat(config.db_version)){
			return true;
		}
		else{
			return false;
		}
    };
    
    var registerApp = function (orgId){
    	var self = this;
    	
    	var params = {
  						inn:{
  							url: config.register_url,
  							type: 'register_application',
  							catalogId: config.catalog_id,
  							orgId: orgId
  						},
  						out:{
  							out:null,
  							readerToken: null,
  							catDownloadUrl: null
  						}
  					};
  		window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
    	
    	if (!params.out.out) {
			var msg = getPropertiesString("azd_ame_regReaderFail");
    		alert(msg);
  			// alert('Failed to register the reader.\nExiting the application');
  			// window.close();
		}
		else{
			config.readerToken = params.out.readerToken;
			self.downloadCat(params.out.catDownloadUrl+orgId);
			// if(config.catalog_id === 'all') {
				// Catalog.displayAll();
			// } else {
				Catalog.load(config.catalog_id)
			// }
		}
    };
    
    var downloadCat = function(cat_download_url) {
    	var params = {
			inn:{
				cat_download_url: cat_download_url,
				type: 'cat_dai',
			},
			out:{
				out:null,
			}
		};
		window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=270, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();

		if (!params.out.out) {
			var msg = getPropertiesString("azd_ame_downloadCatFail");
			alert(msg);
			window.close();
		}
    }
    
    /**
     * Initilizes the bootup preferences found in azardi-bootup.js.
     */
    var setBootupPreferences = function () {
    	config.title = Bootup.reader_title || 'AZARDI';
    	config.splashScreen = Bootup.reader_splash || '../icons/AZARDI-splash.png';
    	UserPref.config.openWidth = Bootup.width || 700;
    	UserPref.config.openHeight = Bootup.height || 700;
    	config.use_catalog = Bootup.use_catalog;
    	if(Bootup.use_catalog){
    		config.catalog_id = Bootup.catalog_id;
    	}
    	config.register_reader = Bootup.register_reader;
    	config.register_url = Bootup.register_url;
    	config.db_version = Bootup.db_version;
    }
	
	var getDBVersion = function() {
		return config.db_version;
	}
	
	var getDefaultDirPath = function(){
		return config.defaultDirPath;
	}
	
	var getDbFilePath = function(){
		return config.dbFilePath;
	}
	
	var getCdpDbFilePath = function () {
		return config.cdpDbFilePath;
	}
	
	var getSearchDbFilePath = function () {
		return config.searchDbFilePath;
	}
	
	var getTempDownloadPath = function () {
		return config.tempDownloadPath;
	}
	
	// var getCfsLocation = function () {
		// // return config.cfs_location;
	// }
	
	var getCatalogId = function () {
		return config.catalog_id;
	}
	
	var getReaderToken = function () {
		return config.readerToken;
	}
	
	var getAppName = function () {
		return config.appName;
	}
	
	/**
	 * Sets the title of the window to the book title
	 * @params title Title of the book
	 * @params author Author of the book
	 */
	var windowTitle = function(title, author){
		if (title.length > 40) {
	        title = title.substring(0, 37) + '...';
	    }
		if(author.length > 15){
			author = author.substring(0, 12) + '...';
		}
	    document.title = config.title+' :: '+title + ' - ' + author;
	};
	
	/**
	 * Displays the library (table/thumbnail)
	 * @params isBookAdded Is the new book added in the library.
	 */
	var showLibrary = function(isBookAdded) {
		var self = this;
		
		if(Library.config.open || Shelf.config.open){
			return;
		}
		
		//return to the normal mode
		var currentState = window.fullScreen;
		var was_fullscreen = window.fullScreen;
		if (currentState == true) {
			currentState = false;
			AME.isfullscreen = currentState;
			window.setTimeout( function(aWindow) {
				 aWindow.fullScreen = currentState;
				 if(config.wasShelfOpen) {
					$('#but_show_shelf').click();
				}
			 },0,window);
		} 
		
		if(config.context == 'AZD'){
			Azd.unload();
			Toolbar.closeAzd();
		}
		
		
		if(config.context == 'ePub'){
			//If the epub was open, save the preferences to database.
			BookPref.save();
			//Unload the epub
			Epub.unload();
			//Hides the epub related buttons from toolbar
			Toolbar.closeEpub();
		}
		
		$('#viewport1-azd').show();
		$('#shelf_label').show();
		//DC - FLO UPDATE - 201211
		$('#doc-flo-viewer-azd').hide();
		$('#doc-viewer-azd').hide();
		
		//	FIXME: Why do we need to show table view here. Not working without this statement. Need to check.
		$('#library-azd').show();
		//Show the buttons related to library.
		Toolbar.showLibrary();
		
		//DC - FLO UPDATE - 201211
		Toolbar.setFLOToolbarDisplay(false);
		Toolbar.enableUserPrefButton();
		
		document.title = config.title;
		
		if(config.wasShelfOpen) {
			//Show the shelf, if shelf was open earlier.
			if(!was_fullscreen) {
				//We want to do the click action after the window resizes to normal.
				//so we put in a additional condition here
				$('#but_show_shelf').click();
			}
			if(isBookAdded) {
				//Show the books in shelf
				Shelf.showBooks(Shelf.config.currentShelfId, Shelf.config.currentShelfName, Shelf.config.currentShelfDescription);
			}
		} else {
			//Initilize the library
			Library.init();
			config.context  = 'library';
			Library.config.open = true;
			Shelf.config.open = false;
		}
		
		var targetbuttonid = "";
		
		//Set the button status
		if(UserPref.config.libraryView == 'table') {
			document.getElementById('but_table').click();
			targetbuttonid = 'but_table';
		} else if(UserPref.config.libraryView == 'thumbnail') {
			document.getElementById('but_thumb').click();
			targetbuttonid = 'but_thumb';
		}
		
		if (targetbuttonid) {
			try {
				var evt = document.createEvent("MouseEvents");
				evt.initMouseEvent("click", true, true, window,
					0, 0, 0, 0, 0, false, false, false, false, 0, null);
				var cb = document.getElementById(targetbuttonid); 
				var canceled = !cb.dispatchEvent(evt);
			} catch(e) {
			
			}
		}
		
		//If new book is added, then scroll to top to show the new books.
		if(isBookAdded) {
			$('#gridbox').scrollTop(0);
			$('#library-azd').scrollTop(0);
		}
	};
	
	/**
	 * Function to hide library related items.
	 */
	var closeLibrary = function(){
		$('#viewport1-azd').hide();
		$('#library-azd').hide();
		$('.gridbox').hide();
		$('#shelf_label').hide();
		Toolbar.closeLibrary();
		Library.config.open = false;
	};
	
	var sortLibrary = function(column){
		Library.sort(column)
		Library.setLibraryView();
	}
	
	/**
	 * Toggles the library view between table and thumbnail.
	 * @params view View to be applied
	 */
	var changeLibraryView = function(view) {
		if(view == 'table'){
			$('#library-azd').show();
			$('.gridbox').hide();
		}
		else if(view == 'thumbnail'){
			$('#library-azd').hide();
			$('.gridbox').show();
		}
		else if(view == 'stack'){
			//	TODO: Implement the stack view.
		}
		Library.config.libraryView = view;
		UserPref.config.libraryView = view;
		//Set the library view. table|thumbnail.
		Library.setLibraryView();
	};
	
	var getTitle = function () {
		return config.title;
	}
	
	var showShelf = function(){
		if(Shelf.config.open){
			return;
		}
		
		var shelfWidth = parseInt(UserPref.config.openWidth) + 250;
		
		window.resizeTo(shelfWidth, UserPref.config.openHeight);
		
		$('#leftcol-aza').show();
		document.title = config.title;
		
		Shelf.init();
		
		config.context = 'shelf';
		// config.wasShelfOpen = true;
		Shelf.config.open = true;
		Library.config.open = false;
	}
	
	/**
	 * Function to hide shelf related elements.
	 */
	var closeShelf = function(){
		if(!Shelf.config.open){
			return;
		}
		
		$('#leftcol-aza').hide();
		
		window.resizeTo(UserPref.config.openWidth, UserPref.config.openHeight);
		
		if(document.getElementById('but_show_shelf').hasAttribute('checked')){
			document.getElementById('but_show_shelf').removeAttribute('checked');
		}
		
		Shelf.config.open = false;
		// config.wasShelfOpen = false;
	};
	
	var toggleShelf = function(){
		var self = this;
		
		if(document.getElementById('but_show_shelf').hasAttribute('checked')){
			self.showShelf();
			config.wasShelfOpen = true;
		}
		else{
			self.closeShelf();
			config.context = 'library';
			Library.config.open = true;
			Shelf.config.open = false;
			config.wasShelfOpen = false;
		}
	}
	
	var resetAddBookForm = function(){
		$('.book-info').val('');
		// $('#cover_thumbnail').attr('src', '');
		$('#cover_thumbnail').css('background-image', '');
	    $('#btn_ok').attr('disabled', true);
	    $('#btn_ok_mul_add').attr('disabled', true);
	    
	    AddMulEpub.clear();
	}
	
	var showAddBookForm = function(){
		var self = this;
		
		self.closeLibrary();
		self.resetAddBookForm();
		self.closeShelf();
		$('#form_add_doc').show();
		$('#file_type').focus();
	}
	
	var closeAddBookForm = function(isBookAdded){
		var self = this;
		
		$('#form_add_doc').hide();
		self.showLibrary(isBookAdded);
	}
	
	/**
	 * Displays the user preferences form
	 */
	var showUserPrefForm = function(){
		var self = this;
		
		if(config.context == 'library')
			self.closeLibrary();
		if (config.context == 'shelf') {
			self.closeLibrary();
			self.closeShelf();
		}
		if (config.context == 'ePub') {
			$('#doc-viewer-azd').hide();
			Toolbar.closeEpub();
		}
		
		UserPref.setForm();
		$('#form_usr_pref').show();
		$('#user_pref_lib_view').focus();
		//Set the current app
		var localeListbox = document.getElementById("locale-selector-menu");
		localeListbox.selectedIndex = config.locale_index;
	}
	
	var closeUserPrefForm = function(){
		var self = this;
		
		// self.resizeWindow(UserPref.config.openWidth, UserPref.config.openHeight);
		if(config.context == 'library' || config.context == 'shelf'){
			$('#form_usr_pref').hide();
			self.showLibrary();	
		}
		else if(config.context == 'ePub'){
			$('#form_usr_pref').hide();
			$('#doc-viewer-azd').show();
			Toolbar.showEpub();
			
			self.showLoading();
			
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.config.showLastPage = false;
				Columns.config.goToDepth = Columns.config.currentReadingDepth;
				Columns.init(BookPref.config.noOfCols);
			}
			else{
				Flow.config.showLastPage = false;
				Flow.init();
			}
			
			self.hideLoading();
		}
	}
	
	var resizeWindow = function(width, height){
		window.resizeTo(width, height)
	}
	
	/**
	 * Intilizes the sequence to save user preferences to database and displays either previously opened book or library
	 */
	var saveUserPrefs = function(){
		var self = this;
		
		UserPref.save();
		if(config.context == 'ePub'){
			BookPref.init();
			BookPref.set();	
			
			Chapter.config.orig_width = UserPref.config.openWidth;
			Chapter.config.orig_height = UserPref.config.openHeight;
			
			if(Chapter.config.azardi_presentation == 'fixed') {
				Chapter.unbindResize();
				
				if(Chapter.config.azardi_width && Chapter.config.azardi_height) {
					if(Chapter.config.azardi_margins != -1) {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * Chapter.config.azardi_margins);
					} else {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = parseInt(Chapter.config.azardi_height)+parseInt(Chapter.config.toolbar_height);
				} else if(Chapter.config.azardi_width) {
					if(Chapter.config.azardi_margins != -1) {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * Chapter.config.azardi_margins);
					} else {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = BookPref.config.openHeight;
				} else if(Chapter.config.azardi_height){
					window.innerWidth = BookPref.config.openWidth;
					window.innerHeight = parseInt(Chapter.config.azardi_height)+parseInt(Chapter.config.toolbar_height);
					
				} else {
					window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				}
			} else if(config.azardi_presentation == 'fullscreen') {
				self.unbindResize();
				width = screen.width || BookPref.config.openWidth;
				height = screen.height || BookPref.config.openHeight;
				window.resizeTo(width, height);
				window.moveTo(0, 0);
			} else if(Chapter.config.azardi_presentation == null) {
				Chapter.unbindResize();
				Chapter.bindResize();
				window.resizeTo(Chapter.config.orig_width, Chapter.config.orig_height);
			}
		} else if(config.context == 'library' || config.context == 'shelf') {
			self.resizeWindow(UserPref.config.openWidth, UserPref.config.openHeight);
		}
		
		self.closeUserPrefForm();
	}
	
	var resetCreateShelfForm = function(){
		$('#shelf_name').val('');
		$('#shelf_description').val('');
		$('#btn_shelf_ok').attr('disabled', 'true');
	};
	
	/**
	 * Displays the add new shelf form.
	 */
	var showCreateShelfForm = function(){
		var self = this;
		
		//Close library and shelf
		self.closeLibrary();
		self.closeShelf();
		//Reset the add new shelf form
		self.resetCreateShelfForm();
		//Finally, show the form
		$('#form_create_shelf').show();
		$('#shelf_name').focus();
	};
	
	/**
	 * Displays the add new shelf form.
	 */
	var showCreatePublisherShelfForm = function(){
		var self = this;
		
		//Close library and shelf
		self.closeLibrary();
		self.closeShelf();
		//Reset the add new shelf form
		$('#pshelf_url').val('');
		$('#pshelf_code').val('');
		$('#pshelf_name').val('');
		$('#pshelf_pas').val('');
		$('#btn_pshelf_ok').attr('disabled', 'true');
		
		if (Bootup.hasOwnProperty("single_bookstore") &&  
				Bootup.hasOwnProperty("single_bookstore_url") && 
				Bootup.single_bookstore && Bootup.single_bookstore_url ) {
			$('#row_pshelf_url').hide();
			$('#row_pshelf_code').hide();
			$('#pshelf_url').val(Bootup.single_bookstore_url);
		}
		
		
		//Finally, show the form
		$('#form_create_pshelf').show();
		$('#pshelf_url').focus();
	};
	
	var refreshPublisherShelf = function() {
		var clickedShelf = Shelf.getClicked();
		if (clickedShelf && (Shelf.config.currentShelfType == "acfs")) {
			Shelf.updatePubShelf(clickedShelf.id);
			Library.showPubShelfBooks(clickedShelf.id, Shelf.config.currentShelfName, Shelf.config.currentShelfDescription);
		}
	};
	var closeCreateShelfForm = function(){
		var self = this;
		
		$('#form_create_shelf').hide();
		self.showLibrary();
	};
	
	var closePubCreateShelfForm = function(){
		var self = this;
		
		$('#form_create_pshelf').hide();
		self.showLibrary();
	};
	
	var createShelf = function(){
		var self = this;
		
		var type = 'user',
		    name = $('#shelf_name').val(),
		    description = $('#shelf_description').val();
		
		var shelfCreated = Shelf.create(type, name, description);
		
		if(!shelfCreated){
			var msg = getFormattedPropertiesString("azd_ame_dupShelf", [name]);
			//var message = AzdMessages.dupShelf.replace('%name%', name); //
			var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
			prompts.alert(null, AME.getTitle(), msg);
			return;
		}
		
		self.closeCreateShelfForm();
	}
	
	var createPubShelf = function(){
		var self = this;
		
		var type = 'acfs',
		    bsurl = $('#pshelf_url').val(),
		    bsacc = $('#pshelf_code').val(),
		    name = $('#pshelf_name').val(),
		    pas = $('#pshelf_pas').val();
		
		azDump("createPubShelf", type, bsurl, bsacc, name, pas);
		
		var shelfCreated = Shelf.createPub(type, bsurl, bsacc, name, pas);
		
		if(!shelfCreated){
			return;
		}
		
		self.closePubCreateShelfForm();
	}
	
	var deleteShelf = function(){
		var clickedShelf = Shelf.getClicked();
		Shelf.deleteShelf(clickedShelf);
		Shelf.init();
	}
	
	var addBook = function(epubtype){
		var self = this;
		
		var bookMetadata = {};
		if (!epubtype) {
			bookMetadata.type = $('#file_type').val();
		} else {
			bookMetadata.type = epubtype;
		}	
		bookMetadata.uid = $('#book_id').val();
		bookMetadata.created_date = $('#created_date_in_ms').val();
		bookMetadata.title = $('#title').val();
		bookMetadata.author = $('#author').val();
		bookMetadata.publisher_identifier = $('#publisher').val();
		bookMetadata.publish_date = $('#publish_date').val();
		bookMetadata.description = $('#description').val();
		bookMetadata.location = $('#file_location').val();
		bookMetadata.cover_image = $('#cover_image').val();
		bookMetadata.cover_type = $('#cover_type').val();
		bookMetadata.version = $('#version').val();
		
		//DC - FLO UPDATE - 201211 - New FLO variables
		bookMetadata.rendition_layout = $('#rendition_layout').val();
		bookMetadata.rendition_orientation = $('#rendition_orientation').val();
		bookMetadata.rendition_spread = $('#rendition_spread').val();
		
		BookMetadata.add(bookMetadata);
		
		Shelf.addBook(bookMetadata, 'All Books');
		
		document.getElementById('library-azd').setAttribute("sortResource", "created_date");
		document.getElementById('library-azd').setAttribute("sortDirection", "descending");
		
		self.closeAddBookForm(true);
	};
	
	var addMultipleBooks = function() {
		var self = this;
		
		var allBookMetadata = AddMulEpub.getAllBooksInfo();
		for(var i=0; i<allBookMetadata.length; i++) {
			var bookMetadata = allBookMetadata[i][1];
			BookMetadata.add(bookMetadata);
			Shelf.addBook(bookMetadata, 'All Books');
		}
		
		document.getElementById('library-azd').setAttribute('sortResource', 'created_date');
		document.getElementById('library-azd').setAttribute('sortDirection', 'descending');
		
		self.closeAddBookForm(true);
	};
	
	/**
	 * Initilizes the sequence to completely delete book from database
	 * @params bookId Id of the book to be deleted 
	 */
	var deleteBook = function(bookId){

		if (Library.config.library_type == "ACFS") {
			AME.deletePubBook(bookId);
			return;
		}
	
		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
		
		var msg = getPropertiesString("azd_ame_confirmBookDelete");
		
		if(prompts.confirm(null, AME.getTitle(), msg)){
			if(typeof bookId == 'undefined') {
				var bookId = Library.getClickedBook();
			}
			
			if(Library.config.open) {
				Library.deleteBook(bookId);
				BookPref.deletePref(bookId);
				Notes.deleteAllNotesFromDatabase(bookId);
			} else if(Shelf.config.open) {
				if(Shelf.config.currentShelfId == Shelf.config.allBooksShelfId) { //This is the allbooks shelf
					Library.deleteBook(bookId);
					BookPref.deletePref(bookId);
					Notes.deleteAllNotesFromDatabase(bookId);
				} else {
					Shelf.deleteBook(bookId, Shelf.config.currentShelfId);
				}
			}
			
			if(config.context == 'library'){
				Library.init();
				Library.setLibraryView();
			}
			else if(config.context == 'shelf'){
				Library.showShelfBooks(Shelf.config.currentShelfId, Shelf.config.currentShelfName, Shelf.config.currentShelfDescription);
			}
		}
	};
	
	var addBookToShelf = function(event, shelfName){
		
		shelfName = unescape(shelfName);
		event.dataTransfer.effectAllowed = "copy";
		var bookId = '';
		if(Library.config.libraryView == 'table'){
			bookId = Library.getClickedBook();
		}
		if(Library.config.libraryView == 'thumbnail'){
			bookId = event.dataTransfer.getData("text/plain");
		}
		var bookMetadata = {};
		bookMetadata['id'] = bookId;
		Shelf.addBook(bookMetadata, shelfName);
	};
	
	/**
	 * Extracts the information from user selected file and fills the add book form
	 */
	var selectFile = function(fileType){
		var self = this;
		
		if (fileType == 'AZD'){
		 	var azdFolder = openSelectFile('Select File', 'directory', 'none');
			//getInfo calls the enterBookInformation from its side as it can not return any value.
			Azd.getInfo(azdFolder);
			//TODO: Do not set image to iframe instead set them directly to the canvas.
	    }
	    else if (fileType == 'ePub'){
			var epubFile = openSelectFile('Select File', 'open', 'ePub');
			
			if(!epubFile) {
				return;
			}
			try{
				var epubInfo = Epub.getInfo(epubFile);
			}
			catch(e){
				//	If any error while loading epubs then display error message and exit.
				var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
				prompts.alert(null, AME.getTitle(), e.message);
				// alert(e.message);
				return;
			}
			self.setCoverThumbnail(epubInfo.coverImagePath);
			self.enterBookInformation(epubInfo.metadata);
			self.changeAddBookOkButtonStatus();
		} else if (fileType == 'E0'){
			var epubFile = openSelectFile('Select File', 'open', 'e0');
			
			if(!epubFile) {
				return;
			}
			try{
				var epubInfo = Epub0.getE0Info(epubFile);
			}
			catch(e){
				//	If any error while loading epubs then display error message and exit.
				var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
				prompts.alert(null, AME.getTitle(), e.message);
				// alert(e.message);
				return;
			}
			self.setCoverThumbnail(epubInfo.coverImagePath);
			self.enterBookInformation(epubInfo.metadata);
			self.changeAddBookOkButtonStatus();
			return;
		}
	}
	
	/**
	 * Set the cover thumbnail in add book form
	 * @params coverPath Path of the cover file
	 */
	var setCoverThumbnail = function(coverPath){
		var self = this;
		
		// coverPath = getFileObjectFromPath(coverPath);
		// var coverUrl = getURL(coverPath);
		
		self.getCoverThumbDataUri(coverPath);
		
		
		// if(coverPath.exists()){
			// //	if the cover file is there then only set the url.
			// //	TODO: in earlier functions when i get the coverpath, need to check if the file exists or not and then
			// //	depending on that should fallback on some default cover.
			// $('#pageContainerIframe').attr('src', coverUrl);
		// }
		setTimeout(function(){
			// var coverThumbDataUri = AME.getCoverThumbDataUri();
			// $('#cover_thumbnail').attr('src', coverThumbDataUri);
			// $('#cover_image').val(coverThumbDataUri);
// 			
			// $('#pageContainerIframe').attr('src', '');
			
			//	Here we can delete the extracted folder.
			deleteFolder(getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder]));
		}, 500);
	}
	
	/**
	 * Returns the scaled dimension
	 * @params maxW Maximum width allowed for element
	 * @params maxH Maximum height allowed for element
	 * @params currW Current width of element
	 * @params currH Current height of element
	 */
	var scaleSize = function(maxW, maxH, currW, currH){

		var ratio = currH / currW;
		
		if(currW >= maxW && ratio <= 1){
			currW = maxW;
			currH = currW * ratio;
		} else if(currH >= maxH){
			currH = maxH;
			currW = currH / ratio;
		}
		
		return [currW, currH];
	};
	
	/**
	 * Sets the datauri to the cover container
	 * @params coverPath Path of the cover file
	 */
	var getCoverThumbDataUri = function(coverPath, callback){
		var self = this;
		try {
			//If the image is svg, do not resize it.
			if(coverPath.split('.').pop() == 'svg') {
				var coverUri = generateDataURI(coverPath);
				$('#cover_thumbnail').css('background-image', 'url('+coverUri+')');
				$('#cover_image').val(coverUri);
				if (typeof(callback) == "function") {
					callback();
				}
			} else {
				var coverObject = getFileObjectFromPath(coverPath);
				var coverUrl = getURL(coverObject);
				var canvas = document.getElementById("thumb_canvas");
				var ctx = canvas.getContext("2d");
				var img = new Image();
				img.onload = function(){
					var currW = img.width;
					var currH = img.height;
					var dim = self.scaleSize(300, 300, currW, currH);
					canvas.width = dim[0];
					canvas.height = dim[1];
					ctx.mozImageSmoothingEnabled = true;
					ctx.drawImage(img, 0, 0, dim[0], dim[1]);
					
					var coverUri = canvas.toDataURL("image/png");
					$('#cover_thumbnail').css('background-image', 'url('+coverUri+')');
					$('#cover_image').val(coverUri);
					if (typeof(callback) == "function") {
						callback();
					}			
				};
				img.src = coverUrl;
			}
		
		} catch(e) {
			if (typeof(callback) == "function") {
				callback();
			}	
		}
	};
	
	/**
	 * Enters the book information in add book form
	 */
	var enterBookInformation = function(bookInfo){
		//DC - FLO UPDATE - 201211
		//handle the new FLO specific metadata
		var location = '',
		    bookId = '',
		    title = '',
		    author = '',
		    publisher = '',
		    publishDate = '',
		    description = '',
		    version = '',
		    cover_type = '',
		    currentTime = new Date(),
			layout = '',
			orientation = '',
			spread = '';
		$.each(bookInfo, function(index, value){
			switch(value[0]){
				case 'dc:location':
					location = value[1];
					break;
				case 'dc:identifier':
					bookId = value[1];
					break;
				case 'dc:title':
					title = value[1];
					break;
				case 'dc:creator':
					author = value[1];
					break;
				case 'dc:publisher':
					publisher = value[1];
					break;
				case 'dcterms.publisher':
					publisher = value[1];
					break;
				case 'dc:date':
					publishDate = value[1];
					break;
				case 'dc:description':
					description = value[1];
					break;
				case 'dc:version':
					version = value[1];
					break;
				case 'dc:cover_type':
					cover_type = value[1];
					break;
				case 'rendition:layout':
					layout = value[1];
					break;
				case 'rendition:orientation':
					orientation = value[1];
					break;
				case 'rendition:spread':
					spread = value[1];
					break;
			}
		});
		
		$('#file_location').val(location);
		$('#book_id').val(bookId);
		$('#version').val(version);
		$('#title').val(title);
		$('#author').val(author);
		$('#publisher').val(publisher);
		$('#created_date').val(currentTime.toLocaleDateString());
		$('#created_date_in_ms').val(currentTime.getTime());
		$('#publish_date').val(publishDate);
		$('#description').val(description);
		$('#cover_type').val(cover_type);
		$('#rendition_layout').val(layout);
		$('#rendition_orientation').val(orientation);
		$('#rendition_spread').val(spread);
	};
	
	var openBookAtFirst = function() {
		var self  = this;
		
		if(typeof bookId === 'undefined'){
			var bookId = Library.getClickedBook();
		}
		
		
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === +bookId){
				var bookType = Library.config.bookMetadata[i]['type'],
					bookTitle = Library.config.bookMetadata[i]['title']
					bookAuthor = Library.config.bookMetadata[i]['author'],
					bookLocation = Library.config.bookMetadata[i]['location'],
					bookImage = Library.config.bookMetadata[i]['cover_image'],
					bookImageType = Library.config.bookMetadata[i]['cover_type'],
					bookVersion = Library.config.bookMetadata[i]['version'],
					rendition_layout = Library.config.bookMetadata[i]['rendition_layout']
					rendition_orientation = Library.config.bookMetadata[i]['rendition_orientation'],
					rendition_spread = Library.config.bookMetadata[i]['rendition_spread'],
					identifier = Library.config.bookMetadata[i]['uid'];
				break;
			}
		}
		
		Library.config.currentBookId = bookId;

		//DC - FLO UPDATE - 201211 - passing addition parameter
		var additionalparams = {};
		additionalparams['rendition_layout'] = rendition_layout;
		additionalparams['rendition_orientation'] = rendition_orientation;
		additionalparams['rendition_spread'] = rendition_spread;
		additionalparams['identifier'] = identifier;
		
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			self.closeLibrary();
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
			self.closeShelf();
			$('#doc-viewer-azd').show();
			
			Toolbar.disableEpubBtn();
			Toolbar.setFLOToolbarDisplay(false);
			self.showLoading();
			setTimeout(function(){
				if (Library.config.library_type == "ACFS") {
					if (Library.config.bookMetadata[i]['bookstate'] != "READY") {
						if(UserPref.config.libraryView == 'table') {
							var msg = getPropertiesString("azd_ame_downloadBookFromTable");
							alert(msg);
						} else {
							var msg = getPropertiesString("azd_ame_downloadBookFromGrid");
							alert(msg);
						}
						AME.showLibrary();
						self.hideLoading();
						return false; 
					}
					 
					try {
						Library.config.currentBookId = Library.config.bookMetadata[i]['workuid'];
						Epub.loadPubBook(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, true, additionalparams,
							Library.config.bookMetadata[i]['full_md']);
					}catch(e){
						//alert(e);
					}
				} else {
					Epub.load(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, true, additionalparams);
				}
			}, 100);
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
		
		config.context = 'ePub';
		
		Chapter.history.chapterNo = null;
		Chapter.history.readingDepth = null;
		self.windowTitle(bookTitle, bookAuthor);
	}
	
	/**
	 * This function initiates the sequence to open a book from library.
	 * @params bookId Id of the book to open. 
	 */
	var openBook = function(bookId){
		var self  = this;
		
		//bookId is undefined when it is opened from the table view.
		if(typeof bookId === 'undefined'){
			var bookId = Library.getClickedBook();
		}
		
		// Get the metadata of the clicked book.
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === +bookId){
				var bookType = Library.config.bookMetadata[i]['type'],
					bookTitle = Library.config.bookMetadata[i]['title']
					bookAuthor = Library.config.bookMetadata[i]['author'],
					bookLocation = Library.config.bookMetadata[i]['location'],
					bookImage = Library.config.bookMetadata[i]['cover_image'],
					bookImageType = Library.config.bookMetadata[i]['cover_type'],
					bookVersion = Library.config.bookMetadata[i]['version']
					rendition_layout = Library.config.bookMetadata[i]['rendition_layout']
					rendition_orientation = Library.config.bookMetadata[i]['rendition_orientation']
					rendition_spread = Library.config.bookMetadata[i]['rendition_spread']
					identifier = Library.config.bookMetadata[i]['uid'];
				break;
			}
		}
		Library.config.currentBookId = bookId;
		//DC - FLO UPDATE - 201211 - passing addition parameter
		var additionalparams = {};
		additionalparams['rendition_layout'] = rendition_layout;
		additionalparams['rendition_orientation'] = rendition_orientation;
		additionalparams['rendition_spread'] = rendition_spread;
		additionalparams['identifier'] = identifier;
		if(bookType == "" || (typeof bookType == "undefined")) {
			bookType = "ePub"; 
		}
		
		if(bookType == "EPUB") {
			bookType = "ePub"; 
		}
		
		if(bookType == "E0") {
			bookType = "e0"; 
		}
		
		//AZD is not used now, see function openBookFromCatalog
		if(bookType == 'AZD'){
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				self.closeLibrary();
				self.closeShelf();
				$('#doc-viewer-azd').show();
				Azd.load(bookLocation, cdpId);
				
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_cdp_doc.html');
			
			Toolbar.showAzd();
			config.context = 'AZD';
		}
		else if(bookType == 'ePub' || bookType == 'DT' || bookType == 'e0'){
			//	This is required for library to close only when the default epub document is fully loaded.
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				// Close the library and the shelf
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
				self.closeLibrary();
				self.closeShelf();
				$('#doc-viewer-azd').show();
				
				Toolbar.disableEpubBtn();
				//show loading overlay
				self.showLoading();
				setTimeout(function(){
					if (Library.config.library_type == "ACFS") {
						if (Library.config.bookMetadata[i]['bookstate'] != "READY") {
							if(UserPref.config.libraryView == 'table') {
								var msg = getPropertiesString("azd_ame_downloadBookFromTable");
								alert(msg);
							} else {
								var msg = getPropertiesString("azd_ame_downloadBookFromGrid");
								alert(msg);
							}
							AME.showLibrary();
							self.hideLoading();
							return false; 
						}
						 
						try {
							Library.config.currentBookId = Library.config.bookMetadata[i]['workuid'];
							Epub.loadPubBook(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, false, additionalparams,
								Library.config.bookMetadata[i]['full_md']);
						}catch(e){
							//alert(e);
						}
					} else {
						Epub.load(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, false, additionalparams);
					}
				}, 100);
			});
			
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			
			config.context = 'ePub';
			
			Chapter.history.chapterNo = null;
			Chapter.history.readingDepth = null;
		}
		//set the window title
		self.windowTitle(bookTitle, bookAuthor);
	};
	
	var openBookFromCatalog = function (cdpId) {
		var self = this;
		
		self.hideStatusBar();
		self.removeAllMessages();
		var bookLocation = getFileObject(config.cdpDbFilePath);
		
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			try{
				Azd.load(bookLocation.path, cdpId);
			}catch(e){alert(e);}
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_cdp_doc.html');
		config.context = 'AZD';
	}
	
	/**
	 * Show the loading overlay.
	 */
	var showLoading = function(){
		$('#doc-viewer-azd').contents().find('#loading_overlay').show();
	};
	
	/**
	 * Hide the loading overlay.
	 */
	var hideLoading = function(){
		$('#doc-viewer-azd').contents().find('#loading_overlay').hide();
	};
	
	/**
	 * Hide/show full page toc
	 * @params show Boolean expression.
	 */
	var toggleFullPageToc = function(show){
		if(show){
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
				if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
					Chapter.history.readingDepth = Columns.config.currentReadingDepth;
					Chapter.config.oldReadingDepth = Columns.config.currentReadingDepth;
				}
				if(Epub.config.version == 3) {
					Epub.setFullPageTocDataEpub3();
				} else {
					Epub.setFullPageTocDataEpub2();
				}
				Notes.getAllNotes(Library.config.currentBookId);
				Notes.sortAllNotes();
				Notes.createAllNotesPage();
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_toc.html');
			Toolbar.showFullPageToc();
		}
		else{
			/*$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				//DC - FLO UPDATE - 201211 - Dont need this as Chapter.loadPageEx sets it up separately
				/*if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				
				
				
				//$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});*/
			//$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			Chapter.goToReadingDepth();
			Toolbar.showEpub();
		}
	}
	
	/**
	 * Show/Hide Landmark page
	 * @params show Boolean expression
	 */
	var toggleLandmark = function (show) {
		var self = this;
		
		if(show) {
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
				
				if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
					Chapter.history.readingDepth = Columns.config.currentReadingDepth;
					Chapter.config.oldReadingDepth = Columns.config.currentReadingDepth;
				}
				
				// Set landmark and page-nav data
				Epub.setLandmarkDataEpub3();
				Epub.setPageNavDataEpub3();
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_landmark.html');
			Toolbar.showLandmark();
		} else {
			/*$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				//DC - FLO UPDATE - 201211 - Dont need this as Chapter.loadPageEx sets it up separately
				if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
				
				
			});*/
			//$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			Chapter.goToReadingDepth();
			Toolbar.showEpub();
		}
	};
	
	/**
	 * Initilizes the sequence to go to next page
	 */
	var nextPage = function(){
		Chapter.next();
	};
	
	/**
	 * Initilizes the sequence to go to previous page
	 */
	var previousPage = function(){
		Chapter.previous();
	}
	
	/**
	 * Initilizes the sequence to go to next chapter
	 */
	var nextChapter = function() {
		Chapter.nextChapter();
	};
	
	/**
	 * Initilizes the sequence to go to previous chapter
	 */
	var previousChapter = function() {
		Chapter.previousChapter();
	};
	
	var changeFontSize = function(size){
		var currentSize = parseFloat(Fonts.config.fontSize);
		if(size === 'plus') {
			currentSize += 0.10;
		} else {
			currentSize -= 0.10;
		}
		currentSize = currentSize.toFixed(1);
		if(currentSize <= Fonts.config.maxValue) {
			Fonts.set(currentSize);
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.initilizeVariables();
				if(Columns.config.noOfCol > 1) {
					Columns.removeEmptyCols();
					Columns.appendEmptyCols(Columns.config.noOfCol);
				}
				Columns.setApproximatePage();
				Columns.getColumnCount();
				Columns.getCurrentColPosition();
				Columns.getCurrentReadingDepth();
				Columns.updateProgressBar();
				Chapter.setNavButtons();
			}
			else{
				Flow.getCurrentReadingDepth();
			}
		}
	}
	/*
	var changeFontSize = function(size){
		Fonts.set(size);
		if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
			Columns.initilizeVariables();
			Columns.getColumnCount();
			Columns.getCurrentColPosition();
			Columns.getCurrentReadingDepth();
			Columns.updateProgressBar();
			Chapter.setNavButtons();
		}
		else{
			Flow.getCurrentReadingDepth();
		}
	}
	*/
	
	var changeLineSpace = function(space){
		var currentSpace = parseFloat(Line.config.lineSpacing);
		if(space === 'plus') {
			currentSpace += 0.10;
		} else {
			currentSpace -= 0.10;
		}
		currentSpace = currentSpace.toFixed(1);
		if(currentSpace >= Line.config.minValue) {
			Line.set(currentSpace);
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.initilizeVariables();
				if(Columns.config.noOfCol > 1) {
					Columns.removeEmptyCols();
					Columns.appendEmptyCols(Columns.config.noOfCol);
				}
				Columns.setApproximatePage();
				Columns.getColumnCount();
				Columns.getCurrentColPosition();
				Columns.getCurrentReadingDepth();
				Columns.updateProgressBar();
				Chapter.setNavButtons();
			}
			else{
				Flow.getCurrentReadingDepth();
			}
		}
	}
	
	/*
	var changeLineSpace = function(space){
		Line.set(space);
		if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
			Columns.initilizeVariables();
			Columns.getColumnCount();
			Columns.getCurrentColPosition();
			Columns.getCurrentReadingDepth();
			Columns.updateProgressBar();
			Chapter.setNavButtons();
		}
		else{
			Flow.getCurrentReadingDepth();
		}
	}
	*/
	// var changeParagraphSpace = function(space){
		// Paragraph.set(space);
		// if(BookPref.config.columns){
			// Columns.initilizeVariables();
			// Columns.getColumnCount();
			// Columns.getCurrentColPosition();
			// Columns.getCurrentReadingDepth();
			// Columns.updateProgressBar();
		// }
	
	var toggleJustify = function(justify){
		Justify.set(justify);
	}
	
	var toggleHyphen = function(hyphenate){
		Hyphen.set(hyphenate);
	}
	
	var toggleAudioControls = function(display) {
		if(display) {
			Toolbar.showAudioControls();
		} else {
			Toolbar.hideAudioControls();
		}
	};
	
	var changeMarginSpace = function(space){
		if(Chapter.config.azardi_margins == -1 || Chapter.config.azardi_presentation != 'fixed') {
			var currentSpace = parseInt(Margin.config.margin);
			if(space === 'plus') {
				currentSpace += 10;
			} else {
				currentSpace -= 10;
			}
			
			if(currentSpace >= Margin.config.minValue) {
				Margin.set(currentSpace);
				if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
					Columns.initilizeVariables();
					if(Columns.config.noOfCol > 1) {
						Columns.removeEmptyCols();
						Columns.appendEmptyCols(Columns.config.noOfCol);
					}
					Columns.setApproximatePage();
					Columns.getColumnCount();
					Columns.getCurrentColPosition();
					Columns.getCurrentReadingDepth();
					Columns.updateProgressBar();
					Chapter.setNavButtons();
				}
				else{
					Flow.getCurrentReadingDepth();
				}
			}
		}
	}
	
	/*
	var changeMarginSpace = function(space){
		if(Chapter.config.azardi_margins == -1 || Chapter.config.azardi_presentation != 'fixed') {
			Margin.set(space);
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.initilizeVariables();
				Columns.getColumnCount();
				Columns.getCurrentColPosition();
				Columns.getCurrentReadingDepth();
				Columns.updateProgressBar();
				Chapter.setNavButtons();
			}
			else{
				Flow.getCurrentReadingDepth();
			}
		}
	}
	*/
	var changeBackgroundColor = function(color){
		
		var colorCode;
		
		switch(color) {
			case 'Normal':
				colorCode = '#FFFFFF';
				break;
			case 'Warm':
				colorCode = '#FAFAD2';
				break;
			case 'Low Contrast':
				colorCode = '#F0F0F0';
				break;
		}
		
		Color.set(colorCode);
	}
	
	/**
	 * Opens a notes dialog
	 */
	var openNotesDialog = function(){
		var self = this;
		
		//Check if valid to create notes
		var create = Notes.isTextSelected();
		if(create){
			var serial = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.serializeSelection();
			var selectionString = Notes.getSelectedString();
			
			$('#doc-viewer-azd').contents().find('#notes-serialized').text(serial);
			$('#doc-viewer-azd').contents().find('#notes-book-id').text(Library.config.currentBookId);
			$('#doc-viewer-azd').contents().find('#notes-current-chapter').text(Chapter.config.currentChapter);
			$('#doc-viewer-azd').contents().find('#notes-selection-string').text(selectionString);
			
			$('#doc-viewer-azd').contents().find('#white_overlay').show();
			$('#doc-viewer-azd').contents().find('#add_notes').show();
			
			$('#doc-viewer-azd').contents().find('#notesDescription').focus();
			
			$('#book_context_menu').hide();
			
			// self.unbindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
		}
	};
	
	/**
	 * Closes the notes dialog
	 */
	var closeAddNotesDialog = function(){
		var self = this;
		
		$('#doc-viewer-azd').contents().find('#white_overlay').hide();
		$('#doc-viewer-azd').contents().find('#add_notes').hide();
		
		$('#doc-viewer-azd').contents().find('#notesTitle').val('');
		$('#doc-viewer-azd').contents().find('#notesDescription').val('');
		
		$('#doc-viewer-azd').contents().find('#notes-serialized').html('');
		$('#doc-viewer-azd').contents().find('#notes-book-id').html('');
		$('#doc-viewer-azd').contents().find('#notes-current-chapter').html('');
		$('#doc-viewer-azd').contents().find('#notes-selection-string').html('');
		
		$('#book_context_menu').show();
		
		// self.bindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
	};
	
	/**
	 * Initilizes the sequence to save notes
	 */
	var saveNotes = function(){
		try {
			var self = this;
			var notesDescription = $('#doc-viewer-azd').contents().find('#notesDescription').val();
			if(notesDescription == '' || notesDescription == ' ' || notesDescription.replace(/\s/g, '') == ''){
				self.closeAddNotesDialog();
				return;	
			}
			
			var serial = $('#doc-viewer-azd').contents().find('#notes-serialized').text(),
				selectionString = $('#doc-viewer-azd').contents().find('#notes-selection-string').text(),
				bookId = $('#doc-viewer-azd').contents().find('#notes-book-id').text(),
				currentChapter = $('#doc-viewer-azd').contents().find('#notes-current-chapter').text();
	//			   notesTitle = $('#doc-viewer-azd').contents().find('#notesTitle').val(),
	//			   notesDescription = $('#doc-viewer-azd').contents().find('#notesDescription').val();
			
			var currentTime = new Date().getTime();
			Notes.save(bookId, currentChapter, notesDescription, selectionString, currentTime, serial, '', false);
			var noteId = Notes.getLatestId();
			Notes.addToArray(notesDescription, selectionString, serial, currentTime, noteId);
			Notes.highlightOnTheFly(serial, noteId);
	//		document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			self.closeAddNotesDialog();
		} catch(e) {
			//alert(e);
		}
	};
	
	var saveEditedNotes = function(){
		var self =this;
		
		var noteId = $('#doc-viewer-azd').contents().find('#id_anno').html();
		var notesDescription = $('#doc-viewer-azd').contents().find('.notes-description-ape').text();
		var currentTime = new Date().getTime();
		
		Notes.save('', '', notesDescription, '', currentTime, '', noteId, true);
		Notes.updateArray(notesDescription, currentTime, noteId);
		self.closeEditNotesDialog();
	}
	
	/**
	 * Opens the notes window in edit mode
	 * @params noteId Id of the note to be edited
	 */
	var openEditNotesDialog = function(noteId){
		var self = this;
		
		Notes.createEditDialog();
		Notes.setEditDialog(noteId);
		
		$('#doc-viewer-azd').contents().find('#white_overlay').show();
		$('#doc-viewer-azd').contents().find('#edit_notes').show();
		
		$('#doc-viewer-azd').contents().find('#edit_notes').css('height', document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight *0.4 + 'px');
		$('#doc-viewer-azd').contents().find('#edit_notes').css('top', document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight *0.3 + 'px');
		
		$(window).resize(function(){
			$('#doc-viewer-azd').contents().find('#edit_notes').css('height', document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight *0.4 + 'px');
			$('#doc-viewer-azd').contents().find('#edit_notes').css('top', document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight *0.3 + 'px');
		});
		
		$('#book_context_menu').hide();
		
		// self.unbindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
	}
	
	var closeEditNotesDialog = function(){
		var self = this;
		
		$('#doc-viewer-azd').contents().find('#white_overlay').hide();
		$('#doc-viewer-azd').contents().find('#edit_notes').remove();
		
		$('#book_context_menu').show();
		
		// self.bindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
	}
	
	var changeShelfOkButtonStatus = function(){
		var shelfName = $('#shelf_name').val();
		
		if (shelfName == '' || shelfName == ' ' || shelfName.replace(/ /g, '') == ''){
			$('#btn_shelf_ok').attr('disabled', true);
		}
		else{
			$('#btn_shelf_ok').attr('disabled', false);
		}
	}
	
	var changePubShelfOkButtonStatus = function(){
		var shelfName = $('#pshelf_name').val();
		
		if (shelfName == '' || shelfName == ' ' || shelfName.replace(/ /g, '') == ''){
			$('#btn_pshelf_ok').attr('disabled', true);
		}
		else{
			$('#btn_pshelf_ok').attr('disabled', false);
		}
	}
	
	/**
	 * Change the status of OK button in add book form
	 */
	var changeAddBookOkButtonStatus = function(){
		var bookName = $('#title').val();
		var bookLocation = $('#file_location').val();
		
		if (bookName == '' || bookName == ' ' || bookName.replace(/ /g, '') == '' || bookLocation == '' || bookLocation == ' '){
			$('#btn_ok').attr('disabled', true);
		}
		else{
			$('#btn_ok').attr('disabled', false);
		}
	};
	
	var changeMulAddBookOkButtonStatus = function(noOfBooks){
		if(noOfBooks > 0) {
			$('#btn_ok_mul_add').attr('disabled', false);
		} else {
			$('#btn_ok_mul_add').attr('disabled', true);
		}
	};
	
	var goBack = function(){
		//DC - FLO UPDATE - 201211 - Back functionality for FLO books
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.goBack();
			return;
		}
		
		if(Chapter.history.chapterNo == null){
			return;
		}
		
		Chapter.goBack();
		
		//	Reset the variables.
		Chapter.history.chapterNo = null;
		Chapter.history.readingDepth = null;
		
		Toolbar.disableBackButton();
	}
	
	var showAllNotes = function(){
		if($('#doc-viewer-azd').contents().find('#notes_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#toc_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#notes_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#content').removeClass('section-active');
		$('#doc-viewer-azd').contents().find('#notes').addClass('section-active');
	}
	
	var showTableOfContent = function(){
		if($('#doc-viewer-azd').contents().find('#toc_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#notes_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#toc_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#content').addClass('section-active');
		$('#doc-viewer-azd').contents().find('#notes').removeClass('section-active');
	}
	
	var showLandmarksColumn = function() {
		if($('#doc-viewer-azd').contents().find('#landmark_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#page-nav_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#landmark_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#landmark').addClass('section-active');
		$('#doc-viewer-azd').contents().find('#page-nav').removeClass('section-active');
	};
	
	var showPageNavColumn = function(){
		if($('#doc-viewer-azd').contents().find('#page-nav_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#landmark_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#page-nav_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#landmark').removeClass('section-active');
		$('#doc-viewer-azd').contents().find('#page-nav').addClass('section-active');
	};
	
	/**
	 * Function to go to note from full-page toc page
	 */
	var goToNote = function(id, chapter){
		var self = this;
		//DC - FLO UPDATE - 201211
		/*$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			$('#but_toc').removeAttr('checked');
			if(Epub.config.version == 3) {
				if(Epub.config.epub3TocData != ''){
					Epub.setInPageTocDataEpub3();
				}
				else{
					Epub.setInPageTocDataEpub2();
				}
			}
			else {
				Epub.setInPageTocDataEpub2();
			}
			//$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');*/
		$('#but_toc').removeAttr('checked');
		Chapter.goToNote(id, chapter);
		Toolbar.showEpub();
	};
	
	var toggleColumnRule = function(display) {
		if(BookPref.config.columns) {
			Columns.toggleColumnRule(display);
		}
	}
	
	var toggleColumn = function(column, noOfCol){
		var self = this;
		
		self.showLoading();
		if(column){
			BookPref.config.columns = true;
			Columns.config.showLastPage = false;
			Columns.config.goToDepth = Flow.config.currentReadingDepth;
			
			Columns.init(noOfCol);
			
			if(Columns.config.displayColumnRule == true) {
				self.toggleColumnRule(true);
			}
		}
		else{
			BookPref.config.columns = false;
			Flow.config.showLastPage = false;
			Flow.config.goToDepth = Columns.config.currentReadingDepth;
			
			Flow.init();
		}
		self.hideLoading();
	}
	
	/**
	 * Function to delete the note
	 * @params noteId Id of the note
	 * @params deleteButton DOM reference of the delete button pressed
	 */
	var deleteNote = function(noteId, deleteButton){
		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
 		
		if(prompts.confirm(null, AME.getTitle(), AzdMessages.deleteNote)){
			Notes.deleteNoteFromDatabase(noteId);
			Notes.deleteNoteFromFullPage(deleteButton);
		}
	};
	
	var clearTempDir = function () {
		Epub.getExtractLocation();
		var extractLocation = Epub.config.extractLocation;
		var fileObj = getFileObjectFromPath(extractLocation);
		if (fileObj.exists()) {
			deleteFolder(fileObj.path);
		}
	}
	
	var openSearchPage = function () {
		var self = this;
		
		self.hideStatusBar();
		$('#doc-viewer-azd').attr('src', '');
		$('#doc-viewer-azd').attr('src', 'html/azardi2_search_doc.html');
	}
	
	var showStatusBar = function () {
		$('#azd-statusbar').show();
	}
	
	var hideStatusBar = function () {
		$('#azd-statusbar').hide();
	}
	
	var showCatalog = function () {
		var self = this;
		
		// //	Reset the books current position.
		// //TODO: Check if need to call unloadAco function.
		// CdpNav.book.curr_pos = -1;
		// //	Load the catalog.
		// if(CdpDownload.config.isDownloading){
			// self.showStatusBar();
		// }
		Catalog.load(self.getCatalogId());
	}
	
	var openCdpBookmarkPage = function () {
		var params = {
						inn:{
							type:'bookmark'
						},
						out:null
					 };
		window.openDialog("chrome://azardi2/content/azd_page_container.xul", "Bookmarks", "chrome,width=500,height=655, dialog, modal, resizable=no, dependent, centerscreen", params).focus();
    }
    
    var openCdpGlossaryPage = function () {
    	if(Azd.book.show_glossary){
	    	var params = {
	    				 	inn:{
	    				 		type:'glossary'
	    				 	},
	    				 	out:null
	    				 };
			window.openDialog("chrome://azardi2/content/azd_page_container.xul", "Glossary", "chrome,width=500,height=655, dialog, modal, resizable=no, dependent, centerscreen", params).focus();
		}
    }
    
    /**
     * 	This function is needed to get the tooltip text of html page set in the xul iframe.
     */
    
    var fillInHTMLTooltip = function (tipElement) {
    	var retVal = false;
		if (tipElement.namespaceURI == "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul") {
			return retVal;
		}
		const XLinkNS = "http://www.w3.org/1999/xlink";
		var titleText = null;
		var XLinkTitleText = null;
		while (!titleText && !XLinkTitleText && tipElement) {
			if (tipElement.nodeType == Node.ELEMENT_NODE) {
				titleText = tipElement.getAttribute("title");
				XLinkTitleText = tipElement.getAttributeNS(XLinkNS, "title");
			}
			tipElement = tipElement.parentNode;
		}
		var texts = [titleText, XLinkTitleText];
		var tipNode = document.getElementById("aHTMLTooltip");
		for (var i = 0; i < texts.length; ++i) {
			var t = texts[i];
			if (t && t.search(/\S/) >= 0) {
				tipNode.setAttribute("label", t.replace(/\s+/g, " "));
				retVal = true;
			}
		}
		return retVal;
    };
    
    var showMessage = function (message, priority, autohide, buttons) {
    	var self = this;
    	
		var nb = document.getElementById('message_pane');
		
		switch(priority){
			case 'info':
				priority = nb.PRIORITY_INFO_MEDIUM;
				break;
			case 'warning':
				priority = nb.PRIORITY_WARNING_MEDIUM;
				break;
			case 'critical':
				priority = nb.PRIORITY_CRITICAL_MEDIUM;
				break;
		}
		
		if(buttons) {
			nb.appendNotification(message, 'azd_notification', null, priority, buttons)
		}else{
			nb.appendNotification(message, 'azd_notification', null, priority, null)
		}
		
		
		if (autohide) {
		    var tim = {
				notify: function(timer) {
					try {
						nb.removeCurrentNotification();
					} 
					catch (e) {
					}
					tim.timer = null;	// break circular reference
				}
		    };
	
		    var timer = Components.classes["@mozilla.org/timer;1"].
			createInstance(Components.interfaces.nsITimer);
		    tim.timer = timer;	// NOTE circular reference
		    timer.initWithCallback(tim, 10*1000, timer.TYPE_ONE_SHOT);
		}
		// var n = nb.getNotificationWithValue('popup-blocked');
		// if(n) {
			// n.label = message;
		// } else {
			// var buttons = [{
				// label : 'Button',
				// accessKey : 'B',
				// popup : 'blockedPopupOptions',
				// callback : null
			// }];
// 
			// priority = nb.PRIORITY_WARNING_BLOCK;
			// nb.appendNotification(message, 'popup-blocked', null, priority, buttons);
		// }
    } 
    
    var removeAllMessages = function () {
    	var self = this;
    	
		var nb = document.getElementById('message_pane');
    	nb.removeAllNotifications(true)
    }
    
    // var bindArrowKeys = function(element) {
		// $(element).keydown(function (e) {
			// var keyCode = e.keyCode || e.which,
				// arrow = {left: 37, up: 38, right: 39, down: 40 };
// 			
			// switch (keyCode) {
				// case arrow.left:
					// // parent.AME.previousPage();
					// e.preventDefault();
					// break;
				// case arrow.up:
					// // parent.AME.previousPage();
					// e.preventDefault();
					// break;
				// case arrow.right:
					// // parent.AME.nextPage();
					// e.preventDefault();
					// break;
				// case arrow.down:
					// // parent.AME.nextPage();
					// e.preventDefault();
					// break;
			// }
		// });
    // }
//     
    // var unbindArrowKeys = function(element) {
    	// $(element).unbind('keydown');
    // }
    
    var showBookInfoDialog = function(bookId) {
    	var self = this;
    	
    	if(typeof bookId == 'undefined') {
    		bookId = Library.getClickedBook();
    	}
		
		var bookInfo = self.getBookInfo(bookId);
		
		var params = {
			inn : {
				bookInfo : bookInfo
			},
			out : {
				out : null
			}
		};
		window.openDialog("chrome://azardi2/content/azd_book_information.xul", "Book Information", "chrome,width=500,height=400, dialog, modal, resizable=yes, dependent, centerscreen", params).focus();
    }
    
    var getBookInfo = function(bookId) {
    	var self = this;
    	
    	var bookInfo = new Array();
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === +bookId){
				bookInfo['id'] = Library.config.bookMetadata[i]['id'],
				bookInfo['type'] = Library.config.bookMetadata[i]['type'],
				bookInfo['uid'] = Library.config.bookMetadata[i]['uid'],
				bookInfo['title'] = Library.config.bookMetadata[i]['title'],
				bookInfo['author'] = Library.config.bookMetadata[i]['author'],
				bookInfo['publisher_identifier'] = Library.config.bookMetadata[i]['publisher_identifier'],
				bookInfo['publish_date'] = Library.config.bookMetadata[i]['publish_date'];
				bookInfo['created_date'] = new Date(parseInt(Library.config.bookMetadata[i]['created_date'])).toLocaleDateString();
				bookInfo['description'] = Library.config.bookMetadata[i]['description'];
				bookInfo['location'] = Library.config.bookMetadata[i]['location'];
				bookInfo['cover_image'] = Library.config.bookMetadata[i]['cover_image'];
				bookInfo['cover_type'] = Library.config.bookMetadata[i]['cover_type'];
				bookInfo['version'] = Library.config.bookMetadata[i]['version'];
				break;
			}
		}
		return bookInfo;
    }
    
    /**
     * Function to navigate from page-nav
     */
    var goToFromPageNav = function(referenceLocation) {
    	var self = this;
    	 
		 //DC - FLO UPDATE - 201211
		 
		/*$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			$('#but_landmark').removeAttr('checked');
			if(Epub.config.version == 3) {
				if(Epub.config.epub3TocData != ''){
					Epub.setInPageTocDataEpub3();
				}
				else{
					Epub.setInPageTocDataEpub2();
				}
			}
			else {
				Epub.setInPageTocDataEpub2();
			}
			
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
		*/
		Chapter.goToFromPageNav(referenceLocation);
		Toolbar.showEpub();
    };
    
    /**
     * Function to navigate from landmark
     */
    var goToFromLandmark = function (referenceLocation) {
    	var self = this;
    	
		 //DC - FLO UPDATE - 201211
		/*$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			$('#but_landmark').removeAttr('checked');
			if(Epub.config.version == 3) {
				if(Epub.config.epub3TocData != ''){
					Epub.setInPageTocDataEpub3();
				}
				else{
					Epub.setInPageTocDataEpub2();
				}
			}
			else {
				Epub.setInPageTocDataEpub2();
			}
			
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');*/
		$('#but_landmark').removeAttr('checked');
		Chapter.goToFromLandmark(referenceLocation);
		Toolbar.showEpub();
    };
    
    /**
     * This function initiates navigation from toc
     * @params referenceLocation Destination location
     * @params tocType Type of toc used to initiate the navigation side|full-page
     */
    var goToFromToc = function(referenceLocation, tocType){
    	var self = this;
    	
    	if(tocType == 'side') {
			Chapter.goToFromToc(referenceLocation, tocType);
    	} else {
			//DC - FLO UPDATE - 201211 - Dont need this as Chapter.loadPageEx sets it up separately
    		/*$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				
				if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						//Set fly-out toc data in epub3
						Epub.setInPageTocDataEpub3();
					}
					else{
						//Set fly-out toc data in epub2
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					//Set fly-out toc data in epub2
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToFromToc(referenceLocation, tocType);
				 //DC - FLO UPDATE - 201211 - Dont need this as Chapter.loadPageEx does its own unbind
				//$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');*/
			//Show epub toolbar
			$('#but_toc').removeAttr('checked');
			Chapter.goToFromToc(referenceLocation, tocType);
			Toolbar.showEpub();
    	}
    };
    
    /**
     * Function to initilize the sequence to go to cover page
     * @params tocType Type of the toc navigating from
     */
    var goToCoverPage = function(tocType) {
    	var self = this;
    	if(tocType == 'side') {
    		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
    			if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToCoverPage(tocType);
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
    	} else {
    		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
    			$('#but_toc').removeAttr('checked');
    			if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToCoverPage(tocType);
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			Toolbar.showEpub();
    	}
    };
    
    var previousPageUsingKey = function (e) {
    	var self = this;
		if (!config.istextbox) {
			self.previousPage();
		}
    }
    
    var nextPageUsingKey = function (e) {
    	var self = this;
		if (!config.istextbox) {
			self.nextPage();
		}
    }
    
    var previousChapterUsingKey = function() {
    	var self = this;
    	
    	self.previousChapter();
    };
    
    var nextChapterUsingKey = function() {
    	var self = this;
    	
    	self.nextChapter();
    };
    
    var openAboutWindow = function () {
    	var self = this;
    	
    	var params = {
			// inn : {
				// bookInfo : bookInfo
			// },
			// out : {
				// out : null
			// }
		};
		window.openDialog("chrome://azardi2/content/about.xul", "About", "chrome,width=500,height=330, dialog, resizable=no, dependent, centerscreen", params).focus();
    }
    
    /**
     * Open image in new viewport
     */
    var openImage = function (src) {
    	var self = this;
    	
    	var params = {
    		inn : {
				src : src,
				type : 'image'
			},
			out : {
				out : null
			}
		};
    	
    	window.openDialog("chrome://azardi2/content/azardi_external_container.xul", "External Container", "chrome, width=1000, height=700, dialog, resizable=yes, centerscreen", params).focus();
    };
	
	/**
     * Open image in new viewport
     */
    var openPDF = function (src) {
    	var self = this;
    	
    	var params = {
    		inn : {
				src : src,
				type : 'pdf'
			},
			out : {
				out : null
			}
		};
    	
    	window.openDialog("chrome://azardi2/content/azardi_external_container.xul", "External Container", "chrome, width=1000, height=700, dialog, resizable=yes, centerscreen", params).focus();
    };
    
    var checkToken = function () {
		var self = this;

		var fileObj = getFileObject(self.getDbFilePath());
		table_name = 'System';
		var conditions = new Array(['key', 'token', 'TEXT']);
		var values = ['value'];
		
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var value;
		while (statement.executeStep()) {
			value = statement.row.value
		}
		statement.finalize();
		
		if(typeof value == 'undefined')
			return false;
		else{
			config.readerToken = value;
			return true;
		}
	};
	
	var resetBookPreferences = function() {
		BookPref.init();
		BookPref.set();
		
		if(BookPref.config.columns){
			Columns.init(BookPref.config.noOfCols);
			Columns.toggleColumnRule(BookPref.config.displayColumnRule);
		} else {
			Flow.init();
		}
	};
	
	var playAudio = function(target) {
		try {
			if($('#but_audio_control').attr('disabled') != 'true') {
				if($('#but_audio_control').attr('checked') != 'true') {
					$('#but_audio_control').click();
				}
				
				var flo_reflow = true;
				if (Epub.is_fixed_layout) {
					flo_reflow = document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.in_reflow_mode;
				}
				
				if (Epub.is_fixed_layout && (!flo_reflow)) {
					document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.playFLOSMILAudio($(target), $('#but_audio_play').hasClass('playing'));
				} else {		
					//DC - FLO UPDATE - 201211 - the playaudio needs to be called using wrappedJSObject
					//Also, We are passing the current state of the button as AIE cant access it directly
					document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.playAudioEx($(target), $('#but_audio_play').hasClass('playing'), config.aud_playback_rate);
				}
			}
		} catch(e) {
			//alert(e);
		}
	};
	
	var stopAudio = function(target) {
		try {
			if($('#but_audio_control').attr('disabled') != 'true') {
				if($('#but_audio_control').attr('checked') != 'true') {
					$('#but_audio_control').click();
				}
				
				var flo_reflow = true;
				if (Epub.is_fixed_layout) {
					flo_reflow = document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.in_reflow_mode;
				}
					
				if (Epub.is_fixed_layout && (!flo_reflow)) {
					document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.stopFLOSMILAudio();
				} else {
					//DC - FLO UPDATE - 201211 - the playaudio needs to be called using wrappedJSObject
					document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.stopAudio($(target));
				}
			}
		} catch(e) {
		
		}
	};
	
	var setDefaultAudioRate = function(target) {
		var ws = 1;
		config.aud_playback_rate = ws;
		var aud_playback_rate = document.getElementById('aud_playback_rate');
		aud_playback_rate.value = 5;
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.changePlaybackRate(ws);
			return;
		}
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.changePlaybackRate(ws);
	};
	
	/**
	 * Function to handle the play event
	 */
	var playing_handle = function() {
		$('#but_audio_play').addClass('playing');
		// show pause button
		$('#but_audio_play').attr('image', 'icons/ico_audio-pause.png');
	};
	
	/**
	 * Function to handle the pause event
	 */
	var paused_handle = function() {
		$('#but_audio_play').removeClass('playing');
		// show playing button
		$('#but_audio_play').attr('image', 'icons/ico_audio-play.png');
	};
	
	/**
	 * Function to handle the End of SMIL content
	 * DC - FLO UPDATE - 201211
	 */
	var finished_handle = function() {
		$('#but_audio_play').removeClass('playing');
		// show playing button
		$('#but_audio_play').attr('image', 'icons/ico_audio-play.png');
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.smilAudioFinished();
		}
	};
	
	/**
	 * Initilizes the keyboard events.
	 */
	var setupKeyEvent = function() {
		$(document).keydown(function(event) {
			switch(event.which){
				case 16:
					//	Shift key
					config.isShiftPressed = true;
					break;
				case 17:
					//	Ctrl key
					config.isCtrlPressed = true;
					break;
				case 32:
					//	Spacebar
					if(config.isShiftPressed) {
						AME.stopAudio($('#but_audio_stop'));
						event.preventDefault();
					} else if(config.isCtrlPressed) {
						AME.playAudio($('#but_audio_play'));
						event.preventDefault();
					}
					break;
				default:
			}
		});
		
		$(document).keyup(function(event) {
			switch(event.which){
				case 16:
					//	Shift key
					config.isShiftPressed = false;
					break;
				case 17:
					//	Ctrl key
					config.isCtrlPressed = false;
					break;
				default:
			}
		});
	};
	
	var showFLOProperties = function() {
		if (($("#doc-flo-viewer-azd").is(":visible")) || ($("#doc-viewer-azd").is(":visible"))) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.showItemProperties();
			return;
		}
	}
	
	var setFLOBackground = function () {
		if ($('#doc-flo-viewer-azd').contents().find('body').hasClass('bg0')) {
			$('#doc-flo-viewer-azd').contents().find('body').addClass('bg1');
			$('#doc-flo-viewer-azd').contents().find('body').removeClass('bg0');
			BookPref.config.floBackground = 'bg1'; 
		} else if ($('#doc-flo-viewer-azd').contents().find('body').hasClass('bg1')) {
			$('#doc-flo-viewer-azd').contents().find('body').addClass('bg2');
			$('#doc-flo-viewer-azd').contents().find('body').removeClass('bg1');
			BookPref.config.floBackground = 'bg2';
		} else if ($('#doc-flo-viewer-azd').contents().find('body').hasClass('bg2')) {
			$('#doc-flo-viewer-azd').contents().find('body').addClass('bg3');
			$('#doc-flo-viewer-azd').contents().find('body').removeClass('bg2');
			BookPref.config.floBackground = 'bg3';
		} else if ($('#doc-flo-viewer-azd').contents().find('body').hasClass('bg3')) {
			$('#doc-flo-viewer-azd').contents().find('body').addClass('bg4');
			$('#doc-flo-viewer-azd').contents().find('body').removeClass('bg3');
			BookPref.config.floBackground = 'bg4';
		} else if ($('#doc-flo-viewer-azd').contents().find('body').hasClass('bg4')) {
			$('#doc-flo-viewer-azd').contents().find('body').addClass('bg0');
			$('#doc-flo-viewer-azd').contents().find('body').removeClass('bg4');
			BookPref.config.floBackground = 'bg0';
		} 
	}

	function SetFLOGutter() {
		if ($('#doc-flo-viewer-azd').contents().find('.zoombox-azd').hasClass('flo0')) {
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').addClass('flo1');
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').removeClass('flo0');
			BookPref.config.floGutter = 'flo1'; 
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.handleFLOGutter(1);
		} else if ($('#doc-flo-viewer-azd').contents().find('.zoombox-azd').hasClass('flo1')) {
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').addClass('flo2');
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').removeClass('flo1');
			BookPref.config.floGutter = 'flo2'; 
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.setFLOState(2);
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.handleFLOGutter(2);
		} else if ($('#doc-flo-viewer-azd').contents().find('.zoombox-azd').hasClass('flo2')) {
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').addClass('flo3');
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').removeClass('flo2');
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.handleFLOGutter(3);
			BookPref.config.floGutter = 'flo3'; 
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.setFLOState(0);
		} else if ($('#doc-flo-viewer-azd').contents().find('.zoombox-azd').hasClass('flo3')) {
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').addClass('flo4');
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').removeClass('flo3');
			BookPref.config.floGutter = 'flo4'; 
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.handleFLOGutter(4);
		} else if ($('#doc-flo-viewer-azd').contents().find('.zoombox-azd').hasClass('flo4')) {
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').addClass('flo0');
			$('#doc-flo-viewer-azd').contents().find('.zoombox-azd').removeClass('flo4');
			BookPref.config.floGutter = 'flo0'; 
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.handleFLOGutter(0);
		} 
	}
	
	function SetFLOPrintPageDisplay() {
		try {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.setFLOPrintPageDisplay(true);
		} catch(e) {
			
		}
		
		try {
			Chapter.showRealPageNums(true);
		} catch(e) {
		
		}
	}
	
	var toggleFullScreen = function() {
		var currentState = window.fullScreen;
		if (currentState == true) {
			currentState = false;
		} else {
			currentState = true;
		}
		
		 window.setTimeout( function(aWindow) {
			if(currentState == false){
				Toolbar.showHideMainToolbar(true);
				document.getElementById('but_fullscreen').removeAttribute('checked');
			} else {
				Toolbar.showHideMainToolbar(false);
				document.getElementById('but_fullscreen').setAttribute("checked", "true");
			}
			aWindow.fullScreen = currentState;
			AME.isfullscreen = currentState;
			//Redisplay the FLO mode to make sure to handle the 30px toolbar
			if (Epub.is_fixed_layout) document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.handleResize();
			
		 },1,window);
	}
	
	trackMousePosition = function(e){
		if (AME.isfullscreen == true) {
			//Show the toolbar when user moves the mouse to the top 30 px 
			if (e.clientY < 30) {
				Toolbar.showHideMainToolbar(true);
			} else {
				Toolbar.showHideMainToolbar(false);
			}
		}
	}
	
	downloadPubBook = function(bookId) {
		try {
			if(typeof bookId === 'undefined'){
				var bookId = Library.getClickedBook();
			}
			var result = CdpDownload.downloadBSItem(bookId);
			if (result.status) {
				//reload the metadata 
				Library.config.bookMetadata = BookMetadata.getBSShelfBooks(Library.config.shelfId);
				try {
					var evt = document.createEvent("MouseEvents");
					evt.initMouseEvent("click", true, true, window,
						0, 0, 0, 0, 0, false, false, false, false, 0, null);
					var cb = document.getElementById(Library.config.shelfId); 
					var canceled = !cb.dispatchEvent(evt);
				} catch(e) {
				
				}
			} else {
				var msg = getPropertiesString("azd_ame_downloadBookFail");
				alert(msg + " " + result["reason"]);
			}
			return false;
		} catch(e) {
			alert(e);
		}
	}
	
	var deletePubBook = function(bookId) {
		try {
			if(confirm('Are you sure you want to delete this book?')){
				if(typeof bookId === 'undefined'){
					var bookId = Library.getClickedBook();
				}
				var result = CdpDownload.deleteBSItem(bookId);
				if (result.status) {
					//reload the metadata 
					Library.config.bookMetadata = BookMetadata.getBSShelfBooks(Library.config.shelfId);
					try {
						var evt = document.createEvent("MouseEvents");
						evt.initMouseEvent("click", true, true, window,
							0, 0, 0, 0, 0, false, false, false, false, 0, null);
						var cb = document.getElementById(Library.config.shelfId); 
						var canceled = !cb.dispatchEvent(evt);
					} catch(e) {
					
					}
				}
			}
			return false;
		} catch(e) {
			alert(e);
		}
	}
	
	
    return {
		init: init,
		config: config,
		setBootupPreferences: setBootupPreferences,
		// getRenewCdpUrl: getRenewCdpUrl,
		// getRenewCatUrl: getRenewCatUrl,
		getDBVersion: getDBVersion,
		getDefaultDirPath: getDefaultDirPath,
		getDbFilePath: getDbFilePath,
		getCdpDbFilePath: getCdpDbFilePath,
		getSearchDbFilePath: getSearchDbFilePath,
		getTempDownloadPath: getTempDownloadPath,
		// getCfsLocation: getCfsLocation,
		getCatalogId: getCatalogId,
		getReaderToken: getReaderToken,
		getAppName: getAppName,
		// createDefaultDbTables: createDefaultDbTables,
		showLibrary: showLibrary,
		closeLibrary: closeLibrary,
		changeLibraryView: changeLibraryView,
		toggleShelf: toggleShelf,
		showShelf: showShelf,
		closeShelf: closeShelf,
		deleteShelf: deleteShelf,
		windowTitle: windowTitle,
		resetAddBookForm: resetAddBookForm,
		showAddBookForm: showAddBookForm,
		closeAddBookForm: closeAddBookForm,
		showUserPrefForm: showUserPrefForm,
		closeUserPrefForm: closeUserPrefForm,
		saveUserPrefs: saveUserPrefs,
		resetCreateShelfForm: resetCreateShelfForm,
		showCreateShelfForm: showCreateShelfForm,
		closeCreateShelfForm: closeCreateShelfForm,
		addBook: addBook,
		deleteBook: deleteBook,
		addBookToShelf: addBookToShelf,
		createShelf: createShelf,
		selectFile: selectFile,
		sortLibrary: sortLibrary,
		enterBookInformation: enterBookInformation,
		setCoverThumbnail: setCoverThumbnail,
		getCoverThumbDataUri: getCoverThumbDataUri,
		openBook: openBook,
		openBookFromCatalog: openBookFromCatalog,
		showLoading: showLoading,
		hideLoading: hideLoading,
		toggleFullPageToc: toggleFullPageToc,
		toggleLandmark: toggleLandmark,
		nextPage: nextPage,
		previousPage: previousPage,
		nextChapter: nextChapter,
		previousChapter: previousChapter,
		changeFontSize: changeFontSize,
//		changeParagraphSpace: changeParagraphSpace,
		changeLineSpace: changeLineSpace,
		changeMarginSpace: changeMarginSpace,
		changeBackgroundColor: changeBackgroundColor,
		toggleJustify: toggleJustify,
		toggleHyphen: toggleHyphen,
		resizeWindow: resizeWindow,
		openNotesDialog: openNotesDialog,
		closeAddNotesDialog: closeAddNotesDialog,
		saveNotes: saveNotes,
		openEditNotesDialog: openEditNotesDialog,
		closeEditNotesDialog: closeEditNotesDialog,
		saveEditedNotes: saveEditedNotes,
		changeShelfOkButtonStatus: changeShelfOkButtonStatus,
		changeAddBookOkButtonStatus: changeAddBookOkButtonStatus,
		changeMulAddBookOkButtonStatus: changeMulAddBookOkButtonStatus,
		goBack: goBack,
		showAllNotes: showAllNotes,
		showTableOfContent: showTableOfContent,
		goToNote: goToNote,
		toggleColumn: toggleColumn,
		deleteNote: deleteNote,
		clearTempDir: clearTempDir,
		openSearchPage: openSearchPage,
		showStatusBar: showStatusBar,
		hideStatusBar: hideStatusBar,
		getTitle: getTitle,
		showCatalog: showCatalog,
		openCdpBookmarkPage: openCdpBookmarkPage,
		openCdpGlossaryPage: openCdpGlossaryPage,
		registerApp: registerApp,
		fillInHTMLTooltip: fillInHTMLTooltip,
		copyDefaultDbTables: copyDefaultDbTables,
		checkDbVersion: checkDbVersion,
		showMessage: showMessage,
		removeAllMessages: removeAllMessages,
		// bindArrowKeys:bindArrowKeys,
		// unbindArrowKeys: unbindArrowKeys,
		showBookInfoDialog: showBookInfoDialog,
		getBookInfo: getBookInfo,
		scaleSize: scaleSize,
		goToFromToc: goToFromToc,
		goToFromLandmark: goToFromLandmark,
		checkUpdates: checkUpdates,
		previousPageUsingKey: previousPageUsingKey,
		nextPageUsingKey: nextPageUsingKey,
		openAboutWindow: openAboutWindow,
		openImage: openImage,
		openPDF: openPDF,
		goToCoverPage: goToCoverPage,
		goToFromPageNav: goToFromPageNav,
		showLandmarksColumn: showLandmarksColumn,
		showPageNavColumn: showPageNavColumn,
		addMultipleBooks: addMultipleBooks,
		previousChapterUsingKey: previousChapterUsingKey,
		nextChapterUsingKey: nextChapterUsingKey,
		checkToken: checkToken,
		downloadCat: downloadCat,
		toggleColumnRule: toggleColumnRule,
		playAudio: playAudio,
		stopAudio: stopAudio,
		setDefaultAudioRate: setDefaultAudioRate,
		playing_handle: playing_handle,
		paused_handle: paused_handle,
		finished_handle: finished_handle,
		resetBookPreferences: resetBookPreferences,
		toggleAudioControls: toggleAudioControls,
		setupKeyEvent: setupKeyEvent,
		openBookAtFirst: openBookAtFirst,
		showFLOProperties: showFLOProperties,
		setFLOBackground: setFLOBackground,
		SetFLOGutter: SetFLOGutter,
		SetFLOPrintPageDisplay: SetFLOPrintPageDisplay,
		toggleFullScreen:toggleFullScreen,
		trackMousePosition:trackMousePosition,
		isfullscreen: isfullscreen,
		showCreatePublisherShelfForm: showCreatePublisherShelfForm,
		refreshPublisherShelf: refreshPublisherShelf,
		closePubCreateShelfForm: closePubCreateShelfForm,
		changePubShelfOkButtonStatus: changePubShelfOkButtonStatus,
		downloadPubBook: downloadPubBook,
		createPubShelf: createPubShelf,
		deletePubBook: deletePubBook,
		setLocale: setLocale,
    };
}();


//DC - FLO UPDATE - 201211
/* Event listerner from the content iframe */
/* source - https://developer.mozilla.org/en-US/docs/Code_snippets/Interaction_between_privileged_and_non-privileged_pages */
var iframelistener = {
  listen_request: function(callback) { // analogue of chrome.extension.onRequest.addListener
    document.addEventListener("azd-query", function(event) {
      var node = event.target;
      if (!node || node.nodeType != Node.TEXT_NODE)
        return;
 
      var doc = node.ownerDocument;
	  var reqObj = JSON.parse(node.nodeValue);
	  
	  if (reqObj.hasOwnProperty('action')){
		var action = reqObj.action;
		
			
		if (action == "nextPage") {
			AME.nextPage();
		} else if (action == "previousPage") {
			AME.previousPage();
		} else if (action == "goToCoverPage") {
			var param = reqObj.param1;
			AME.goToCoverPage(param);
		} else if (action == "goToFromToc") {
			var param1 = reqObj.param1;
			var param2 = reqObj.param2;
			AME.goToFromToc(param1, param2);
		} else if (action == "saveEditedNotes") {
			AME.saveEditedNotes();
		} else if (action == "saveNotes") {
			AME.saveNotes();
		} else if (action == "note_clicked") {
			if (reqObj.hasOwnProperty('note_id')){
				AME.openEditNotesDialog(reqObj.note_id);
			}
		} else if (action == "playing_handle") {
			AME.playing_handle();
		} else if (action == "paused_handle") {
			AME.paused_handle();
		} else if (action == "finished_handle") {
			AME.finished_handle();
		} else if (action == "getmetainfo") {
			return Epub.config.metainfo;
		} else if (action == "settargetbox"){
			if (reqObj.hasOwnProperty('status')){
				AME.config.istextbox = reqObj.status;
			}
		} else if (action == "aie_playing_handle") {
			if (reqObj.hasOwnProperty('elid')){
				try {				
					var flo_reflow = true;
					if (Epub.is_fixed_layout) {
						flo_reflow = document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.in_reflow_mode;
					}
					
					//if in fixed layout and not in normal reflow mode
					if ((Epub.is_fixed_layout) && (!flo_reflow)) {
						document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.smilAudioTrackPara(reqObj.elid);
						return;
					}
				
					if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
						Columns.goToInternalLinkReference(reqObj.elid);	
					}
					else{
						Flow.goToInternalLinkReference(reqObj.elid);
					}
				} catch(e) {
					//alert(e)
				}
			}
		} 
	  }
	  
	callback(JSON.parse(node.nodeValue), doc, function(response) {
        node.nodeValue = JSON.stringify(response);
        var event = doc.createEvent("HTMLEvents");
        event.initEvent("azd-response", true, false);
        return node.dispatchEvent(event);
      });
	  
    }, false, true);
    return listener;
  },
  
  callback: function(request, sender, callback) {
	return callback(null);
  }
}
 
iframelistener.listen_request(iframelistener.callback);

/*
 * Process a series of command line arguments and modify default settings
 */
var openbook = "";
var loadedepubInfo = {};
function processCommandLineArgs() {
	try{	
		 var isMac = false;
		var osString = Components.classes["@mozilla.org/xre/app-info;1"]
               .getService(Components.interfaces.nsIXULRuntime).OS;
        if(osString.indexOf("Darwin") > -1){
			isMac = true;
			AME.config.is_mac = true;
		}
       var cmdLine = window.arguments[0];
        cmdLine = cmdLine.QueryInterface(Components.interfaces.nsICommandLine);
        if(cmdLine.length > 0){
		   var fileloc = cmdLine.getArgument(0);

		   if (isMac) {
			 /* in mac we get a file type as arg 1. values are: -url|-file
				in case of -url, we remove the localhost bit to get the actual file path
				also we get -url when app is launched for the first time and then
				later we get the -file when app is running
			 */
			 var filetype = fileloc;
			 var fileloc = cmdLine.getArgument(1);
			 fileloc = unescape(fileloc);
			 if(filetype == "-url") fileloc = fileloc.replace("file://localhost", "");
		   }
			
		   var file_ext = fileloc.split(".").pop();
			
			
		   var allbooks = BookMetadata.getAllBooks();
			//First check if this book is already added to the library
			//We do this by comparing the file location in db to load epub location
			for (var i=0; i<allbooks.length; i++) {
					var bookinfo = allbooks[i];
					if (bookinfo.location == fileloc){
							//show library
							AME.showLibrary(true);
							openbook = bookinfo.id;
							setTimeout(function() {
									//open book from library
									AME.openBook(openbook);
							}, 250);
							return;
					};
			}

			//So we do not have the epub in the library. So lets try to
			//Add it to the library and then open it
			var file = Components.classes["@mozilla.org/file/local;1"].
									createInstance(Components.interfaces.nsILocalFile);
			file.followLinks = true;
		    file.initWithPath(fileloc);
			
			AME.showLoading();
			
			if (file_ext === "e0") {
				loadedepubInfo = Epub0.getE0Info(file);
				AME.setCoverThumbnail(loadedepubInfo.coverImagePath);
				AME.getCoverThumbDataUri(loadedepubInfo.coverImagePath, addE0Book);
			} else {
				loadedepubInfo = Epub.getInfo(file);
				AME.getCoverThumbDataUri(loadedepubInfo.coverImagePath, addEPUBBook);
			}
        }
	} catch(e) {
	
  }
}

function addE0Book() {
	setTimeout(function() {
		if (loadedepubInfo) {
			AME.enterBookInformation(loadedepubInfo.metadata);
			AME.addBook("E0");
			var newbookid = BookMetadata.config.lastInsertedBookID;
			AME.showLibrary(true);
			loadedepubInfo = "";
			if (newbookid) {
				openbook = newbookid;
				setTimeout(function() {
					AME.openBook(openbook);
				}, 250);
			}
		}
	}, 500);
}

function addEPUBBook() {
	setTimeout(function() {
		if (loadedepubInfo) {
			AME.enterBookInformation(loadedepubInfo.metadata);
			AME.addBook();
			var newbookid = BookMetadata.config.lastInsertedBookID;
			AME.showLibrary(true);
			if (newbookid) {
				openbook = newbookid;
				setTimeout(function() {
					AME.openBook(openbook);
				}, 250);
			}
		}
	}, 500);
}
