/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

var BookMetadata = function(){
	
	/**
	 * Private properties and methods
	 */
	
	var config = {
		bookMetadata: [],
		lastInsertedBookID: ""
	}
	/**
	 * Public properties and methods
	 */
	
	var clear = function(){
		config.bookMetadata.splice(0, config.bookMetadata.length);
	}
	
	var add = function(bookMetadata){
		var fileObj = getFileObject(AME.getDbFilePath());
		
		//DC - FLO UPDATE - 201211 - Create new bookinfo table [if not exists]
		createBookInfoTable(fileObj, 'book_info');
		
	    var table_name = 'library_master';
	    var values = new Array(['created_date', bookMetadata.created_date, 'TEXT'],
		   					  ['title', bookMetadata.title, 'TEXT'],
		   					  ['author', bookMetadata.author, 'TEXT'],
		  					  ['publisher_identifier', bookMetadata.publisher_identifier, 'TEXT'],
		 					  ['publish_date', bookMetadata.publish_date, 'TEXT'],
		 					  ['description', bookMetadata.description, 'TEXT'],
		 					  ['location', bookMetadata.location, 'TEXT'],
		 					  ['uid', bookMetadata.uid, 'TEXT'],
		 					  ['type', bookMetadata.type, 'TEXT'],
		 					  ['cover_image', bookMetadata.cover_image, 'TEXT'],
		 					  ['cover_type', bookMetadata.cover_type, 'TEXT'],
		 					  ['version', bookMetadata.version, 'TEXT']);
	    var bookid = insertRowIntoDatabase(fileObj, values, table_name);
		 
		 //DC - FLO UPDATE - 201211 - Insert additional metadata into table
		if (bookid != -1) {
			var book_extended_md = {};
			 
			book_extended_md["rendition_layout"] = bookMetadata.rendition_layout;
			book_extended_md["rendition_orientation"] = bookMetadata.rendition_orientation;
			book_extended_md["rendition_spread"] = bookMetadata.rendition_spread;
			
			var book_extended_md_s = JSON.stringify(book_extended_md);
			
			var values = new Array(['book_id', bookid, 'TEXT'],
		   					  ['bookinfo', book_extended_md_s, 'TEXT']);	
			
			insertRowIntoDatabase(fileObj, values, 'book_info');
		}
		
		config.lastInsertedBookID = bookid;
		
		return bookid;
	}
	
	var getAll = function(){
		var self = this;
		
		self.clear();
		
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'type', 'uid', 'created_date', 'title', 'author', 'publisher_identifier', 'publish_date', 'description', 'location', 'cover_image', 'cover_type', 'version'];
		var tableName = 'library_master';
		var statement = getValuesFromDatabase(fileObj, values, tableName);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['type'] = statement.row.type;	
			data['uid'] = statement.row.uid;
			data['created_date'] = statement.row.created_date;
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['publisher_identifier'] = statement.row.publisher_identifier;
			data['publish_date'] = statement.row.publish_date;
			data['description'] = statement.row.description;
			data['location'] = statement.row.location;
			data['cover_image'] = statement.row.cover_image;
			data['cover_type'] = statement.row.cover_type;
			data['version'] = statement.row.version;
			config.bookMetadata.push(data);
		}
		statement.finalize();
		
		 //DC - FLO UPDATE - 201211 - get the additional bookmetadata and update the config
		for(var i = 0; i < config.bookMetadata.length; i++){
			var bookId = config.bookMetadata[i]['id'];
			var ext_metadata = getAdditionalBookMetadata(fileObj, bookId);
			if (ext_metadata) {
				$.extend(config.bookMetadata[i], ext_metadata);
			}
		}
		
		return config.bookMetadata;
	}
	
	var getAdditionalBookMetadata = function(dbFileObj, bookid) {
		//Predefine default values
		var bookinfo_obj = {};
		bookinfo_obj["rendition_layout"] = "reflowable";
		bookinfo_obj["rendition_orientation"] = "auto";
		bookinfo_obj["rendition_spread"] = "auto";

		try {
			var bookinfo_obj_db;
			
			var values = ['book_id', 'bookinfo'];
			var tableName = 'book_info';
			var conditions = new Array(['book_id', bookid, 'TEXT']);
			var statement = getValuesFromDatabase(dbFileObj, values, tableName, conditions);
			
			while(statement.executeStep()){ 
				var bookinfo = statement.row.bookinfo;
				bookinfo_obj_db = JSON.parse(bookinfo);
			}
			statement.finalize();
			
			//if we get proper values from db, return that
			if (bookinfo_obj_db.hasOwnProperty('rendition_layout')) {
				return bookinfo_obj_db;
			}
			
			//return a predefined object to handle books that dont have ext metadata
			return bookinfo_obj;
		} catch(e) {
			return bookinfo_obj;
		}
	}
	
	var getShelfBooks = function(shelfId){
		var self = this;
		
		self.clear();
		
		var fileObj = getFileObject(AME.getDbFilePath());
		
		if (!fileObj.exists()) {
			return;
		}
		var queryStatement = 'select id, type, uid, created_date, title, author, publisher_identifier, publish_date, description, location, cover_image, cover_type, version from library_master'+
							 ' where id in (select lib_id from shelf_document_join where shelf_id=?1);';
		var condition = new Array([shelfId, 'INTEGER']);
		
		var statement = getValuesFromDatabaseJoin(fileObj, queryStatement, condition);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['type'] = statement.row.type;	
			data['uid'] = statement.row.uid;
			data['created_date'] = statement.row.created_date;
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['publisher_identifier'] = statement.row.publisher_identifier;
			data['publish_date'] = statement.row.publish_date;
			data['description'] = statement.row.description;
			data['location'] = statement.row.location;
			data['cover_image'] = statement.row.cover_image;
			data['cover_type'] = statement.row.cover_type;
			data['version'] = statement.row.version;
			config.bookMetadata.push(data);
		}
		statement.finalize();
		
		//DC - FLO UPDATE - 201211 - get the additional bookmetadata and update the config
		for(var i = 0; i < config.bookMetadata.length; i++){
			var bookId = config.bookMetadata[i]['id'];
			var ext_metadata = getAdditionalBookMetadata(fileObj, bookId);
			if (ext_metadata) {
				$.extend(config.bookMetadata[i], ext_metadata);
			}
		}
		return config.bookMetadata;
	}
	
	var getBSBookAdditionalMetadata = function(h) {
		var destinationDb = getFileObject(AME.getCdpDbFilePath());
		var table_name = "store";
		var conditions = new Array(['uid', h, 'TEXT']);
		var values = ['data'];
		var statement = getValuesFromDatabase(destinationDb, values, table_name, conditions);
		var value = {};
		while (statement.executeStep()) {
			value_s = statement.row.data;
			value = JSON.parse(value_s);
		}
		statement.finalize();
		return value;
	}
	
	var getBSShelfBooks = function(shelfId){
		try{
		var self = this;
		
		self.clear();
		
		var fileObj = getFileObject(AME.getDbFilePath());
		
		if (!fileObj.exists()) {
			return;
		}
		var queryStatement = 'select id, type, bookstate, workuid, created_date, title, author, publisher_identifier, publish_date, description, location, cover_image, cover_type, version, docinfo from pub_docs'+
							 ' where shelf_id=?1;';
		var condition = new Array([shelfId, 'INTEGER']);
		
		var statement = getValuesFromDatabaseJoin(fileObj, queryStatement, condition);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['type'] = statement.row.type;	
			data['uid'] = statement.row.publisher_identifier;
			data['workuid'] = statement.row.workuid;
			data['created_date'] = statement.row.created_date;
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['publisher_identifier'] = statement.row.publisher_identifier;
			data['publish_date'] = statement.row.publish_date;
			data['description'] = statement.row.description;
			data['location'] = statement.row.location;
			data['version'] = statement.row.version;
			data['cover_type'] = statement.row.cover_type;
			data['cover_image'] = statement.row.cover_image;
			try {
				data['bookstate'] = statement.row.bookstate;
			} catch(e) {
				alert(e);
			}
			
			try {
				var docinfo = JSON.parse(statement.row.docinfo);
				for (keyval in docinfo) {
					data[keyval] = docinfo[keyval];
				}
				if (docinfo.hasOwnProperty("publisher")) {
					data['publisher_identifier'] = docinfo["publisher"];
				}
				
				if (docinfo.hasOwnProperty("identifier")) {
					data['uid'] = docinfo["identifier"];
				}
				
			} catch(e) {
				alert(e);
			}
			
			try {
				var s_docinfo = statement.row.docinfo;
				var docinfo = JSON.parse(s_docinfo);
				data['bookstate'] = statement.row.bookstate;
			} catch(e) {
				
			}
			data['workuid'] = statement.row.workuid;
			config.bookMetadata.push(data);
		}
		statement.finalize();
		
		shelfmd = Shelf.getPubAcctMd(shelfId);
		
		//DC - FLO UPDATE - 201211 - get the additional bookmetadata and update the config
		for(var i = 0; i < config.bookMetadata.length; i++){
			var bookId = config.bookMetadata[i]['workuid'];
			var h = shelfmd[bookId];
			
			if (h) {
				var md = getBSBookAdditionalMetadata(h);
			
				var bookinfo_obj = {};
				bookinfo_obj["rendition_layout"] = "reflowable"; 
				bookinfo_obj["rendition_orientation"] = "auto";
				bookinfo_obj["rendition_spread"] = "auto";
				bookinfo_obj["full_md"] = md;
				
				if (md.hasOwnProperty("rendition-layout") && md["rendition-layout"]) bookinfo_obj["rendition_layout"] = md["rendition-layout"]; 
				if (md.hasOwnProperty("rendition-orientation") && md["rendition-orientation"]) bookinfo_obj["rendition_orientation"] = md["rendition-orientation"]; 
				if (md.hasOwnProperty("rendition-spread") && md["rendition-spread"]) bookinfo_obj["rendition_spread"] = md["rendition-spread"]; 
				if (md.hasOwnProperty("version") && md["version"]) config.bookMetadata[i]['version'] = md["version"];
				
				if (bookinfo_obj) {
					$.extend(config.bookMetadata[i], bookinfo_obj);
				}
			}
		}
		
		return config.bookMetadata;
		} catch(e) {
			alert(e);
		}
	}
	
	var getAllBooks = function(){
		var self = this;
		var results = [];
		self.clear();
		
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'type', 'uid', 'created_date', 'title', 'author', 'publisher_identifier', 'publish_date', 'description', 'location', 'cover_image', 'cover_type', 'version'];
		var tableName = 'library_master';
		var statement = getValuesFromDatabase(fileObj, values, tableName);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['type'] = statement.row.type;	
			data['uid'] = statement.row.uid;
			data['created_date'] = statement.row.created_date;
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['publisher_identifier'] = statement.row.publisher_identifier;
			data['publish_date'] = statement.row.publish_date;
			data['description'] = statement.row.description;
			data['location'] = statement.row.location;
			data['cover_image'] = statement.row.cover_image;
			data['cover_type'] = statement.row.cover_type;
			data['version'] = statement.row.version;
			results.push(data);
		}
		statement.finalize();
		
		//DC - FLO UPDATE - 201211 - get the additional bookmetadata and update the config
		for(var i = 0; i < config.bookMetadata.length; i++){
			var bookId = config.bookMetadata[i]['id'];
			var ext_metadata = getAdditionalBookMetadata(fileObj, bookId);
			if (ext_metadata) {
				$.extend(config.bookMetadata[i], ext_metadata);
			}
		}
		return results;
	}
	
	var getBookId = function(bookTitle) {
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id'];
		var tableName = 'library_master';
		var conditions = new Array(['title', bookTitle, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
		var id;
		while(statement.executeStep()){
			id = statement.row.id;
		}
		statement.finalize();
		
		return id;
	}
	
	return{
		config: config,
		clear: clear,
		add: add,
		getAll: getAll,
		getShelfBooks: getShelfBooks,
		getAllBooks: getAllBooks,
		getBSShelfBooks: getBSShelfBooks,
		getBSBookAdditionalMetadata: getBSBookAdditionalMetadata,
		getBookId: getBookId
	}
}();