/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for manipulating Book Preferences.
 */
var BookPref = function(){
	var config = {
        columns: true,
        noOfCols: 2,
        displayColumnRule: true,
        font: 'DejaVu Serif',
        fontSize: 1,
		lineSpacing: 1.2,
        margin: 30,
        hyphenationStatus: true,
        justifyStatus: true,
        backgroundColor: '#FFFFFF',
		openWidth: '',
		openHeight: '',
		readingDepth: 0,
		currentChapter: 0,
		floGutter: "flo0",
		floBackground: "bg2",
		floShowPageNum: 3,
	};
	
	/**
	 * Initilize Book Preferences.
	 */
	var init = function(){
		var self = this;
		config.floGutter = "flo0";
		config.floBackground = "bg2";
		config.floShowPageNum = 3;
		//Override the book preferences with user preferences.
		self.overRide(UserPref.config);
	};
	
	/**
	 * Overrides the book preferences with new preferences.
	 * @param newPref An object containing user preferences.
	 */
	var overRide = function(newPref){
		var self = this;
		
		//If newPref is empty object then do not set it.
		if (!$.isEmptyObject(newPref)) {
            $.extend(self.config, newPref);
        }
	};
	
	/**
	 * Initilize the book preferences with the stored values
	 * @params bookPref Stored book preferences 
	 */
	var initWithSavedPreferences = function(bookPref){
		for (i = 0; i < bookPref.length; i++) {
	        switch (bookPref[i][0]) {
				case 'currentChapter':
					if(bookPref[i][1].indexOf('azd-outofspine-') == -1) {
						config.currentChapter = parseInt(bookPref[i][1]);
					} else {
						config.currentChapter = bookPref[i][1];
					}
	                break;
				case 'fontSize':
	                config.fontSize = bookPref[i][1];
	                break;
//	            case 'paragraphSpacing':
//	                config.paragraphSpacing = bookPref[i][1];
//	                break;
				case 'lineSpacing':
				    config.lineSpacing = bookPref[i][1];
				    break;
	            case 'margin':
	                config.margin = bookPref[i][1];
	                break;
	            case 'columns':
	            	if(bookPref[i][1] == 'true') {
                		config.columns = true;
                		config.noOfCols = 1;
                	} else if(bookPref[i][1] == 'false') {
                		config.columns = false;
                		config.noOfCols = 0;
                	} else {
                		config.noOfCols = parseInt(bookPref[i][1]);
                		if(config.noOfCols == 0) {
                			config.columns = false;
                		} else {
                			config.columns = true;
                		}
                	}
	                break;
	            case 'displayColumnRule':
	            	if(bookPref[i][1] == 'true') {
	            		config.displayColumnRule = true;
	            	} else {
	            		config.displayColumnRule = false;
	            	}
	            	break;
	            case 'hyphenationStatus':
	                if(bookPref[i][1] == 'true'){
						config.hyphenationStatus = true;
					}
					else{
						config.hyphenationStatus = false;
					}
	                break;
				case 'justifyStatus':
	                if(bookPref[i][1] == 'true'){
						config.justifyStatus = true;
					}
					else{
						config.justifyStatus = false;
					}
	                break;
	            case 'backgroundColor':
	                config.backgroundColor = bookPref[i][1];
	                break;
	            case 'openWidth':
	                config.openWidth = bookPref[i][1];
	                break;
	            case 'openHeight':
	                config.openHeight = bookPref[i][1];
	                break;
	            case 'readingDepth':
	                config.readingDepth = bookPref[i][1];
	                break;
				case 'floGutter':
	                config.floGutter = bookPref[i][1];
	                break;
				case 'floBackground':
	                config.floBackground = bookPref[i][1];
	                break;
				case 'floShowPageNum':
	                config.floShowPageNum = bookPref[i][1];
	                break;
	            default:
	        }
	    }
	};
	
	/**
	 * Sets the book preferences in page
	 */
	var set = function(){
		Fonts.setFont(config.font);
		Justify.set(config.justifyStatus);
		Justify.setButton(config.justifyStatus);
		Hyphen.set(config.hyphenationStatus);
		Hyphen.setButton(config.hyphenationStatus);
		Fonts.set(config.fontSize);
		Line.set(config.lineSpacing);
		Color.set(config.backgroundColor);
		
		if(Chapter.config.azardi_margins != -1) {
			//If available, set the margin from fixed layout properties
			Margin.setFromMeta(Chapter.config.azardi_margins);
		} else {
			Margin.set(config.margin);
			Margin.config.margin = config.margin;
			config.margin = config.margin;
		}
		
		Columns.setButton(config.noOfCols);
		Columns.toggleColumnRule(config.displayColumnRule);
		Columns.setColumnRuleButton(config.displayColumnRule);
	};
	
	/**
	 * Get the book preferences from the db for current book
	 * @returns {object} Saved book preferences
	 */
	var getFromDB = function(){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'doc_user_data';
		var values = ['book_pref'];
		var conditions = new Array(['book_id', Library.config.currentBookId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while (statement.executeStep()) {
			var bookPref = statement.row.book_pref;
		}
		return eval(bookPref);
	};
	
	var getAllFromDB = function() {
		var doc_user_data = [];
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'doc_user_data';
		var values = ['id', 'book_id', 'book_pref'];
		var statement = getValuesFromDatabase(fileObj, values, table_name);
		while (statement.executeStep()) {
			var temp = [];
			temp.push(statement.row.id);
			temp.push(statement.row.book_id);
			temp.push(statement.row.book_pref);
			doc_user_data.push(temp);
		}
		return doc_user_data;
	};
	
	/**
	 * Delete the saved book preferences
	 * @params bookId Id of the book to be deleted
	 */
	var deletePref = function(bookId){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'doc_user_data';
		var condition = new Array(['book_id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, condition);
	};
	
	/**
	 * Saves the current book preferences to database.
	 */
	var save = function(){
		//This is not the best way to save the preferences.
		//Preferences can be saved as a stringified json object, but that needs updation of database.
		
		var bookPref = "new Array(['currentChapter', '"+Chapter.config.currentChapter+"'],"+
							      "['fontSize', '"+Fonts.config.fontSize+"'],"+
								  "['lineSpacing', '"+Line.config.lineSpacing+"'],"+
							      "['margin', '"+Margin.config.margin+"'],"+
							      "['columns', '"+config.noOfCols+"'],"+
							      "['displayColumnRule', '"+Columns.config.displayColumnRule+"'],"+
							      "['hyphenationStatus', '"+Hyphen.config.hyphenationStatus+"'],"+
							      "['justifyStatus', '"+Justify.config.justifyStatus+"'],"+
							      "['backgroundColor', '"+Color.config.backgroundColor+"'],"+
							      "['openWidth', '"+Chapter.config.orig_width+"'],"+ 
							      "['openHeight', '"+Chapter.config.orig_height+"'],"+
							      "['readingDepth', '"+Columns.config.currentReadingDepth+"']," +
								  "['floGutter', '"+config.floGutter+"'],"+
								  "['floBackground', '"+config.floBackground+"'],"+
								  "['floShowPageNum', '"+config.floShowPageNum+"'])";
													  
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'doc_user_data';
		var values = ['book_id'];
		var conditions = new Array(['book_id', Library.config.currentBookId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while (statement.executeStep()){
			var book_id = statement.row.book_id;
		}
		//	if row is not already created then create new row and then insert.
		if(typeof book_id == 'undefined'){
			values = new Array(['book_id', Library.config.currentBookId, 'TEXT'],
							   ['book_pref', bookPref, 'TEXT']);
			insertRowIntoDatabase(fileObj, values, table_name);
		}
		//	if row already exists then update the row.
		else{
			values = new Array(['book_pref', bookPref, 'TEXT']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
	};
	
	return{
		config: config,
		init: init,
		overRide: overRide,
		set: set,
		save: save,
		deletePref: deletePref,
		getFromDB: getFromDB,
		getAllFromDB: getAllFromDB,
		initWithSavedPreferences: initWithSavedPreferences
	};
}();
