/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

var CatDAI = function(){
	/**
	 * Private properties and methods
	 */
	
	var config = {
		catObjects: []
	};
	
	var install = function() {
		for(var i=0; i<config.catObjects.length; i++) {
			var fileObj = window.opener.getFileObject(window.opener.AME.getCdpDbFilePath());
			installHTML(fileObj, config.catObjects[i]);
			installCSS(fileObj, config.catObjects[i]);
			installJS(fileObj, config.catObjects[i]);
			installInfo(fileObj, config.catObjects[i]);
		}
	};
	
	var installHTML = function(fileObj, jsObject) {
		var table_name = 'CAT_Master';
		
		var values = new Array(['cat_id', jsObject.catalog.id, 'TEXT'],
							   ['title', jsObject.catalog.name, 'TEXT'],
							   ['description', jsObject.catalog.description, 'TEXT'],
							   ['publisher', jsObject.catalog.publisher, 'TEXT'],
							   ['cat_content', jsObject.catalog.html, 'TEXT']);
		window.opener.insertRowIntoDatabase(fileObj, values, table_name);
	};
	
	var installCSS = function(fileObj, jsObject) {
		var table_name = 'CAT_Presentation_css';
		
		for(var i=0; i<jsObject.catalog.stylesheet.length; i++){
			var values = new Array(['cat_id', jsObject.catalog.id, 'TEXT'],
								   ['file_content', jsObject.catalog.stylesheet[i], 'TEXT']);
			window.opener.insertRowIntoDatabase(fileObj, values, table_name);
		}
	};
	
	var installJS = function(fileObj, jsObject) {
		var table_name = 'CAT_Presentation_js';
		
		for(var i=0; i<jsObject.catalog.javascript.length; i++){
			var values = new Array(['cat_id', jsObject.catalog.id, 'TEXT'],
								   ['file_content', jsObject.catalog.javascript[i], 'TEXT']);
			window.opener.insertRowIntoDatabase(fileObj, values, table_name);
		}
	};
	
	var installInfo = function(fileObj, jsObject) {
		var table_name = 'CAT_Info';
		
		$.each(jsObject.catalog.info, function(key, value) {
			if(typeof value === 'object') {
				value = JSON.stringify(value);
			}
			var values = new Array(['cat_id', jsObject.catalog.id, 'TEXT'],
								   ['key', key, 'TEXT'],
								   ['value', value, 'TEXT']);
			window.opener.insertRowIntoDatabase(fileObj, values, table_name);
		});
	};
	
	var exists = function(cat_id) {
		var exists = true;
		var fileObj = window.opener.getFileObject(window.opener.AME.getCdpDbFilePath());
		var table_name = 'CAT_Master';
		var values = ['cat_id'];
		var conditions = new Array(['cat_id', cat_id, 'TEXT']);
		var statement = window.opener.getValuesFromDatabase(fileObj, values, table_name, conditions);
		while (statement.executeStep()){
			var catId = statement.row.cat_id;
		}
		
		if (typeof catId === 'undefined') {
			exists = false;
		}
		
		return exists;
	};
	
	var downloadAllCat = function(cat_download_url) {
		cat_download_url = cat_download_url + '/catalog.json';
		var request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
				  .createInstance(Components.interfaces.nsIXMLHttpRequest);
		request.onload = function(aEvent) {
			var text = aEvent.target.responseText;
			try{
				var jsObject = JSON.parse(text);
			}
			catch(e){
				window.arguments[0].out.out = false;
				window.close();
			}
			allCatObj = jsObject.catalogs;
			$.each(allCatObj, function(key, value) {
				var catObj = {
					"catalog": value
				};
				config.catObjects.push(catObj);
			});
		};
		request.onerror = function(aEvent) {
			// window.alert("Error Status: " + aEvent.target.status);
			window.arguments[0].out.out = false;
			window.close();
		};
		request.open("GET", cat_download_url, false);
		request.send(null);
	};
	
	var downloadMasterCat = function(cat_download_url) {
		cat_download_url = cat_download_url + '/mastercatalog.json';
		var request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
				  .createInstance(Components.interfaces.nsIXMLHttpRequest);
		request.onload = function(aEvent) {
			var text = aEvent.target.responseText;
			try{
				var jsObject = JSON.parse(text);
			}
			catch(e){
				window.arguments[0].out.out = false;
				window.close();
			}
			
			config.catObjects.push(jsObject);
			
		};
		request.onerror = function(aEvent) {
			// window.alert("Error Status: " + aEvent.target.status);
			window.arguments[0].out.out = false;
			window.close();
		};
		request.open("GET", cat_download_url, false);
		request.send(null);
	};
	
	
	/**
	 * Public properties and methods
	 */
	
	var init = function(cat_download_url) {
		var self = this;
		
		document.title = window.opener.AME.getTitle();
		$('#modal-message').attr('value', 'Downloading catalog(s). Please Wait...');
		
		downloadAllCat(cat_download_url);
		downloadMasterCat(cat_download_url);
		
		install();
		window.arguments[0].out.out = true;
		window.close();
		
		// for(var i=0; i<cat_download_url.length; i++) {
// 			
			// if(!exists(cat_download_url[i][0])) {
				// var request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
						  // .createInstance(Components.interfaces.nsIXMLHttpRequest);
				// request.onload = function(aEvent) {
					// var text = aEvent.target.responseText;
					// try{
						// var jsObject = JSON.parse(text);
					// }
					// catch(e){
						// window.arguments[0].out.out = false;
						// window.close();
					// }
// 					
					// config.catObjects.push(jsObject);
// 					
					// if(i === cat_download_url.length - 1) {
						// install();
						// window.arguments[0].out.out = true;
						// window.close();
					// }
				// };
				// request.onerror = function(aEvent) {
					// // window.alert("Error Status: " + aEvent.target.status);
					// window.arguments[0].out.out = false;
					// window.close();
				// };
				// request.open("GET", cat_download_url[i][1], false);
				// // request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
				// request.send(null);
			// } else {
				// if(i === cat_download_url.length - 1) {
					// install();
					// window.arguments[0].out.out = true;
					// window.close();
				// }
			// }
		// }
	};
	
	return {
		init: init
	};
}();