/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

var Glossary = function(){

	/**
	 * 	Private properties and methods.
	 */
	
	var config = {
		glossary_acoid: '',
		book_location: '',
		presentationCss: new Array(),
		presentationJs: new Array(),
		html: new Array(),
		XHTML_NS: "http://www.w3.org/1999/xhtml"
	}
	
	var glossaryPref = {
		openWidth : '500',
		openHeight: '620'
	}

	
	var load = function () {
		var self = this;
		
		config.glossary_acoid = window.opener.Azd.book.glossary_acoid;
		config.book_location = window.opener.Azd.book.location;
		
		self.getJsData(config.book_location, config.glossary_acoid);
		self.setJs();
		self.getConfiguration();
		self.setConfiguration();
		self.getCssData(config.book_location, config.glossary_acoid);
		self.getHtmlData(config.book_location, config.glossary_acoid);
		self.setCss();
		self.setHtml();
		
		document.getElementById('page_container_iframe').contentWindow.initGlossary();
	}
	
	var getConfiguration = function () {
		if(typeof document.getElementById('page_container_iframe').contentWindow.acoPref != 'undefined'){
			$.extend(glossaryPref, document.getElementById('page_container_iframe').contentWindow.acoPref);
		}
	}
	
	var setConfiguration = function (){
		if (typeof glossaryPref.openWidth != 'undefined' && typeof glossaryPref.openHeight != 'undefined') {
			window.resizeTo(glossaryPref.openWidth, glossaryPref.openHeight);
		}
	}
	
	var getCssData = function (bookLocation, acoId) {
		var self = this;
		
		var fileObj = window.opener.getFileObjectFromPath(bookLocation);
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'aco_id', 'filename', 'file_content'];
		var tableName = 'ACO_Presentation_css';
		var conditions = new Array(['aco_id', acoId, 'TEXT']);
		var statement = window.opener.getValuesFromDatabase(fileObj, values, tableName, conditions);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['aco_id'] = statement.row.aco_id;	
			data['filename'] = statement.row.filename;
			data['file_content'] = statement.row.file_content;
			config.presentationCss.push(data);
		}
		statement.finalize();
	}
	
	var getJsData = function (bookLocation, acoId) {
		var self = this;
		
		var fileObj = window.opener.getFileObjectFromPath(bookLocation);
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'aco_id', 'filename', 'file_content'];
		var tableName = 'ACO_Presentation_js';
		var conditions = new Array(['aco_id', acoId, 'TEXT']);
		var statement = window.opener.getValuesFromDatabase(fileObj, values, tableName, conditions);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['aco_id'] = statement.row.aco_id;	
			data['filename'] = statement.row.filename;
			data['file_content'] = statement.row.file_content;
			config.presentationJs.push(data);
		}
		statement.finalize();
	}
	
	var getHtmlData = function (bookLocation, acoId) {
		var self = this;
		
		var fileObj = window.opener.getFileObjectFromPath(bookLocation);
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'aco_id', 'title', 'aco_content', 'aco_info'];
		var tableName = 'ACO_Master';
		var conditions = new Array(['aco_id', acoId, 'TEXT']);
		var statement = window.opener.getValuesFromDatabase(fileObj, values, tableName, conditions);
		
		while(statement.executeStep()){
			config.html['id'] = statement.row.id;
			config.html['aco_id'] = statement.row.aco_id;	
			config.html['title'] = statement.row.title;
			config.html['aco_content'] = statement.row.aco_content;
			config.html['aco_info'] = statement.row.aco_info;
		}
		statement.finalize();
	}
	
	var setCss = function () {
		for (var i=0; i < config.presentationCss.length; i++) {
			// if(book.ACOPresentation[i]['content_type'] == 'css'){
				var styleTag = '<style type="text/css" class="acoCss_'+ i +'">' + config.presentationCss[i]['file_content'] + '</style>';
				$('#page_container_iframe').contents().find('head').append(styleTag);
			// }
		};
	}
	
	var setJs = function () {
		for (var i=0; i < config.presentationJs.length; i++) {
			// if(book.ACOPresentation[i]['content_type'] == 'js'){
				var scriptTag = document.createElementNS(config.XHTML_NS, 'script');
				// scriptTag.setAttribute("id", "acoJs_"+i);
				var scriptContent = document.createTextNode(config.presentationJs[i]['file_content']);
				scriptTag.appendChild(scriptContent);
				document.getElementById('page_container_iframe').contentWindow.document.getElementsByTagName('head')[0].appendChild(scriptTag);
			// };
		}
	}
	
	var setHtml = function () {
		$('#page_container_iframe').contents().find('body').html(config.html['aco_content']);
	}
	
	return{
		config: config,
		load: load,
		getCssData: getCssData,
		getJsData: getJsData,
		getHtmlData: getHtmlData,
		setCss: setCss,
		setJs: setJs,
		setHtml: setHtml,
		getConfiguration: getConfiguration,
		setConfiguration: setConfiguration
	}
}();