/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

var Azd = function(){

	/**
	 * 	Private properties and methods.
	 */
	
	var config = {
		openWidth : '',
		openHeight: '',
		sizeLock: 'minimum',
		azardiControls: false,
		library: false,
		documentNavigation: false,
		userOptions: false,
	    aco_display: 1,
	    aco_content: ''
	}
	
	var book = {
		CDPHtml: new Array(),
		CDPPresentationCss: new Array(),
		CDPPresentationJs: new Array(),
		XHTML_NS: "http://www.w3.org/1999/xhtml",
		location: '',
		spine: new Array(),
		rpnMap: new Array(),
		opener_aco: '',
		cdpId: '',
		// glossary_audio: new Array(),
		glossary_acoid: '',
		show_glossary: false
	}
	
	
	/**
	 * 	Public properties and methods.
	 */
	
	var getInfo = function(azdFolder){
		var self = this;
		
		var metadata = new Array();
		var manifestFile = azdFolder.clone();
		manifestFile.append(azdFolder.leafName +  '-Manifest.html');
		var manifestFileURL = getURL(manifestFile);
		$('#pageContainerIframe').bind('DOMContentLoaded', function(e){
			metadata.push(['dc:location', azdFolder.path]);
			metadata.push(['dc:identifier', $('#pageContainerIframe').contents().find('.Identifier:first').text()]);
			metadata.push(['dc:title', $('#pageContainerIframe').contents().find('.Title').text()]);
			metadata.push(['dc:creator', $('#pageContainerIframe').contents().find('.Author').text()]);
			metadata.push(['dc:publisher', $('#pageContainerIframe').contents().find('.Publisher').text()]);
			metadata.push(['dc:date', $('#pageContainerIframe').contents().find('.Published-Date').text()]);
			metadata.push(['dc:description', $('#pageContainerIframe').contents().find('.Description').text()]);
			
			$('#pageContainerIframe').unbind('DOMContentLoaded');
			$('#pageContainerIframe').attr('src', '');
			
			AME.enterBookInformation(metadata);
			AME.changeAddBookOkButtonStatus();
	    });
		self.setDefaultCover();
		$('#pageContainerIframe').attr('src', manifestFileURL);
	};
	
	var setDefaultCover = function(){
		var defaultCoverPath = getFileObject(['CurProcD', 'chrome', 'content', 'icons', 'default_azd_cover.png']).path;
		var azdURI = generateDataURI(defaultCoverPath);
		
		$('#cover_thumbnail').attr('src', azdURI);
		$('#cover_image').val(azdURI);
	};
	
	/**
	 * Commenting following two functions as using the same function for OUP customization.
	 */
	
	// var load = function(bookLocation){
// 		
		// bookLocation = getFileObjectFromPath(bookLocation);
		// var cdpName = bookLocation.leafName + '.html';
		// bookLocation.append(cdpName);
		
		// var cdpURL = getURL(bookLocation);
// 		
		// $('#doc-viewer-azd').attr('src', cdpURL);
	// }
// 	
	// var unload = function(){
		// $('#doc-viewer-azd').attr('src', '');
	// }
	
	var load = function (bookLocation, cdpId, rpn) {
		var self = this;
		
		self.unload();
		book.location = bookLocation;
		book.cdpId = cdpId;
		// $('#doc-viewer-azd').attr('src', 'html/azardi2_default_cdp_doc.html');
		self.getCDPData(bookLocation, cdpId);
		self.setCDP();
		
		book.spine = document.getElementById('doc-viewer-azd').contentWindow.SPINE;
		
		book.rpnMap = document.getElementById('doc-viewer-azd').contentWindow.RPNMAP;
		
		book.opener_aco = document.getElementById('doc-viewer-azd').contentWindow.opener_aco;
		
		// book.glossary_audio = document.getElementById('doc-viewer-azd').contentWindow.glossary_audio;
		book.glossary_acoid = document.getElementById('doc-viewer-azd').contentWindow.glossary_acoid;
		book.show_glossary = document.getElementById('doc-viewer-azd').contentWindow.show_glossary;
		
		self.getApplicationConfiguration();
		self.getCDPConfiguration();
		
		document.getElementById('doc-viewer-azd').contentWindow.initCDP(rpn);
	};
	
	var unload = function () {
		book.CDPHtml.splice(0, book.CDPHtml.length);
		book.CDPPresentationCss.splice(0, book.CDPPresentationCss.length);
		book.CDPPresentationJs.splice(0, book.CDPPresentationJs.length);
		book.spine.splice(0, book.spine.length);
		book.rpnMap.splice(0, book.rpnMap.length);
		book.location = '';
		book.opener_aco = '';
		book.cdpId = '';
		book.glossary_acoid = '';
		book.show_glossary = false;
	};
	
	var getApplicationConfiguration = function () {
		$.extend(config, UserPref.config);
	};
	
	var getCDPConfiguration = function () {
		if(typeof document.getElementById('doc-viewer-azd').contentWindow.cdpPref != 'undefined'){
			$.extend(config, document.getElementById('doc-viewer-azd').contentWindow.cdpPref)
		}
	};
	
	var getCDPData = function(bookLocation, cdpId){
		var self = this;
		
		self.getCDPHtmlData(bookLocation, cdpId);
		self.getCDPCssData(bookLocation, cdpId);
		self.getCDPJsData(bookLocation, cdpId);
		// self.getCDPPresentationData(bookLocation, cdpId);
	};
	
	var getCDPHtmlData = function (bookLocation, cdpId) {
		var self = this;
		
		var fileObj = getFileObjectFromPath(bookLocation);
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'cdp_id', 'title', 'author', 'description', 'identifier', 'published_date', 'publisher', 'cdp_content'];
		var tableName = 'CDP_Master';
		var conditions = new Array(['cdp_id', cdpId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['cdp_id'] = statement.row.cdp_id;	
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['description'] = statement.row.description;
			data['identifier'] = statement.row.identifier;
			data['published_date'] = statement.row.published_date;
			data['publisher'] = statement.row.publisher;
			data['cdp_content'] = statement.row.cdp_content;
			book.CDPHtml.push(data);
		}
		statement.finalize();
	};
	
	var getCDPCssData = function (bookLocation, cdpId) {
		var self = this;
		
		var fileObj = getFileObjectFromPath(bookLocation);
		if (!fileObj.exists()) {
			return;
		}
		
		var values = ['id', 'cdp_id', 'filename', 'file_content'];
		var tableName = 'CDP_Presentation_css';
		var conditions = new Array(['cdp_id', cdpId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['cdp_id'] = statement.row.cdp_id;	
			data['filename'] = statement.row.filename;
			data['file_content'] = statement.row.file_content;
			book.CDPPresentationCss.push(data);
		}
		statement.finalize();
	};
	
	var getCDPJsData = function (bookLocation, cdpId) {
		var self = this;
		
		var fileObj = getFileObjectFromPath(bookLocation);
		if (!fileObj.exists()) {
			return;
		}
		
		var values = ['id', 'cdp_id', 'filename', 'file_content'];
		var tableName = 'CDP_Presentation_js';
		var conditions = new Array(['cdp_id', cdpId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['cdp_id'] = statement.row.cdp_id;	
			data['filename'] = statement.row.filename;
			data['file_content'] = statement.row.file_content;
			book.CDPPresentationJs.push(data);
		}
		statement.finalize();
	};
	
	// var getCDPPresentationData = function (bookLocation, cdpId) {
		// var self = this;
// 		
		// var fileObj = getFileObjectFromPath(bookLocation);
		// if (!fileObj.exists()) {
			// return;
		// }
		// var values = ['id', 'type', 'content_type', 'filename', 'file_path', 'file'];
		// var tableName = 'presentation';
		// var conditions = new Array(['type', 'CDP', 'TEXT']);
		// var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
// 		
		// while(statement.executeStep()){
			// var data = new Array();
			// data['id'] = statement.row.id;
			// data['type'] = statement.row.type;	
			// data['content_type'] = statement.row.content_type;
			// data['filename'] = statement.row.filename;
			// data['file_path'] = statement.row.file_path;
			// data['file'] = statement.row.file;
			// book.CDPPresentation.push(data);
		// }
		// statement.finalize();
	// }
	
	var setCDP = function () {
		var self = this;
		
		self.setCDPCss();
		self.setCDPJs();
		self.setCDPHtml();
	};
	
	var setCDPHtml = function () {
		
		$('#doc-viewer-azd').contents().find('body').html(book.CDPHtml[0]['cdp_content']);
		// dump($('#doc-viewer-azd').contents().find('html').html());
		// for (var i=0; i < book.CDPHtml.length; i++) {
			// if (book.CDPHtml[i]['use'] == 'main') {
				// $('#doc-viewer-azd').contents().find('body').html(book.CDPHtml[i]['file']);
				// break;
			// };
		// };
	};
	
	var setCDPCss = function () {
		for (var i=0; i < book.CDPPresentationCss.length; i++) {
			// if(book.CDPPresentation[i]['content_type'] == 'css'){
				var styleTag = '<style type="text/css">' + book.CDPPresentationCss[i]['file_content'] + '</style>';
				$('#doc-viewer-azd').contents().find('head').append(styleTag);
			// }
		};
	};
	
	var setCDPJs = function () {
		for (var i=0; i < book.CDPPresentationJs.length; i++) {
			// if(book.CDPPresentation[i]['content_type'] == 'js'){
				var scriptTag = document.createElementNS(book.XHTML_NS, 'script');
				var scriptContent = document.createTextNode(book.CDPPresentationJs[i]['file_content']);
				scriptTag.appendChild(scriptContent);
				document.getElementById('doc-viewer-azd').contentWindow.document.getElementsByTagName('head')[0].appendChild(scriptTag)
			// };
		};
	};
	
	var setConfiguration = function(){
		if (typeof config.openWidth != 'undefined' && typeof config.openHeight != 'undefined') {
			window.resizeTo(config.openWidth, config.openHeight);
		}
	};
	
	var getCDPID = function() {
		var self = this;
		
		return self.book.cdpId;
	};
	
	return{
		config: config,
		book: book,
		getInfo: getInfo,
		setDefaultCover: setDefaultCover,
		load: load,
		unload: unload,
		getApplicationConfiguration: getApplicationConfiguration,
		getCDPConfiguration: getCDPConfiguration,
		setConfiguration: setConfiguration,
		getCDPData: getCDPData,
		getCDPHtmlData: getCDPHtmlData,
		// getCDPPresentationData: getCDPPresentationData,
		getCDPCssData: getCDPCssData,
		getCDPJsData: getCDPJsData,
		setCDP: setCDP,
		setCDPHtml: setCDPHtml,
		setCDPCss: setCDPCss,
		setCDPJs: setCDPJs,
		getCDPID: getCDPID
	};
}();