/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for column manipulation
 */
var Columns = function(){
	var config = {
		showLastPage: false,
		goToReference: '',
		goToNoteId: '',
		goToDepth: '',
		currentWidth : '',
		currentHeight : '',
		documentWidth : '',
		documentHeight : '',
		currentColumnCount : 0,
		currentColumnHeight : 0,
		currentColumnPosition : 0,
		currentReadingDepth : 0,
		columnGap : 0,
		xOffset : '',
		yOffset : '',
		noOfCol: 1,
		displayColumnRule: true
	};
	
	/**
	 * Initlizes the column mode
	 * @params noOfCol Number of column on the viewport
	 */
	var init = function(noOfCol){
		try {
			var self = this;
			
			config.noOfCol = noOfCol;
			BookPref.config.noOfCols = noOfCol;
			self.unbindResize();
			self.resetProgressBar();
			if($('#doc-viewer-azd').contents().find('#container-ape').length == 0){
				$('#doc-viewer-azd').contents().find('#custom-style-igp').wrap('<div id="container-ape" />');
			}
			
			//setTimeout(function () {
				$('#doc-viewer-azd').contents().find('body').css('overflow', 'hidden');
				// $('#doc-viewer-azd').css('overflow', 'hidden');
				
				self.initilizeVariables();
				//	resize images here.
				
				self.create(noOfCol);
				
				var extraCols = 0;
				self.removeEmptyCols();
				if(noOfCol > 0) {
					extraCols = self.appendEmptyCols(noOfCol);
				}
				
				//Need to set the margin again after creating the columns.
				if(Chapter.config.azardi_margins != -1) {
					Margin.setFromMeta(Chapter.config.azardi_margins);
				} else {
					Margin.set(BookPref.config.margin);
				}
				
				if(config.showLastPage){
					self.last(extraCols);
				}
				else{
					self.first();
				}
				self.getColumnCount();
				
				self.getCurrentColPosition();
				self.getCurrentReadingDepth();
				self.updateBookProgressBar();
				self.updateProgressBar();
				
				self.bindResize();
				
				if(config.goToReference != ''){
					self.goToInternalLinkReference(config.goToReference);
					config.goToReference = '';
				}
				
				if(config.goToDepth != ''){
					config.currentReadingDepth = config.goToDepth;
					config.goToDepth = '';
					self.setApproximatePage();
					self.getColumnCount();
					self.getCurrentColPosition();
					self.getCurrentReadingDepth();
					self.updateProgressBar();
				}
			//}, 100);
		} catch(e) {
			
		}
	};
	
	/**
	 * Bind the resize event
	 */
	var bindResize = function(){
		var self = this;
		
		$(window).bind('resize', function(){
			
			self.initilizeVariables();
			self.create(config.noOfCol);
			if(config.noOfCol > 0) {
				self.removeEmptyCols();
				self.appendEmptyCols(config.noOfCol);
			}
			// self.resizeImages();
			self.setApproximatePage();
			self.getColumnCount();
			self.getCurrentColPosition();
			self.getCurrentReadingDepth();
			self.updateProgressBar();
			Chapter.setNavButtons();
		});
	};
	
	/**
	 * Unbind the resize event
	 */
	var unbindResize = function(){
		$(window).unbind('resize');
	};
	
	/**
	 * Initilize the latest window variables
	 */
	var initilizeVariables = function(){
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.initPageVariables();
		$.extend(config, document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config);
	};
	
	/**
	 * Set the approximate column in viewport.
	 */
	var setApproximatePage = function(){
		var self = this;
		
		self.initilizeVariables();
		self.getColumnCount();
		config.currentColumnPosition = Math.round((config.currentReadingDepth * config.currentColumnCount) / 100);
	    var scrollToXVal = (config.currentColumnPosition - 1) * (config.currentWidth + config.columnGap);
	    document.getElementById('doc-viewer-azd').contentWindow.scrollTo(scrollToXVal, 0);
	}
	
	/**
	 * Create the columns
	 * @params noOfCol Number of columns to be created per viewport
	 */
	var create = function(noOfCol){
		$('#doc-viewer-azd').contents().find('#container-ape').css('-moz-column-gap', config.columnGap+'px');
		$('#doc-viewer-azd').contents().find('#container-ape').attr("class", "columns");
		$('#doc-viewer-azd').contents().find('#container-ape').css("-moz-column-width", (config.currentWidth/noOfCol)-config.columnGap);
		$('#doc-viewer-azd').contents().find('#container-ape').css("height", config.currentHeight);
	};
	
	var reduceColHeight = function(reduceby) {
		config.currentColumnHeight = config.currentHeight;
		var newheight = config.currentHeight - reduceby;
		$('#doc-viewer-azd').contents().find('#container-ape').css("height", newheight);
	}
	
	var resetColHeight = function(reduceby) {
		$('#doc-viewer-azd').contents().find('#container-ape').css("height", config.currentColumnHeight);
	}
	
	/**
	 * Calculates the number of column created
	 * NOTE: This function actually calculates number of viewport.
	 * To get the correct column count, multiply this value with number of columns per viewport
	 */
	var getColumnCount = function(){
		var self = this;
		
		self.initilizeVariables();
		var totalScrollVal = config.documentWidth - config.currentWidth;
		config.currentColumnCount = (totalScrollVal / (config.currentWidth + config.columnGap)) + 1;
	};
	
	/**
	 * Calculates the position of current column
	 */
	var getCurrentColPosition = function(){
		var self = this;
		
		self.initilizeVariables();
	    config.currentColumnPosition = Math.round((config.xOffset / (config.currentWidth + config.columnGap)) + 1);
	};
	
	/**
	 * Calculate the reading depth(in %) for the current chapter
	 */
	var getCurrentReadingDepth = function(){
		var self = this;
		
		self.initilizeVariables();
		config.currentReadingDepth = (config.currentColumnPosition / config.currentColumnCount) * 100;
	};
	
	/**
	 * Updates the book progress bar.
	 */
	var updateBookProgressBar = function(){
		var percentComplete = ((Chapter.config.currentChapter + 1) / Epub.config.spineList.length) * 100;
		$('#doc-viewer-azd').contents().find('.bookProgressbar').css({
			'width': percentComplete + '%'});
		return;
		
		var percentComplete = ((Chapter.config.currentChapter + 1) / Epub.config.spineList.length) * 100;
		$('#doc-viewer-azd').contents().find('.bookProgressbar').animate({
			'width': percentComplete+'%'
		}, { duration: 400, queue: false }, 'swing');
	};
	
	/**
	 * Update the chapter progress bar.
	 */
	var updateProgressBar = function(){
		$('#doc-viewer-azd').contents().find('.progressbar').css({
			'width': config.currentReadingDepth+'%'});
		return;
		
		$('#doc-viewer-azd').contents().find('.progressbar').animate({
			'width': config.currentReadingDepth+'%'
		}, { duration: 400, queue: false }, 'swing');
	};
	
	/**
	 * Reset the chapter progressbar and book progressbar
	 */
	var resetProgressBar = function(){
		$('#doc-viewer-azd').contents().find('.progressbar').show();
		$('#doc-viewer-azd').contents().find('.bookProgressbar').show();
		$('#doc-viewer-azd').contents().find('.progressbar').css('width', '0');
	};
	
	/**
	 * Go to the first column of the page.
	 */
	var first = function(){
		document.getElementById('doc-viewer-azd').contentWindow.scrollTo(0, 0);
	}
	
	/**
	 * Scroll to next column
	 */
	var next = function(){
		var self = this;
		
		var scrollByXVal = config.currentWidth + config.columnGap;
		document.getElementById('doc-viewer-azd').contentWindow.scrollBy(scrollByXVal, 0);
		
		self.getColumnCount();
		self.getCurrentColPosition();
		self.getCurrentReadingDepth();
		self.updateProgressBar();
	};
	
	/**
	 * Scroll to previous column
	 */
	var previous = function(){
		var self = this;
		
		var scrollByXVal = 0 - (config.currentWidth + config.columnGap);
        document.getElementById('doc-viewer-azd').contentWindow.scrollBy(scrollByXVal, 0);
		
		self.getColumnCount();
		self.getCurrentColPosition();
		self.getCurrentReadingDepth();
		self.updateProgressBar();
	}
	
	/**
	 * Go to the last column
	 * @params extraCols Number of empty columns appended at the end
	 */
	var last = function(extraCols){
		if (config.noOfCol > 1) {
			var scrollToXVal = config.documentWidth + ((config.currentWidth/config.noOfCol)*extraCols) - config.currentWidth;
			document.getElementById('doc-viewer-azd').contentWindow.scrollTo(scrollToXVal, 0);
		} else {
			var scrollToXVal = config.documentWidth + ((config.currentWidth/config.noOfCol)) - config.currentWidth;
			document.getElementById('doc-viewer-azd').contentWindow.scrollTo(scrollToXVal, 0);
		}
	};
	
	/**
	 * Goes to the requested link references.
	 * @params reference Reference location
	 */
	var goToInternalLinkReference = function(reference){
		var self  = this;
		self.initilizeVariables();
		try{
			var offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
			var epubtype = $('#doc-viewer-azd').contents().find('#'+reference).attr("epub:type");
			
			if (epubtype && epubtype == "footnote") {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.left =  offset.left + item_offset.left;
			}
			
			if (epubtype && epubtype == "rearnote") {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.left =  offset.left + item_offset.left;
			}
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-note-rw")) {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.left =  offset.left + item_offset.left;
			}
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-note")) {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.left =  offset.left + item_offset.left;
			}
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-footnote-rw")) {
				//it seems that the offset for actual num-footnote-rw return it with reference to container div
				//So we will get the offset position of container div and that of element and then use it to get actual nav position
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.left =  offset.left + item_offset.left;
			}
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-footnote")) {
				//it seems that the offset for actual num-footnote-rw return it with reference to container div
				//So we will get the offset position of container div and that of element and then use it to get actual nav position
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.left =  offset.left + item_offset.left;
			}
			
		} catch(e) {
			//	jQuery's .position() throws an error if the element has style 'display:none',
			//	use of 'visibility:hidden' instead is encouraged.
			return false;
		}
		
		if(offset == null){
			// Did not find the element.
			return false;
		}
		var elementXPos = offset.left;
		var columnNumber = parseInt((elementXPos / (config.currentWidth + config.columnGap)));
		var scrollToXVal = (columnNumber * config.currentWidth) + (config.columnGap * columnNumber);
		document.getElementById('doc-viewer-azd').contentWindow.scrollTo(scrollToXVal, 0);
		
		self.getColumnCount();
		self.getCurrentColPosition();
		self.getCurrentReadingDepth();
		self.updateProgressBar();
	}
	
	/**
	 * Sets the status of column buttons
	 * @params noOfCols Number of columns this chapter shows in a viewport
	 */
	var setButton = function(noOfCols){
		$('#scroll_view').removeAttr('checked');
		$('#column_1').removeAttr('checked');
		$('#column_2').removeAttr('checked');
		$('#column_3').removeAttr('checked');
		$('#column_4').removeAttr('checked');
		
		switch(parseInt(noOfCols)) {
			case 0:
				$('#scroll_view').attr('checked', true);
				break;
			case 1:
				$('#column_1').attr('checked', true);
				break;
			case 2:
				$('#column_2').attr('checked', true);
				break;
			case 3:
				$('#column_3').attr('checked', true);
				break;
			case 4:
				$('#column_4').attr('checked', true);
				break;
			default:
		}
		
		if(parseInt(noOfCols) == 0) {
			BookPref.config.columns = false;
		} else {
			BookPref.config.columns = true;
		}
		BookPref.config.noOfCols = parseInt(noOfCols);
		config.noOfCol = parseInt(noOfCols);
	};
	
	/**
	 * Hides/Shows the column rule
	 * @params display Whether to hide or show the column rule
	 */
	var toggleColumnRule = function(display) {
		$('#doc-viewer-azd').contents().find('body')
				.removeClass('rule-on')
				.removeClass('column-count-2')
				.removeClass('column-count-3')
				.removeClass('column-count-4');
		if(display) {
			switch(config.noOfCol) {
				case 2:
					$('#doc-viewer-azd').contents().find('body').addClass('rule-on').addClass('column-count-2');
					break;
				case 3:
					$('#doc-viewer-azd').contents().find('body').addClass('rule-on').addClass('column-count-3');
					break;
				case 4:
					$('#doc-viewer-azd').contents().find('body').addClass('rule-on').addClass('column-count-4');
					break;
				default:
			}
			config.displayColumnRule = true;
			BookPref.config.displayColumnRule = true;
		} else {
			config.displayColumnRule = false;
			BookPref.config.displayColumnRule = false;
		}
	};
	
	var setColumnRuleButton = function(columnRule) {
		if(columnRule)
			$('#but_column_rule').attr('checked', columnRule);
		else
			$('#but_column_rule').removeAttr('checked');
			
		config.columnRule = columnRule;
		BookPref.config.columnRule = columnRule;
	};
	
	/**
	 * Create and add empty columns at the end of the page
	 * @params noOfCol Number of columns per viewport
	 * @returns {number} Total number of empty columns added.
	 */
	var appendEmptyCols = function(noOfCol) {
		var self = this;
		
		self.getColumnCount();
		var noOfColToAppend = parseInt(noOfCol) - (Math.round(config.currentColumnCount*parseInt(noOfCol)) % parseInt(noOfCol));
		
		if(noOfColToAppend != noOfCol) {
			var ht = config.currentHeight - parseInt(Margin.config.margin)
			for(var i=0; i<noOfColToAppend; i++) {
				var lastPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('div'))
								.addClass('last-page-'+i)
								.css("height", ht)
				
				$('#doc-viewer-azd').contents().find('#custom-style-igp').append(lastPage);
			}
			return parseInt(noOfColToAppend);
		} else {
			return 0;
		}
	};
	
	/**
	 * Remove all the empty columns previously applied
	 */
	var removeEmptyCols = function() {
		var self = this;
		
		$('#doc-viewer-azd').contents().find('div[class^="last-page"]').remove();
	};
	
	return{
		config: config,
		init: init,
		bindResize: bindResize,
		unbindResize: unbindResize,
		// resizeImages: resizeImages,
		initilizeVariables: initilizeVariables,
		create: create,
		getColumnCount: getColumnCount,
		getCurrentColPosition: getCurrentColPosition,
		getCurrentReadingDepth: getCurrentReadingDepth,
		updateBookProgressBar: updateBookProgressBar,
		updateProgressBar: updateProgressBar,
		resetProgressBar: resetProgressBar,
		first: first,
		next: next,
		previous: previous,
		last: last,
		goToInternalLinkReference: goToInternalLinkReference,
		setApproximatePage: setApproximatePage,
		setButton: setButton,
		toggleColumnRule: toggleColumnRule,
		setColumnRuleButton: setColumnRuleButton,
		appendEmptyCols: appendEmptyCols,
		reduceColHeight: reduceColHeight,
		resetColHeight: resetColHeight,
		removeEmptyCols: removeEmptyCols
	}
}();