/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 Function to get values from database.
 @param fileObj			-----	object of .sqlite file	
 @param values			-----	array of values to be retrieved
 @param table_name		-----	table name
 @param condition		-----	2d array of column name and value and column type
  
 @returns statement object
 */

function getValuesFromDatabase(fileObj, values, table_name, condition){
	var valueString = values.join(', ');
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	if(condition != undefined){
		var conditionStatement = 'where '+condition[0][0]+'=?1';
		for(var i=1; i<condition.length; i++){
			conditionStatement += ' and '+condition[i][0]+'=?'+(i+1);
		}
		try {
			var statement = mDBConn.createStatement('SELECT ' + valueString + ' FROM ' + table_name + ' ' + conditionStatement);
		}
		catch(e){
			dump(e+' '+e.message);
			return false;
		}
		for(var i=0; i<condition.length; i++){
			switch (condition[i][2]){
				case 'TEXT':
					statement.bindStringParameter(i, condition[i][1]);
					break;
				case 'INTEGER':
					statement.bindInt32Parameter(i, condition[i][1]);
					break;
				case 'DOUBLE':
					statement.bindDoubleParameter(i, condition[i][1]);
					break;
				case 'BLOB':
					statement.bindBlobParameter(i, condition[i][1]);
					break;
				default:
					
			}
		}
	}
	else{
		try {
			var statement = mDBConn.createStatement('SELECT ' + valueString + ' FROM ' + table_name);
		}
		catch(e){
			dump(e+' '+e.message);
			return false;
		}
	}
	return statement;
}

/**
 Function to create table in database.
 @param {Object} fileObj
 @param {Object} query
 */

function createTable(fileObj, query){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.close();
}

/**
 Function to delete row from database.
 @param {Object} fileObj
 @param {Object} table_name
 @param {Object} conditions
 */

function deleteRowFromDatabase(fileObj, table_name, conditions){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var conditionString = '';
	
	for(i=0; i<conditions.length; i++){
		if(i>0){
			conditionString += 'and ';
		}
		conditionString += conditions[i][0];
		conditionString += '=';
		conditionString += '?'+(i+1);
	}
	
	var query =  'DELETE FROM '+table_name+' WHERE '+conditionString;
	
	var statement = mDBConn.createStatement(query);
	
	for(var i=0; i<conditions.length; i++){
		switch (conditions[i][2]){
			case 'TEXT':
				statement.bindStringParameter(i, conditions[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(i, conditions[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(i, conditions[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(i, conditions[i][1], conditions[i][1].length);
				break;
			default:
		}
	}
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
	}
	statement.finalize();
	mDBConn.close();
}

/**
 * Function to delete all rows from table.
 */

function deleteAllRowsFromTable(fileObj, table_name) {
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var query =  'DELETE FROM '+table_name;
	
	var statement = mDBConn.createStatement(query);
	
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
	}
	statement.finalize();
	mDBConn.close();
}

/**
 * 	Function to vacuum the database
 * @param fileObj		-----	object of .sqlite file
 */
function vacuumDatabase(fileObj) {
	return;
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);
	var mDBConn = storageService.openDatabase(fileObj);
	var query = 'VACUUM';
	var statement = mDBConn.createStatement(query);
	statement.execute();
	statement.finalize();
	mDBConn.close();
}

function createPubTables(fileObj) {
	try{
	var query = 'CREATE TABLE IF NOT EXISTS shelf_info ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"shelf_id" INTEGER, "shelf_uid" TEXT, "shelf_url" TEXT, "email" TEXT, "shelf_metadata" TEXT);';
	var query1 = 'CREATE TABLE IF NOT EXISTS pub_docs ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"shelf_id" INTEGER, "type" TEXT, "workuid" TEXT, "bookstate" TEXT, "created_date" TEXT, "title" TEXT, "author" TEXT, "publisher_identifier" TEXT, "publish_date" TEXT, "description" TEXT, "location" TEXT, "cover_image" TEXT, "cover_type" TEXT, "version" TEXT, "docinfo" TEXT);';
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.executeSimpleSQL(query1);
	mDBConn.close();
	}catch(e){
		alert(e);
	}
}

function createPubDataTables(fileObj) {
	try{
	var query = 'CREATE TABLE IF NOT EXISTS store ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"uid" TEXT, "type" TEXT, "data" TEXT);';
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.close();
	}catch(e){
		alert(e);
	}
}

/**
 Function to insert row into database
 @param fileObj			-----	object of .sqlite file
 @param values			-----	2d array of column name, value and column type
 @param table_name		-----	table name
 */

function insertRowIntoDatabase(fileObj, values, table_name){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	var lastrowid = -1;
	var columnString = '(';
	var valueString = '(';
	
	for(i=0; i<values.length; i++){
		if(i>0){
			columnString += ', ';
			valueString += ', ';
		}
		columnString += values[i][0];
		valueString += '?'+(i+1);
	}
	columnString += ')';
	valueString += ')';
	
	var query =  'INSERT INTO '+table_name+' '+columnString+' VALUES '+valueString;
	var statement = mDBConn.createStatement(query);
	
	for(var i=0; i<values.length; i++){
		switch (values[i][2]){
			case 'TEXT':
				statement.bindStringParameter(i, values[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(i, values[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(i, values[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(i, values[i][1], values[i][1].length);
				break;
			default:
				
		}
	}
	try{
		statement.execute();
		 //DC - FLO UPDATE - 201211 - get the inserted row id
		try {
			lastrowid = mDBConn.lastInsertRowID;
		} catch(e){
		}
	}
	catch(e){
		lastrowid = -1;
	}
	statement.finalize();
	mDBConn.close();
	 //DC - FLO UPDATE - 201211 - return the last inserted row id
	return lastrowid;
}

/**
 Function to update row of table in database.
 @param {Object} fileObj
 @param {Object} values
 @param {Object} table_name
 @param {Object} conditions
 */

function updateRowIntoDatabase(fileObj, values, table_name, conditions){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var valueToInsert = '';
	var conditionString = '';
	var totalParameters = 0;
	
	for(i=0; i<values.length; i++){
		totalParameters++;
		if(i>0){
			valueToInsert += ', ';
		}
		valueToInsert += values[i][0];
		valueToInsert += '=';
		valueToInsert += '?'+totalParameters;
	}
	
	for(i=0; i<conditions.length; i++){
		totalParameters++;
		if(i>0){
			conditionString += 'and ';
		}
		conditionString += conditions[i][0];
		conditionString += '=';
		conditionString += '?'+totalParameters;
	}
	
	var query =  'UPDATE '+table_name+' SET '+valueToInsert+' WHERE '+conditionString;
	
	var statement = mDBConn.createStatement(query);
	
	totalParameters = -1;
	
	//	binding values.
	for(var i=0; i<values.length; i++){
		totalParameters++;
		switch (values[i][2]){
			case 'TEXT':
				statement.bindStringParameter(totalParameters, values[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(totalParameters, values[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(totalParameters, values[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(totalParameters, values[i][1], values[i][1].length);
				break;
			default:
		}
	}
	
	//	binding conditions
	for(var i=0; i<conditions.length; i++){
		totalParameters++;
		switch (conditions[i][2]){
			case 'TEXT':
				statement.bindStringParameter(totalParameters, conditions[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(totalParameters, conditions[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(totalParameters, conditions[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(totalParameters, conditions[i][1], conditions[i][1].length);
				break;
			default:
		}
	}
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
	}
	statement.finalize();
	mDBConn.close();
}

function getValuesFromDatabaseJoin(fileObj, queryString, condition){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	if(condition != undefined){
		try {
			var statement = mDBConn.createStatement(queryString);
		}
		catch(e){
			dump(e+' '+e.message+'\n');
			return false;
		}
		for(var i=0; i<condition.length; i++){
			switch (condition[i][1]){
				case 'TEXT':
					statement.bindStringParameter(i, condition[i][0]);
					break;
				case 'INTEGER':
					statement.bindInt32Parameter(i, condition[i][0]);
					break;
				case 'DOUBLE':
					statement.bindDoubleParameter(i, condition[i][0]);
					break;
				case 'BLOB':
					statement.bindBlobParameter(i, condition[i][0]);
					break;
				default:
					
			}
		}
	}
	else{
		try {
			var statement = mDBConn.createStatement(queryString);
		}
		catch(e){
			dump(e+' '+e.message);
			return false;
		}
	}
	
	return statement;
}

function createBookInfoTable(fileObj, tablename) {
	var query = 'CREATE TABLE IF NOT EXISTS ' + tablename + ' ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"book_id" TEXT, "bookinfo" TEXT)';
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.close();
}

