/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Deepak Chandran
 */

/**
 * @class Contains all the functions necessary for loading epub.
 */
var Epub0 = function(){
	/*----------------------------*/
	/*----E0 Functions STARTS ----*/
	/*----------------------------*/
	/*----------------------------*/
	/*----------------------------*/
	
	var completehtmlleaflist = [];
	var completehtmlfileslist = [];
	
	var loadE0 = function(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, openAtFirst, additionalparams) {
		try {
		var self = this;
		completehtmlleaflist.splice(0, completehtmlleaflist.length);
		completehtmlfileslist.splice(0, completehtmlfileslist.length);
		
		var epubFile = getFileObjectFromPath(bookLocation);
		try{
			unzipFile(epubFile.path, false);
			completehtmlleaflist = extractFileList(epubFile.path, []);
			completehtmlfileslist = [];
			for (var i=0; i<completehtmlleaflist.length; i++) {
				var leafname = completehtmlleaflist[i];
				var entry_leaf_fn = leafname.split("/").pop();
				completehtmlfileslist.push(entry_leaf_fn); 
			}
		}
		catch(e){
			//	Throw error if archive file is corrupted.
			throw new Error('An error occurred while trying to load the book.');
		}
		
		Epub.getExtractLocation();
		Epub.getExtractedFolderName(epubFile.leafName);
		
		var containerXmlPath = getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder ,'index.html']);
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(containerXmlPath);
		
		if( !file.exists() || !file.isFile() ) {   // if it doesn't exist, create
		   containerXmlPath = getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder, Epub.config.extractedFolder ,'index.html']);
		}
		
		try{
			var containerXmlContent = readFileUTF(containerXmlPath);
			containerXmlContent = containerXmlContent.replace(/property=/g, "name=");
		}
		catch(e){
			//	Throw error if META-INF folder or container.xml file is missing from the archive. 
			throw new Error('An error occured while trying to load the book1.');
		}
		
		if(containerXmlContent == '') {
			//	Throw error if conatainer.xml file is empty.
			throw new Error('An error occured while trying to load the book.');
		}
				
		Epub.config.spineList.splice(0, Epub.config.spineList.length);

		var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
							   .createInstance(Components.interfaces.nsIDOMParser);
		var domTree = parser.parseFromString(containerXmlContent, "text/html");
				

		self.getE0Spine(domTree);
		
		self.loadE0MetaProperties(domTree);
		
		self.getE0Manifest(domTree);
		
		Epub.config.lang = "en";
		Epub.setLang(Epub.config.lang);
		
		self.createE0TOC(domTree);
		
		var coverImageInfo = self.getE0CoverImage(domTree, containerXmlPath);
		var cover_type = 'image';
		var coverImagePath = coverImageInfo.coverImagePath;
		Epub.config.cover_image = generateDataURI(coverImagePath);
		
		for(var i = 0; i < Epub.config.bookData.length; i++){
			if(Epub.config.bookData[i]['mediaType'] == 'text/html' || Epub.config.bookData[i]['mediaType'] == 'application/xhtml+xml'){
				try{
					var htmlPath = Epub.getEntityAbsolutePath(containerXmlPath, Epub.config.bookData[i]['href']);
					var htmlObj = getFileObjectFromPath(htmlPath);
					var filename = htmlObj.leafName.split('.');
					var name = filename[0];
					var extension = filename[1];
					Epub.filename_lookup[htmlObj.leafName] = i;
					var htmlXmlContent = readFileUTF(htmlPath);
					Epub.config.bookData[i]["htmlXmlContent"] = htmlXmlContent;
					
					var appcss = Epub.getAppCustomCSS();
					headers = Epub.getAppHeaderComponents(htmlXmlContent);
					htmlXmlContent = htmlXmlContent.replace("<head>", "<head>" + appcss);
					htmlXmlContent = htmlXmlContent.replace("</body>", headers + "</body>\n");
					//Force the smil aie_textsound to override the book aie textsound js
					htmlXmlContent = htmlXmlContent.replace("</body>", Epub.appSMILJS + "</body>");
					writeFileUTF(htmlPath, htmlXmlContent);
				} catch(e) {
				}
			}
		}
		self.getE0HtmlData(containerXmlPath);
		
		}catch(e){
			
		}
	}
	
	var createE0TOC = function(indexTree) {
		Epub.config.epub2FullPageTocData = '<ol><li><a href="javascript: void(0);" class="azd-cover-link" onclick="javascript:parent.AME.goToCoverPage(\'full-page\');">COVER</a></li></ol>';
			
		Epub.config.epub2InPageTocData = '<div id="toc-azd" class="toc-azd"><ol><li><a href="javascript: void(0);" class="azd-cover-link" onclick="javascript:parent.AME.goToCoverPage(\'side\');">COVER</a></li></ol></div>';
			
		var template = "<li><a class=\"$h1$\"  onclick=\"xulconnector.send_request({action: \'goToFromToc\', param1: \'$h$\', param2: \'side\'});\" href=\"javascript: void(0);\">$c$</a></li>";
		var ft_template = "<li><a class=\"$h1$\"  onclick=\"javascript:parent.AME.goToFromToc('$h$','full-page');\" href=\"javascript: void(0);\">$c$</a></li>";
		
		var navNodes = indexTree.getElementsByTagName("nav");
		if (navNodes.length < 0) {
			throw new Error('An error occured while trying to load the book.');
		}
		var navNodesCh = navNodes[0].children;
		
		var olnode = "";
		for (var i=0; i<navNodesCh.length; i++) {
			if (navNodesCh[i].nodeName === "OL") {
				olnode = navNodesCh[i];
				break;
			}
		}
		
		if (!olnode) {
			return;
		}
		var tocitems = [];
		for (i=0; i<olnode.children.length; i++) {
			var node = olnode.children[i];
			var nodename = node.nodeName;
			if (nodename == "LI") {
				var hidden = false;
				var anode = node.children[0];
				var href = anode.attributes.getNamedItem('href').nodeValue;
				try {
					var role = node.attributes.getNamedItem('role').nodeValue;
				} catch(e){
					var role = "";
				}
				
				try {
					//check for hidden attribute in the li tag
					hidden = node.hidden;
				} catch(e){
	
				}
				
				var href_id = "";
				var nodetext = "";
				if (role) {
					if (role == "toc") {
						href_id = href.replace(".html", "");
						nodetext = anode.childNodes[0].nodeValue;
					}
				} else {
					href_id = href.replace(".html", "");
					nodetext = anode.childNodes[0].nodeValue;
				}
				if (href && nodetext && !hidden) {
					tocitems.push([href, nodetext]);
				}
			}
		}
		
		var sidetoc = "";
		for (i=0; i<tocitems.length; i++) {
			var t_s = template;
			var tocitem = tocitems[i];
			t_s = t_s.replace("$h$", tocitem[0]);
			t_s = t_s.replace("$h1$", tocitem[0]);
			t_s = t_s.replace("$c$", tocitem[1]);
			sidetoc = sidetoc + t_s;
		}
		if (sidetoc) {
			sidetoc = '<div id="toc-azd" class="toc-azd"><ol>' + sidetoc + '</ol></div>';
			Epub.config.epub2InPageTocData = sidetoc;
		}
		
		var fulltoc = "";
		for (i=0; i<tocitems.length; i++) {
			var t_s = ft_template;
			var tocitem = tocitems[i];
			t_s = t_s.replace("$h$", tocitem[0]);
			t_s = t_s.replace("$h1$", tocitem[0]);
			t_s = t_s.replace("$c$", tocitem[1]);
			fulltoc = fulltoc + t_s;
		}
		
		if(fulltoc) {
			fulltoc = '<ol>' + fulltoc + '</ol>';
			Epub.config.epub2FullPageTocData = fulltoc;
		}
	}
	
	var getE0HtmlData = function(opfFilePath){
		var self = this;
		
		for(var i = 0; i < Epub.config.bookData.length; i++){
			Epub.config.htmlCounter = i;
			if(Epub.config.bookData[Epub.config.htmlCounter]['mediaType'] == 'text/html' || Epub.config.bookData[Epub.config.htmlCounter]['mediaType'] == 'application/xhtml+xml'){
				try {
					var htmlPath = Epub.getEntityAbsolutePath(opfFilePath, Epub.config.bookData[Epub.config.htmlCounter]['href'])
					var htmlObj = getFileObjectFromPath(htmlPath);
					var filename = htmlObj.leafName.split('.');
					var name = filename[0];
					var extension = filename[1];
					
					if(Epub.config.bookData[Epub.config.htmlCounter]['mediaType'] === 'application/xhtml+xml' && extension !== 'xhtml') {
						//	Mozilla doesn't recognize XML declaration if the file-extension is incorrect, hence renaming the file.
						htmlObj.moveTo(null, name+'.xhtml');
					}
					
					var htmlUrl = getURL(htmlObj);
					//	If the required file does not exists then set available as false.
					if(!htmlObj.exists()) {
						Epub.config.bookData[Epub.config.htmlCounter]['available'] = 'false';
					}
					var parser = new DOMParser();
					var stringContainingXMLSource = Epub.config.bookData[Epub.config.htmlCounter]["htmlXmlContent"];			
					var doc = parser.parseFromString(stringContainingXMLSource, "application/xml");
					Epub.config.bookData[Epub.config.htmlCounter]['available'] = 'true';
					Epub.config.bookData[Epub.config.htmlCounter]['htmlUrl'] = htmlUrl;
				} catch(e) {
					Epub.config.bookData[Epub.config.htmlCounter]['available'] = 'false';
				}
			}
		}
		
		Epub.config.htmlCounter = 0;
		Epub.config.customCss = 1;
	}
	
	var loadE0MetaProperties = function(domTree) {
		try {
			var metNodes = domTree.getElementsByTagName("meta");
			var metadata1 = {};	
			for (var i=0; i<metNodes.length; i++) {
				try {
					try {
						var name = metNodes[i].attributes.getNamedItem('name').nodeValue
						var content = metNodes[i].attributes.getNamedItem('content').nodeValue
					}catch(e){
					
					}
					if (name) {
						metadata1[name] = content;
					}
				} catch(e) {
		
				}
			}
			Epub.config.metainfo = metadata1;
		}catch(e) {
			
		}
		
	}
	
	/**
	 * Get manifest information from .opf file for epub 2
	 * @params opfFileContent Content of the .opf file
	 */
	var getE0Manifest = function(indexTree){
		var navNodes = indexTree.getElementsByTagName("nav");
		var manifestlist = [];
		if (navNodes.length < 0) {
			throw new Error('An error occured while trying to load the book.');
		}
		var navNodesCh = navNodes[0].children;
		
		var olnode = "";
		for (var i=0; i<navNodesCh.length; i++) {
			if (navNodesCh[i].nodeName === "OL") {
				olnode = navNodesCh[i];
				break;
			}
		}
		
		if (!olnode) {
			throw new Error('An error occured while trying to load the book.');
		}
		
		for (i=0; i<olnode.children.length; i++) {
			var node = olnode.children[i];
			var nodename = node.nodeName;
			if (nodename == "LI") {
				var data = {};
				var anode = node.children[0];
				var href = anode.attributes.getNamedItem('href').nodeValue;
				var href_id = href.replace(".html", "");
				data['id'] = href_id;
				data['href'] = href;
				data['mediaType'] = "text/html";
				data['data'] = '';
				data['cssRef'] = '';
				data['jsRef'] = '';
				data['available'] = 'false';
				data['azardi-presentation'] = '';
				data['azardi-width'] = '';
				data['azardi-height'] = '';
				data['azardi-margins'] = '';
				data['azardi-toc'] = '';
				data['properties'] = '';
				data['mediaOverlay'] = '';
				Epub.config.bookData.push(data);
				manifestlist.push(href);
			}
		}
		
		for (var i=0; i<completehtmlfileslist.length; i++) {
			var href = completehtmlfileslist[i];
			found = $.inArray(href, manifestlist);
			if (found === -1) {
				//We found a file that is present in zip and missing from index.html
				var data = {};
				var leafname = completehtmlfileslist[i];
				var href = leafname;
				var href_id = href.replace(".html", "");
				data['id'] = href_id;
				data['href'] = href;
				data['mediaType'] = "text/html";
				data['data'] = '';
				data['cssRef'] = '';
				data['jsRef'] = '';
				data['available'] = 'false';
				data['azardi-presentation'] = '';
				data['azardi-width'] = '';
				data['azardi-height'] = '';
				data['azardi-margins'] = '';
				data['azardi-toc'] = '';
				data['properties'] = '';
				data['mediaOverlay'] = '';
				Epub.config.bookData.push(data);
			}
		}
	};
	
	var getE0Spine = function(indexTree) {
		var navNodes = indexTree.getElementsByTagName("nav");
		if (navNodes.length < 0) {
			throw new Error('An error occured while trying to load the book.');
		}
		var navNodesCh = navNodes[0].children;
		
		var olnode = "";
		for (var i=0; i<navNodesCh.length; i++) {
			
			if (navNodesCh[i].nodeName === "OL") {
				olnode = navNodesCh[i];
				break;
			}
		}
		
		if (!olnode) {
			throw new Error('An error occured while trying to load the book.');
		}
		
		for (i=0, k=0; i<olnode.children.length; i++) {
			var node = olnode.children[i];
			var nodename = node.nodeName;
			if (nodename == "LI") {
				var anode = node.children[0];
				var href = anode.attributes.getNamedItem('href').nodeValue;
				try {
					//check the role attribute in the li tag
					var role = node.attributes.getNamedItem('role').nodeValue;
				} catch(e){
					var role = "";
				}
				var addToSpine = false;
				if (role) {
					if (role == "spine") {
						addToSpine = true;
					}
				} else {
					addToSpine = true;
				}	
				
				if (addToSpine) {
					href = href.replace(".html", "");
					Epub.config.spineList.push(href);
				}
				
				
				var display_item = {};
				display_item['page-spread'] = 'center';
				display_item["width"] = -1;
				display_item["height"] = -1;
				Epub.config.spineProperties[k] = display_item;
				k = k + 1;
			}
		}
	}
	
	/**
	 * Returns the cover image path for the E0 book
	 * @params opfFileContent Content of the .opf file
	 * @params opfFilePath Path of the .opf file
	 * @params version Version info of the epub
	 * @returns {object} conver image path, cover type
	 */
	var getE0CoverImage = function(indexTree, indexFilePath){
		var self = this;
		
		var coverImage = '';
		var coverImagePath = '';
		var coverObject;
		var cover_type = 'image';
		
		var sectNodes = indexTree.getElementsByTagName("link");
		for (var i=0; i<sectNodes.length; i++) {
			try {
				var role = sectNodes[i].attributes.getNamedItem('rel').nodeValue;
				if (role && (role == "cover")) {
					var coverImage = sectNodes[i].href;
					coverImagePath = Epub.getEntityAbsolutePath(indexFilePath, coverImage);
					coverObject = getFileObjectFromPath(coverImagePath);
					if(!coverObject.exists()) {
						coverImage = '';
						coverImagePath = '';
					}
					break;
				}
			}catch(e){
				
			}
		}
				
		//If the cover image is not found then use default cover as image, cover type as avatar.
		if(coverImage == '') {
			coverImage = 'cover-avatar.png';
			coverObject = getFileObject(['CurProcD', 'chrome', 'content', 'icons', coverImage]);
			coverImagePath = coverObject.path;
			cover_type = 'avatar';
		}
		
		return {
			coverImagePath: coverImagePath,
			cover_type: cover_type
		};
	};
	
	/**
	 * Extracts the information from e0 file
	 * @params epubFile File object of the epub selected
	 * @returns {object} metadata and cover image path
	 */
	var getE0Info = function(epubFile){
		try{
		var self = this;
		try{
			unzipFile(epubFile.path, false);
		}
		catch(e){
			//	Throw error if archive file is corrupted.
			throw new Error('An error occured while trying to load the book.');
		}
		Epub.getExtractLocation();
		Epub.getExtractedFolderName(epubFile.leafName);
		
		var containerXmlPath = getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder ,'index.html']);
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(containerXmlPath);
		if( !file.exists() || !file.isFile() ) {   // if it doesn't exist, create
		   containerXmlPath = getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder, Epub.config.extractedFolder ,'index.html']);
		}
		try{
			var containerXmlContent = readFileUTF(containerXmlPath);
			containerXmlContent = containerXmlContent.replace(/property=/g, "name=");
		}
		catch(e){
			//	Throw error if META-INF folder or container.xml file is missing from the archive. 
			throw new Error('An error occured while trying to load the book1.');
		}
		
		if(containerXmlContent == '') {
			//	Throw error if conatainer.xml file is empty.
			throw new Error('An error occured while trying to load the book.');
		}
		
		var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
							   .createInstance(Components.interfaces.nsIDOMParser);
		var domTree = parser.parseFromString(containerXmlContent, "text/html");
		
		var metNodes = domTree.getElementsByTagName("meta");
		var metadata1 = new Array();	
		for (var i=0; i<metNodes.length; i++) {
			try {
				try {
					var name = metNodes[i].attributes.getNamedItem('name').nodeValue
					var content = metNodes[i].attributes.getNamedItem('content').nodeValue
				}catch(e){
				
				}
				if (name) {
					var mdInfo = new Array();
					mdInfo.push(name);
					mdInfo.push(content);
					metadata1.push(mdInfo);
				}
			} catch(e) {
				
			}
		}
		var version = "e0";
		var coverImageInfo = self.getE0CoverImage(domTree, containerXmlPath);
		var cover_type = 'image';
		var coverImagePath = coverImageInfo.coverImagePath;
		var cover_type = coverImageInfo.cover_type;
		metadata1.push(['dc:location', epubFile.path]);
		metadata1.push(['dc:version', version]);
		metadata1.push(['dc:cover_type', cover_type]);
		
		// deleteFolder(getAppendFilePath(config.extractLocation, [config.extractedFolder]));
		
		return {
			metadata: metadata1,
			coverImagePath: coverImagePath
		};
		}catch(e){
		}
	};
	
	/*----E0 Functions ENDS ----*/
		
	return{
		/*---------------------------*/
		getE0Info: getE0Info,
		getE0CoverImage: getE0CoverImage,
		getE0Spine: getE0Spine,
		loadE0: loadE0,
		loadE0MetaProperties: loadE0MetaProperties,
		getE0Manifest: getE0Manifest,
		getE0HtmlData: getE0HtmlData,
		createE0TOC: createE0TOC,
	}
}();