/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for loading particular chapter.
 */
var Chapter = function(){
	var config = {
		openAtFirst: false,
		currentChapter: 0,
		// containsMathMl: false,
		// containsSVG: false,
		standAlonePage: false,
		XHTML_NS: "http://www.w3.org/1999/xhtml",
		azardi_presentation: null,
		azardi_width: '',
		azardi_height: '',
		azardi_margins: -1,
		azardi_toc: null,
		oldReadingDepth: null,
		orig_width: null,
		orig_height: null,
		toolbar_height: 30,
		viewport_width: null,
		viewport_height: null,
		currentNavPos: 0,
		forcedcolmode: false,
		colmodeprevstate: "",
		colmodeprev_colcount: 0,
		colmodeprev_displayColumnRule: "",
		colmodeprev_columnRule: "",
		colmodeprev_margin: "",
	};
	
	var curr_nav_item = null;
	
	var flo_pageref_display_index = 3;
	
	var history = {
		chapterNo: null,
		readingDepth: null
	};
	
	/**
	 * Initilize the chapter
	 */
	var init = function(){
		try{
			var self = this;
			var bookPref = BookPref.getFromDB();
			//DC - FLO UPDATE - 201211
			if(typeof bookPref == 'undefined'){
				BookPref.init();	// It overrides book pref with user pref.
			} else {
				BookPref.initWithSavedPreferences(bookPref);	// It overrides book pref with saved book prefs.
				var floShowPageNum = parseInt(BookPref.config.floShowPageNum);	
				self.flo_pageref_display_index = floShowPageNum;
				width = parseInt(BookPref.config.openWidth) || window.outerWidth;
				height =  parseInt(BookPref.config.openHeight) || window.outerHeight;
				//Resizing to full screen width seems to make the screen junp around
				//so reduce the size by 5px
				if (width >= window.screen.width) width = width-5;
				if (height >= window.screen.height) height = height-5;
				BookPref.config.openWidth = width;
				BookPref.config.openHeight = height;
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				if(BookPref.config.columns){
					Columns.config.showLastPage = false;
					Columns.config.goToDepth = BookPref.config.readingDepth;
				}
				else{
					Flow.config.showLastPage = false;
				}
			}

			if(!config.openAtFirst) {
				config.currentChapter = BookPref.config.currentChapter;
			}
				
			if (Epub.is_fixed_layout) {
				config.viewport_width = window.screen.width;
				config.viewport_height = window.screen.height;
				$("#doc-viewer-azd").hide();
				displayFixedLayoutDoc();
				Toolbar.setFLOToolbarDisplay(true);
			} else {
				$("#doc-flo-viewer-azd").hide();
				$("#doc-viewer-azd").show();
				self.loadPageEx("init");
				Toolbar.setFLOToolbarDisplay(false);
			}
			return;
		}catch(e){
			//alert(e);
		}
	}
	
	var displayFixedLayoutDoc = function() {
		$("#doc-flo-viewer-azd").attr('src', "html/azardi2_flo_epub_doc.html");
		$("#doc-flo-viewer-azd").bind('DOMContentLoaded', function(e) {
			try {
				$("#doc-flo-viewer-azd").unbind('DOMContentLoaded');
				$("#doc-flo-viewer-azd").show();
				document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.init();
				setTimeout(function(){
					document.getElementById('doc-flo-viewer-azd').contentWindow.focus();
					}, 100);
			} catch(e) {
				//alert(e);
			}
		});
	};
	
	var displayFlowItem = function(floc) {
		try {
			$("#doc-flo-viewer-azd").hide();
			$("#doc-viewer-azd").show();
			var idref = Epub.getCurrentChapterFromSpine();
			if (!idref) {
				Chapter.loadStandAlonePage(floc);
			} else {
				Chapter.loadPageEx("flolink", floc);
			}
			Toolbar.enableUserPrefButton();
			Toolbar.setFLOToolbarDisplay(false);
		}catch(e){
			//alert(e);
		}
	};
	
	var switchToFixedMode = function(floc) {
		try {
			$("#doc-flo-viewer-azd").show();
			$("#doc-viewer-azd").hide();
			AME.paused_handle();
			Toolbar.disableUserPrefButton();
			Toolbar.disableColumnsButton();
			Toolbar.disableColumnRuleButton();
			Toolbar.disableFontSizeButton();
			Toolbar.disableResetButton();
			Toolbar.disableMarginSpaceButton();
			Toolbar.disableLineSpaceButton();
			Toolbar.disableJustifyButton();
			Toolbar.disableHyphenateButton();
			Toolbar.disableBookPrefButton();
			//We will disable audio options inside the flo engine
			//Toolbar.disableAudioOptionsButton();
			Toolbar.disableLandmarkButton();
			Toolbar.disableTocButton();
			Toolbar.setFLOToolbarDisplay(true);
			document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.stopAudio();
		} catch(e) {
			
		}
	};
	/*----------------FLO DONE--------------------------*/
	
	var loadPageEx = function (calledfrom, floc) {
		try {
			var self = this;
			calledfrom = typeof calledfrom !== 'undefined' ? calledfrom : '';
			floc = typeof floc !== 'undefined' ? floc : '';
			//Do not remove the line, used to calculate correct innerHeight
			config.toolbar_height = parseInt(window.innerHeight) - parseInt(document.defaultView.getComputedStyle(document.	getElementById("doc-viewer-azd"), null).getPropertyValue("height"));
			
			removeNoteFNPopup();
			
			if (calledfrom == "standalone") {
				Columns.config.showLastPage = false;
				Flow.config.showLastPage = false;
				var hrefUrl = floc;
				var mediaOverlay = "";
				//Lets get the id of the standalone item here to get the other rendition properties
				for(var i = 0; i < Epub.config.bookData.length; i++){	
					var currhtml = Epub.config.bookData[i]['href'].split('/').pop();
					var hrefUrl_fn = hrefUrl.split('/').pop();
					if(currhtml === hrefUrl_fn){
						var idref = Epub.config.bookData[i]['id'];
						break;
					}
				}
				
			} else {
				//Get spine of current chapter
				var idref = Epub.getCurrentChapterFromSpine();
				if(!idref) {
					config.currentChapter = 0;
					config.standAlonePage = false;
					idref = Epub.getCurrentChapterFromSpine();
				} else {
					config.standAlonePage = false;
				}
				
				var hrefUrl = Epub.getChapterHrefUrl(idref);
				//Get fixed layout details of current chapter
				var chapterPresentation = Epub.getChapterPresentationDetails(idref);
				Chapter.setChapterPresentation(chapterPresentation);
				//Get media overlay of current chapter
				var mediaOverlay = Epub.getChapterOverlay(idref);
			}
			var displayurl = hrefUrl + "?" + generateRandString();
			$('#doc-viewer-azd').attr('src', displayurl);
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
				try {
					azLogTime("PAGE LOAD STARTS");
					$('#doc-viewer-azd').unbind('DOMContentLoaded');
					$('#doc-viewer-azd').contents().find('body').wrapInner('<div id="custom-style-igp"  class="custom-style-igp"></div>');
					$('#doc-viewer-azd').contents().find('#custom-style-igp').before(Epub.appPrevControl);
					$('#doc-viewer-azd').contents().find('#custom-style-igp').after(Epub.appNextControl);
					var note_label = getPropertiesString("azd_epub_makeNoteDlgTitle");
					var note_done = getPropertiesString("azd_epub_makeNoteDlgDone");
					Epub.appControlStructure = Epub.appControlStructure.replace("$makeanote_label$", note_label);
					Epub.appControlStructure = Epub.appControlStructure.replace("$makeanote_done$", note_done);
					$('#doc-viewer-azd').contents().find('body').append(Epub.appControlStructure);
					if (Chapter.flo_pageref_display_index>=0) {
						self.showRealPageNums(false);
					}
					//Initilize the reader's audio player if chapter has smil data associated with it
					self.initAudioPlayer(mediaOverlay);
					if(mediaOverlay != '') {
						try {
							var srcpage = hrefUrl.split("/").pop();
							Smil.init(mediaOverlay, srcpage);
						} catch(e) {
						
						}
					}
					try{
						if(Epub.config.version == 3) {
							if(Epub.config.epub3TocData != ''){
								//Set fly-out toc for epub3
								Epub.setInPageTocDataEpub3();
							}
							else{
								//Set fly-out toc for epub2
								Epub.setInPageTocDataEpub2();
							}
						}
						else {
							//Set fly-out toc for epub2
							
							Epub.setInPageTocDataEpub2();
						}
						
						if(config.azardi_toc == 'false') {
							self.hideToc();
							Toolbar.disableLandmarkButton();
						} else {
							self.showToc();
							if(Epub.config.version == 3) {
								Toolbar.enableLandmarkButton();
							}
						}
					} catch(e) {
						//alert(e)
					}
					if(config.azardi_presentation == null) {
						self.unbindResize();
												
						self.bindResize();
						
					}
					config.orig_width = BookPref.config.openWidth;
					config.orig_height = BookPref.config.openHeight;
					Justify.getParagraphsDefaultAlignment();
					BookPref.set();
					self.processInternalLinks();
					self.processParagraphs();
					Notes.getNotes(Library.config.currentBookId, config.currentChapter);
					
					self.loadAudio();
					self.loadVideo();
					self.loadImages(idref);
					try {
						var metainfo = Epub.config.metainfo;
						$('#doc-viewer-azd').contents().find('body').attr("azdtemp", JSON.stringify(metainfo));
						document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.initAIE(metainfo);
					} catch(e) {
						
					}
					if (calledfrom == "sidetoc") {
						document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.showSideToc();
					}
					document.getElementById('doc-viewer-azd').contentWindow.focus();
					
				} catch(e) {
					azDump("******************loadPageEx inside Exception", e);
				}
			});
			
			
		} catch(e) {
			alert(e);
			azDump("******************loadPageEx Exception", e);
		}
	};
	
	/**
	 * Loads the current chapter
	 */
	var load = function(){
		var self = this;
		
		//Get spine of current chapter
		var idref = Epub.getCurrentChapterFromSpine();
		//Get fixed layout details of current chapter
		var chapterPresentation = Epub.getChapterPresentationDetails(idref);
		self.setChapterPresentation(chapterPresentation);
		//Get media type of current chapter
		var mediaType = Epub.getChapterMediaType(idref);
		//Get properties of current chapter
		var properties = Epub.getChapterProperties(idref);
		//Get href of current chapter
		var href = Epub.getChpaterHref(idref);
		//Get html data of current chapter
		var data = Epub.getChapterData(idref);
		//Get css references of current chapter
		var cssRef = Epub.getChapterCssRef(idref);
		//Get js references of current chapter
		var jsRef = Epub.getChapterJsRef(idref);
		//Get media overlay of current chapter
		var mediaOverlay = Epub.getChapterOverlay(idref);
		
		//Initilize the reader's audio player if chapter has smil data associated with it
		self.initAudioPlayer(mediaOverlay);
		
		//Remove old css references
		self.removeCssReference();
		// self.removeJsReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');

		if(cssRef != ''){
			//Set new css references
			self.setCssReference(cssRef);
		}
		
		//If current chapter contains mathml then set the stix fonts
		if(properties.search(/mathml/i) >= 0) {
			self.setStixFont();
		}
		
		if(jsRef != ''){
			self.setJsReference(jsRef);
		}
		
		if(mediaType.split('/')[0] == 'image') {
			self.loadImageAsPage(data);
		} else if(mediaType == 'application/x-shockwave-flash') {
			self.loadSwfAsPage(href);
		} else if(mediaType == 'application/pdf') {
			self.loadPdfAsPage(href);
		} else {
			$('#doc-viewer-azd').contents().find('#custom-style-igp').html(data);
		}
		
		if(mediaOverlay != '') {
			var srcpage = href.split("/").pop();
			Smil.init(mediaOverlay, srcpage);
		}
		
		if(config.azardi_toc == 'false') {
			self.hideToc();
			Toolbar.disableLandmarkButton();
		} else {
			self.showToc();
			if(Epub.config.version == 3) {
				Toolbar.enableLandmarkButton();
			}
		}
	};
	
	/**
	 * Set the css for stix fonts
	 */
	var setStixFont = function() {
		var styleTag = '<link rel="stylesheet" type="text/css" href="../css/STIXfonts.css" id="stixCss" />';
		$('#doc-viewer-azd').contents().find('head').append(styleTag);
	};
	
	/**
	 * Hide flyout toc
	 */
	var hideToc = function() {
		$('#doc-viewer-azd').contents().find('.toc-azd').hide();
		Toolbar.disableTocButton();
	}
	
	/**
	 * Show flyout toc
	 */
	var showToc = function() {
		$('#doc-viewer-azd').contents().find('.toc-azd').show();
		Toolbar.enableTocButton();
	}
	
	/**
	 * Set fixed layout properties for current chapter
	 * @params chapterPresentation Fixed layout details of current chapter
	 */
	var setChapterPresentation = function(chapterPresentation) {
		var self = this;
		
		config.azardi_presentation = (chapterPresentation['azardi-presentation'] != '') ? chapterPresentation['azardi-presentation'] : null;
		config.azardi_width = chapterPresentation['azardi-width'];
		config.azardi_height = chapterPresentation['azardi-height'];
		config.azardi_margins = (chapterPresentation['azardi-margins'] != '') ? parseInt(chapterPresentation['azardi-margins']) : -1;
		config.azardi_toc = (chapterPresentation['azardi-toc'] != '') ? chapterPresentation['azardi-toc'] : null;
	};
	
	/**
	 * Loads the .pdf file as a page
	 
	 * @params href href path of .pdf file
	 */
	var loadPdfAsPage = function(href) {
		var self = this;
		
		var id = href.split('/').pop();
		var pdfUrl = '';
		var pdfObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, id]);
		if(pdfObj.exists()){
			pdfUrl = getURL(pdfObj);
		}
		
		var data = pdfUrl + '?#scrollbar=1&toolbar=1&navpanes=0&view=FitV&statusbar=0';
		
		var pdfPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('div'))
						.addClass('azardi-pdf-page')
						.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('object'))
							.attr('id', 'pdf')
							.attr('data', data)
							.attr('type', 'application/pdf')
							// .attr('width', '550')
							// .attr('height', '400')
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('p'))
								.html('This PDF cannot be display. You must install Adobe Reader.')
							)
						)
						
		//$('#doc-viewer-azd').contents().find('#custom-style-igp').append(pdfPage);
		AME.openPDF(data);
	};
	
	/**
	 * Loads the .swf file as a page
	 * @params href path of the .swf file
	 */
	var loadSwfAsPage = function(href) {
		var self = this;
		
		var id = href.split('/').pop();
		var swfUrl = '';
		var swfObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, id]);
		if(swfObj.exists()){
			swfUrl = getURL(swfObj);
		}
		
		var flashPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('object'))
							.attr('classid', 'clsid:d27cdb6e-ae6d-11cf-96b8-444553540000')
							.attr('width', '550')
							.attr('height', '400')
							.attr('id', id)
							.attr('align', 'middle')
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('param'))
								.attr('name', id)
								.attr('value', swfUrl)
							)
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('a'))
								.attr('href', 'http://www.adobe.com/go/getflash')
								.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('img'))
									.attr('src', 'http://www.adobe.com/images/shared/download_buttons/get_flash_player.gif')
									.attr('alt', 'Get Adobe Flash player')
								)
							)
		$('#doc-viewer-azd').contents().find('#custom-style-igp').append(flashPage);
	};
	
	/**
	 * Loads the image as page
	 * @params imageData Datauri of the image to be used as a page
	 */
	var loadImageAsPage = function(imageData) {
		var self = this;
		
		var imagePage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('div'))
							.addClass('azardi-image-page')
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('img'))
								.attr('src', imageData)
							)
		
		$('#doc-viewer-azd').contents().find('#custom-style-igp').append(imagePage);
	};
	
	var loadPreviewPage = function() {
		var self = this;
		
		var idref = 'preview';
		var href = Epub.getChpaterHref(idref);
		var data = Epub.getChapterData(idref);
		var cssRef = Epub.getChapterCssRef(idref);
		var jsRef = Epub.getChapterJsRef(idref);
		
		self.removeCssReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');

		if(cssRef != ''){
			self.setCssReference(cssRef);
		}
		
		if(jsRef != ''){
			self.setJsReference(jsRef);
		}
		
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html(data);
		self.showToc();
		if(Epub.config.version == 3) {
			Toolbar.enableLandmarkButton();
		}
	};
	
	/**
	 * This function is used to load the chapter which are not in spine order.
	 * @params fileLocation href of the file to be loaded
	 */
	var loadStandAlonePage = function(fileLocation) {
		try{
			var self = this;
			
			//Get the presentation details of current chapter
			var chapterPresentation = Epub.getChapterPresentationDetailsFromFileLocation(fileLocation);
			self.setChapterPresentation(chapterPresentation);
			
			//Get media type of current chapter
			var mediaType = Epub.getChapterMediaTypeFromFileLocation(fileLocation);
			if(mediaType == 'application/pdf') {
				self.loadPdfAsPage(fileLocation);
				AME.hideLoading();
				return false;
			}
			//Get properties of the current chapter
			var properties = Epub.getChapterPropertiesFromFileLocation(fileLocation);
			//Get html data of the current chapter
			var data = Epub.getChapterDataFromFileLocation(fileLocation);
			//Get the css references of the current chapter
			var cssRef = Epub.getChapterCssRefFromFileLocation(fileLocation);
			//Get the js references of the current chapter
			var jsRef = Epub.getChapterJsRefFromFileLocation(fileLocation);
			//Remove the old css references
			self.removeCssReference();
			// self.removeJsReference();
			$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');

			if(cssRef){
				self.setCssReference(cssRef);
			}
			
			if(properties.search(/mathml/i) >= 0) {
				self.setStixFont();
			}
			
			if(jsRef){
				self.setJsReference(jsRef);
			}
			
			if(mediaType.split('/')[0] == 'image') {
				self.loadImageAsPage(data);
			} else if(mediaType == 'application/x-shockwave-flash') {
				self.loadSwfAsPage(fileLocation);
			} else {
				config.standAlonePage = true;
				//Change currentChapter to some custom name, need it when we save chapter information on closing
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				//DC - FLO UPDATE - 201211
				//var hrefUrl = Epub.getChapterUrlFromFileLocation(fileLocation);
				//$('#doc-viewer-azd').attr('src', hrefUrl);
				var hrefUrl = Epub.getChapterUrlFromFileLocation(fileLocation);
				self.loadPageEx("standalone", hrefUrl);
			}
			
			if(config.azardi_toc != null) {
				self.hideToc();
			} else {
				self.showToc();
			}
		} catch(e) {
			azDump("loadStandAlonePage exception", e);
		}
	};
	
	/**
	 * Process all the paragraphs of the chapter.
	 * Assign a custom notes class to each paragraph, so it can be used while creating notes.
	 */
	var processParagraphs = function(){
		var self = this;
		
		var $p = $('#doc-viewer-azd').contents().find('#custom-style-igp p');
		$p.each(function(index){
			$(this).addClass('azd-notes-paragraph-'+index);
		});
	};
	
	var removeNoteFNPopup = function() {
		var popup = $('#doc-viewer-azd').contents().find('.azardi-popup');
		if (popup.length) {
			$('#doc-viewer-azd').contents().find('.azardi-popup').remove();
		}
	};
	
	var createPopup = function (notelink, epubtype) {
		try {
			$('#doc-viewer-azd').contents().find('.azardi-popup').remove();
			var popidref = $(notelink).attr('href');
			var copy = $('#doc-viewer-azd').contents().find(popidref).clone(true);
			var poptemp = $('<div class="azardi-popup ' + epubtype + '" />').append(copy);
			
			$('#doc-viewer-azd').contents().find(popidref).after(poptemp);
			
			$('#doc-viewer-azd').contents().find('body').click(function(e){
				var popup = $('#doc-viewer-azd').contents().find('.azardi-popup');
				if (!popup.is(e.target) 
					&& popup.has(e.target).length === 0) {
					removeNoteFNPopup();
				}
			});
			
		} catch(e) {
		
		}
	}
	
	/**
	 * Process the links in the chapter
	 */
	var processInternalLinks = function(){
		try {
			var self = this;
			var $links = $('#doc-viewer-azd').contents().find('#custom-style-igp a');
			$links.each(function(){
				var linkReference = $(this).attr('href');
				
				if(typeof linkReference != 'undefined'){
					if(linkReference.indexOf('http://') == 0 || linkReference.indexOf('https://') == 0){
						$(this).bind('click', function(event){
							event.preventDefault();
							window.open(linkReference, "IEV", "chrome, width=1000, height=700, resizable=yes, centerscreen, scrollbars=yes").focus();
						});
					} else if (linkReference != 'javascript: void(0);'){
						if(linkReference.split('#')[0] != ''){
							$(this).bind('click', function(event){
								event.preventDefault();
								self.goToFromInternalLinks(linkReference);
							});
						} else {
							$(this).bind('click', function(event){
								event.preventDefault();
								var ref = linkReference.split('#')[1];
								var epubtype = $('#doc-viewer-azd').contents().find('#'+ref).attr("epub:type");
								var currtype = $(this).attr("epub:type");
								if (currtype && currtype=="noteref" && epubtype) {
									createPopup($(this), currtype);
									return false;
								}
								
								if (currtype && currtype=="glossterm" && epubtype) {
									createPopup($(this), currtype);
									return false;
								}
								
								if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
									Columns.goToInternalLinkReference(linkReference.split('#')[1]);	
								}
								else{
									Flow.goToInternalLinkReference(linkReference.split('#')[1]);
								}
							});
						}
					}
				}
			});
		} catch(e) {
			//alert("processInternalLinks" + e);
		}
	}
	
	/**
	 * Removes old css references from the page
	 */
	var removeCssReference = function(){
		$('#doc-viewer-azd').contents().find('style').each(function(){
			$(this).remove();
		});
		
		//Remove the stix fonts as well
		$('#doc-viewer-azd').contents().find('#stixCss').remove();
	};
	
	var removeJsReference = function() {
		return;
		$('#doc-viewer-azd').contents().find('script').each(function(){
			$(this).remove();
		});
	}
	
	/**
	 * Sets the new css references
	 * @params cssRef css references for the current page
	 */
	var setCssReference = function(cssRef){
		for(var i = 0; i < cssRef.length; i++){
			var cssData = Epub.getCssData(cssRef[i]);
			
			var styleTag = '<style type="text/css">' + cssData + '</style>';
			$('#doc-viewer-azd').contents().find('head').append(styleTag);
		}
	};
	
	var setJsReference = function(jsRef) {
		for(var i = 0; i < jsRef.length; i++){
			if(typeof jsRef[i] != 'undefined') {
				var jsData = Epub.getJsData(jsRef[i]);
				var scriptTag = document.createElementNS(config.XHTML_NS, 'script');
				var scriptContent = document.createTextNode(jsData);
				scriptTag.appendChild(scriptContent);
				document.getElementById('doc-viewer-azd').contentWindow.document.getElementsByTagName('head')[0].appendChild(scriptTag);
			}
		}
	}
	
	/**
	 * Function to go to next viewport/chapter
	 */
	var next = function(){
		var self = this;
		removeNoteFNPopup();
		
		if (Epub.is_fixed_layout) {
			if (document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.in_reflow_mode) {
				if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
					var xOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.xOffset;
					var documentWidth = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.documentWidth;
					var currentWidth = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentWidth;
					
					if(documentWidth == (currentWidth + xOffset)){
						if (config.standAlonePage) {
							return false;
						}
						Columns.config.showLastPage = false;
						Flow.config.showLastPage = false;
						document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayNext();
					} else {
						Columns.next();
						self.setNavButtons();
					}
				} else {
					var yOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.yOffset;
					var documentHeight = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.documentHeight;
					var currentHeight = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight;
					
					if(documentHeight <= (currentHeight + yOffset) || documentHeight - (currentHeight + yOffset) == 1){
						if (config.standAlonePage) {
							return false;
						}
						Columns.config.showLastPage = false;
						Flow.config.showLastPage = false;
						document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayNext();			
					} else {
						Flow.next();
						self.setNavButtons();
					}
				}
			} else {
				Columns.config.showLastPage = false;
				Flow.config.showLastPage = false;
				document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayNext();
			}
			return;
		} 
			
		
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.initPageVariables();
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			var xOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.xOffset;
			var documentWidth = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.documentWidth;
			var currentWidth = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentWidth;
			
			if(documentWidth == (currentWidth + xOffset)){
				//Go to next chapter
				if(config.currentChapter < Epub.config.spineList.length - 1 && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter += 1;
					AME.showLoading();
					
					//DC - FLO UPDATE - 201211
					config.currentNavPos += 1;
					var idref = Epub.getCurrentChapterFromSpine();
					var navpos = Epub.nav_lookup[idref];
					Columns.config.showLastPage = false;
					Flow.config.showLastPage = false;
					self.loadPageEx();
					return;
				}
				else{
					//	TODO: Disable the next button here.
				}
			}
			else{
				Columns.next();
				self.setNavButtons();
			}
		}
		else{
			var yOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.yOffset;
			var documentHeight = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.documentHeight;
			var currentHeight = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight;
			
			if(documentHeight <= (currentHeight + yOffset) || documentHeight - (currentHeight + yOffset) == 1){
				//Go to next chapter
				if(config.currentChapter < Epub.config.spineList.length - 1 && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter += 1;
					AME.showLoading();
					//DC - FLO UPDATE - 201211
					config.currentNavPos += 1;
					var idref = Epub.getCurrentChapterFromSpine();
					var navpos = Epub.nav_lookup[idref];
					Columns.config.showLastPage = false;
					Flow.config.showLastPage = false;
					self.loadPageEx();
					return;
				}
				else{
					//	TODO: Disable the next button here.
				}
			}
			else{
				Flow.next();
				self.setNavButtons();
			}
		}
	}
	
	/**
	 * Function to go to previous viewport/chapter
	 */
	var previous = function(){
		var self = this;
		removeNoteFNPopup();
		
		if (Epub.is_fixed_layout) {
		    if (document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.in_reflow_mode) {
				if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
					var xOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.xOffset;
						
					if(xOffset == 0){
						if (config.standAlonePage) {
							return false;
						}
						Columns.config.showLastPage = true;
						document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayPrevious();
					} else {
						Columns.previous();
						self.setNavButtons();
					}
				} else {
					var yOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.yOffset;
				
					if(yOffset == 0){
						if (config.standAlonePage) {
							return false;
						}
						Flow.config.showLastPage = true;
						document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayPrevious();
				
					} else {
						Flow.previous();
						self.setNavButtons();
					}
				}
			} else {
				Columns.config.showLastPage = true;
				Flow.config.showLastPage = true;
				document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayPrevious();
			}
			return;
		} 
		
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.initPageVariables();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			var xOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.xOffset;
			if(xOffset == 0){
				//Go to previous chapter
				if(config.currentChapter > 0  && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter -= 1;
					AME.showLoading();
					//DC - FLO UPDATE - 201211
					config.currentNavPos -= 1;
					var idref = Epub.getCurrentChapterFromSpine();
					var navpos = Epub.nav_lookup[idref];
					Columns.config.showLastPage = true;
					Flow.config.showLastPage = true;
					self.loadPageEx();
					return;
				}
				else{
					Columns.config.showLastPage = false;
					Flow.config.showLastPage = false;
				}
			}
			else{
				Columns.previous();
				self.setNavButtons();
			}
		}
		else{
			var yOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.yOffset;
			if(yOffset == 0){
				//Go to previous chapter
				if(config.currentChapter > 0 && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter -= 1;
					AME.showLoading();
					//DC - FLO UPDATE - 201211
					config.currentNavPos -= 1;
					var idref = Epub.getCurrentChapterFromSpine();
					var navpos = Epub.nav_lookup[idref];
					Flow.config.showLastPage = true;
					self.loadPageEx();
					return;
				}
				else{
					//	TODO: Disable the previous button here.
					Flow.config.showLastPage = false;
				}
			}
			else{
				Flow.previous();
				self.setNavButtons();
			}
		}
	
	}
	
	/**
	 * Function to go to next chapter
	 */
	var nextChapter = function() {
		var self = this;
		removeNoteFNPopup();
		
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayNext();
			return;
		} 
		
		if(config.currentChapter < Epub.config.spineList.length - 1 && !config.standAlonePage){
			history.chapterNo = null;
			history.readingDepth = null;
			Toolbar.disableBackButton();
			
			config.currentChapter += 1;
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			self.loadPageEx();
			return;
		}
	};
	
	/**
	 * Function to go to previous chapter
	 */
	var previousChapter = function() {
		var self = this;
		removeNoteFNPopup();
		
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.displayPrevious();
			return;
		} 
		
		if(config.currentChapter > 0  && !config.standAlonePage){
			history.chapterNo = null;
			history.readingDepth = null;
			Toolbar.disableBackButton();
			
			config.currentChapter -= 1;
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			self.loadPageEx();
			return;
		}
	};
	
	/**
	 * Function to initilize the sequence to go to reading depth
	 */
	var goToReadingDepth = function() {
		var self = this;
		if(String(config.currentChapter).indexOf('azd-outofspine-') == -1){
			config.standAlonePage = false;
			//DC - FLO UPDATE - 201211
			self.loadPageEx();
			return false;
		} else {
			var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
			config.standAlonePage = true;
			self.loadStandAlonePage(fileLocation);
		}
	}
	
	/**
	 * Function to load the chapter and go to note
	 * @params id Id of the note
	 * @params chapter Chapter no.
	 */
	var goToNote = function(id, chapter){
		var self = this;

		// $('#but_toc').click();
		
		if(chapter.indexOf('azd-outofspine-') == -1){
			config.currentChapter = parseInt(chapter);
			config.standAlonePage = false;
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			if (BookPref.config.columns) {
				Columns.config.showLastPage = false;
				Columns.config.goToNoteId = id;
			} else {
				Flow.config.showLastPage = false;
				Flow.config.goToNoteId = id;
			}
			self.loadPageEx();
			return;
		} else {
			config.currentChapter = chapter;
			var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
			config.standAlonePage = true;
			AME.showLoading();
			self.loadStandAlonePage(fileLocation);
		}	
	}
	
	/**currentChapter
	 * Function used to navigate from internal links
	 * @params reerenceLocation Location of the next chapter.
	 */
	var goToFromInternalLinks = function(referenceLocation){
		var self = this;
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.navigateToFixedView(fileLocation);
			return;
		} 
		
		history.chapterNo = config.currentChapter;
		Toolbar.enableBackButton();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			history.readingDepth = Columns.config.currentReadingDepth;
		}
		
		
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		
		if(typeof spineId == 'undefined') {
			Toolbar.disableBackButton();
			history.chapterNo = null;
			history.readingDepth = null;
			return;
		}
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			if(BookPref.config.columns){
				Columns.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Columns.config.goToReference = goToReference;
				}	
			}
			else{
				Flow.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Flow.config.goToReference = goToReference;
				}
			}
			self.loadPageEx();
			return false;
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				//Change currentChapter to some custom name, need it when we save chapter information on closing
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				//Load chapter page that is not in spine order
				self.loadStandAlonePage(fileLocation);
			}
		}
	}
	
	/**
	 * Function used for navigation using page-nav.
	 * @params referenceLocation Destination location
	 */
	var goToFromPageNav = function (referenceLocation) {
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.navigateToFixedView(fileLocation);
			return;
		} 
		
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			self.loadPageEx();
			if(BookPref.config.columns){
				Columns.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Columns.config.goToReference = goToReference;
				}	
			}
			else{
				Flow.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Flow.config.goToReference = goToReference;
				}
			}
			return false;
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				self.loadStandAlonePage(fileLocation);
			}
		}
	};
	
	/**
	 * Function used for navigation using landmark.
	 * @params referenceLocation Destination location
	 */
	var goToFromLandmark = function (referenceLocation) {
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.navigateToFixedView(fileLocation);
			return;
		} 
		
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			if(BookPref.config.columns){
				Columns.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Columns.config.goToReference = goToReference;
				}	
			} else {
				Flow.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Flow.config.goToReference = goToReference;
				}
			}
			
			self.loadPageEx();
			return false;
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				self.loadStandAlonePage(fileLocation);
			}
		}
	}
	
	/**
	 * Loads the cover page as current chapter
	 * @params tocType Type of the toc navigating from
	 */
	var goToCoverPage = function(tocType) {
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		Toolbar.disableColumnsButton();
		Toolbar.disableColumnRuleButton();
		Toolbar.disableFontSizeButton();
		Toolbar.disableResetButton();
		Toolbar.disableMarginSpaceButton();
		Toolbar.disableLineSpaceButton();
		Toolbar.disableJustifyButton();
		// Toolbar.disableColorButton();
		Toolbar.disableHyphenateButton();
		Toolbar.disableBookPrefButton();
		Toolbar.disableAudioOptionsButton();
		
		if(BookPref.config.columns && tocType == 'side'){
			history.readingDepth = Columns.config.currentReadingDepth;
		}
		
		config.standAlonePage = true;
		// config.currentChapter = 'azd-outofspine-'+fileLocation;
		AME.showLoading();
		self.removeCssReference();
		// self.removeJsReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');
		
		var snippet = '';
		if(Epub.config.cover_type === 'image') {
			snippet = '<div class="azardi-cover-image" style="background-image:url(\'$background-image$\');"><div class="azardi-cover-details"><p>$title$</p><p>$author$</p></div></div>';
		} else {
			snippet = '<div class="azardi-cover-avatar" style="background-image:url(\'$background-image$\');"><div class="azardi-cover-details"><p>$title$</p><p>$author$</p></div></div>';
		}
		snippet = snippet.replace('$background-image$', Epub.config.cover_image);
		snippet = snippet.replace('$title$', Epub.config.title);
		snippet = snippet.replace('$author$', Epub.config.author);
		
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html(snippet);
		
		if(BookPref.config.columns){
			Columns.init(BookPref.config.noOfCols);
		} else {
			Flow.init();
		}
		
		Fonts.setMin();
		Hyphen.setNone();
		Line.setMin();
		Margin.setMin();
		
		self.setNavButtons();
		
		self.setCoverDim();
		
		AME.hideLoading();
	};
	
	/**
	 * Sets the height of the cover inside cover page
	 */
	var setCoverDim = function() {
		var Dim = document.getElementById('doc-viewer-azd').contentWindow.getInnerDim();
		
		$('#doc-viewer-azd').contents().find('.azardi-cover-image, .azardi-cover-avatar').css({
			// 'width': Dim.width,
			'height': Dim.height
		});
		
		$(window).bind('resize', function(){
			var Dim = document.getElementById('doc-viewer-azd').contentWindow.getInnerDim();
		
			$('#doc-viewer-azd').contents().find('.azardi-cover-image, .azardi-cover-avatar').css({
				// 'width': Dim.width,
				'height': Dim.height
			});
		});
	};
	
	/**
	 * Function used for navigation using toc.
	 * @params referenceLocation Destination location
     * @params tocType Type of toc used to initiate the navigation side|full-page
	 */
	var goToFromToc = function(referenceLocation, tocType){
		try {
		var self = this;
		history.chapterNo = config.currentChapter;
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		
		
		Toolbar.enableBackButton();
		
		if (Epub.is_fixed_layout) {
			document.getElementById('doc-flo-viewer-azd').contentWindow.wrappedJSObject.FLO.navigateToFixedView(fileLocation);
			return;
		} 
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed' && tocType == 'side'){
			history.readingDepth = Columns.config.currentReadingDepth;
		}
		
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			if(BookPref.config.columns){
				Columns.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Columns.config.goToReference = goToReference;
				}	
			} else {
				Flow.config.showLastPage = false;
				if(typeof goToReference != 'undefined'){
					Flow.config.goToReference = goToReference;
				}
			}
			
			self.loadPageEx("sidetoc");
			return false;
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				//Change currentChapter to some custom name, need it when we save chapter information on closing
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				//Load chapter page that is not in spine order
				self.loadStandAlonePage(fileLocation);
			}
		}
		}catch(e){
			//alert(e);
		}
	};
	
	var goBack = function(){
		try {
		var self = this;
		
		config.currentChapter = history.chapterNo;
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.goToDepth = history.readingDepth;
		}
		 
		if(String(config.currentChapter).indexOf('azd-outofspine-') == -1){
			config.standAlonePage = false;
			AME.showLoading();
			//DC - FLO UPDATE - 201211
			self.loadPageEx();
			return;
		} else {
			var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
			config.standAlonePage = true;
			AME.showLoading();
			self.loadStandAlonePage(fileLocation);
		}
		}catch(e) {
			//alert(e);
		}
	}
	
	/**
	 * Change the src of audio tag in this chapter to point to audios in buffer location
	 */
	var loadAudio = function(){
		var self = this;
		
		var audioObj, audioUrl, filename;
		
		var $audio = $('#doc-viewer-azd').contents().find('#custom-style-igp audio');
		//	If no audio is found on the page.
		if($audio.length == 0) {
			return false;
		} else {
			$audio.each(function() {
				var audRef = $(this).attr('src');
				if(typeof audRef == 'undefined') {
					//	If audio tag does not have any src attribute then check if it contains source tag.
					var $sources = $(this).find('source');
					for (var i=0; i< $sources.length; i++) {
						var $source = $($sources[0]);
						//	If the source tag is found, then check if it contains src.
						var sourceRef = $source.attr('src');
						if(typeof sourceRef == 'undefined') {
							//	If source tag does not have any src attribute then skip this audio.
							
						} else {
							if(sourceRef.indexOf("http") == -1) {
								// If the source attribute is not empty then set the new src of this source tag.
								$source.remove();
								filename = sourceRef.split('/').pop();
								audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
								if(audioObj.exists()){
									audioUrl = getURL(audioObj);
									// $(this).attr('preload', 'none');
									$(this).attr('src', audioUrl);
								} else {
									Epub.config.containErrors = true;
								}
							}
						}
					}
				} else {
					//	If the source attribute is not empty then set the new src of this audio tag.
					filename = audRef.split('/').pop();
					audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
					if(audioObj.exists()) {
						audioUrl = getURL(audioObj);
						// $(this).attr('preload', 'none');
						$(this).attr('src', audioUrl);
					} else {
						Epub.config.containErrors = true;
					}
				}
			});
		}
		
		// var audioObj;
		// var audioUrl;
		// var $audio = $('#doc-viewer-azd').contents().find('#custom-style-igp audio source');
		// if($audio.length == 0){
			// return false;
		// }
		// else{
			// $audio.each(function(){
				// for(var i = 0; i < Epub.config.bookData.length; i++){
					// var filename = $(this).attr('src').split('/').pop();
					// if(unescape(filename) == unescape(Epub.config.bookData[i]['href'].split('/').pop())){
						// audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
						// audioUrl = getURL(audioObj);
						// // alert(audioUrl);
						// $(this).parent().attr('preload', 'none')
						// $(this).parent().attr('src', audioUrl);
						// $(this).remove();
						// break;
					// }
				// }
			// });
		// }
	};
	
	/**
	 * Change the src of video tag in this chapter to point to audios in buffer location
	 */
	var loadVideo = function(){
		var self = this;
		
		var videoObj, videoUrl, filename;
		
		var $video = $('#doc-viewer-azd').contents().find('#custom-style-igp video');
		//	If no video is found on the page.
		if($video.length == 0) {
			return false;
		} else {
			$video.each(function() {
				var vidRef = $(this).attr('src');
				if(typeof vidRef == 'undefined') {
					//	If video tag does not have any src attribute then check if it contains source tag.
					var $sources = $(this).find('source');
					for (var i=0; i< $sources.length; i++) {
						var $source = $($sources[i]);
						//	If the source tag is found, then check if it contains src.
						var sourceRef = $source.attr('src');
						if(typeof sourceRef == 'undefined') {
							//	If source tag does not have any src attribute then skip this video.
							//return true;
						} else {
							// If the source attribute is not empty then set the new src of this source tag.
							if(sourceRef.indexOf("http") == -1) {
								//$source.remove();
								filename = sourceRef.split('/').pop();
								videoObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
								var poster_name = $(this).attr('poster').split('/').pop();
								if (poster_name && Epub.config.pubdoc_ext_images.hasOwnProperty(poster_name)) {
									$(this).attr('poster', Epub.config.pubdoc_ext_images[poster_name]);
								}	
								if(videoObj.exists()){
									var video_type = $source.attr("type");
									videoUrl = getURL(videoObj); 
									if (AME.config.is_mac && video_type && video_type == "video/mp4") {
									
									} else {
										$(this).attr('preload', 'none');
										//$(this).removeAttr('autoplay');
										$(this).attr('src', videoUrl);
										$source.attr('src', videoUrl);
										$source.attr('preload', 'none');
									}
								} else {
									Epub.config.containErrors = true;
								}
							}
						}
					}
				} else {
					//	If the source attribute is not empty then set the new src of this video tag.
					filename = vidRef.split('/').pop();
					videoObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
					if(videoObj.exists()) {
						videoUrl = getURL(videoObj);
						$(this).attr('preload', 'none');
						// $(this).removeAttr('autoplay');
						$(this).attr('src', videoUrl);
					} else {
						Epub.config.containErrors = true;
					}
					
				}
			});
		}
	};
	
	/**
	 * This function loads all the images in chapter with the data-uris, and when all the images are loaded then
	 * it calls columns/flow init function
	 */
	var loadImages = function(idref){
		try{
			var self = this;
			
			var $img = $('#doc-viewer-azd').contents().find('#custom-style-igp img');
			var  j = 0;
			var totalImages = $img.length;
			
			var page_in_flowmode = false;
			if (config.forcedcolmode) {
				config.forcedcolmode = false;
				if (config.colmodeprevstate === "colmode") {
					BookPref.config.columns = true;
					BookPref.config.noOfCols = config.colmodeprev_colcount;
					BookPref.config.displayColumnRule = config.colmodeprev_displayColumnRule;
					BookPref.config.columnRule = config.colmodeprev_columnRule;
					BookPref.config.margin = config.colmodeprev_margin;
				} else {
					BookPref.config.columns = false;
				}
				config.colmodeprevstate = "";
				config.colmodeprev_colcount = 0;
				config.colmodeprev_displayColumnRule = "";
				config.colmodeprev_columnRule = "";
				config.colmodeprev_margin = "";
			}
			
			if (idref && Epub.config.spineProperties.hasOwnProperty(idref) && Epub.config.spineProperties[idref]["layout_scrolling"]) {
				page_in_flowmode = true;
				config.forcedcolmode = true;
				if (BookPref.config.columns) {
					config.colmodeprevstate = "colmode";
					config.colmodeprev_colcount = BookPref.config.noOfCols;
					config.colmodeprev_displayColumnRule = BookPref.config.displayColumnRule;
					config.colmodeprev_columnRule = BookPref.config.columnRule;
					config.colmodeprev_margin = BookPref.config.margin;
			
				} else {
					config.colmodeprevstate = "flomode";
				}
				BookPref.config.columns = false;
			}
			
			if(totalImages >= 0){
				Toolbar.enableUserPrefButton();
				if(config.azardi_presentation == 'fixed') {
					Toolbar.disableColumnsButton();
					Toolbar.disableColumnRuleButton();
					Toolbar.disableFontSizeButton();
					Toolbar.disableResetButton();
					Toolbar.disableMarginSpaceButton();
					Toolbar.disableLineSpaceButton();
					Toolbar.disableJustifyButton();
					// Toolbar.disableColorButton();
					Toolbar.disableHyphenateButton();
					//Toolbar.disableBookPrefButton();
					Toolbar.disableAudioOptionsButton();
						
					Flow.init();
				} else if(Chapter.config.azardi_margins != -1) {
					Toolbar.disableMarginSpaceButton();
				} else {
					Toolbar.enableColumnsButton();
					Toolbar.enableColumnRuleButton();
					Toolbar.enableFontSizeButton();
					Toolbar.enableResetButton();
					Toolbar.enableMarginSpaceButton();
					Toolbar.enableLineSpaceButton();
					Toolbar.enableJustifyButton();
					// Toolbar.enableColorButton();
					Toolbar.enableHyphenateButton();
					Toolbar.enableBookPrefButton();
					// Toolbar.enableAudioOptionsButton();
					if(BookPref.config.columns && (!page_in_flowmode)){
						Columns.init(BookPref.config.noOfCols);
						Columns.toggleColumnRule(BookPref.config.displayColumnRule);
						
					} else {
						Flow.init();
					}
				}
				self.setNavButtons();
				
	//			Justify.getParagraphsDefaultAlignment();
	//			BookPref.set();
				Notes.highlight();
				if(BookPref.config.columns && config.azardi_presentation != 'fixed' && (!page_in_flowmode)){
					if(Columns.config.goToNoteId != '' ){
						Columns.goToInternalLinkReference(Columns.config.goToNoteId);
						Columns.config.goToNoteId = '';
					}
				}
				else{
					if(Flow.config.goToNoteId != ''){
						Flow.goToInternalLinkReference(Flow.config.goToNoteId);
						Flow.config.goToNoteId = '';
					}
				}
				
				AME.hideLoading();
				if(Epub.config.containErrors){
					////DC - FLO UPDATE - 201211 - do not display the warnings
					//Epub.displayWarning();
					Epub.config.containErrors = false;
				}
			}
			
			$img.each(function(){
				if (Epub.config.isexternalimages) {
					var destObj = getFileObject(['ProfD', 'quizfx', 'buffer', config.epubId]);
					var imgname = $(this).attr('src').split('/').pop();
					if (Epub.config.pubdoc_ext_images.hasOwnProperty(imgname)) {
						$(this).attr('src', Epub.config.pubdoc_ext_images[imgname]);
						$(this).bind('dblclick', function () {
							var src = $(this).attr('src');
							if (src.indexOf("data:image") != -1) {
								AME.openImage(src);
								return;
							}
						});
						
					} else {
						azDump("Failed to set", imgname);
					}
				} else {
					$(this).bind('dblclick', function () {
						var foundInManifest = false;
						if (Epub.config.version == "e0") {
							var src = $(this).attr('src');
							var srclist = src.split("/");
							srclist.unshift(Epub.config.extractedFolder); 
							var imgPath = getAppendFilePath(Epub.config.extractLocation, srclist);
							try {
								var imgObject = getFileObjectFromPath(imgPath);
								var imgUrl = getURL(imgObject);
								AME.openImage(imgUrl);
							} catch(e) {
								
							}
							return;
						}
						
						for(var i = 0; i < Epub.config.bookData.length; i++){	
							var src = $(this).attr('src');
							if (src.indexOf("data:image") != -1) {
								AME.openImage(src);
								return;
							}
							if(unescape($(this).attr('src').split('/').pop()) == unescape(Epub.config.bookData[i]['href'].split('/').pop())){
								foundInManifest = true;
								break
							}
						}
						if (foundInManifest) {
							AME.openImage(Epub.config.bookData[i]['data']);
						}
					});
				}
			});
			
		}catch(e){
			
		}
		return;
		//Below code is not required anymore. but kept for debug
		$img.each(function(){
			var foundInManifest = false;
			for(var i = 0; i < Epub.config.bookData.length; i++){
				if(unescape($(this).attr('src').split('/').pop()) == unescape(Epub.config.bookData[i]['href'].split('/').pop())){
					foundInManifest = true;
					if(Epub.config.bookData[i]['available'] == 'true'){
						$(this).bind('dblclick', function () {
							AME.openImage($(this).attr('src'));
						});
						$(this).attr('src', Epub.config.bookData[i]['data']).load(function(){
							j = j + 1;
							if(j == totalImages){
								Toolbar.enableUserPrefButton();
								if(config.azardi_presentation == 'fixed') {
									Toolbar.disableColumnsButton();
									Toolbar.disableColumnRuleButton();
									Toolbar.disableFontSizeButton();
									Toolbar.disableResetButton();
									Toolbar.disableMarginSpaceButton();
									Toolbar.disableLineSpaceButton();
									Toolbar.disableJustifyButton();
									// Toolbar.disableColorButton();
									Toolbar.disableHyphenateButton();
									//Toolbar.disableBookPrefButton();
									Toolbar.disableAudioOptionsButton();
										
									Flow.init();
								} else if(Chapter.config.azardi_margins != -1) {
									Toolbar.disableMarginSpaceButton();
								} else {
									Toolbar.enableColumnsButton();
									Toolbar.enableColumnRuleButton();
									Toolbar.enableFontSizeButton();
									Toolbar.enableResetButton();
									Toolbar.enableMarginSpaceButton();
									Toolbar.enableLineSpaceButton();
									Toolbar.enableJustifyButton();
									// Toolbar.enableColorButton();
									Toolbar.enableHyphenateButton();
									Toolbar.enableBookPrefButton();
									// Toolbar.enableAudioOptionsButton();
									
									if(BookPref.config.columns){
										Columns.init(BookPref.config.noOfCols);
									} else {
										Flow.init();
									}
								}
								self.setNavButtons();
	//							Justify.getParagraphsDefaultAlignment();
	//							BookPref.set();
								Notes.highlight();
								if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
									if(Columns.config.goToNoteId != ''){
										Columns.goToInternalLinkReference(Columns.config.goToNoteId);
										Columns.config.goToNoteId = '';
									}
								}
								else{
									if(Flow.config.goToNoteId != ''){
										Flow.goToInternalLinkReference(Flow.config.goToNoteId);
										Flow.config.goToNoteId = '';
									}
								}
								
								AME.hideLoading();
								if(Epub.config.containErrors){
									Epub.displayWarning();
									Epub.config.containErrors = false;
								}
							}
						});
					} else {
						j = j + 1;
						if(j == totalImages){
							Toolbar.enableUserPrefButton();
							if(config.azardi_presentation == 'fixed') {
								Toolbar.disableColumnsButton();
								Toolbar.disableColumnRuleButton();
								Toolbar.disableFontSizeButton();
								Toolbar.disableResetButton();
								Toolbar.disableMarginSpaceButton();
								Toolbar.disableLineSpaceButton();
								Toolbar.disableJustifyButton();
								// Toolbar.disableColorButton();
								Toolbar.disableHyphenateButton();
								//Toolbar.disableBookPrefButton();
								Toolbar.disableAudioOptionsButton();
									
								Flow.init();
							} else if(Chapter.config.azardi_margins != -1) {
								Toolbar.disableMarginSpaceButton();
							} else {
								Toolbar.enableColumnsButton();
								Toolbar.enableColumnRuleButton();
								Toolbar.enableFontSizeButton();
								Toolbar.enableResetButton();
								Toolbar.enableMarginSpaceButton();
								Toolbar.enableLineSpaceButton();
								Toolbar.enableJustifyButton();
								// Toolbar.enableColorButton();
								Toolbar.enableHyphenateButton();
								Toolbar.enableBookPrefButton();
								// Toolbar.enableAudioOptionsButton();
				
								if(BookPref.config.columns){
									Columns.init(BookPref.config.noOfCols);
								} else {
									Flow.init();
								}
							}
							self.setNavButtons();
//							Justify.getParagraphsDefaultAlignment();
//							BookPref.set();
							Notes.highlight();
							if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
								if(Columns.config.goToNoteId != ''){
									Columns.goToInternalLinkReference(Columns.config.goToNoteId);
									Columns.config.goToNoteId = '';
								}
							}
							else{
								if(Flow.config.goToNoteId != ''){
									Flow.goToInternalLinkReference(Flow.config.goToNoteId);
									Flow.config.goToNoteId = '';
								}
							}
							
							AME.hideLoading();
							if(Epub.config.containErrors){
								Epub.displayWarning();
								Epub.config.containErrors = false;
							}
						}
					}
					break;
				}
			}
			if(!foundInManifest) {
				Epub.config.containErrors = true;
				j = j + 1;
				if(j == totalImages){
					Toolbar.enableUserPrefButton();
					if(config.azardi_presentation == 'fixed') {
						Toolbar.disableColumnsButton();
						Toolbar.disableColumnRuleButton();
						Toolbar.disableFontSizeButton();
						Toolbar.disableResetButton();
						Toolbar.disableMarginSpaceButton();
						Toolbar.disableLineSpaceButton();
						Toolbar.disableJustifyButton();
						// Toolbar.disableColorButton();
						Toolbar.disableHyphenateButton();
						//Toolbar.disableBookPrefButton();
						Toolbar.disableAudioOptionsButton();
							
						Flow.init();
					} else if(Chapter.config.azardi_margins != -1) {
						Toolbar.disableMarginSpaceButton();
					} else {
						Toolbar.enableColumnsButton();
						Toolbar.enableColumnRuleButton();
						Toolbar.enableFontSizeButton();
						Toolbar.enableResetButton();
						Toolbar.enableMarginSpaceButton();
						Toolbar.enableLineSpaceButton();
						Toolbar.enableJustifyButton();
						// Toolbar.enableColorButton();
						Toolbar.enableHyphenateButton();
						Toolbar.enableBookPrefButton();
						// Toolbar.enableAudioOptionsButton();
		
						if(BookPref.config.columns){
							Columns.init(BookPref.config.noOfCols);
						} else {
							Flow.init();
						}
					}
					self.setNavButtons();
	//							Justify.getParagraphsDefaultAlignment();
	//							BookPref.set();
					Notes.highlight();
					if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
						if(Columns.config.goToNoteId != ''){
							Columns.goToInternalLinkReference(Columns.config.goToNoteId);
							Columns.config.goToNoteId = '';
						}
					}
					else{
						if(Flow.config.goToNoteId != ''){
							Flow.goToInternalLinkReference(Flow.config.goToNoteId);
							Flow.config.goToNoteId = '';
						}
					}
					
					AME.hideLoading();
					if(Epub.config.containErrors){
						Epub.displayWarning();
						Epub.config.containErrors = false;
					}
				}
			}
		});
	};
	
	// var checkForMathMl = function () {
		// var self = this;
// 		
		// if ($('#doc-viewer-azd').contents().find('math').length > 0){
		  	// config.containsMathMl = true;
		// }
		// else{
			// config.containsMathMl = false;
		// }
	// }
// 	
	// var checkForSVG = function () {
		// var self = this;
// 		
		// if ($('#doc-viewer-azd').contents().find('svg').length > 0){
		  	// config.containsSVG = true;
		// }
		// else{
			// config.containsSVG = false;
		// }
	// }
	
	/**
	 * Set the states of navigation buttons
	 */
	var setNavButtons = function() {
		var self = this;
		
		if (!$("#doc-viewer-azd").is(":visible")) {
			//This event gets fired from resize event of 
			//Column and Flow. We dont want to take action
			//when the normal epub viewer is hidden
			return;
		}
		
		Toolbar.enablePreviousButton();
		Toolbar.enableNextButton();
		
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.initPageVariables();
		
		var currentChapter, spineListLength;
		if(config.standAlonePage) {
			currentChapter = 0;
			spineListLength = 1;
		} else {
			currentChapter = config.currentChapter;
			spineListLength = Epub.config.spineList.length;
		}
		
		if(currentChapter == 0) {
			//	If on the first page.
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				//	If in columns mode.
				var xOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.xOffset;
				if(xOffset == 0) {
					Toolbar.disablePreviousButton();
				}
			} else {
				//	If in flow mode.
				var yOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.yOffset;
				if(yOffset == 0){
					Toolbar.disablePreviousButton();
				}
			}
		} 
		
		if(currentChapter == spineListLength - 1) {
			//	If on the last page.
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				//	If in columns mode.
				var xOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.xOffset;
				var documentWidth = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.documentWidth;
				var currentWidth = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentWidth;
				if(documentWidth == (currentWidth + xOffset)){
					Toolbar.disableNextButton();
				}
			} else {
				//	If in flow mode.
				var yOffset = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.yOffset;
				var documentHeight = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.documentHeight;
				var currentHeight = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config.currentHeight;
				
				if(documentHeight <= (currentHeight + yOffset) || documentHeight - (currentHeight + yOffset) == 1){
					Toolbar.disableNextButton();
				}
			}
		}
	};
	
	/**
	 * Bind resize event
	 */
	var bindResize = function() {
		var self = this;
		
		$('#doc-viewer-azd').bind('resize', function() {
			config.orig_width = window.outerWidth;
			config.orig_height = window.outerHeight;
			BookPref.config.openHeight = window.outerHeight;
			BookPref.config.openWidth = window.outerWidth
		});
	};
	
	/**
	 * Unbind resize event
	 */
	var unbindResize = function() {
		var self = this;
		
		$('#doc-viewer-azd').unbind('resize');
	};
	
	/**
	 * Initilize the reader's audio player if chapter has smil data associated with it
	 * @params mediaOverlay media overlay property of the current chapter
	 */
	var initAudioPlayer = function(mediaOverlay) {
		if(mediaOverlay != '') {
			Toolbar.enableAudioOptionsButton();
		} else {
			Toolbar.disableAudioOptionsButton();
		}
		Toolbar.hideAudioControls();
		AME.paused_handle();
	};
	
	var showRealPageNums = function(updateindex) {
		try {
		if(!($("#doc-viewer-azd :visible"))) {
			return;
		}
		
		var pref_display_state = ["ppb-show-block", "ppb-show-inline", "ppb-show-margin"];
		
		if (updateindex) {
			Chapter.flo_pageref_display_index = Chapter.flo_pageref_display_index + 1;
			if (Chapter.flo_pageref_display_index > 3) Chapter.flo_pageref_display_index = 0
		}
		
		var newclassname = "";
		var rem_classname = "";
		if (Chapter.flo_pageref_display_index == 0) {
			newclassname = pref_display_state[Chapter.flo_pageref_display_index];
			rem_classname = pref_display_state[2];
		} else if (Chapter.flo_pageref_display_index == 3){
			rem_classname = pref_display_state[Chapter.flo_pageref_display_index-1];
			var newclassname = "";
		} else {
			newclassname = pref_display_state[Chapter.flo_pageref_display_index];
			rem_classname = pref_display_state[Chapter.flo_pageref_display_index-1];
		}
		
		//TODO - Replace this loop with a better method
		$('#doc-viewer-azd').contents().find('body *:not(script, style, noscript)').each(function() {
			var epubtype = $(this).attr("epub:type");
			if (epubtype == "pagebreak") {
				if (rem_classname) $(this).removeClass(rem_classname);
				if (newclassname) $(this).addClass(newclassname);
			}
		});
		
		BookPref.config.floShowPageNum = Chapter.flo_pageref_display_index;
		} catch(e) {
			//alert(e);
		}
	}
	
	var setEpubIframeEmpty = function() {
		$('#doc-viewer-azd').attr('src', "html/page-blank.html");
	}
	
	var setFocusToFLOFrame = function() {
		document.getElementById('doc-flo-viewer-azd').contentWindow.focus();
	}
	
	return {
		config: config,
		flo_pageref_display_index: flo_pageref_display_index,
		history: history,
		init: init,
		load: load,
		loadStandAlonePage: loadStandAlonePage,
		processInternalLinks: processInternalLinks,
		processParagraphs: processParagraphs,
		removeCssReference: removeCssReference,
		removeJsReference: removeJsReference,
		setCssReference: setCssReference,
		next: next,
		previous: previous,
		nextChapter: nextChapter,
		previousChapter: previousChapter,
		goToReadingDepth: goToReadingDepth,
		goToFromToc: goToFromToc,
		goToFromLandmark: goToFromLandmark,
		goToFromInternalLinks: goToFromInternalLinks,
		goBack: goBack,
		goToNote: goToNote,
		loadImages: loadImages,
		loadAudio: loadAudio,
		loadVideo: loadVideo,
		// checkForMathMl: checkForMathMl,
		// checkForSVG: checkForSVG,
		setNavButtons: setNavButtons,
		setJsReference: setJsReference,
		loadImageAsPage: loadImageAsPage,
		loadSwfAsPage: loadSwfAsPage,
		loadPdfAsPage: loadPdfAsPage,
		setChapterPresentation: setChapterPresentation,
		showToc: showToc,
		hideToc: hideToc,
		bindResize: bindResize,
		unbindResize: unbindResize,
		setStixFont: setStixFont,
		goToCoverPage: goToCoverPage,
		setCoverDim: setCoverDim,
		goToFromPageNav: goToFromPageNav,
		initAudioPlayer: initAudioPlayer,
		loadPreviewPage: loadPreviewPage,
		loadPageEx: loadPageEx,
		displayFlowItem:displayFlowItem,
		switchToFixedMode:switchToFixedMode,
		showRealPageNums:showRealPageNums,
		setEpubIframeEmpty:setEpubIframeEmpty,
		setFocusToFLOFrame:setFocusToFLOFrame,
	}
}();


function azDump(prefix){
	if(!indebugmode)return;
	dump("\n" + prefix + ">>>>  ");
	result = ""
	 // iterate through non-separator arguments
	  for (var i = 1; i < arguments.length; i++) {
			result += JSON.stringify(arguments[i]) + "\n";
	  }
	dump("\n" + result);
	dump("----------------------------------------------");
}

function azLogTime(prefix){
	if(!indebugmode)return;
	dump("\n" + prefix + "->  ");
	
	result = ""
	var dd = new Date();
    var mm = dd.getMinutes();
    var ss = dd.getSeconds();
    var ms = dd.getMilliseconds();
    result = mm + ":" + ss + ":" + ms;
	dump(result);
	dump("----------------------------------------------");
}

var indebugmode = false;

function encode_utf8(s) {
  return unescape( encodeURIComponent( s ) );
}

function utf8tobin(st) {
    let utf8 = decodeURIComponent(encodeURIComponent(st));
    let len = utf8.length;
    let bin = new Array(len);
    for (let i = 0; i < len; i++)
	bin[i] = utf8.charCodeAt(i);
    return bin;
}

function string2Bin(str) {
  var result = [];
  for (var i = 0; i < str.length; i++) {
    result.push(str.charCodeAt(i)&0xff);
  }
  return result;
}

function string2ArrayBuffer(str){
    var ba=new ArrayBuffer(str.length);
    var bytes = new Uint8Array(ba);
    for(var i=0;i<str.length; i++){
        bytes[i] = str.charCodeAt(i);
    }
    return ba;
}

function bin2String(array) {
  return String.fromCharCode.apply(String, array);
}

function getRandomInt(min, max) {
	return Math.floor(Math.random() * (max - min + 1)) + min;
}

function generateRandString(range)
{
	range = typeof range !== 'undefined' ? range : 5;
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for( var i=0; i < range; i++ )
        text += possible.charAt(Math.floor(Math.random() * possible.length));

    return text;
}

function base64tobin(input) {
    let alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
    let len = input.length;
    let bin = new Array();
    let j = 0;
    for (let i = 0; i < len - 3;) {
	let c1 = alphabet.indexOf(input[i++]);
	let c2 = alphabet.indexOf(input[i++]);
	let c3 = alphabet.indexOf(input[i++]);
	let c4 = alphabet.indexOf(input[i++]);

	bin[j++] = (c1 << 2) | (c2 >> 4);
	if (c3 == 64)
	    break;
	bin[j++] = ((c2 << 4) | (c3 >> 2)) & 0xff;
	if (c4 == 64)
	    break;
	bin[j++] = ((c3 << 6) | c4) & 0xff;
    }
    return bin;
}