/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

var obj_Persist;

// Output folder for extracting zip content.
var store2 = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("ProfD", Components.interfaces.nsIFile);
store2.append("quizfx");

// Output folder for extracting CDP ZIP content.
var downloadCdpPath = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsIFile);
downloadCdpPath.append("AZARDI");


/**
 Function to return file object from path array.
 @param {Array} filePathArray
 */

function getFileObject(filePathArray)
{
	var file = Components.classes["@mozilla.org/file/directory_service;1"]
				.getService(Components.interfaces.nsIProperties)
				.get(filePathArray[0], Components.interfaces.nsIFile);  
	for(i=1; i<=filePathArray.length; i++)
	{
		file.append(filePathArray[i]);
	}
	return file;
}

/**
 Function to return file object from path string. 
 @param {String} filePath
 */

function getFileObjectFromPath(filePath)
{
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);  
	file.initWithPath(filePath);
	return file;
}

/**
 Function to return concatenated file path.
 @param {String} oldFilePath
 @param {Array} filePathArray
 */

function getAppendFilePath(oldFilePath, filePathArray)
{
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	if (oldFilePath != '')
	{
		file.initWithPath(oldFilePath);
	}
	for(i=0; i<=filePathArray.length; i++)
	{
		file.append(filePathArray[i]);
	}
	return file.path;
}

/**
 Function to write content into file in UTF-8 format.
 @param {String} path
 @param {String} content
 */

function writeFileUTF(path, content)
{
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(path);
    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
	if( !file.exists() || !file.isFile() ) {   // if it doesn't exist, create
	   file.create(Components.interfaces.nsIFile.FILE_TYPE, 0777);
	}
    if (!file.isWritable())
	{
        file.permissions = 0384;
    }
    foStream.init(file, 0x04 | 0x08 | 0x20, 0666, 0);
    var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
    converter.init(foStream, "UTF-8", 0, 0);
    converter.writeString(content);
    converter.close();
}

/**
 Function to append content into file in UTF-8 format.
 @param {String} path
 @param {String} content
 */

function appendFileUTF(path, content)
{
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(path);
    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    if (!file.isWritable())
	{
        file.permissions = 0384;
    }
    foStream.init(file, 0x02 | 0x10, 0666, 0);
    var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
    converter.init(foStream, "UTF-8", 0, 0);
    converter.writeString(content);
    converter.close();
}


function createFile(filePathArray)
{
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
				.getService(Components.interfaces.nsIProperties)
				.get(filePathArray[0], Components.interfaces.nsIFile);  
	for(i=1; i<=filePathArray.length; i++)
	{
		file.append(filePathArray[i]);
	}
    if( !file.exists() || !file.isFile() ) {   // if it doesn't exist, create
	   file.create(Components.interfaces.nsIFile.FILE_TYPE, 0777);
	}
}

function createDirectory(dirPathArray){
	var file = Components.classes["@mozilla.org/file/directory_service;1"].
    	       getService(Components.interfaces.nsIProperties).
        	   get(dirPathArray[0], Components.interfaces.nsIFile);
	
	for(i=1; i<=dirPathArray.length; i++){
		file.append(dirPathArray[i]);
	}
	
	if( !file.exists() || !file.isDirectory() ) {
	   file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
	}
}

/**
 Function to return content of a local file.
 @param {String} savefile
 */

function readFileUTF(savefile) 
{
	var file = Components.classes["@mozilla.org/file/local;1"]
		.createInstance(Components.interfaces.nsILocalFile);
	file.initWithPath( savefile );
	if ( file.exists() == false ){
		throw new Error(file.leafName+' not found.');
	}
	
	var data = "";
	var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].
	              createInstance(Components.interfaces.nsIFileInputStream);
	var cstream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].
	              createInstance(Components.interfaces.nsIConverterInputStream);
	fstream.init(file, -1, 0, 0);
	cstream.init(fstream, "UTF-8", 0, 0);
	let (str = {})
	{
		var read = 0;
		do
		{ 
// read as much as we can and put it in str.value	  
			read = cstream.readString(0xffffffff, str);
			data += str.value;
	  	} while (read != 0);
	}
	cstream.close();
	return data;
}

// FIXME: make isCDP a fileType with values either AZD or ePub.
// FIXME: the function should return an error if not able to open a zip file.
function unzipFile(ZipFilePath1,isCDP)
{
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(ZipFilePath1);
	if(!file.exists()){
		throw new Error('Document not found.');
	}
    
    //  To Open Dialog and Get Input from User.
    //    var file = selectZipFile();    
    
    var fileName = file.leafName;
    var extensionDelimiter = fileName.lastIndexOf(".");
    var prefix = fileName.substr(0, extensionDelimiter);

// if file doesn't exist, create
	if(isCDP)
	{
		if (!downloadCdpPath.exists() || !downloadCdpPath.isDirectory())
		{
			downloadCdpPath.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
		}
		open_doc_path = downloadCdpPath;
		extractExtensionFiles(prefix, file, isCDP);
		downloadCdpPath = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsIFile);
		downloadCdpPath.append("AZARDI");
	}
	else
	{
		store2.append(prefix);
		if (!store2.exists() || !store2.isDirectory()) {
			store2.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
		} else {
			//delete the existing directory to handle caching issues
			try {
				deleteFolder(store2.path);
			} catch(e) {
				
			}
		}
		open_doc_path = store2
		extractExtensionFiles(prefix, file);
		store2 = Components.classes["@mozilla.org/file/directory_service;1"]
				.getService(Components.interfaces.nsIProperties)
				.get("ProfD", Components.interfaces.nsIFile);
		store2.append("quizfx");
	}
    if(isCDP)
    {
    	online2LocalCDPManifest(downloadCdpPath.path, ZipFilePath1);
    }
	return true;
}

/**
 To get zip file using file picker dialog.
 */

function selectZipFile()
{
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
  	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, "Select File", nsIFilePicker.modeOpen);
    fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterText);
    var rv = fp.show();
    if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace)
	{
        var file = fp.file;
// Get the path as string. Note that you usually won't
// need to work with the string paths.
        var path = fp.file.path;
// work with returned nsILocalFile...
        return file;
    }
}

/**
 Function to open file picker dialog. 
 @param {Object} title
 @param {Object} mode
 @param {Object} fileType
 */

function openSelectFile(title, mode, fileType)
{
	const nsIFilePicker = Components.interfaces.nsIFilePicker;
	var fp = Components.classes["@mozilla.org/filepicker;1"]
	               .createInstance(nsIFilePicker);
	if (mode == 'open'){
		mode = nsIFilePicker.modeOpen;
	}
	else if (mode == 'save'){
		mode = nsIFilePicker.modeSave;
	}
	else if (mode == 'directory'){
		mode = nsIFilePicker.modeGetFolder;
	}
	else if (mode == 'multiple'){
		mode = nsIFilePicker.modeOpenMultiple;
	}
	fp.init(window, title, mode);
	if(fileType != 'none'){
		var filter = fileType.toLowerCase();
		filter = '*.'+filter
		fp.appendFilter(fileType, filter)
	}
	var rv = fp.show();
	if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace){
		 var file = fp.file;
// Get the path as string. Note that you usually won't
// need to work with the string paths.
//		 var path = fp.file.path;
// work with returned nsILocalFile...
		 return file;
	}
	else{
		//	return false if user press cancel.
		return false;
	}
}

/**
 Function to extract contents of zip file.
 @param {Object} extensionID
 @param {Object} xpiFile
 */

function extractExtensionFiles(extensionID, xpiFile, isCDP)
{
    var zipReader = Components.classes["@mozilla.org/libjar/zip-reader;1"].createInstance(Components.interfaces.nsIZipReader);
    zipReader.open(xpiFile);
    zipReader.test(null);
    // create directories first
    var entries = zipReader.findEntries("*/");
    while (entries.hasMore())
	{
        var entryName = entries.getNext();
		
        var target = getItemFile(entryName, isCDP);
        if (!target.exists())
		{
            try
			{
                target.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
            } 
            catch (e)
			{
				var msg = getPropertiesString("azd_file_op_unZipFail");
                alert(msg + " file = " + target.path + ", exception = " + e + "\n");//azd_file_op_unZipFail
            }
        }
    }
    
    entries = zipReader.findEntries(null);
    while (entries.hasMore())
	{
        var entryName = entries.getNext();
        target = getItemFile(entryName, isCDP);
        if (target.exists())
		{
			continue;
		}
        
        try
		{
			target.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
        } 
        catch (e)
		{
			var msg = getPropertiesString("azd_file_op_unZipFileFail");
            alert(msg + " file = " + target.path + ", exception = " + e + "\n");
		}
        zipReader.extract(entryName, target);
    }
    zipReader.close();
}


function extractFileList(ZipFilePath1)
{
	var results = [];
	var xpiFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    xpiFile.initWithPath(ZipFilePath1);
	
    var zipReader = Components.classes["@mozilla.org/libjar/zip-reader;1"].createInstance(Components.interfaces.nsIZipReader);
    zipReader.open(xpiFile);
    zipReader.test(null);
    entries = zipReader.findEntries(null);
    while (entries.hasMore())
	{
        var entryName = entries.getNext();
        var entryName_leaf = entryName.split("/").pop();
		if (entryName_leaf.endsWith(".html") && (entryName_leaf !== "index.html")) {
			results.push(entryName);
		}
    }
    zipReader.close();
	return results;
}


/**
 Function to get the location of file (where to extract file.)
 @param {Object} filePath
 */

function getItemFile(filePath,isCDP)
{
	if(isCDP)
	{
		var itemLocation = downloadCdpPath.clone();
	}
	else
	{
		var itemLocation = store2.clone();
	}
    var parts = filePath.split("/");
    for (var i = 0; i < parts.length; ++i)
	{
		itemLocation.append(parts[i]);
	}
	return itemLocation;
}

/**
 Function to delete a folder.
 @param {String} folderPath
 */

function deleteFolder(folderPath)
{
    var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
    if (aFile instanceof Components.interfaces.nsILocalFile)
	{
        aFile.initWithPath(folderPath);
        try{
        	aFile.remove(true);
        } catch(e) {
        	
        }
    }
}

/**
 Function to return dataURI of a local file.
 @param {String} filePath
 */

function generateDataURI(filePath)
{
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(filePath);
    
    if(!file.exists()) {
    	throw new Error('FILE_NOT_FOUND');
    }
    
    try{
    	var contentType = Components.classes["@mozilla.org/mime;1"].getService(Components.interfaces.nsIMIMEService).getTypeFromFile(file);
    }
    catch(e){
    	var contentType = 'application/octet-stream';
    }
    var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    inputStream.init(file, 0x01, 0600, 0);
    var stream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
    stream.setInputStream(inputStream);
    var encoded = btoa(stream.readBytes(stream.available()));
	inputStream.close();
    return "data:" + contentType + ";base64," + encoded;
}

function loadBinaryFile(filePath)
{
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(filePath);
    
    if(!file.exists()) {
    	throw new Error('FILE_NOT_FOUND');
    }
    var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    inputStream.init(file, 0x01, 0600, 0);
    var stream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
    stream.setInputStream(inputStream);
	var bytes = stream.readBytes(stream.available());
	inputStream.close();
	stream.close();
	return bytes;
}

function downloadMe(downloadCDPId)
{	
	downloadFile(downloadCDPId,srcDownloadUrl);
}

// Function to download files.
function downloadFile(cdpid, httpLoc) 
{	
	httpLoc = httpLoc+cdpid+'.zip';
//	document.getElementById('aza-frame').contentWindow.setIframeSrc();
//	document.getElementById('aza-frame').contentWindow.showModalWindow(cdpid); 
//	document.getElementById('aza-frame').contentWindow.showProgressMessage('Starting download');	

//	get profile directory  
	var prof_path = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsIFile);  
	var app_os = osDetection();

	if(app_os=='Windows')
	{ 
		var outputFilename = prof_path.path+'\\AZARDI\\azd\\'+cdpid+'.zip';
		var zipOP = prof_path.path+'\\AZARDI\\';
	}
	else
	{
		var outputFilename = prof_path.path+'/AZARDI/azd/'+cdpid+'.zip'; 
		var zipOP = prof_path.path+'/AZARDI/';
	}
	try {			
		//new obj_URI object
		var obj_URI = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(httpLoc, null, null);
		//new file object
		var obj_TargetFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

		//set file with path
		obj_TargetFile.initWithPath(outputFilename);
		//if file doesn't exist, create
		if(!obj_TargetFile.exists()) {
			obj_TargetFile.create(0x00,0644);
		}

		//new persitence object
		obj_Persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);

		// with persist flags if desired
		const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
		const flags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
		obj_Persist.persistFlags = flags | nsIWBP.PERSIST_FLAGS_FROM_CACHE;

alert('Download Started...');

		obj_Persist.progressListener = {
			onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {
				var percentComplete = (aCurTotalProgress/aMaxTotalProgress)*100;				 
//				window.title = parseInt(percentComplete);

//				document.getElementById('aza-frame').contentWindow.showProgressMessage('Downloading Book');			
//				document.getElementById('aza-frame').contentWindow.showCancelDownloadButton();
			},

			onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
				if(obj_Persist.currentState == obj_Persist.PERSIST_STATE_FINISHED)
				{
					var chan =	aRequest.QueryInterface(Components.interfaces.nsIHttpChannel);
					if (!chan.requestSucceeded)
					{
//						document.getElementById('aza-frame').contentWindow.showProgressMessage('Book not available for Downloading');
//						setTimeout(function(){document.getElementById('aza-frame').contentWindow.closeModalWindow(cdpid)}, 2000);
						delete_File(obj_TargetFile.path);
						return;
					}
					if(aStatus == 0){		//		aStatus gives '0' if download is finished.
// Request for Unzip on finish download.
alert('Download Finish...');
unzipFile(outputFilename,true);
//						document.getElementById('aza-frame').contentWindow.showProgressMessage('Download Complete');
//						writeDownloadedFileInfo(cdpid);		//	edited 21 jan
//						setTimeout(function(){ setCatalogDownloadStatus(cdpid); document.getElementById('aza-frame').contentWindow.hideCancelDownloadButton();},'2000');
					}
					else{		//		if download is cancelled.
//						document.getElementById('aza-frame').contentWindow.showProgressMessage('Cancelling Download');
//						cancelDownloadCleanUp(cdpid);
//						document.getElementById('aza-frame').contentWindow.closeModalWindow(cdpid);
//						document.getElementById('aza-frame').contentWindow.hideCancelDownloadButton();
//						setTimeout(function(){document.getElementById('aza-frame').contentWindow.closeModalWindow(cdpid)}, 2000);
//						cancelDownloadCleanUp(cdpid);
					}
				} 
			}
		  }

		//save file to target
		obj_Persist.saveURI(obj_URI,null,null,null,null,obj_TargetFile, null);

//		obj_Persist.cancelSave();
	} catch (e) {
		alert(e);
	}
}

function cancelDownloadFile(){
	obj_Persist.cancelSave();
}

function cancelDownloadCleanUp(cdpid){
	var file = Components.classes["@mozilla.org/file/directory_service;1"].
	    getService(Components.interfaces.nsIProperties).
	    get("Home", Components.interfaces.nsIFile);
	file.append('AZARDI');
	file.append('azd');
	file.append(cdpid+'.azd');
	if( file.exists()) {
		try{
			delete_Folder(file.path);
		}
		catch(e){}
	}
}

function delete_Folder(folderPath){
	var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
	if(aFile instanceof Components.interfaces.nsILocalFile){
		aFile.initWithPath(folderPath);
		aFile.remove(true);
	}
}

function getURL(file){
	var ios = Components.classes["@mozilla.org/network/io-service;1"]
					  	.getService(Components.interfaces.nsIIOService);
	var URL = ios.newFileURI(file);
	return URL.spec
}

function getFileObjFromURL(URL) {
	// URL is a nsIURI; see nsIIOService::newURI for getting a string into a nsIURI.
	var file = URL.QueryInterface(Components.interfaces.nsIFileURL).file;
	// file is now a nsIFile.
	return file;
}


function chromeToPath (aPath) {

   if (!aPath || !(/^chrome:/.test(aPath)))
      return; //not a chrome url
   var rv;

      var ios = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces["nsIIOService"]);
        var uri = ios.newURI(aPath, "UTF-8", null);
        var cr = Components.classes['@mozilla.org/chrome/chrome-registry;1'].getService(Components.interfaces["nsIChromeRegistry"]);
        rv = cr.convertChromeURL(uri).spec;

        if (/^file:/.test(rv))
          rv = this.urlToPath(rv);
        else
          rv = this.urlToPath("file://"+rv);

      return rv;
}

function urlToPath (aPath) {

    if (!aPath || !/^file:/.test(aPath))
      return ;
    var rv;
   var ph = Components.classes["@mozilla.org/network/protocol;1?name=file"]
        .createInstance(Components.interfaces.nsIFileProtocolHandler);
    rv = ph.getFileFromURLSpec(aPath).path;
    return rv;
}

function getAbsPathFromChromeURL(chromeURL) {
	var path = chromeToPath(chromeURL);
	var pathObject = getFileObjectFromPath(path);
	var abspath = getURL(pathObject);
	return abspath;
}

var guidGenerator = function() {
	var S4 = function() {
		return Math.floor(Math.random() * 0x10000 /* 65536 */
		).toString(16);
	};
	return (S4() + S4() + S4() + S4() + S4() + S4() + S4() + S4()
	);
}

var getPropertiesString=function(name) {
	try {
		var strbundle = document.getElementById("azardiproperties");
		var val=strbundle.getString(name);
		return val;
	} catch(e) {
		return "";
	}
} 
var getFormattedPropertiesString=function(name, props) {
	try {
		var strbundle = document.getElementById("azardiproperties");
		var val=strbundle.getFormattedString(name, props);
		return val;
	} catch(e) {
		return "";
	}
} 

function saveBindata(fn, data) {
	try {
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(fn);
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		if( !file.exists() || !file.isFile() ) {   // if it doesn't exist, create
		   file.create(Components.interfaces.nsIFile.FILE_TYPE, 0777);
		}
		if (!file.isWritable())
		{
			file.permissions = 0384;
		}

		var fileOutStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		fileOutStream.init(file, 0x04 | 0x08 | 0x20, 0755, 0);
		
		var binaryOutStream = Components.classes['@mozilla.org/binaryoutputstream;1'].createInstance(Components.interfaces.nsIBinaryOutputStream);
		binaryOutStream.setOutputStream(fileOutStream);
		binaryOutStream.writeByteArray(data, data.length);

		binaryOutStream.close();
		fileOutStream.close();

	} catch(e) {
		
	}

}