/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Deepak Chandran
 */

/**
 * @class Contains all the functions necessary for loading particular chapter.
 */
var AZ_DIM = function () {
	
	var IS_LANDSCAPE = 1;
	var IS_PORTRAIT = 2;
	var IS_SQUARE = 3;
	
	var spine_items = [];
	var spine_properties = {};
	var aco_dim = [];
	var zbox_dim = [];
	var scale = 1;
	var lmargin = 0;
	var offsetX, offsetY;
	var zxWidth, zxHeight;
	var orig_left_scroll_pos = 0;
	var orig_right_scroll_pos = 0;
	
	var pbox_orientation = ""; //Presentation box/Device orientation
	
	var setWindowDim = function() {
		aco_dim = getPresentationDim();
	}
	
	var getPresentationDim = function() {
		try {
			var wheight =  parent.window.innerHeight;
			if (!parent.AME.isfullscreen) {
				//adjust the top toolbar height
				wheight = wheight - 30;
			}
			var wwidth =  parent.window.innerWidth-10;
			$(".aco-presentation").css("height", wheight);
			$(".aco-presentation").css("width", wwidth);
			var height = $(".aco-presentation").css("height");
			var width = $(".aco-presentation").css("width");
			if(!width){
				width=wwidth-10;
				$(".aco-presentation").css("width", width);
			}
			height = parseInt(height);
			width = parseInt(width);
			if (width > height) pbox_orientation = IS_LANDSCAPE;
			else if (height > width) pbox_orientation = IS_PORTRAIT;
			else pbox_orientation = IS_SQUARE;
			return [width, height];
		} catch(e) {
			azDump("****************getPresentationDim Exception", e);
		}
	}
	
	var calculateZoombox = function(lspread, rspread) {
		try {
			var lspineitem = spine_properties[lspread];
			if (!lspineitem){
				return;
			}
			var rspineitem = spine_properties[rspread];
			if (!rspineitem) rspineitem = lspineitem;
			var lw =  0;
			var rw =  0;
			if (lspineitem.hasOwnProperty("width")) lw = parseInt(lspineitem["width"]);
			if (rspineitem.hasOwnProperty("width")) rw = parseInt(rspineitem["width"]);
			var zbox_w = 0;
			
			if ((lspineitem["spread"] == "none") && (rspineitem["spread"] == "none")) {
				//Special - TOP BOTTOM
				if (lw > zbox_w)zbox_w = lw;
				if (rw > zbox_w)zbox_w = rw;
			} else {
				 //LEFT RIGHT
				 zbox_w = lw + rw;
			}
			
			
			var lh, rh = 0;
			if (lspineitem.hasOwnProperty("height")) lh = parseInt(lspineitem["height"]);
			if (rspineitem.hasOwnProperty("height")) rh = parseInt(rspineitem["height"]); 
			
			var zbox_h = 0;
			if (lh > zbox_h)zbox_h = lh;
			if (rh > zbox_h)zbox_h = rh;
			if ((lspineitem["spread"] == "none") && (rspineitem["spread"] == "none")) {
				zbox_h = lh + rh;
				
			}
			
			zbox_dim = [zbox_w, zbox_h];
			//calculate the scale transformation
			if ((zbox_dim[0] / zbox_dim[1]) >= ((aco_dim[0] / aco_dim[1]))) {
				//Portait mode
				scale = aco_dim[0]/zbox_dim[0];
				offsetY = Math.abs((aco_dim[1] - (zbox_dim[1]*scale))/2);
				offsetX = 0;
				zxWidth = zbox_dim[0];
				zxHeight = zbox_dim[1];	
			} else {
				//Landscape mode mode
				scale = aco_dim[1]/zbox_dim[1];
				offsetX = Math.abs((aco_dim[0] - (zbox_dim[0]*scale))/2);
				offsetY = 0;
				zxWidth = zbox_dim[0] ;
				zxHeight = zbox_dim[1];	
			} 
			//calculate margins
			lmargin = (aco_dim[0] - (zbox_dim[0]*scale))/2;
			
			return zbox_dim;
		} catch(e) {
			azDump("****************calculateZoombox Exception", e);
		}
	};
	
	var calculateZoomboxCenter = function(cspread) {
		var cspineitem = spine_properties[cspread];
		if (!cspineitem)return;
		var lw =  0;
		if (cspineitem.hasOwnProperty("width")) lw = parseInt(cspineitem["width"]);
		if (lw <= 0) lw = aco_dim[0];
		var zbox_w = lw;
		
		var lh, rh = 0;
		if (cspineitem.hasOwnProperty("height")) lh = parseInt(cspineitem["height"]);
		
		var zbox_h = 0;
		if (lh > zbox_h)zbox_h = lh;
		if (rh > zbox_h)zbox_h = rh;
		
		if (zbox_h <= 0) zbox_h = aco_dim[1];
		
		zbox_dim = [zbox_w, zbox_h];
		//calculate the scale transformation
		if ((zbox_dim[0] / zbox_dim[1]) >= ((aco_dim[0] / aco_dim[1]))) {
			scale = aco_dim[0]/zbox_dim[0];
			offsetY = Math.abs((aco_dim[1] - (zbox_dim[1]*scale))/2);
			offsetX = 0;
			zxWidth = zbox_dim[0];
			zxHeight = zbox_dim[1];	
		} else {
			scale = aco_dim[1]/zbox_dim[1];
			offsetX = Math.abs((aco_dim[0] - (zbox_dim[0]*scale))/2);
			offsetY = 0;
			zxWidth = zbox_dim[0] ;
			zxHeight = zbox_dim[1];	
		} 
		
		//calculate margins
		lmargin = (aco_dim[0] - (zbox_dim[0]*scale))/2;
		return zbox_dim;
	};
	
	var init = function(spineProperties, sp_items) {
		spine_items = sp_items;
		spine_properties = spineProperties;
		
		//If bookpref available, set the open width and height
		var init_wh_set = false;
		if ((parseInt(parent.BookPref.config.openWidth)) && (parseInt(parent.BookPref.config.openHeight))) {
			parent.window.resizeTo(parseInt(parent.BookPref.config.openWidth), parseInt(parent.BookPref.config.openHeight));
			init_wh_set = true;
		} else {
		
		}
		
		//get presentation aco dimensions
		aco_dim = getPresentationDim();
		
		var first_spine_item_ref = spine_items[0];
		var first_spine_item = spine_properties[first_spine_item_ref];
		
		if (first_spine_item["page-spread"] == "center") {
			zbox_dim = calculateZoomboxCenter(first_spine_item_ref);
		} else {
			zbox_dim = calculateZoombox(first_spine_item_ref,spine_items[1]);
		}
		
		//This happens for the first time. If we dont have user pref height and width
		//we use the zoombox dim and screen dim to get the initial loading dim
		if (!init_wh_set) {
			var init_width = zxWidth-10;
			if (init_width > parent.screen.availWidth) {
				init_width = parent.screen.availWidth-10;
			}
			var init_height = zxHeight-20;
			if (zxHeight > parent.screen.availHeight) {
				init_height = parent.screen.availHeight-20;
			}
			parent.window.resizeTo(init_width, init_height);
			//Update the var in parent to be saved to the book prefs
			parent.Chapter.config.orig_width = init_width;
			parent.Chapter.config.orig_height = init_height;
		}
	};
	
	var setDimensions = function() {
		$(".zoombox-azd").css("width", zxWidth + "px");
		$(".zoombox-azd").css("height", zxHeight + "px");
		$(".zoombox-azd").css("transform", "scale(" + scale +")");
		$(".zoombox-azd").css("transform-origin", "0% 0%");
		$(".zoombox-azd").css("margin-left", offsetX + "px");
		$(".zoombox-azd").css("margin-top", offsetY + "px");
	};
	
	var setCenterFramePos = function (pw, ph) {
		$('#page-center-az').css("height", zxHeight + "px");
		$('#page-center-az').css("width",  zxWidth + "px");
		$(".zoombox-azd").css("transform", "scale(" + scale +")");
		$(".zoombox-azd").css("transform-origin", "0% 0%");
		$(".zoombox-azd").css("margin-left", offsetX + "px");
		$(".zoombox-azd").css("margin-top", offsetY + "px");
	}
	
	var getScaleTransform = function(){
		return scale;
	}
	
	return {
		init: init,
		setDimensions: setDimensions,
		setCenterFramePos: setCenterFramePos,
		getScaleTransform: getScaleTransform,
		calculateZoombox: calculateZoombox,
		calculateZoomboxCenter: calculateZoomboxCenter,
		setWindowDim: setWindowDim,
	};
}();

function azDump(prefix){
	return;
	dump("\n" + prefix + ">>>>  ");
	result = ""
	 // iterate through non-separator arguments
	  for (var i = 1; i < arguments.length; i++) {
			result += JSON.stringify(arguments[i]) + "\t";
	  }
	dump(result);
	dump("\n");
	dump("----------------------------------------------");
}

var FLO = function(){
	var spineProperties = {};
	var spineList = [];
	var nav_list = [];
	var current_spine_pos = 0;
	var curr_nav_item = "";
	var doc_ppd = "ltr";
	var lctr,rctr,cctr = 0;
	var spreadtype = "";
	var is_fixed_layout = false;
	var back_spine_pos = "";
	var in_reflow_mode = false;
	var is_lspread_reflowable = false;
	var is_rspread_reflowable = false;
	var flo_state = 0;
	var flo_pageref_display_index = 3;
	var flo_current_gutter_state = 0;
	var flo_smil_lookup = {
		lspread: "",
		rspread: "",
		cspread: "",
		current_playing: ""
	};
	var playBackRate = 1;
	
	var loadinggif = getAbsPathFromChromeURL("chrome://azardi2/content/icons/loading.gif");
	/*var loadingtag = '<div id="loading_overlay" class="loading_overlay">\
			<img id="loading_image" alt="" class="loading_image" src="'+loadinggif+'" border="0" />\
		</div>';*/
		
	var loadingtag = '<div id="loading_overlay" class="loading_overlay"></div>';
	
	  
	var init = function () {
		self = this;
		self.spineProperties = parent.Epub.config.spineProperties;
		self.spineList = parent.Epub.config.spineList;
		self.nav_list = parent.Epub.nav_list;
		doc_ppd = parent.Epub.config.page_progression_dir
		
		AZ_DIM.init(self.spineProperties, self.spineList);
		is_fixed_layout = parent.Epub.is_fixed_layout;
		
		AZ_DIM.setDimensions();
		
		if (!parent.Chapter.config.openAtFirst) {
			//TODO - Implement load from Previously closed location
			try {
				var cpos = parent.Chapter.config.currentChapter;
				if (cpos>= 0)current_spine_pos = cpos;
			} catch(e){
			
			}
		}	
		
		if (parent.BookPref.config.floGutter) {
			$('.zoombox-azd').removeClass('flo0');
			$('.zoombox-azd').addClass(parent.BookPref.config.floGutter);
			switch(parent.BookPref.config.floGutter){
				case 'flo1':
					flo_current_gutter_state = 1;
					break
				case 'flo2':
					flo_current_gutter_state = 2;
					FLO.setFLOState(2);
					break
				case 'flo3':
					flo_current_gutter_state = 3;
					break
				case 'flo4':
					flo_current_gutter_state = 4;
					break
				case 'flo0':
					flo_current_gutter_state = 0;
					break
			}
			FLO.handleFLOGutter(flo_current_gutter_state);
		}

		if (parent.BookPref.config.floBackground) {
			$('body').removeClass('bg2');
			$('body').removeClass('bg0');
			$('body').addClass(parent.BookPref.config.floBackground);
		}
		
		var floShowPageNum = parseInt(parent.BookPref.config.floShowPageNum);
		
		if (floShowPageNum >= 0) {
			flo_pageref_display_index = floShowPageNum;
			setFLOPrintPageDisplay();
		}	
		
		setNavButtons(current_spine_pos);
		
		displayFixedLayoutDoc();
	};

	var displayPageInFrame = function(frameid, pageref) {
		if (pageref == "page-blank.html" ) {
			$("#" + frameid).attr("src", "");
		} else {
			$("#" + frameid).attr("src", pageref + "?" + generateRandString());
		}
	};
	
	var setPageLeftPageDim = function (calledby) {
		try {
			if (!jQuery.isEmptyObject(curr_nav_item)){
				var spine_pos = curr_nav_item["left"];
				//Turn SMIL data into AIE playsound data structure
				if (calledby == "domload") {
					var mediaOverlay = parent.Epub.getChapterOverlay(spine_pos);
					if (mediaOverlay) {
						var display_item = self.spineProperties[spine_pos];
						var srcpage = display_item["href"].split("/").pop();
						
						SmilFLO.init(mediaOverlay, 'page-left-az', srcpage);
						loadAudio('page-left-az');
						loadVideo('page-left-az');
						flo_smil_lookup.lspread = "mediaOverlay";
					} else {
						loadAudio('page-left-az');
						loadVideo('page-left-az');
						flo_smil_lookup.lspread = "";
					}
					initAIE('page-left-az');
				}
				setBodyWidthHeight(spine_pos, 'page-left-az');	
				
				/*do the iscroll setting here. we get a delay when we have a difference */
				var display_item = FLO.spineProperties[spine_pos];
				if (display_item && (display_item["layout"] == "reflowable")) {
					document.getElementById('page-left-az').contentWindow.wrappedJSObject.initIScroll('page-left-az', orig_left_scroll_pos);
				}
			} 
		}catch(e) {
			azDump("****************setPageLeftPageDim Exception", e);
		}
	};
	
	var setPageRightPageDim = function (calledby) {
		try {
			if (!jQuery.isEmptyObject(curr_nav_item)) {
				var spine_pos = curr_nav_item["right"];
				//Turn SMIL data into AIE playsound data structure
				var mediaOverlay = parent.Epub.getChapterOverlay(spine_pos);
				if (calledby == "domload") {
					if (mediaOverlay) {
						var display_item = self.spineProperties[spine_pos];
						var srcpage = display_item["href"].split("/").pop();
						
						SmilFLO.init(mediaOverlay, 'page-right-az', srcpage);
						loadAudio('page-right-az');
						loadVideo('page-right-az');
						flo_smil_lookup.rspread = mediaOverlay;
					} else {
						loadAudio('page-right-az');
						loadVideo('page-right-az');
						flo_smil_lookup.rspread = "";
					}
					initAIE('page-right-az');
				}
				setBodyWidthHeight(spine_pos, 'page-right-az');
				
			}
		}catch(e){
			azDump("****************setPageRightPageDim Exception", e);
		}
	};
	
	var setPageCenterPageDim = function (calledby) {
		try {
			if (!jQuery.isEmptyObject(curr_nav_item)) {
				var spine_pos = curr_nav_item["center"];
				setBodyWidthHeight(spine_pos, 'page-center-az');
				var mediaOverlay = parent.Epub.getChapterOverlay(spine_pos);
				//Turn SMIL data into AIE playsound data structure
				if (calledby == "domload") {
					if (mediaOverlay) {
						var display_item = self.spineProperties[spine_pos];
						var srcpage = display_item["href"].split("/").pop();
						
						SmilFLO.init(mediaOverlay, 'page-center-az', srcpage);
						loadAudio('page-center-az');
						loadVideo('page-center-az');
						flo_smil_lookup.cspread = mediaOverlay;
					} else {
						loadAudio('page-center-az');
						loadVideo('page-center-az');
						flo_smil_lookup.cspread = mediaOverlay;
					}
					initAIE('page-center-az');		
				}
				
				if (spine_pos) {
					var display_item = FLO.spineProperties[spine_pos];
					if (display_item && (display_item["layout"] == "reflowable")) {
						document.getElementById('page-center-az').contentWindow.wrappedJSObject.initIScroll('page-center-az');
					}
				}
						
			}
		} catch (e) {
			azDump("****************setPageCenterPageDim Exception", e);
		}
	};
	
	/**
	 * Change the src of audio tag in this chapter to point to audios in buffer location
	 */
	var loadAudio = function(frameid){
		try{
		var self = this;
		
		var audioObj, audioUrl, filename;
		
		var $audio = $('#' + frameid).contents().find('#custom-style-igp audio');
		//	If no audio is found on the page.
		if($audio.length == 0) {
			return false;
		} else {
			$audio.each(function() {
				var audRef = $(this).attr('src');
				
				if(typeof audRef == 'undefined') {
					//	If audio tag does not have any src attribute then check if it contains source tag.
					var $source = $(this).find('source');
					if($source.length == 0) {
						//	If no source tag then skip this audio.
						return true;
					} else {
						//	If the source tag is found, then check if it contains src.
						var sourceRef = $source.attr('src');
						
						if(typeof sourceRef == 'undefined') {
							//	If source tag does not have any src attribute then skip this audio.
							return true;
						} else {
							// If the source attribute is not empty then set the new src of this source tag.
							$source.remove();
							filename = sourceRef.split('/').pop();
							audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', parent.Epub.config.epubId, filename]);
							if(audioObj.exists()){
								audioUrl = getURL(audioObj);
								// $(this).attr('preload', 'none');
								$(this).attr('src', audioUrl);
							} else {
								parent.Epub.config.containErrors = true;
							}
						}
					}
				} else {
					//	If the source attribute is not empty then set the new src of this audio tag.
					filename = audRef.split('/').pop();
					audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', parent.Epub.config.epubId, filename]);
					if(audioObj.exists()) {
						audioUrl = getURL(audioObj);
						// $(this).attr('preload', 'none');
						$(this).attr('src', audioUrl);
					} else {
						parent.Epub.config.containErrors = true;
					}
				}
			});
		}
		}catch(e){
			
		}
	}
	
		/**
	 * Change the src of audio tag in this chapter to point to audios in buffer location
	 */
	var loadVideo = function(frameid){
		try{
			var videoObj, videoUrl, filename;
			var $video = $('#' + frameid).contents().find('#custom-style-igp video');
			//	If no video is found on the page.
			if($video.length == 0) {
				return false;
			} else {
				$video.each(function() {
					var vidRef = $(this).attr('src');
					if(typeof vidRef == 'undefined') {
						//	If video tag does not have any src attribute then check if it contains source tag.
						var $sources = $(this).find('source');
						if($sources.length == 0) {
							//	If no source tag then skip this video.
							return true;
						} else {
							for (var x=0; x < $sources.length; x++) {
								var source = $sources[x];
								//	If the source tag is found, then check if it contains src.
								var sourceRef = $(source).attr('src');
								if(typeof sourceRef == 'undefined') {
									//	If source tag does not have any src attribute then skip this video.
									return true;
								} else {
									// If the source attribute is not empty then set the new src of this source tag.
									$(source).remove();
									filename = sourceRef.split('/').pop();
									videoObj = getFileObject(['ProfD', 'quizfx', 'buffer', parent.Epub.config.epubId, filename]);
									videoUrl = getURL(videoObj);
									var poster_name = $(this).attr('poster').split('/').pop();
									if (poster_name && parent.Epub.config.pubdoc_ext_images.hasOwnProperty(poster_name)) {
										$(this).attr('poster',parent. Epub.config.pubdoc_ext_images[poster_name]);
									}
									if(videoObj.exists()){
										$(this).attr('preload', 'none');
										// $(this).removeAttr('autoplay');
										$(this).attr('src', videoUrl);
									} else {
										
									}
									
								}
							}
						}
					} else {
						//	If the source attribute is not empty then set the new src of this video tag.
						filename = vidRef.split('/').pop();
						videoObj = getFileObject(['ProfD', 'quizfx', 'buffer', parent.Epub.config.epubId, filename]);
						if(videoObj.exists()) {
							videoUrl = getURL(videoObj);
							$(this).attr('preload', 'none');
							// $(this).removeAttr('autoplay');
							$(this).attr('src', videoUrl);
						} else {
							
						}
						
					}
				});
			}
		
		}catch(e){
			
		}
	}
	
	var displayCenterSpread = function(spine_pos){
		try {
			$("#" + "page-left-az").hide();
			stopAudio("page-left-az");
			$("#" + "page-right-az").hide();
			stopAudio("page-right-az");
			
			parent.Chapter.setEpubIframeEmpty();
			
			var mediaOverlay = parent.Epub.getChapterOverlay(spine_pos);
			var smilData = parent.Epub.getChapterSmilData(mediaOverlay);
				
			var display_item = self.spineProperties[spine_pos];
			
			displayPageInFrame("page-center-az", display_item["href"]);
			cctr = spine_pos;
			spreadtype = "CENTER";
			AZ_DIM.calculateZoomboxCenter(spine_pos);
			AZ_DIM.setDimensions();
			//Set this for saving to the book pref
			var currChapter = getNavListPos(cctr);
			if (currChapter) {
				//parent.Chapter.config.currentChapter = currChapter;
			}
			
			if (FLO.in_reflow_mode) {
				parent.Chapter.switchToFixedMode();
				FLO.in_reflow_mode = false;
			}
			
			
		}catch(e){
			azDump("****************displayCenterSpread Exception", e);
		}
	};
	
	var setBodyWidthHeight = function (spine_pos, frameid) {
		try {
			var display_item = FLO.spineProperties[spine_pos];
			
			if(display_item) {	
				var pw =  0;
				var ph =  0;
				var scale = AZ_DIM.getScaleTransform();
				if (display_item.hasOwnProperty("width")) pw = parseInt(display_item["width"]);
				if (display_item.hasOwnProperty("height")) ph = parseInt(display_item["height"]);
				if ((pw <= 0) || (ph <= 0)) {
					ph = parseInt($(".aco-presentation").css("height"));
					pw = parseInt($(".aco-presentation").css("width"));
					
				}
				
				var isvertical = false;
				//Special Vertical handling case
				if (display_item["spread"] == "none") {
					pw = "100%";
					if ((lctr!="") && (frameid == "page-right-az")) {
						var ldisplay_item = FLO.spineProperties[lctr];
						var lph = parseInt(ldisplay_item["height"]);
						$('#' + frameid).css("top", (lph) + "px");
						$('#' + frameid).css("left", (0) + "px");
						isvertical = true;
					}
				} else {
					if (flo_state == 2) {
						if (pw != "100%") pw = pw - 6;
						ph = ph - 6;
					}
					pw = pw + "px";
					$('#' + frameid).css("top", "0px");
				}
				
				
				//Calculate and set the left position of the right spread
				//This is to stack them next to each other without overlapping
				if ((frameid == "page-right-az") && (!isvertical)) {
					if (lctr!="") {
						var ldisplay_item = FLO.spineProperties[lctr];
						var lpw = parseInt(ldisplay_item["width"]);
						if (flo_state) lpw = lpw - 3;
						$('#' + frameid).css("left", (lpw) + "px");
					} else {
						var rdisplay_item = FLO.spineProperties[rctr];
						var rpw = parseInt(rdisplay_item["width"]);
						$('#' + frameid).css("left", (rpw) + "px");
					}
				} 
				
				if (ph) $('#' + frameid).css("height", (ph) + "px");
				if (ph) $('#' + frameid).contents().find('body').css("height", (ph) + "px");
				if (pw) $('#' + frameid).contents().find('body').css("width",  pw);
				if (pw) $('#' + frameid).css("width",  pw);
				
				if (frameid != "page-center-az") {					
					$("#" + frameid).show();
				} else {
					AZ_DIM.setCenterFramePos(pw, ph);
					$("#" + frameid).show();
				}
			}
		} catch(e){
			azDump("****************setBodyWidthHeight Exception", e);
		}
	};
	
	
	
	var displayDocument = function() {
		if (is_fixed_layout) {
			displayFixedLayoutDoc();
		} else {
			displayFlowableDoc();
		}
	};
	
	var displayFixedLayoutDoc = function() {
		var self = this;
		var navitem = FLO.nav_list[current_spine_pos];
		curr_nav_item = navitem;
		
		if (navitem["navtype"] == "flow"){
			displayCenterSpread(navitem["center"]);
			$(".zoombox-azd").css("display", "block");
		} else if (navitem["navtype"] == "spread"){
			displaySpread(navitem);
			$(".zoombox-azd").css("display", "block");
		} else if (navitem["navtype"] == "reflow"){
			$(".zoombox-azd").css("display", "none");
			displayFlowItem(navitem["center"]);
		} 
	};
	
	var displayFlowableDoc = function() {
		var self = this;
		
		var navitem = FLO.nav_list[current_spine_pos];
		curr_nav_item = navitem;
		if (navitem["navtype"] == "flow"){
			displayCenterSpread(navitem["center"]);
			$(".zoombox-azd").css("display", "block");
			
		} else if (navitem["navtype"] == "spread"){
			displaySpread(navitem);
			$(".zoombox-azd").css("display", "block");
		} else if (navitem["navtype"] == "reflow"){
			$(".zoombox-azd").css("display", "none");
			displayFlowItem(navitem["center"]);
		} 
	};
	
	var displayFlowItem = function(spine_pos){
		try {
			
			stopAudio("page-center-az");
			stopAudio("page-left-az");
			stopAudio("page-right-az");
			$("#" + "page-center-az").css("display", "none");
			var display_item = FLO.spineProperties[spine_pos];
			var currChapter = getNavListPos(spine_pos);
			parent.Chapter.history.chapterNo = parent.Chapter.config.currentChapter;
			parent.Chapter.config.currentChapter = currChapter;
			FLO.in_reflow_mode = true;
			parent.Chapter.displayFlowItem(display_item["href"]);
			parent.Chapter.config.currentChapter = current_spine_pos;
		
		} catch(e) {
			azDump("****************displayFlowItem Exception", e);
		}
	};
	
	var setupFlowPage = function () {
		if (!jQuery.isEmptyObject(curr_nav_item)) {
			$("#" + "doc-viewer-azd").css("display", "block");
		}
	};
	
	var displayLeftSpread = function(navitem) {
		lctr = "";
		/*grab the current scroll positions if you are going to display a linear no page
		we will use it to set the nav page back to the current scroll position
		*/
		if ((navitem.hasOwnProperty("ext-spread-pos")) && (navitem["ext-spread-ref"] != "")) {
			if (navitem["ext-spread-pos"] == "left") {
				try {
					orig_right_scroll_pos = document.getElementById('page-right-az').contentWindow.wrappedJSObject.getIScrollPos();
				} catch(e) {
					orig_right_scroll_pos = 0;
				}
			}
			
			if (navitem["ext-spread-pos"] == "right") {
				try {
					orig_left_scroll_pos = document.getElementById('page-left-az').contentWindow.wrappedJSObject.getIScrollPos();
				} catch(e) {
					orig_left_scroll_pos = 0;
				}
			}
		}
		
		if ((navitem.hasOwnProperty("ext-spread-pos")) && (navitem["ext-spread-pos"] == "left") && (navitem["ext-spread-ref"] != "")) {
			var leftitem = navitem["ext-spread-ref"];
			navitem["ext-spread-ref"] = "";
			navitem["ext-spread-pos"] = "";
			FLO.nav_list[current_spine_pos] = navitem;
			
		} else {
			var leftitem = navitem["left"];
			orig_right_scroll_pos = 0;
		}
				
		if ((leftitem == "") || (leftitem == "page-blank.html")) {
			$("#" + "page-left-az").attr("src", "");
			lctr = "";
		} else {
			var display_item = FLO.spineProperties[leftitem];
			lctr = leftitem;
			if (display_item['layout'] == "reflowable") {
				FLO.is_lspread_reflowable = true; 
			} else {
				
			}
			displayPageInFrame("page-left-az", display_item["href"]);
			return true;
		}
		return false;
	};
	
	var displayRightSpread = function() {
		try{
			navitem = curr_nav_item;
			if (jQuery.isEmptyObject(curr_nav_item))return;
		
			if ((navitem.hasOwnProperty("ext-spread-pos")) && (navitem["ext-spread-pos"] == "right") && (navitem["ext-spread-ref"] != "")) {
				var rightitem = navitem["ext-spread-ref"];
				navitem["ext-spread-ref"] = "";
				navitem["ext-spread-pos"] = "";
				FLO.nav_list[current_spine_pos] = navitem;
			} else {
				var rightitem = navitem["right"];
				orig_left_scroll_pos = 0;
			}
			
			
			if ((rightitem == "")|| (rightitem == "page-blank.html")) {
				$("#" + "page-right-az").attr("src", "");
				//Special case: Prev spread was vertically stacked and this
				//right spread is empty. So the setPageWidthHeight is not triggered
				//So we explicitly set the top and left position to push the right back to original position
				if (lctr!="") {
					var ldisplay_item = FLO.spineProperties[lctr];
					var lpw = parseInt(ldisplay_item["width"]);
					var lph = parseInt(ldisplay_item["height"]);
					if (flo_state == 2) {
						if (lpw != "100%") lpw = lpw - 6;
						lph = lph - 6;
					}
					
					if (flo_state) lpw = parseInt(ldisplay_item["width"]) - 3;
					
					$('#' + "page-right-az").css("left", (lpw) + "px");
					$('#' + "page-right-az").css("top", (0) + "px");
					
					if (lph) $('#' + frameid).contents().find('body').css("height", (lph) + "px");
					if (lph) $('#' + frameid).css("height", (lph) + "px");
					if (pw) $('#' + frameid).contents().find('body').css("width",  lpw + "px");
					if (pw) $('#' + frameid).css("width",  lpw + "px");
					
				} 
					
				if (lctr) rctr = lctr;
			} else {
				var display_item = FLO.spineProperties[rightitem];		
				rctr = rightitem;
				if (lctr ==  "")lctr = rctr;
				if (display_item['layout'] == "reflowable") {
					FLO.is_rspread_reflowable = true;
				} else {
				}
				
				displayPageInFrame("page-right-az", display_item["href"]);
				return true;
			}
			return false;
		}catch(e){
			azDump("****************displayRightSpread Exception", e);
		}
	}
	
	var postSpreadload = function() {
		try {
			var navitem = curr_nav_item;
			var rightitem = navitem["right"];
			var leftitem = navitem["left"];
			if ((rightitem == "")|| (rightitem == "page-blank.html")) {
				if ((leftitem == "") || (leftitem == "page-blank.html")) {
					return;
				}
			} 
			
			//Set this for saving to the book pref
			var currChapter = getNavListPos(lctr);
			//parent.Chapter.config.currentChapter = currChapter;
			
			AZ_DIM.calculateZoombox(lctr, rctr);
			AZ_DIM.setDimensions();
			
			if (rightitem) {
				var display_item = FLO.spineProperties[rightitem];
				if (display_item && (display_item["layout"] == "reflowable")) {
					document.getElementById('page-right-az').contentWindow.wrappedJSObject.initIScroll('page-right-az', orig_right_scroll_pos);
				}
			}
			
			if (FLO.in_reflow_mode) {
				parent.Chapter.switchToFixedMode(); //TODO
				FLO.in_reflow_mode = false;
			}	
			
			setImagesHandler('page-left-az');
			setImagesHandler('page-right-az');
			
			FLO.setSMILState();
			
			FLO.setFLOPrintPageDisplay(false);
			
			FLO.handleFLOGutter(flo_current_gutter_state);
			document.getElementById('page-left-az').contentWindow.focus();
			document.getElementById('page-right-az').contentWindow.focus();
			
		}catch(e){
			azDump("****************postSpreadload Exception", e);
		}
	}
	
	var displaySpread = function(navitem) {
		try {
			var self = this;
			lctr = "";
			rctr = "";
			parent.Chapter.setEpubIframeEmpty();
			$("#" + "page-center-az").hide();
			stopAudio("page-center-az");
			$("#" + "page-center-az").attr("src", "");
			
			spreadtype = "LEFTRIGHT";
			var lspread_status = displayLeftSpread(navitem);
			if (!lspread_status) FLO.displayRightSpread(navitem);
		} catch(e) {
			azDump("****************displaySpread Exception", e);
		}
	};
	
	var stopAudio = function(targetid) {
		try {
			document.getElementById(targetid).contentWindow.wrappedJSObject.stopAudio(null);
		} catch(e) {
		
		}
	}
	
	var initAIE = function(targetid) {
		try {
			document.getElementById(targetid).contentWindow.wrappedJSObject.initAIEFLO();
		} catch(e) {
		
		}
	}
	
	var initAIETextSound = function(targetid) {
		try {
			var metainfo = parent.Epub.config.metainfo;
			$('#' + targetid).contents().find('body').attr("azdtemp", JSON.stringify(metainfo));
			document.getElementById(targetid).contentWindow.wrappedJSObject.initAIETextSoundFLO();
		} catch(e) {
				
		}
	}
	
	var handleResize = function() {
		//Set these variables so that they can be saved to the Book pref 
		parent.Chapter.config.orig_width = window.outerWidth;
		parent.Chapter.config.orig_height = window.outerHeight;
		parent.BookPref.config.openWidth = window.outerWidth;
		parent.BookPref.config.openHeight = window.outerHeight;
		AZ_DIM.setWindowDim();
		
		var c_scrollpos  = 0;
		var l_scrollpos  = 0;
		var r_scrollpos  = 0;
		
		try {
			l_scrollpos = document.getElementById('page-left-az').contentWindow.wrappedJSObject.getIScrollPos();
		} catch(e) {
			l_scrollpos = 0;
		}
		
		try {
			r_scrollpos = document.getElementById('page-right-az').contentWindow.wrappedJSObject.getIScrollPos();
		} catch(e) {
			r_scrollpos  = 0;
		}
		try {
			c_scrollpos = document.getElementById('page-center-az').contentWindow.wrappedJSObject.getIScrollPos();
		} catch(e) {
			c_scrollpos = 0;
		}
		
		if (spreadtype == "LEFTRIGHT") {
			AZ_DIM.calculateZoombox(lctr, rctr);
			AZ_DIM.setDimensions();
			setPageLeftPageDim("resize");
			setPageRightPageDim("resize");
		} else if (spreadtype == "CENTER") {
			AZ_DIM.calculateZoomboxCenter(cctr);
			AZ_DIM.setDimensions();
			setPageCenterPageDim("resize");
		}
		
		if (lctr) {
			var display_item = FLO.spineProperties[lctr];
			if (display_item && (display_item["layout"] == "reflowable")) {
				document.getElementById('page-left-az').contentWindow.wrappedJSObject.initIScroll('page-right-az', l_scrollpos);
			}
		}
		
		if (rctr) {
			var display_item = FLO.spineProperties[rctr];
			if (display_item && (display_item["layout"] == "reflowable")) {
				document.getElementById('page-right-az').contentWindow.wrappedJSObject.initIScroll('page-right-az', r_scrollpos);
			}
		}
		
		if (cctr) {
			var display_item = FLO.spineProperties[cctr];
			if (display_item && (display_item["layout"] == "reflowable")) {
				document.getElementById('page-center-az').contentWindow.wrappedJSObject.initIScroll('page-center-az', c_scrollpos);
			}
			document.getElementById('page-center-az').contentWindow.focus();
		}
			
			
	};
	
	var getNavPos = function(target_fname) {
		curr_nav_item = navitem;
		var found = false;
		var targetitem = "";
		for (nav_id in self.spineProperties) {
			var navitem = self.spineProperties[nav_id];
			if (navitem["orighref"].indexOf(target_fname) != -1) {
				targetitem = nav_id;
				found = true;
				break;
			}
		}
		
		if (!found) return -1;
		
		for(var i=0; i < FLO.nav_list.length; i++) {
			var navitem = FLO.nav_list[i];
			
			if ((navitem["left"] == targetitem) || (navitem["right"] == targetitem) || (navitem["center"] == targetitem)) {
				return i;
			}
		}
		
		return -1;
	}
	
	var getNavListPos = function(target_ref) {
		for(var i=0; i < FLO.spineList.length; i++) {
			var navitem = FLO.spineList[i];
			if (navitem == target_ref) {
				return i;
			}
		}
		
		return -1;
	}
	
	var handleLinkEvents = function(frameid) {
		self = this;
		if (frameid == "") {
			var $links = $('a');
		} else {
			var $links = $('#' + frameid).contents().find('a');
		}
		$links.each(function(){
			var linkReference = $(this).attr('href');
			if(typeof linkReference != 'undefined'){
				if(linkReference.indexOf('http://') == 0 || linkReference.indexOf('https://') == 0){
					$(this).bind('click', function(event){
						event.preventDefault();
						window.open(linkReference, "IEV", "chrome, width=1000, height=700, resizable=yes, centerscreen, scrollbars=yes").focus();
					});
				}
				else if(linkReference != 'javascript: void(0);'){
					if(linkReference.split('#')[0] != ''){
						$(this).bind('click', function(event){
							//actual link event handling happens here
							event.preventDefault();
							self.navigateToFixedView(linkReference.split('#')[0]);
						});
					}
				}
			}
		});
	}
	
	var navigateToFixedView = function(htmlref) {
		
		var newpos = getNavPos(htmlref);
					
		//grab the current position to handle the back button
		back_spine_pos = current_spine_pos;
		if (newpos != -1) {
			parent.Toolbar.enableBackButton();
			current_spine_pos = newpos;
			parent.Chapter.config.currentChapter = current_spine_pos;
			displayDocument();
			setNavButtons(current_spine_pos);
		} else {
			//We disable the nav buttons and enable the back button
			parent.Toolbar.enableBackButton();
			//This means we have found a section that is not there in the 
			//Spine navigation. So this will be a special case
			loadStandAlonePage(htmlref);
			
			return;
		}
	}
	
	var loadStandAlonePage = function (target_fname) {
		var navitem = {};
		var targetitem = "";
		var found = false; 
		for (nav_id in self.spineProperties) {
			var navitem = self.spineProperties[nav_id];
			if (navitem["orighref"].indexOf(target_fname) != -1) {
				targetitem = nav_id;
				display_item = navitem;
				found = true;
				break
			}
		}
		
		if (!found) return -1;
		
		//here we add a special item to the navitem to show the spread none page inside a 
		//spread. Scenario: Left spread contains links to linear no spine items.
		//After updating the var, we just use the regular displaydocument function
		if ((navitem["page-spread"] == "left") || (navitem["page-spread"] == "right")) {
			var navitem1 = FLO.nav_list[current_spine_pos];
			navitem1["ext-spread-ref"] = targetitem;
			navitem1["ext-spread-pos"] = navitem["page-spread"];
			FLO.nav_list[current_spine_pos] = navitem1;
			displayDocument();
			setNavButtons(current_spine_pos);
			return;
		}
		
		curr_nav_item = {};
		var tempnav = {};
		tempnav["navtype"] = "spread";
		tempnav["left"] = "";
		tempnav["center"] = "";
		tempnav["right"] = "";
		
		if (navitem["page-spread"] == "flow"){
			displayCenterSpread(targetitem);
			$(".zoombox-azd").css("display", "block");
			tempnav["center"] = targetitem;
		} else if (navitem["page-spread"] == "center"){
			tempnav["center"] = targetitem;
			displayCenterSpread(targetitem);
			$(".zoombox-azd").css("display", "block");
		} else if ((navitem["page-spread"] == "left") || (navitem["page-spread"] == "right")){
			$(".zoombox-azd").css("display", "block");	
			if (navitem["page-spread"] == "center")tempnav["center"] = targetitem;
			if (navitem["page-spread"] == "left")tempnav["left"] = targetitem;
			if (navitem["page-spread"] == "right")tempnav["right"] = targetitem;
			displaySpread(tempnav);
		} else if ((navitem["navtype"] == "reflow")|| (navitem["page-spread"] == "reflow")){
			displayFlowItem(targetitem);
			tempnav["center"] = targetitem;
			$(".zoombox-azd").css("display", "none");
		} else {
			displayFlowItem(targetitem);
			tempnav["center"] = targetitem;
			$(".zoombox-azd").css("display", "none");
		}
		
		curr_nav_item = tempnav;
		
		parent.Toolbar.disablePreviousButton();
		parent.Toolbar.disableNextButton();
		
	}
	
	var setNavButtons = function(tpos) {
		if (doc_ppd == "rtl"){
			//Right to Left mode
			if (tpos > 0) {
				parent.Toolbar.enablePreviousButton();
				parent.Toolbar.enableNextButton();
			} else if (tpos == 0){
				parent.Toolbar.disableNextButton();
				parent.Toolbar.enablePreviousButton();
			}else {
				parent.Toolbar.disableNextButton();
				parent.Toolbar.enablePreviousButton();
			}
			
			if (tpos >= FLO.nav_list.length - 1) {
				parent.Toolbar.enableNextButton();
				parent.Toolbar.disablePreviousButton();
			} 			
		} else {
			//Left to Right mode
			if (tpos <= 0) {
				parent.Toolbar.disablePreviousButton();
				parent.Toolbar.enableNextButton();
				return;
			} else {
				parent.Toolbar.enableNextButton();
				parent.Toolbar.enablePreviousButton();
			}
			
			if (tpos >= FLO.nav_list.length - 1) {
				parent.Toolbar.disableNextButton();
				parent.Toolbar.enablePreviousButton();
			} 
		}
	}
	
	var setImagesHandler = function(frameid){
		var self = this;
		
		var $img = $('#'+ frameid).contents().find('#custom-style-igp img');
		var  j = 0;
		var totalImages = $img.length;
		$img.each(function(){
			if (parent.Epub.config.isexternalimages) {
				var destObj = getFileObject(['ProfD', 'quizfx', 'buffer', config.epubId]);
				var imgname = $(this).attr('src').split('/').pop();
				if (parent.Epub.config.pubdoc_ext_images.hasOwnProperty(imgname)) {
					$(this).attr('src', parent.Epub.config.pubdoc_ext_images[imgname]);
					$(this).bind('dblclick', function () {
						var src = $(this).attr('src');
						if (src.indexOf("data:image") != -1) {
							parent.AME.openImage(src);
							return;
						}
					});
				} else {
					azDump("Failed to set", imgname);
				}
			} else {
				var foundInManifest = false;
				var src = $(this).attr('src');
				if (src.indexOf("data:image") != -1) {
					$(this).bind('dblclick', function () {
					parent.AME.openImage(src);
					return;
					});
				}

				for(var i = 0; i < parent.Epub.config.bookData.length; i++){
					if(unescape($(this).attr('src').split('/').pop()) == unescape(parent.Epub.config.bookData[i]['href'].split('/').pop())){
						foundInManifest = true;
						if(parent.Epub.config.bookData[i]['available'] == 'true'){
							$(this).bind('dblclick', function () {
								try {
									for(var i = 0; i < parent.Epub.config.bookData.length; i++){
										if(unescape($(this).attr('src').split('/').pop()) == unescape(parent.Epub.config.bookData[i]['href'].split('/').pop())){
											var imgPath = parent.Epub.getEntityAbsolutePath(parent.Epub.config.opfFilePath, parent.Epub.config.bookData[i]['href']);
											var imgObj = getFileObjectFromPath(imgPath);
											var imgUrl = getURL(imgObj);
											parent.AME.openImage(imgUrl);
											return;
										}
									}
								}	catch(e) {
								
								}
								
							});
						}
					}
				}
			}
		});
	};
	
	/*-------------------------NAV FUNCTIONS-----------------------------*/
	var goBack = function() {
		if (back_spine_pos != "") {
			current_spine_pos = back_spine_pos;
			parent.Chapter.config.currentChapter = current_spine_pos;
			displayDocument();
			setNavButtons(current_spine_pos);
		}
		back_spine_pos = "";
		parent.Toolbar.disableBackButton();
		
	}
	
	var displayPrevious = function() {
		
		
		var tpos = current_spine_pos;
		if (doc_ppd == "rtl"){
			tpos = tpos + 1;
		} else {	
			tpos = tpos - 1;
		}
		
		parent.Toolbar.enablePreviousButton();
		
		setNavButtons(tpos);
		
		//Dont display anything if the following conditions are true
		if (tpos < 0) {
			//In LTR, No prev item left
			return;
		} if (tpos > FLO.nav_list.length - 1){
			if (doc_ppd == "rtl"){
				//In RTL, No next item left
				return;
			}	
		}
		$('#page-left-az').contents().find('#custom-style-igp').html(loadingtag);
		$('#page-right-az').contents().find('#custom-style-igp').html(loadingtag);
		$('#page-center-az').contents().find('#custom-style-igp').html(loadingtag);
		
		current_spine_pos = tpos;
		parent.Chapter.config.currentChapter = current_spine_pos;
		
		displayDocument();
	};

	var displayNext = function() {
		
		
		var tpos = current_spine_pos;
		if (doc_ppd == "rtl"){
			tpos = tpos - 1;
		} else {	
			tpos = tpos + 1;
		}
		
		parent.Toolbar.enableNextButton();
		
		setNavButtons(tpos);
		
		//Dont display anything if the following conditions are true
		if (tpos > FLO.nav_list.length - 1){
			//In LTR, No next item left
			return;
		} else if (tpos < 0) {
			if (doc_ppd == "rtl"){
				//In RTL, No prev item left
				return;
			}	
		} 
		
		$('#page-left-az').contents().find('#custom-style-igp').html(loadingtag);
		$('#page-right-az').contents().find('#custom-style-igp').html(loadingtag);
		$('#page-center-az').contents().find('#custom-style-igp').html(loadingtag);
	
		
		current_spine_pos = tpos;
		parent.Chapter.config.currentChapter = current_spine_pos;
		displayDocument();
	};	
	
	var getSpreadInfoForDisplay = function(idref){
		try{
			var s = "";
			var display_item = self.spineProperties[idref];
			for (k in display_item) {
				if(k == "newhref") {
					var fn = display_item[k].split("/").pop();
					s = s + "href" + ": ";
					s = s + fn + "\n";
				} else if(k == "href") {
				
				} else {
					s = s + k + ": ";
					s = s + display_item[k] + "\n";
				}
			}
			return s;
		} catch(e) {
			return "";
		}
	};
	
	var showItemProperties = function() {
		var display = "";
		var navitem = FLO.nav_list[current_spine_pos];
		if ((navitem["left"] != "page-blank.html") && (navitem["left"] != "")) {
			var idref = navitem["left"];
			display = getSpreadInfoForDisplay(idref);
		}
		display = display + "\n\n";
		
		if ((navitem["right"] != "page-blank.html") && (navitem["right"] != "")) {
			var idref = navitem["right"];
			display = display + getSpreadInfoForDisplay(idref);
		}
		display = display + "\n\n";
		
		if ((navitem["center"] != "page-blank.html") && (navitem["center"] != "")) {
			var idref = navitem["center"];
			display = display + getSpreadInfoForDisplay(idref);
		}
		
		alert(display);
	};
	
	var setMarginOnSpreads = function(frameid) {
		//When the margins get set reduce the margin dim from left and right frame to make sure
		//that we dont get overlaps between spreads
		var ph = parseInt($('#' + frameid).contents().find('body').css("height"));
		var pw = parseInt($('#' + frameid).contents().find('body').css("width"));
		if (flo_state == 2) {
			ph = ph - 6;
			pw = pw - 6;
		} else {
			ph = ph + 6;
			pw = pw + 6;		
		}
		$('#' + frameid).contents().find('body').css("height", ph+"px");
		$('#' + frameid).css("height", ph+"px");
		$('#' + frameid).contents().find('body').css("width", pw+"px");
		$('#' + frameid).css("width", pw+"px");
	};
	
	var setFLOState = function(state) {
		flo_state = state;
		setMarginOnSpreads("page-left-az");
		setMarginOnSpreads("page-right-az");
		setMarginOnSpreads("page-center-az");
	};
	
	var setPageRefDisplayStyle = function(frameid) {
		var pref_display_state = ["ppb-show-block", "ppb-show-inline", "ppb-show-margin"];
		
		var newclassname = "";
		var rem_classname = "";
		if (flo_pageref_display_index == 0) {
			newclassname = pref_display_state[flo_pageref_display_index];
			rem_classname = pref_display_state[2];
		} else if (flo_pageref_display_index == 3){
			rem_classname = pref_display_state[flo_pageref_display_index-1];
			var newclassname = "";
		} else {
			newclassname = pref_display_state[flo_pageref_display_index];
			rem_classname = pref_display_state[flo_pageref_display_index-1];
		}
		
		//TODO - Replace this loop with a better method
		$('#' + frameid).contents().find('body *:not(script, style, noscript)').each(function() {
			var epubtype = $(this).attr("epub:type");
			if (epubtype == "pagebreak") {
				if (rem_classname) $(this).removeClass(rem_classname);
				if (newclassname) $(this).addClass(newclassname);
			}
		});
	};
	
	var setFLOPrintPageDisplay = function(updateindex) {
		try {
		if (updateindex) {
			flo_pageref_display_index = flo_pageref_display_index + 1;
			if (flo_pageref_display_index > 3) flo_pageref_display_index = 0
		}
		switch(flo_pageref_display_index){
			case 0:
				setPageRefDisplayStyle("page-left-az");
				setPageRefDisplayStyle("page-right-az");
				setPageRefDisplayStyle("page-center-az");
				break;
			case 1:
				setPageRefDisplayStyle("page-left-az");
				setPageRefDisplayStyle("page-right-az");
				setPageRefDisplayStyle("page-center-az");
				break;
			case 2:
				setPageRefDisplayStyle("page-left-az");
				setPageRefDisplayStyle("page-right-az");
				setPageRefDisplayStyle("page-center-az");
				break;
			case 3:
				setPageRefDisplayStyle("page-left-az");
				setPageRefDisplayStyle("page-right-az");
				setPageRefDisplayStyle("page-center-az");
				break;
			default:
				setPageRefDisplayStyle("page-left-az");
				setPageRefDisplayStyle("page-right-az");
				setPageRefDisplayStyle("page-center-az");
				break;
			}
			parent.BookPref.config.floShowPageNum = flo_pageref_display_index;
		}catch(e){
			//alert(e);
		}
	};
	
	var playFLOSMILAudio = function(target, status){
		
		if (spreadtype == "LEFTRIGHT") {
			if (flo_smil_lookup.lspread != "") {
				//Play the lspread audio
				initAIETextSound('page-left-az');
				document.getElementById('page-left-az').contentWindow.wrappedJSObject.playAudioEx(target, status, playBackRate);
				flo_smil_lookup.current_playing = "lspread";
				return;
			}
			if (flo_smil_lookup.rspread != "") {
				initAIETextSound('page-right-az');
				document.getElementById('page-right-az').contentWindow.wrappedJSObject.playAudioEx(target, status, playBackRate);
				flo_smil_lookup.current_playing = "rspread";
				return;
			}
		} else if (spreadtype == "CENTER") {
			if (flo_smil_lookup.cspread != ""){
				initAIETextSound('page-center-az');
				document.getElementById('page-center-az').contentWindow.wrappedJSObject.playAudioEx(target, status, playBackRate);
				flo_smil_lookup.current_playing = "cspread";
				return;
			}
		}		
	};

	var stopFLOSMILAudio = function(){
		if (spreadtype == "LEFTRIGHT") {
			if (flo_smil_lookup.lspread != "") {
				document.getElementById('page-left-az').contentWindow.wrappedJSObject.stopAudio(null);
				flo_smil_lookup.current_playing = "";
			}
			if (flo_smil_lookup.rspread != "") {
				document.getElementById('page-right-az').contentWindow.wrappedJSObject.stopAudio(null);
				flo_smil_lookup.current_playing = "";
			}
		} else if (spreadtype == "CENTER") {
			if (flo_smil_lookup.cspread != ""){
				document.getElementById('page-center-az').contentWindow.wrappedJSObject.stopAudio(null);
				flo_smil_lookup.current_playing = "";
				return;
			}
		}	
	};
	
	/*
	* Set the top toolbar audio control status if the fixed spread has SMIL audio
	*/
	var setSMILState = function () {
		if ((flo_smil_lookup.lspread != "") || (flo_smil_lookup.rspread != "") || (flo_smil_lookup.cspread != "")) {
			parent.Toolbar.enableAudioOptionsButton();
			parent.AME.paused_handle();
			parent.Toolbar.hideAudioControls();
		} else {
			parent.AME.paused_handle();
			parent.Toolbar.disableAudioOptionsButton();
			parent.Toolbar.hideAudioControls();
		}
	}
	
	/*
	 * Called from ame.js when the audio finished event is triggered
	*/
	var smilAudioFinished = function () {
		if (flo_smil_lookup.current_playing != "") {
			if (flo_smil_lookup.current_playing == "lspread") {
				//Leftspread finished playing. Start the right spread audio if it exists
				if (flo_smil_lookup.rspread != "") {
					initAIETextSound('page-right-az');
					document.getElementById('page-right-az').contentWindow.wrappedJSObject.playAudioEx("", false, playBackRate);
					flo_smil_lookup.current_playing = "rspread";
				} else {
					flo_smil_lookup.current_playing = "";
				}
				return;
			} else if (flo_smil_lookup.current_playing == "rspread") {
				flo_smil_lookup.current_playing = "";
				return;
			} else if (flo_smil_lookup.current_playing == "cspread") {
				flo_smil_lookup.current_playing = "";
				return;
			}
		}
	}
	
	var smilAudioTrackPara = function (blockid) {
		if (flo_smil_lookup.current_playing != "") {
			if (flo_smil_lookup.current_playing == "lspread") {
				scrollToInternalLinkReference("page-left-az", blockid);
			} else if (flo_smil_lookup.current_playing == "rspread") {
				scrollToInternalLinkReference("page-right-az", blockid);
			} else if (flo_smil_lookup.current_playing == "cspread") {
				scrollToInternalLinkReference("page-center-az", blockid);
			}
		}
	};
	
	/*
	* This is used to scroll the reflowable pages when displaying smil blocks
	* Called from ame.js on event::aie_playing_handle inside iframelistener
	*/
	var scrollToInternalLinkReference = function(frameid, reference){
		try {
			document.getElementById(frameid).contentWindow.wrappedJSObject.scrollToElement(reference);
		} catch(e) {
			
		}
	}
	
	var getCurrentGutterState = function() {
		return flo_current_gutter_state;
	}
	
	var setFrameAudioRate = function(frameid, rate) {
		try {
			document.getElementById(frameid).contentWindow.wrappedJSObject.changePlaybackRate(rate);
		} catch(e) {
			
		}
	}
	
	var changePlaybackRate = function(rate) {
		if (!playBackRate) {
			playBackRate = 1.0;
		}
		
		playBackRate = rate;
		setFrameAudioRate("page-left-az", rate);
		setFrameAudioRate("page-right-az", rate);
		setFrameAudioRate("page-center-az", rate);
	}
	
	var handleFLOGutter = function(state) {
		flo_current_gutter_state = state;
		parent.BookPref.config.floGutterSetting = flo_current_gutter_state;
		switch(state){
			case 0:
				$('#page-left-az').contents().find('body .shadowleft').remove();
				$('#page-right-az').contents().find('body .shadowright').remove();
				break;
			case 1:
				$('#page-left-az').contents().find('body').append('<div class="borderleft"></div>');
				$('#page-right-az').contents().find('body').append('<div class="borderright"></div>');
				break;
			case 2:
				$('#page-left-az').contents().find('body .borderleft').remove();
				$('#page-right-az').contents().find('body .borderright').remove();
				break;
			case 3:
				$('#page-left-az').contents().find('body .borderleft').remove();
				$('#page-right-az').contents().find('body .borderright').remove();
				break;
			case 4:
				$('#page-left-az').contents().find('body').append('<div class="shadowleft"></div>');
				$('#page-right-az').contents().find('body').append('<div class="shadowright"></div>');
				break;
			default:
				break;
		}
	}
	
	return {
		init: init,
		in_reflow_mode: in_reflow_mode,
		nav_list: nav_list,
		is_lspread_reflowable: is_lspread_reflowable,
		is_rspread_reflowable: is_rspread_reflowable,
		spineList: spineList,
		spineProperties: spineProperties,
		displayPageInFrame: displayPageInFrame,
		setPageLeftPageDim: setPageLeftPageDim,
		setPageRightPageDim: setPageRightPageDim,
		setPageCenterPageDim: setPageCenterPageDim,
		setBodyWidthHeight: setBodyWidthHeight,
		displayCenterSpread: displayCenterSpread,
		displayPrevious: displayPrevious,
		displayNext: displayNext,
		handleResize: handleResize,
		setupFlowPage: setupFlowPage,
		handleLinkEvents: handleLinkEvents,
		goBack: goBack,
		navigateToFixedView: navigateToFixedView,
		showItemProperties: showItemProperties,
		displayRightSpread: displayRightSpread,
		postSpreadload: postSpreadload,
		setFLOState: setFLOState,
		setFLOPrintPageDisplay: setFLOPrintPageDisplay,
		playFLOSMILAudio: playFLOSMILAudio,
		stopFLOSMILAudio: stopFLOSMILAudio,
		setSMILState: setSMILState,
		smilAudioFinished: smilAudioFinished,
		smilAudioTrackPara: smilAudioTrackPara,
		setImagesHandler: setImagesHandler,
		handleFLOGutter: handleFLOGutter,
		getCurrentGutterState: getCurrentGutterState,
		changePlaybackRate: changePlaybackRate,
	};
	
}();

function generateRandString()
{
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for( var i=0; i < 15; i++ )
        text += possible.charAt(Math.floor(Math.random() * possible.length));

    return text;
}