/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for flow mode
 */
var Flow = function(){
	
	var config = {
		showLastPage: false,
		goToDepth: '',
		goToReference: '',
		goToNoteId: '',
		currentWidth: '',
		currentHeight: '',
		documentWidth: '',
		documentHeight: '',
		xOffset: '',
		yOffset: '',
		currentReadingDepth: 0
	};
	
	/**
	 * Intilizes the flow mode
	 */
	var init = function(){
		var self = this;
		
		BookPref.config.noOfCols = 0;
		self.unbindResize();
		self.unbindScroll();
		
		$('#doc-viewer-azd').contents().find('.progressbar').hide();
		$('#doc-viewer-azd').contents().find('.bookProgressbar').hide();
		
		if($('#doc-viewer-azd').contents().find('#container-ape').length != 0){
			$('#doc-viewer-azd').contents().find('#custom-style-igp').unwrap();
		}
		
		$('#doc-viewer-azd').contents().find('body').css('overflow', 'auto');
		
		self.removeEmptyCols();
		
		self.initilizeVariables();
		
		if(config.showLastPage){
			self.last();
		}
		else{
			self.first();
		}
		
		self.getCurrentReadingDepth();
		
		self.bindResize();
		self.bindScroll();
		
		if(config.goToReference !== ''){
			self.goToInternalLinkReference(config.goToReference);
			config.goToReference = '';
		}
		
		if(config.goToDepth !== ''){
			config.currentReadingDepth = parseFloat(config.goToDepth);
			config.goToDepth = '';
			self.setApproximatePage();
			self.getCurrentReadingDepth();
		}
		
		$('#doc-viewer-azd').contents().find('body')
			.removeClass('rule-on')
			.removeClass('column-count-2')
			.removeClass('column-count-3')
			.removeClass('column-count-4');
	};
	
	/**
	 * Initilize the latest window variables
	 */
	var initilizeVariables = function(){
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.initPageVariables();
		$.extend(config, document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.config);
	};
	
	/**
	 * Calculate the reading depth(in %) for the current chapter
	 */
	var getCurrentReadingDepth = function(){
		var self = this;
		
		self.initilizeVariables();
		var totalScrollVal = config.documentHeight - config.currentHeight;
		if(totalScrollVal == 0){
			config.currentReadingDepth = 0;
		}
		else{
			config.currentReadingDepth = config.yOffset / totalScrollVal * 100;
		}
	};
	
	/**
	 * Sets the approximate location in page using current reading depth
	 */
	var setApproximatePage = function(){
		var self = this;
		
		self.initilizeVariables();
		var totalScrollVal = config.documentHeight - config.currentHeight;
		
		var scrollToYVal = config.currentReadingDepth / 100 * totalScrollVal;
		
	    document.getElementById('doc-viewer-azd').contentWindow.scrollTo(0, scrollToYVal);
	};
	
	/**
	 * Scroll to the first page
	 */
	var first = function(){
		document.getElementById('doc-viewer-azd').contentWindow.scrollTo(0, 0);
	};
	
	/**
	 * Scroll to previous viewport
	 */
	var previous = function(){
		var self = this;
		
		self.initilizeVariables();
		var scrollByYVal = 0 - config.currentHeight;
		
        var lineHeight = $('#doc-viewer-azd').contents().find('p').css('line-height');
		if(typeof lineHeight != 'undefined'){
			lineHeight = lineHeight.replace('px', '');
			scrollByYVal = parseFloat(scrollByYVal) - parseFloat(lineHeight);
		}
		
		document.getElementById('doc-viewer-azd').contentWindow.scrollBy(0, scrollByYVal);
		
		self.getCurrentReadingDepth();
	}
	
	/**
	 * Scroll to next viewport.
	 */
	var next = function(){
		var self = this;
		
		self.initilizeVariables();
		var scrollByYVal = config.currentHeight;
        
		var lineHeight = $('#doc-viewer-azd').contents().find('p').css('line-height');
		if(typeof lineHeight != 'undefined'){
			lineHeight = lineHeight.replace('px', '');
			scrollByYVal = parseFloat(scrollByYVal) - parseFloat(lineHeight);
		}
        
		document.getElementById('doc-viewer-azd').contentWindow.scrollBy(0, scrollByYVal);
		
		self.getCurrentReadingDepth();
	}
	
	/**
	 * Scroll to the last page
	 */
	var last = function(){
        var scrollToYVal = config.documentHeight - config.currentHeight;
		document.getElementById('doc-viewer-azd').contentWindow.scrollTo(0, scrollToYVal);
	};
	
	/**
	 * Goes to the requested link references.
	 * @params reference Reference location
	 */
	var goToInternalLinkReference = function(reference){
		try {
		var self  = this;
		self.initilizeVariables();
		try{
			var offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
			var epubtype = $('#doc-viewer-azd').contents().find('#'+reference).attr("epub:type");
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-note-rw")) {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.top =  offset.top + item_offset.top;
			}
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-note")) {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.top =  offset.top + item_offset.top;
			}
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-footnote-rw")) {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.top =  offset.top + item_offset.top;
			}
			
			if ($('#doc-viewer-azd').contents().find('#'+reference).hasClass("num-footnote")) {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.top =  offset.top + item_offset.top;
			}
			
			if (epubtype && epubtype == "footnote") {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.top =  offset.top + item_offset.top;
			}
			
			if (epubtype && epubtype == "rearnote") {
				var offset = $('#doc-viewer-azd').contents().find('#'+reference).parent().position();
				var item_offset = $('#doc-viewer-azd').contents().find('#'+reference).position();
				offset.top =  offset.top + item_offset.top;
			}
			
		} catch(e) {
			//	jQuery's .position() throws an error if the element has style 'display:none',
			//	use of 'visibility:hidden' instead is encouraged.
			return false;
		}
		
		//DC - FLO UPDATE - 201211 - Bug fix - if (offset) was always returning true
		if (offset == null) {
			//	Did not find the element.
			return false;
		}
		var elementYPos = offset.top;
		var scrollToYVal = elementYPos - (config.currentHeight / 2);
		document.getElementById('doc-viewer-azd').contentWindow.scrollBy(0, scrollToYVal);
		
		self.getCurrentReadingDepth();
		} catch(e) {
			alert(e);
		}
	}
	
	/**
	 * Unbind the resize event
	 */
	var unbindResize = function(){
		$(window).unbind('resize');
	};
	
	/**
	 * Bind the resize event
	 */
	var bindResize = function() {
		var self = this;
		
		$(window).bind('resize', function(){
			Chapter.setNavButtons();
		});
	};
	
	/**
	 * Unbind scroll event
	 */
	var unbindScroll = function() {
		var self = this;
		
		$(window).unbind('scroll');
	};
	
	/**
	 * Bind scroll event
	 */
	var bindScroll = function() {
		var self = this;
		
		$(window).bind('scroll', function(){
			Chapter.setNavButtons();
		});
	};
	
	/**
	 * Remove the empty columns which might have been added in column mode
	 */
	var removeEmptyCols = function() {
		var self = this;
		
		$('#doc-viewer-azd').contents().find('div[class^="last-page"]').remove();
	};
	
	return{
		config: config,
		init: init,
		initilizeVariables: initilizeVariables,
		first: first,
		previous: previous,
		next: next,
		last: last,
		goToInternalLinkReference: goToInternalLinkReference,
		unbindResize: unbindResize,
		getCurrentReadingDepth: getCurrentReadingDepth,
		setApproximatePage: setApproximatePage,
		bindResize: bindResize,
		bindScroll: bindScroll,
		unbindScroll: unbindScroll,
		removeEmptyCols: removeEmptyCols
	}
}();