/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains the functions necessary for font manipulations.
 */
var Fonts = function(){
	var config = {
		XUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
		font: '',
		fontSize: '',
		maxValue: 10.0
	};
	
	/**
	 * Initilizes the fonts.
	 */
	var init = function(){
		var self = this;
		
		//Get all fonts list installed on the system.
		var fontList = self.getSystemFonts();
		//Set the fonts list in a drop down box of default settings window.
		self.setFontList(fontList);
	};
	
	/**
	 * Set the font-family for this chapter
	 * @params font Name of the font to set
	 */
	var setFont = function(font){
		$('#doc-viewer-azd').contents().find('body').css('font-family', font);
		config.font = font;
		BookPref.config.font = font;
	};
	
	/**
	 * Set the minimum font size for the chpater
	 */
	var setMin = function() {
		$('#doc-viewer-azd').contents().find('#custom-style-igp').css('font-size', '0.75em');
	};
	
	/**
	 * Set the font size of this chapter
	 * @params size Font size to set
	 */
	var set = function(size) {
		var fontSize = size+'em';
		config.fontSize = size;
		BookPref.config.fontSize = size;
		$('#doc-viewer-azd').contents().find('#custom-style-igp').css('font-size', fontSize);
	};
	
	/*
	var set = function(size){
		var fontSize;
		switch(parseInt(size)){
			case 1:
				fontSize = '0.75em';
				break;
			case 2:
				fontSize = '0.9em';
				break;
			case 3:
				fontSize = '1em';
				break;
			case 4:
				fontSize = '1.1em';
				break;
			case 5:
				fontSize = '1.2em';
				break;
			case 6:
				fontSize = '1.4em';
				break;
			case 7:
				fontSize = '1.6em';
				break;
			case 8:
				fontSize = '1.8em';
				break;
			case 9:
				fontSize = '2em';
				break;
		}

		config.fontSize = size;
		BookPref.config.fontSize = size;
//		size = size / 10 + 'em';
// alert($('#doc-viewer-azd').contents().find('#custom-style-igp').css('font-size'))
		$('#doc-viewer-azd').contents().find('#custom-style-igp').css('font-size', fontSize);
		// alert($('#doc-viewer-azd').contents().find('#custom-style-igp').css('font-size'))
	}
	*/
	
	// var setSlider = function(size){
		// config.fontSize = size;
		// BookPref.config.fontSize = size;
		// $('#font-scale').attr('value', size);
	// };
	
	/**
	 * Set the font list in pop up box of default settings window.
	 * @param fontList Array of all the local fonts. 
	 */
	var setFontList = function(fontList){
		for(var i=0; i<fontList.length; i++){
			var option = document.createElementNS(config.XUL_NS, "menuitem");
			option.setAttribute("label", fontList[i]);
			option.setAttribute("value", fontList[i]);
			
			$('#user_pref_font_list > menupopup').append(option);
		}
	};
	
	/**
	 * Get the list of fonts available on the system.
	 * @returns {array} Array of all local fonts.
	 */
	var getSystemFonts = function(){
		var sys_font_list = Components.classes["@mozilla.org/gfx/fontenumerator;1"]
							.createInstance(Components.interfaces.nsIFontEnumerator);
		var localFontCount = { value: 0 };
		var gLocalFonts = sys_font_list.EnumerateAllFonts(localFontCount);
		return gLocalFonts;
	};
	
	return{
		config: config,
		init: init,
		setFont: setFont,
		set: set,
		// setSlider: setSlider,
		getSystemFonts: getSystemFonts,
		setFontList: setFontList,
		setMin: setMin
	};
}();