/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for creating, updating, saving and displaying notes.
 */
var Notes = function(){
	var config = {
		notesData: '',
		allNotesData: '',
		prompts: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),
		XHTML_NS: "http://www.w3.org/1999/xhtml"
	}
	
	/**
	 * Tests if the selected text is valid to create notes
	 * @returns {boolean} Valid/Invalid to create notes
	 */
	var isTextSelected = function(){
		var self = this;
		
		var notParagraph = false;
		
		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
		if(selectionObj.rangeCount > 1){
			var msg = getPropertiesString("azd_notes_mulSelNoteError");
			config.prompts.alert(null, AME.getTitle(),msg);
			// alert('Can\'t create note on multiple selections.');
			return false;
		}
		else if(selectionObj.toString() == ''){
			var msg = getPropertiesString("azd_notes_noTextSelNoteError");
			config.prompts.alert(null, AME.getTitle(), msg);
			// alert('Select some text to create note.');
			return false;
		}
		else if(selectionObj.toString().length < 6){
			var msg = getPropertiesString("azd_notes_minTextSelNoteError");
			config.prompts.alert(null, AME.getTitle(), msg);
			// alert('Select at least 6 characters.');
			return false;
		}
		else{
			var startNode = selectionObj.anchorNode;
			while(typeof startNode.tagName == 'undefined' || startNode.tagName.toLowerCase() != 'p' ){
				startNode = startNode.parentNode;
				if(startNode.tagName.toLowerCase() == 'body'){
					notParagraph = true;
					break;
				}
			}
			
			var endNode = selectionObj.focusNode;
			while(typeof endNode.tagName == 'undefined' || endNode.tagName.toLowerCase() != 'p'){
				endNode = endNode.parentNode;
				if(endNode.tagName.toLowerCase() == 'body'){
					notParagraph = true;
					break;
				}
			}
			
			if(notParagraph){
				var msg = getPropertiesString("azd_notes_noParaNoteError");
				config.prompts.alert(null, AME.getTitle(), msg);
				// alert('Can\'t create note here.');
				return false;
			}
			else if(startNode.getAttribute('class') != endNode.getAttribute('class')){
				var msg = getPropertiesString("azd_notes_mulParaSelectNoteError");
				config.prompts.alert(null, AME.getTitle(), msg);
				// alert('Select only one paragraph for creating note.');
				return false;
			}
		}
		
//		var selectedPara = self.getSelectedParagraph();
//		var paraContent = $('#doc-viewer-azd').contents().find('.'+selectedPara).html();
//		if(paraContent.search(selectionObj.toString()) == -1){
//			alert('5 Can\'t create note here.');
//			return false;
//		}
		
		return true;
	}
	
//	var getSelectedParagraph = function(){
//		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
//		
//		var startNode = selectionObj.anchorNode;
////		alert(selectionObj.anchorNode.text())
//		while(typeof startNode.tagName == 'undefined' || startNode.tagName.toLowerCase() != 'p' ){
//			startNode = startNode.parentNode;
//			if(startNode.tagName.toLowerCase() == 'body'){
//				break;
//			}
//		}
//		
//		var className = '';
//		var classes = startNode.getAttribute('class').split(' ');
//		for(var i = 0; i < classes.length; i++){
//			if(classes[i].search('azd-notes-paragraph-') != -1){
//				className = classes[i];
//				break;
//			}
//		}
//		
//		return className;
//	}
	
//	var getStartOffset = function(paragraphClass){
//		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
//		var paragraphContent = $('#doc-viewer-azd').contents().find('.'+paragraphClass).html();
//		var selectionObj = selectionObj.toString().replace(/\n/g, ' ');
//		var startOffset = paragraphContent.search(selectionObj);
//		
//		return startOffset;
//	}
//	
//	var getSelectionLength = function(){
//		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
//		
//		return selectionObj.toString().length;
//	}
	
	/**
	 * Returns the selected string
	 * @returns {string} selected string
	 */
	var getSelectedString = function(){
		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
		var selRange = selectionObj.getRangeAt(0);
		
		var selection = selectionObj.toString();
		if(selection.length > 170){
			selection = selection.substr(0, 168) + '...';
		}
		
		return selection;
	};
	
	
//	function saveAnnotation(annotationTitle, annotationText, spineId, paragraphClass, timeCreated, annoId, edit){
//	var filePathArray = ['Home', 'AZARDI2', 'azardi2.azd'];
//	var fileObj = getFileObject(filePathArray);
//	var query = 'CREATE TABLE IF NOT EXISTS annotation ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"book_id" TEXT, "spine_id" TEXT, "para_class" ' +
//	'TEXT, "annotation_title" TEXT, "annotation_text" TEXT, "time_created" TEXT)';
//	createTable(fileObj, query);
//	var table_name = 'annotation';
//	if (edit) {
//		var conditions = new Array(['id', annoId, 'INTEGER']);
//		var values = new Array(['annotation_text', annotationText, 'TEXT'],
//							   ['time_created', timeCreated, 'TEXT']);
//		updateRowIntoDatabase(fileObj, values, table_name, conditions);
//	}
//	else {
//		var values = new Array(['book_id', currentBookId, 'TEXT'],
//							   ['spine_id', spineId, 'TEXT'],
//							   ['para_class', paragraphClass, 'TEXT'],
//							   ['annotation_title', annotationTitle, 'TEXT'],
//							   ['annotation_text', annotationText, 'TEXT'],
//							   ['time_created', timeCreated, 'TEXT']);
//		insertRowIntoDatabase(fileObj, values, table_name);
//	}
//}
	
	/**
	 * Saves the notes data to database
	 * @params bookId Id of the book
	 * @params currentChapter Current chapter of the book
	 * @params notesDescription Description of the notes
	 * @params selectionString Selected string
	 * @params currentTime Current time
	 * @params serial Serialized values of notes
	 * @params noteId Id of the note
	 * @params edit new save | edited save
	 */
	var save = function(bookId, currentChapter, notesDescription, selectionString, currentTime, serial, noteId, edit){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'annotation';
		if(edit){
			var conditions = new Array(['id', noteId, 'INTEGER']);
			var values = new Array(['notes_description', notesDescription, 'TEXT'],
			  					   ['time_created', currentTime, 'TEXT']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
		else{
			var values = new Array(['book_id', bookId, 'TEXT'],
								   ['current_chapter', currentChapter, 'TEXT'],
								   // ['notes_title', notesTitle, 'TEXT'],
								   ['notes_description', notesDescription, 'TEXT'],
						  		   ['selection_string', selectionString, 'TEXT'],
								   ['serial', serial, 'TEXT'],
								   ['time_created', currentTime, 'TEXT']);
			insertRowIntoDatabase(fileObj, values, table_name);
		}
	}
	
	/**
	 * Retrieve all the notes of current chapter from database
	 */
	function getNotes(bookId, currentChapter){
		var fileObj = getFileObject(AME.getDbFilePath());
	    if(!fileObj.exists()){
	        return false;
	    }
	    var table_name = 'annotation';
		var condition = new Array(['book_id', bookId, 'TEXT'],
								  ['current_chapter', currentChapter, 'TEXT']);
//		var values = ['serial', 'notes_title', 'notes_description', 'selection_string', 'time_created', 'id'];
		var values = ['serial', 'notes_description', 'selection_string', 'time_created', 'id'];
		config.notesData = new Array();
		statement = getValuesFromDatabase(fileObj, values, table_name, condition);
		
		if(statement !== undefined && statement !== false){
			while (statement.executeStep()){
				var data = new Array();
				data.push(statement.row.serial);
//				data.push(statement.row.notes_title);
				data.push(statement.row.notes_description);
				data.push(statement.row.selection_string);
				data.push(statement.row.time_created);
				data.push(statement.row.id);
				config.notesData.push(data);
		    }
		    statement.finalize();
		}
	};
	
	/**
	 * Get all the notes data from database
	 * @params bookId Id of the current book
	 */
	var getAllNotes = function(bookId){
		var fileObj = getFileObject(AME.getDbFilePath());
	    if(!fileObj.exists()){
	        return false;
	    }
	    var table_name = 'annotation';
		var condition = new Array(['book_id', bookId, 'TEXT']);
//		var values = ['serial', 'notes_title', 'notes_description', 'selection_string', 'time_created', 'id'];
		var values = ['serial', 'notes_description', 'selection_string', 'time_created', 'id', 'current_chapter'];
		config.allNotesData = new Array();
		statement = getValuesFromDatabase(fileObj, values, table_name, condition);
		
		if(statement !== undefined && statement !== false){
			while (statement.executeStep()){
				var data = new Array();
				data.push(statement.row.serial);
				data.push(statement.row.notes_description);
				data.push(statement.row.selection_string);
				data.push(statement.row.time_created);
				data.push(statement.row.id);
				data.push(statement.row.current_chapter);
				config.allNotesData.push(data);
		    }
		    statement.finalize();
		}
	}
	
	/**
	 * Highlight the notes in the current page
	 */
	var highlight = function(){
		for(var i = 0; i < config.notesData.length; i++){
			var serial = config.notesData[i][0],
//				   noteTitle = config.notesData[i][1],
				   noteDescription = config.notesData[i][1],
				   noteTime = config.notesData[i][3],
				   noteId = config.notesData[i][4];
				   
			try{
				var sel = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.deserializeSelection(serial);
				var range = sel.getRangeAt(0);
				var newNode = document.getElementById('doc-viewer-azd').contentWindow.document.createElementNS(config.XHTML_NS, 'span');
				newNode.setAttribute('id', noteId);
				newNode.setAttribute('class', 'yellowBg');
				newNode.setAttribute('onclick', 'noteinstanceclicked(this.id);');
				newNode.appendChild(range.extractContents());
				range.insertNode(newNode);
			}
			catch(e){
			}
		}
	};
	
	/**
	 * Highlights selected notes string on the fly
	 * @params serial Serialized value of notes
	 * @params noteId Id of the note
	 */
	var highlightOnTheFly = function(serial, noteId ){
		var sel = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.deserializeSelection(serial);
		var range = sel.getRangeAt(0);
		var newNode = document.getElementById('doc-viewer-azd').contentWindow.document.createElementNS(config.XHTML_NS, 'span');
//		var newNode = document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span');
		newNode.setAttribute('id', noteId);
		newNode.setAttribute('class', 'yellowBg');
		newNode.setAttribute('onclick', 'noteinstanceclicked(this.id);');
		newNode.appendChild(range.extractContents());
		range.insertNode(newNode);
	}
	
	/**
	 * Creates the edit note dialog
	 */
	var createEditDialog = function(){
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.createNoteEditDialog();
	};
	
	/**
	 * Fills the notes information in the note dialog
	 * @params noteId Id of the note to be edited
	 */
	var setEditDialog = function(noteId){
		var dateObj = new Date();
		
		for(var i = 0; i < config.notesData.length; i++){
			if(noteId == config.notesData[i][4]){
				dateObj.setTime(config.notesData[i][3]);
				var shortTitle = config.notesData[i][2];
				if(shortTitle.length > 27){
					var shortTitle  = shortTitle.substring(0, 25) + '...';
				}
				$('#doc-viewer-azd').contents().find('.notes-title-ape').html(shortTitle);
				var noteDescription = config.notesData[i][1].replace( /\n/g, '<br/>' );
				$('#doc-viewer-azd').contents().find('.notes-description-ape').html(noteDescription);
				$('#doc-viewer-azd').contents().find('.notes-date-ape').html(dateObj.toDateString());
				$('#doc-viewer-azd').contents().find('#id_anno').html(noteId);
			}
		}
	};
	
	/**
	 * Returns the id of the latest note added to database
	 * @returns {string} Id of the latest note
	 */
	var getLatestId = function(){
		var fileObj = getFileObject(AME.getDbFilePath());
	    if (!fileObj.exists()){
	        return false;
	    }
	    var table_name = 'annotation';
		var values = ['id'];
		statement = getValuesFromDatabase(fileObj, values, table_name);
		var noteId = new Array();
		if(typeof statement !== 'undefined' && statement !== false){
			while (statement.executeStep()){
				noteId.push(statement.row.id);
		    }
		    statement.finalize();
		}
		return noteId.pop();
	}
	
	/**
	 * Adds the new note to array for quick reference
	 * @params notesDescription Description of notes
	 * @params selectionString String selected
	 * @params serial Serialized value of notes
	 * @params currentTime Current time
	 * @params noteId Id of the note
	 */
	var addToArray = function(notesDescription, selectionString, serial, currentTime, noteId){
		var data = new Array();
		data.push(serial);
//		data.push(notesTitle);
		data.push(notesDescription);
		data.push(selectionString);
		data.push(currentTime);
		data.push(noteId);
		config.notesData.push(data);
	};
	
	var updateArray = function(notesDescription, currentTime, noteId){
		for(var i = 0; i < config.notesData.length; i++){
			if(noteId == config.notesData[i][4]){
				config.notesData[i][1] = notesDescription;
				config.notesData[i][3] = currentTime;
				break;
			}
		}
	}
	
//		<ul id="note-container">
//			<li id="notes-item">
//				<h4><span id="notes-section-title">Chapter 1:: Tiger Tiger</span> <span id="notes-date">Thu Jun 16 2011</span> <span class="delete_note">Delete</span></h4>
//				<p id="notes-highlight">for discussions over the past couple years about feature detection and particularly to David Mark for the long discussions over the past three months. David's code has stood the scrutiny of the group and are </p>
//		    </li>
//		</ul>
	
	/**
	 * Create and append all notes page
	 */
	var createAllNotesPage = function(){
		try{
			var dateObj = new Date();
			
			for(var i = 0; i < config.allNotesData.length; i++){
				dateObj.setTime(config.allNotesData[i][3]);
				var shortDescription = config.allNotesData[i][1];
				if(shortDescription.length > 20){
					shortDescription = shortDescription.substring(0, 18) + '...';
				}
				
				var notesPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('ul'))
								  .attr('id', 'note-container')
								  .data('id', config.allNotesData[i][4])
								  .data('chapter', config.allNotesData[i][5])
								  .append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('li'))
							  			.attr('id', 'notes-item')
										.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('h4'))
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.attr('id', 'notes-section-title')
												.click(function(){
													AME.goToNote($(this).parent().parent().parent().data('id'), $(this).parent().parent().parent().data('chapter'));
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(shortDescription)))
											)
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.attr('id', 'notes-date')
												.click(function(){
													AME.goToNote($(this).parent().parent().parent().data('id'), $(this).parent().parent().parent().data('chapter'));
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(dateObj.toDateString())))
											)
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.addClass('delete_note')
												.click(function(){
													AME.deleteNote($(this).parent().parent().parent().data('id'), this);
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode('Delete')))
											)
										)
										.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('p'))
											.attr('id', 'notes-highlight')
											.click(function(){
												AME.goToNote($(this).parent().parent().data('id'), $(this).parent().parent().data('chapter'));
											})
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(config.allNotesData[i][2])))
										)
								   )
				$('#doc-viewer-azd').contents().find('#notes_placeholder').append(notesPage);
			}
		}catch(e){alert(e)}
	}
	
	/**
	 * Sort all the notes data depending upon the date added
	 */
	var sortAllNotes = function(){
		var notesSort = function(a, b){
			if(parseInt(a[3]) < parseInt(b[3]))
	            return 1;
	        else if(parseInt(a[3]) > parseInt(b[3]))
	        	return -1;
	        else
	        	return 0;
	    }
		
		config.allNotesData.sort(notesSort);
	};
	
	/**
	 * 	Deletes the note from the database
	 * @params noteId Id of the note to be deleted
	 */
	var deleteNoteFromDatabase = function(noteId){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'annotation';
		var conditions = new Array(['id', noteId, 'INTEGER']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
	};
	
	/**
	 * Deletes all the notes for this book from database
	 * @params bookId Id of the book to be deleted
	 */
	var deleteAllNotesFromDatabase = function(bookId){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'annotation';
		var conditions = new Array(['book_id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
	}
	
	/**
	 * Delete the note from the full page
	 */
	var deleteNoteFromFullPage = function(deleteButton){
		$(deleteButton).parent().parent().parent().remove();
	};
	
	return{
		config: config,
		isTextSelected: isTextSelected,
//		getSelectedParagraph: getSelectedParagraph,
//		getStartOffset: getStartOffset,
//		getSelectionLength: getSelectionLength,
		createEditDialog: createEditDialog,
		getSelectedString: getSelectedString,
		save: save,
		getNotes: getNotes,
		getAllNotes: getAllNotes,
		highlight: highlight,
		highlightOnTheFly: highlightOnTheFly,
		setEditDialog: setEditDialog,
		getLatestId: getLatestId,
		addToArray: addToArray,
		updateArray: updateArray,
		createAllNotesPage: createAllNotesPage,
		sortAllNotes: sortAllNotes,
		deleteNoteFromDatabase: deleteNoteFromDatabase,
		deleteAllNotesFromDatabase: deleteAllNotesFromDatabase,
		deleteNoteFromFullPage: deleteNoteFromFullPage
	}
}();