/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary to renew catalog.
 */
var RenewCat = function(){
	var config = {
		
	}
	
	/**
	 * Initilizes renewal of the catalog
	 * @params renewCode Renew code entered by the user
	 * @params catId Id of the catalog to be renewed
	 * @params readerToken Token Id of the reader
	 * @params renewCatUrl URL to send catalog renewal information
	 */
	var init = function (renewCode, catId, readerToken, renewCatUrl) {
		var self = this;
		
		var request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
					  .createInstance(Components.interfaces.nsIXMLHttpRequest);
		request.onload = function(aEvent) {
			var text = aEvent.target.responseText;
			try{
				var jsObject = JSON.parse(text);
			}
			catch(e){
				//If the json is invalid.
				var msg = getPropertiesString("azd_renew_cat_Fail");
				alert(msg);
				window.arguments[0].out.out = false;
				window.close();
			}
			var rc = jsObject.expiry_date;
			
			if(rc == 'False') {
				var msg = getPropertiesString("azd_renew_cat_invalidKey");
				alert(msg);
			}
			else if(rc == ''){
				var msg = getPropertiesString("azd_renew_cat_Fail");
				alert(msg);
				window.arguments[0].out.out = false;
				window.close();
			}
			else{
				//Update expiry date of catalog
				self.updateDatabase(rc, catId);
				window.arguments[0].out.out = true;
				window.close();
			}
		};
		request.onerror = function(aEvent) {
			var msg = getPropertiesString("azd_renew_cat_errStatus");
			window.alert(msg + " " + aEvent.target.status);
			window.arguments[0].out.out = false;
			window.close();
		};
		request.open("POST", renewCatUrl, true);
		request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		request.send("readerid=" + readerToken + "&activationkey=" + renewCode + "&id=" + catId+ "&type=catalog");
	};
	
	/**
	 * Updates the expiry date in database.
	 * @params renewCode The new expiry date
	 * @params catId Id of the catalog to be renewed
	 */
	var updateDatabase = function (renewCode, catId) {
		var self = this;
		
		//Encrypt the expiry date using catalog id as a key
		renewCode = window.opener.GibberishAES.enc(renewCode, catId);
		var fileObj = window.opener.getFileObject(window.opener.AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';
		var values = ['raf_value'];
		var conditions = new Array(['field_id', catId, 'TEXT'],
								   ['raf_key', 'expiry_date', 'TEXT']);
		var statement = window.opener.getValuesFromDatabase(fileObj, values, table_name, conditions);
		while (statement.executeStep()){
			var raf_value = statement.row.raf_value;
		}
		
		var values = new Array(['field_id', catId, 'TEXT'],
							   ['raf_key', 'expiry_date', 'TEXT'],
							   ['raf_value', renewCode, 'TEXT'],
							   ['type', 'CAT', 'TEXT']);
							   
		//	Store value for first time.
		if (typeof raf_value == 'undefined'){
			window.opener.insertRowIntoDatabase(fileObj, values, table_name)
		}
		
		//	If value exists then update it.
		else{
			window.opener.updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
	};
	
	return{
		config: config,
		init: init,
		updateDatabase: updateDatabase
	};
}();