/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary to handle the smil file
 */
var SmilFLO = function() {
	/**
	 * Initilize the smil for use
	 * @params smilId Manifest Id of the .smil file
	 */
	var init = function(smilId, targetframe, srcpage) {
		if (smilId == "") return;
		
		targetframe = "#" + targetframe;
		//Get SMIL Data for current chapter.
		var smilData = parent.Epub.getChapterSmilData(smilId);
		if (!smilData) return;
		var audioPath = smilData[0];
		var mediatype = parent.Epub.getMediaType(audioPath);
		
		//Change audio file extension .ogg.
		audioPathResp = processAudioPath(audioPath, mediatype);
		audioPath =  audioPathResp[0];
		if (audioPathResp[1]) {
			mediatype = mediatype.split("/").shift() + "/ogg";
		}
		
		//Create the time-line from the smil data.
		var timeLine = createTimeline(smilData, srcpage);
		if (timeLine) {
			//Set the audio tag in page.
			setAudioTag(audioPath, targetframe, mediatype);
			//Set the timeline in page.
			setTimeline(timeLine, targetframe);
			addClass(targetframe);
		}
	};
	/**
	 * Change the audio file extension to .ogg
	 * @params audioPath Path of the audio file
	 * @returns {string} New audio path
	 */
	var processAudioPath = function(audioPath, mediatype) {
		var audPathArr = audioPath.split('.');
		var ext = audPathArr.pop();
		var isWin = false;
		var isWinMedia = false;
		var replaced = false;
		
		//We no longer need to fall back to ogg format for windows as xulrunner22+
		//supports mpeg. But we need to provide fallback for the other two platforms
		var osString = Components.classes["@mozilla.org/xre/app-info;1"]
               .getService(Components.interfaces.nsIXULRuntime).OS;
		if(osString.indexOf("WINNT") > -1)isWin = true;
		
		var mtype = mediatype.split("/").pop();
		if (mtype == "mpeg" || mtype == "mp3" || mtype == "mp4") {
			isWinMedia = true;
		}
		
		if (!isWin && isWinMedia) {
			if(ext !== 'ogg') {
				audPathArr.push('ogg');
				audioPath = audPathArr.join('.');
				replaced = true;
			}
		} 
		
		return [audioPath, replaced];
	};
	/**
	 * Create and append audio tag with new audio path
	 * @params audioPath New audio path for the current chapter
	 */
	var setAudioTag = function(audioPath, targetframe, mediatype) {
		var targetframeid = targetframe.replace("#", "");
		var audTag = $(document.getElementById(targetframeid).contentWindow.document.createElement('audio')).addClass('audioPlayer').append($(document.getElementById(targetframeid).contentWindow.document.createElement('source')).attr('type', mediatype).attr('src', audioPath));
		$(targetframe).contents().find('#custom-style-igp').prepend(audTag);
	};
	/**
	 * Create the tile line which AIE can understand
	 * @params smilData The smil data for the current chapter
	 * @return {string} Tileline
	 */
	var createTimeline = function(smilData, srcpage) {
		var timeLine = '<pre class="timeline">\n';
		var timeLineInner = '';
		
		for(var i = 1; i < smilData.length; i++) {
			var t_data = smilData[i].slice(0, smilData[i].length);
			var currfn = t_data[0].split("#").shift();
			var found = false;
			if (currfn.indexOf(srcpage) > -1) {
				t_data[0] = "#" + t_data[0].split("#").pop();
				found = true;
			}
			if (found) {
				var instruction = t_data.join(" ");
				timeLineInner += instruction + '\n';
			}
		}
		
		if (timeLineInner ) {
			timeLine += timeLineInner;
			timeLine += '</pre>';
		} else {
			timeLine  = "";
		}
		
		return timeLine;
	};
	/**
	 * Set the timeline in the page
	 * @params timeLine timeLine string which AIE understand
	 */
	var setTimeline = function(timeLine, targetframe) {
		$(targetframe).contents().find('#custom-style-igp').prepend(timeLine);
	};
	/**
	 * Wrap the content of page with the class that AIE understands
	 */
	var addClass = function(targetframe) {
		$(targetframe).contents().find('#custom-style-igp').wrapInner('<div class="aie-audio-text-rw" />');
	};
	return {
		init : init
	};
}();
