/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for interacting with toolbar.
 */
var Toolbar = function(){
	var config = {
		mainToolbar: '#toolbar-nav-azd',
		bookToolbar: '#toolbar-tools-azd',
		bookFontPlus: '#font-size-plus',
		bookFontMinus: '#font-size-minus',
		bookReset: '#reset-all',
		bookAlign: '#but_align_ctrl',
		bookHyphen: '#but_hypnate',
		bookMarginPlus: '#margin-plus',
		bookMarginMinus: '#margin-minus',
		bookColor: '#but_color_ctrl',
		bookFlow: '#scroll_view',
		bookColumn1: '#column_1',
		bookColumn2: '#column_2',
		bookColumn3: '#column_3',
		bookColumn4: '#column_4',
		columnRule: '#but_column_rule',
		bookToc: '#but_toc',
		bookLandmark: '#but_landmark',
		bookPrevious: '#but_previous',
		bookNext: '#but_next',
		tableView: '#but_table',
		thumbView: '#but_thumb',
		back: '#but_back',
		userPref: '#but_usr_pref',
		myAccount: '#but_my_acc',
		manageShelf: '#but_manage_shelf',
		addShelf: '#but_add_new_shelf',
		addPubShelf: '#but_add_new_pubshelf',
		refreshPubShelf: '#but_refresh_pubshelf',
		showShelf: '#but_show_shelf',
		addBook: '#but_add_doc',
		library: '#but_library',
		about: '#but_about',
		bookLinePlus: '#lineheight-plus',
		bookLineMinus: '#lineheight-minus',
		bookPref: '#but_book_pref',
		audioOptions: '#but_audio_control',
		audioPlay: '#but_audio_play',
		audioStop: '#but_audio_stop',
		audioDefPlayRate: '#but_audio_def_rate',
		aud_playback_rate: '#aud_playback_rate',
		floBackground: '#but_background',
		floAspectEase: '#but_aspect-easing',
		floZoom: '#but_zoom',
		floGutter: '#but_gutter',
		floFullScreen: '#but_fullscreen',
		floPrintPageNum: '#but_print-pagenum'
	};
	
	/**
	 * Initlize the toolbar for first time.
	 */
	var init = function(){
		//Only show toolbar necessary for library, hide others.
		$(config.bookToolbar).hide()
		$(config.floFullScreen).hide()
		$(config.floPrintPageNum).hide()
		$(config.mainToolbar).show();
		$(config.library).show();
	};
	
	var showHideMainToolbar = function(show) {
		if (show) {
			$(config.mainToolbar).show();
		} else {
			$(config.mainToolbar).hide();
		}
	}
	
	/**
	 * Show buttons related to library.
	 */
	var showLibrary = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'mainToolbar':
			    case 'tableView':
				case 'thumbView':
				case 'userPref':
				case 'manageShelf':
				case 'addShelf':
				case 'addPubShelf':
				case 'refreshPubShelf':
				case 'showShelf':
				case 'addBook':
				case 'about':
				case 'library':
			    	$(value).show();
			        break;
			    default:
					$(value).hide();
			}
		});
	}
	
	/**
	 * Function to close the library toolbar.
	 */
	var closeLibrary = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'mainToolbar':
			    case 'tableView':
				case 'thumbView':
				case 'userPref':
				case 'manageShelf':
				case 'addShelf':
				case 'addPubShelf':
				case 'showShelf':
				case 'refreshPubShelf':
				case 'addBook':
				case 'about':
				case 'library':
			    	$(value).hide();
			        break;
			    default:
					$(value).show();
			}
		});
	};
	
	var showAzd = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'mainToolbar':
				case 'library':
			    	$(value).show();
			        break;
			    default:
					$(value).hide();
			}
		});
	}
	
	var closeAzd = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'mainToolbar':
				case 'library':
			    	$(value).hide();
			        break;
			    default:
					$(value).show();
			}
		});
	}
	
	/**
	 * Shows the epub related buttons.
	 */
	var showEpub = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'floFullScreen':
				case 'floPrintPageNum':
				case 'mainToolbar':
			    case 'bookToolbar':
				case 'bookFontPlus':
				case 'bookFontMinus':
				case 'bookReset':
				case 'bookLinePlus':
				case 'bookLineMinus':
				case 'bookAlign':
				case 'bookHyphen':
				case 'bookMarginPlus':
				case 'bookMarginMinus':
				case 'bookColor':
				case 'bookFlow':
				case 'bookColumn1':
				case 'bookColumn2':
				case 'bookColumn3':
				case 'bookColumn4':
				case 'columnRule':
				case 'bookToc':
				case 'bookLandmark':
				case 'bookPrevious':
				case 'bookNext':
				case 'back':
				case 'userPref':
				case 'library':
				case 'bookPref':
				case 'audioOptions':
			    	$(value).show();
			        break;
			    default:
					$(value).hide();
			}
		});
	};
	
	/**
	 * Hides epub related buttons from toolbar.
	 */
	var closeEpub = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'floFullScreen':
				case 'floPrintPageNum':
				case 'mainToolbar':
			    case 'bookToolbar':
				case 'bookFontPlus':
				case 'bookFontMinus':
				case 'bookReset':
				case 'bookLinePlus':
				case 'bookLineMinus':
				case 'bookAlign':
				case 'bookHyphen':
				case 'bookMarginPlus':
				case 'bookMarginMinus':
				case 'bookColor':
				case 'bookFlow':
				case 'bookColumn1':
				case 'bookColumn2':
				case 'bookColumn3':
				case 'bookColumn4':
				case 'columnRule':
				case 'bookToc':
				case 'bookLandmark':
				case 'bookPrevious':
				case 'bookNext':
				case 'back':
				case 'userPref':
				case 'bookPref':
				case 'library':
			    	$(value).hide();
			        break;
			    default:
					$(value).show();
			}
		});
	};
	
	/**
	 * Show full-page toc related buttons on toolbar
	 */
	var showFullPageToc = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'mainToolbar':
				case 'bookToc':
				case 'library':
			    	$(value).show();
			        break;
			    default:
					$(value).hide();
			}
		});
	}
	
	/**
	 * Shows landmark related buttons on toolbar
	 */
	var showLandmark = function(){
		$.each(config, function(key, value){
			switch(key){
				case 'mainToolbar':
				case 'bookLandmark':
				case 'library':
			    	$(value).show();
			        break;
			    default:
					$(value).hide();
			}
		});
	}
	
	var hideFullPageToc = function(){
		
	}
	
	/**
	 * Disables the back button
	 */
	var disableBackButton = function(){
		$(config.back).attr('disabled', 'true');
		$(config.back).attr('image', 'icons/ico_back-mute.png');
	}
	
	var enableBackButton = function(){
		$(config.back).attr('disabled', 'false');
		$(config.back).attr('image', 'icons/ico_back.png');
	}
	
	/**
	 * Disables the hyphenate button
	 */
	var disableHyphenateButton = function(){
		$(config.bookHyphen).attr('disabled', 'true');
		$(config.bookHyphen).attr('image', 'icons/ico_hyphenation-mute.png');
	}
	
	/**
	 * Enables the hyphenate button
	 */
	var enableHyphenateButton = function(){
		$(config.bookHyphen).attr('disabled', 'false');
		$(config.bookHyphen).attr('image', 'icons/ico_hyphenation.png');
	}
	
	/**
	 * Disables the next button
	 */
	var disableNextButton = function(){
		$(config.bookNext).attr('disabled', 'true');
		$(config.bookNext).attr('image', 'icons/ico_next-page-mute.png');
	};
	
	/**
	 * Enables the next button
	 */
	var enableNextButton = function(){
		$(config.bookNext).attr('disabled', 'false');
		$(config.bookNext).attr('image', 'icons/ico_next-page.png');
	};
	
	/**
	 * Disables the previous button
	 */
	var disablePreviousButton = function(){
		$(config.bookPrevious).attr('disabled', 'true');
		$(config.bookPrevious).attr('image', 'icons/ico_previous-page-mute.png');
	};
	
	/**
	 * Enable the previous button
	 */
	var enablePreviousButton = function(){
		$(config.bookPrevious).attr('disabled', 'false');
		$(config.bookPrevious).attr('image', 'icons/ico_previous-page.png');
	};
	
	/**
	 * Disables the columns buttons
	 */
	var disableColumnsButton = function(){
		$(config.bookFlow).attr('disabled', 'true');
		$(config.bookColumn1).attr('disabled', 'true');
		$(config.bookColumn2).attr('disabled', 'true');
		$(config.bookColumn3).attr('disabled', 'true');
		$(config.bookColumn4).attr('disabled', 'true');
		$(config.bookFlow).attr('image', 'icons/ico_page-scrolling-mute.png');
		$(config.bookColumn1).attr('image', 'icons/ico_column1-mute.png');
		$(config.bookColumn2).attr('image', 'icons/ico_column2-mute.png');
		$(config.bookColumn3).attr('image', 'icons/ico_column3-mute.png');
		$(config.bookColumn4).attr('image', 'icons/ico_column4-mute.png');
	};
	
	/**
	 * Enables the columns buttons
	 */
	var enableColumnsButton = function(){
		$(config.bookFlow).attr('disabled', 'false');
		$(config.bookColumn1).attr('disabled', 'false');
		$(config.bookColumn2).attr('disabled', 'false');
		$(config.bookColumn3).attr('disabled', 'false');
		$(config.bookColumn4).attr('disabled', 'false');
		$(config.bookFlow).attr('image', 'icons/ico_page-scrolling.png');
		$(config.bookColumn1).attr('image', 'icons/ico_column1.png');
		$(config.bookColumn2).attr('image', 'icons/ico_column2.png');
		$(config.bookColumn3).attr('image', 'icons/ico_column3.png');
		$(config.bookColumn4).attr('image', 'icons/ico_column4.png');
	};
	
	/**
	 * Disables the column rule button
	 */
	var disableColumnRuleButton = function() {
		$(config.columnRule).attr('disabled', 'true');
		$(config.columnRule).attr('image', 'icons/ico_column-rule-mute.png');
	}
	
	/**
	 * Enables the column rule button
	 */
	var enableColumnRuleButton = function() {
		$(config.columnRule).attr('disabled', 'false');
		$(config.columnRule).attr('image', 'icons/ico_column-rule.png');
	}
	
	/**
	 * Disables the TOC button
	 */
	var disableTocButton = function(){
		$(config.bookToc).attr('disabled', 'true');
		$(config.bookToc).attr('image', 'icons/ico_toc-mute.png');
	}
	
	/**
	 * Enables the TOC button
	 */
	var enableTocButton = function(){
		$(config.bookToc).attr('disabled', 'false');
		$(config.bookToc).attr('image', 'icons/ico_toc.png');
	}
	
	/**
	 * Disables the landmark button
	 */
	var disableLandmarkButton = function () {
		$(config.bookLandmark).attr('disabled', 'true');
		$(config.bookLandmark).attr('image', 'icons/ico_landmark-mute.png');
	}
	
	/**
	 * Enables the landmark button
	 */
	var enableLandmarkButton = function () {
		$(config.bookLandmark).attr('disabled', 'false');
		$(config.bookLandmark).attr('image', 'icons/ico_landmark.png');
	}
	
	var disableColorButton = function(){
		$(config.bookColor).attr('disabled', 'true');
		$('#but_color_normal').attr('disabled', 'true');
		$('#but_color_warm').attr('disabled', 'true');
		$('#but_color_low_contrast').attr('disabled', 'true');
		$(config.bookColor).attr('image', 'icons/ico_background-color-mute.png');
	}
	
	var enableColorButton = function(){
		$(config.bookColor).attr('disabled', 'false');
		$('#but_color_normal').attr('disabled', 'false');
		$('#but_color_warm').attr('disabled', 'false');
		$('#but_color_low_contrast').attr('disabled', 'false');
		$(config.bookColor).attr('image', 'icons/ico_background-color.png');
	}
	
	/**
	 * Disables the justify button
	 */
	var disableJustifyButton = function(){
		$(config.bookAlign).attr('disabled', 'true');
		$(config.bookAlign).attr('image', 'icons/ico_justfy-mute.png');
	}
	
	/**
	 * Enables the justify button
	 */
	var enableJustifyButton = function(){
		$(config.bookAlign).attr('disabled', 'false');
		$(config.bookAlign).attr('image', 'icons/ico_justfy.png');
	}
	
	/**
	 * Disables the line space button
	 */
	var disableLineSpaceButton = function(){
		$(config.bookLinePlus).attr('disabled', 'true');
		$(config.bookLineMinus).attr('disabled', 'true');
		$(config.bookLinePlus).attr('image', 'icons/ico_line-spacing-mute+.png');
		$(config.bookLineMinus).attr('image', 'icons/ico_line-spacing-mute-.png');
	}
	
	/**
	 * Enables the line space button
	 */
	var enableLineSpaceButton = function(){
		$(config.bookLinePlus).attr('disabled', 'false');
		$(config.bookLineMinus).attr('disabled', 'false');
		$(config.bookLinePlus).attr('image', 'icons/ico_line-spacing+.png');
		$(config.bookLineMinus).attr('image', 'icons/ico_line-spacing-.png');
	}
	
	/**
	 * Disables the margin button
	 */
	var disableMarginSpaceButton = function(){
		$(config.bookMarginPlus).attr('disabled', 'true');
		$(config.bookMarginMinus).attr('disabled', 'true');
		$(config.bookMarginPlus).attr('image', 'icons/ico_margin-mute+.png');
		$(config.bookMarginMinus).attr('image', 'icons/ico_margin-mute-.png');
	}
	
	/**
	 * Enables the margin button
	 */
	var enableMarginSpaceButton = function(){
		$(config.bookMarginPlus).attr('disabled', 'false');
		$(config.bookMarginMinus).attr('disabled', 'false');
		$(config.bookMarginPlus).attr('image', 'icons/ico_margin+.png');
		$(config.bookMarginMinus).attr('image', 'icons/ico_margin-.png');
	}
	
	/**
	 * Disables the font size buttons
	 */
	var disableFontSizeButton = function(){
		$(config.bookFontPlus).attr('disabled', 'true');
		$(config.bookFontMinus).attr('disabled', 'true');
		$(config.bookFontPlus).attr('image', 'icons/ico_font-size-mute+.png');
		$(config.bookFontMinus).attr('image', 'icons/ico_font-size-mute-.png');
	}
	
	/**
	 * Enables the font size buttons
	 */
	var enableFontSizeButton = function(){
		$(config.bookFontPlus).attr('disabled', 'false');
		$(config.bookFontMinus).attr('disabled', 'false');
		$(config.bookFontPlus).attr('image', 'icons/ico_font-size+.png');
		$(config.bookFontMinus).attr('image', 'icons/ico_font-size-.png');
	}
	
	/**
	 * Disables the reset button
	 */
	var disableResetButton = function() {
		$(config.bookReset).attr('disabled', 'true');
		$(config.bookReset).attr('image', 'icons/ico_reset-mute.png');
	};
	
	/**
	 * Enables the reset button
	 */
	var enableResetButton = function() {
		$(config.bookReset).attr('disabled', 'false');
		$(config.bookReset).attr('image', 'icons/ico_reset.png');
	};
	
	/**
	 * Disables the user pref button
	 */
	var disableUserPrefButton = function() {
		$(config.userPref).attr('disabled', 'true');
		$(config.userPref).attr('image', 'icons/ico_user-preferences-mute.png');
	};
	
	/**
	 * Enable the user preferences button
	 */
	var enableUserPrefButton = function() {
		$(config.userPref).attr('disabled', 'false');
		$(config.userPref).attr('image', 'icons/ico_user-preferences.png');
	};
	
	/**
	 * Disables the audio options button
	 */
	var disableAudioOptionsButton = function() {
		$(config.audioOptions).removeAttr('checked');
		$(config.audioOptions).attr('disabled', 'true');
		$(config.audioOptions).attr('image', 'icons/ico_audio-speaker-mute.png');
	};
	
	/**
	 * Enables the audio options button
	 */
	var enableAudioOptionsButton = function() {
		$(config.audioOptions).removeAttr('checked');
		$(config.audioOptions).attr('disabled', 'false');
		$(config.audioOptions).attr('image', 'icons/ico_audio-speaker.png');
	};
	
	/**
	 * Disables the book preferences button
	 */
	var disableBookPrefButton = function() {
		$(config.bookPref).attr('disabled', 'true');
		$(config.bookPref).attr('image', 'icons/ico_book-preferences-mute.png');
	}
	
	/**
	 * Enables the book preferences button
	 */
	var enableBookPrefButton = function() {
		$(config.bookPref).attr('disabled', 'false');
		$(config.bookPref).attr('image', 'icons/ico_book-preferences.png');
	}
	
	/**
	 * Disables all the epub related buttons
	 */
	var disableEpubBtn = function() {
		var self = this;
		
		self.disableBackButton();
		self.disableHyphenateButton();
		self.disableNextButton();
		self.disablePreviousButton();
		self.disableColumnsButton();
		self.disableColumnRuleButton();
		self.disableTocButton();
		self.disableLandmarkButton();
		self.disableFontSizeButton();
		self.disableResetButton();
		self.disableMarginSpaceButton();
		self.disableLineSpaceButton();
		self.disableJustifyButton();
		// self.disableColorButton();
		self.disableUserPrefButton();
		self.disableBookPrefButton();
		self.disableAudioOptionsButton();
	};
	
	var enableEpubBtn = function() {
		var self = this;
		
		self.enableHyphenateButton();
		self.enableNextButton();
		self.enablePreviousButton();
		self.enableColumnsButton();
		self.enableColumnRuleButton();
		self.enableTocButton();
		self.enableLandmarkButton();
		self.enableFontSizeButton();
		self.enableResetButton();
		self.enableMarginSpaceButton();
		self.enableLineSpaceButton();
		self.enableJustifyButton();
		// self.enableColorButton();
		self.enableUserPrefButton();
		self.enableBookPrefButton();
		self.enableAudioOptionsButton();
	}
	
	var showAudioControls = function() {
		$.each(config, function(key, value){
			switch(key){
				case 'audioPlay':
				case 'audioStop':
				case 'audioDefPlayRate':
				case 'aud_playback_rate':
			    	$(value).show();
			        break;
			    default:
			}
		});
	};
	
	/**
	 * Hides the Audio Controls
	 */
	var hideAudioControls = function() {
		$.each(config, function(key, value){
			switch(key){
				case 'audioPlay':
				case 'audioStop':
				case 'audioDefPlayRate':
				case 'aud_playback_rate':
			    	$(value).hide();
			        break;
			    default:
			}
		});
	};
	
	/**
	 * Shows FLO related buttons from toolbar.
	 */
	var setFLOToolbarDisplay = function(show){
		$.each(config, function(key, value){
			switch(key){
				case 'floGutter':
				case 'floZoom':
				case 'floAspectEase':
				case 'floBackground':
					if (show) {
						$(value).show();
					} else {
						$(value).hide();
					}
			        break;
			    default:
					break;
			}
		});
	};
	
	return{
		init: init,
		showLibrary: showLibrary,
		closeLibrary: closeLibrary,
		showAzd: showAzd,
		closeAzd: closeAzd,
		showEpub: showEpub, 
		closeEpub: closeEpub,
		showFullPageToc: showFullPageToc,
		hideFullPageToc: hideFullPageToc,
		disableBackButton: disableBackButton,
		enableBackButton: enableBackButton,
		disableHyphenateButton: disableHyphenateButton,
		enableHyphenateButton: enableHyphenateButton,
		disableNextButton: disableNextButton,
		enableNextButton: enableNextButton,
		disablePreviousButton: disablePreviousButton,
		enablePreviousButton: enablePreviousButton,
		disableColumnsButton: disableColumnsButton,
		enableColumnsButton: enableColumnsButton,
		disableColumnRuleButton: disableColumnRuleButton,
		enableColumnRuleButton: enableColumnRuleButton,
		disableTocButton: disableTocButton,
		enableTocButton: enableTocButton,
		disableLandmarkButton: disableLandmarkButton,
		enableLandmarkButton: enableLandmarkButton,
		enableFontSizeButton: enableFontSizeButton,
		disableFontSizeButton: disableFontSizeButton,
		enableMarginSpaceButton: enableMarginSpaceButton,
		disableMarginSpaceButton: disableMarginSpaceButton,
		enableLineSpaceButton: enableLineSpaceButton,
		disableLineSpaceButton: disableLineSpaceButton,
		enableJustifyButton: enableJustifyButton,
		disableJustifyButton: disableJustifyButton,
		enableColorButton: enableColorButton,
		disableColorButton: disableColorButton,
		disableEpubBtn: disableEpubBtn,
		enableEpubBtn: enableEpubBtn,
		disableUserPrefButton: disableUserPrefButton,
		enableUserPrefButton: enableUserPrefButton,
		showLandmark: showLandmark,
		hideAudioControls: hideAudioControls,
		showAudioControls: showAudioControls,
		disableAudioOptionsButton: disableAudioOptionsButton,
		enableAudioOptionsButton: enableAudioOptionsButton,
		disableBookPrefButton: disableBookPrefButton,
		enableBookPrefButton: enableBookPrefButton,
		disableResetButton: disableResetButton,
		enableResetButton: enableResetButton,
		setFLOToolbarDisplay: setFLOToolbarDisplay,
		showHideMainToolbar: showHideMainToolbar,
	}
}();